# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: WAProto.proto
# Protobuf Python Version: 6.32.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    32,
    1,
    '',
    'WAProto.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rWAProto.proto\x12\x05proto\"\x81\x02\n\x11\x41\x44VDeviceIdentity\x12\x12\n\x05rawId\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x16\n\ttimestamp\x18\x02 \x01(\x04H\x01\x88\x01\x01\x12\x15\n\x08keyIndex\x18\x03 \x01(\rH\x02\x88\x01\x01\x12\x32\n\x0b\x61\x63\x63ountType\x18\x04 \x01(\x0e\x32\x18.proto.ADVEncryptionTypeH\x03\x88\x01\x01\x12\x31\n\ndeviceType\x18\x05 \x01(\x0e\x32\x18.proto.ADVEncryptionTypeH\x04\x88\x01\x01\x42\x08\n\x06_rawIdB\x0c\n\n_timestampB\x0b\n\t_keyIndexB\x0e\n\x0c_accountTypeB\r\n\x0b_deviceType\"\xdf\x01\n\x0f\x41\x44VKeyIndexList\x12\x12\n\x05rawId\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x16\n\ttimestamp\x18\x02 \x01(\x04H\x01\x88\x01\x01\x12\x19\n\x0c\x63urrentIndex\x18\x03 \x01(\rH\x02\x88\x01\x01\x12\x18\n\x0cvalidIndexes\x18\x04 \x03(\rB\x02\x10\x01\x12\x32\n\x0b\x61\x63\x63ountType\x18\x05 \x01(\x0e\x32\x18.proto.ADVEncryptionTypeH\x03\x88\x01\x01\x42\x08\n\x06_rawIdB\x0c\n\n_timestampB\x0f\n\r_currentIndexB\x0e\n\x0c_accountType\"\xdb\x01\n\x17\x41\x44VSignedDeviceIdentity\x12\x14\n\x07\x64\x65tails\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12 \n\x13\x61\x63\x63ountSignatureKey\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x1d\n\x10\x61\x63\x63ountSignature\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x1c\n\x0f\x64\x65viceSignature\x18\x04 \x01(\x0cH\x03\x88\x01\x01\x42\n\n\x08_detailsB\x16\n\x14_accountSignatureKeyB\x13\n\x11_accountSignatureB\x12\n\x10_deviceSignature\"\x9f\x01\n\x1b\x41\x44VSignedDeviceIdentityHMAC\x12\x14\n\x07\x64\x65tails\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x11\n\x04hmac\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x32\n\x0b\x61\x63\x63ountType\x18\x03 \x01(\x0e\x32\x18.proto.ADVEncryptionTypeH\x02\x88\x01\x01\x42\n\n\x08_detailsB\x07\n\x05_hmacB\x0e\n\x0c_accountType\"\xa7\x01\n\x15\x41\x44VSignedKeyIndexList\x12\x14\n\x07\x64\x65tails\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x1d\n\x10\x61\x63\x63ountSignature\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12 \n\x13\x61\x63\x63ountSignatureKey\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x42\n\n\x08_detailsB\x13\n\x11_accountSignatureB\x16\n\x14_accountSignatureKey\"\xfb\x04\n\x0b\x41IHomeState\x12\x1a\n\rlastFetchTime\x18\x01 \x01(\x03H\x00\x88\x01\x01\x12:\n\x11\x63\x61pabilityOptions\x18\x02 \x03(\x0b\x32\x1f.proto.AIHomeState.AIHomeOption\x12<\n\x13\x63onversationOptions\x18\x03 \x03(\x0b\x32\x1f.proto.AIHomeState.AIHomeOption\x1a\xc3\x03\n\x0c\x41IHomeOption\x12\x43\n\x04type\x18\x01 \x01(\x0e\x32\x30.proto.AIHomeState.AIHomeOption.AIHomeActionTypeH\x00\x88\x01\x01\x12\x12\n\x05title\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x17\n\npromptText\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x16\n\tsessionId\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x1f\n\x12imageWdsIdentifier\x18\x05 \x01(\tH\x04\x88\x01\x01\x12\x1b\n\x0eimageTintColor\x18\x06 \x01(\tH\x05\x88\x01\x01\x12!\n\x14imageBackgroundColor\x18\x07 \x01(\tH\x06\x88\x01\x01\"U\n\x10\x41IHomeActionType\x12\n\n\x06PROMPT\x10\x00\x12\x10\n\x0c\x43REATE_IMAGE\x10\x01\x12\x11\n\rANIMATE_PHOTO\x10\x02\x12\x10\n\x0c\x41NALYZE_FILE\x10\x03\x42\x07\n\x05_typeB\x08\n\x06_titleB\r\n\x0b_promptTextB\x0c\n\n_sessionIdB\x15\n\x13_imageWdsIdentifierB\x11\n\x0f_imageTintColorB\x17\n\x15_imageBackgroundColorB\x10\n\x0e_lastFetchTime\"\xa2\x01\n\rAIQueryFanout\x12*\n\nmessageKey\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x12$\n\x07message\x18\x02 \x01(\x0b\x32\x0e.proto.MessageH\x01\x88\x01\x01\x12\x16\n\ttimestamp\x18\x03 \x01(\x03H\x02\x88\x01\x01\x42\r\n\x0b_messageKeyB\n\n\x08_messageB\x0c\n\n_timestamp\"\x8b\x01\n\x14\x41IRegenerateMetadata\x12*\n\nmessageKey\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x12 \n\x13responseTimestampMs\x18\x02 \x01(\x03H\x01\x88\x01\x01\x42\r\n\x0b_messageKeyB\x16\n\x14_responseTimestampMs\"\xfd\x04\n\x1a\x41IRichResponseCodeMetadata\x12\x19\n\x0c\x63odeLanguage\x18\x01 \x01(\tH\x00\x88\x01\x01\x12M\n\ncodeBlocks\x18\x02 \x03(\x0b\x32\x39.proto.AIRichResponseCodeMetadata.AIRichResponseCodeBlock\x1a\xb4\x01\n\x17\x41IRichResponseCodeBlock\x12]\n\rhighlightType\x18\x01 \x01(\x0e\x32\x41.proto.AIRichResponseCodeMetadata.AIRichResponseCodeHighlightTypeH\x00\x88\x01\x01\x12\x18\n\x0b\x63odeContent\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x10\n\x0e_highlightTypeB\x0e\n\x0c_codeContent\"\xac\x02\n\x1f\x41IRichResponseCodeHighlightType\x12+\n\'AI_RICH_RESPONSE_CODE_HIGHLIGHT_DEFAULT\x10\x00\x12+\n\'AI_RICH_RESPONSE_CODE_HIGHLIGHT_KEYWORD\x10\x01\x12*\n&AI_RICH_RESPONSE_CODE_HIGHLIGHT_METHOD\x10\x02\x12*\n&AI_RICH_RESPONSE_CODE_HIGHLIGHT_STRING\x10\x03\x12*\n&AI_RICH_RESPONSE_CODE_HIGHLIGHT_NUMBER\x10\x04\x12+\n\'AI_RICH_RESPONSE_CODE_HIGHLIGHT_COMMENT\x10\x05\x42\x0f\n\r_codeLanguage\"\xe5\x04\n\"AIRichResponseContentItemsMetadata\x12\x62\n\ritemsMetadata\x18\x01 \x03(\x0b\x32K.proto.AIRichResponseContentItemsMetadata.AIRichResponseContentItemMetadata\x12O\n\x0b\x63ontentType\x18\x02 \x01(\x0e\x32\x35.proto.AIRichResponseContentItemsMetadata.ContentTypeH\x00\x88\x01\x01\x1a\x96\x01\n!AIRichResponseContentItemMetadata\x12T\n\x08reelItem\x18\x01 \x01(\x0b\x32@.proto.AIRichResponseContentItemsMetadata.AIRichResponseReelItemH\x00\x42\x1b\n\x19\x61IRichResponseContentItem\x1a\xb6\x01\n\x16\x41IRichResponseReelItem\x12\x12\n\x05title\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x1b\n\x0eprofileIconUrl\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x19\n\x0cthumbnailUrl\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x15\n\x08videoUrl\x18\x04 \x01(\tH\x03\x88\x01\x01\x42\x08\n\x06_titleB\x11\n\x0f_profileIconUrlB\x0f\n\r_thumbnailUrlB\x0b\n\t_videoUrl\"(\n\x0b\x43ontentType\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x00\x12\x0c\n\x08\x43\x41ROUSEL\x10\x01\x42\x0e\n\x0c_contentType\"\xa1\x03\n\x1d\x41IRichResponseDynamicMetadata\x12Y\n\x04type\x18\x01 \x01(\x0e\x32\x46.proto.AIRichResponseDynamicMetadata.AIRichResponseDynamicMetadataTypeH\x00\x88\x01\x01\x12\x14\n\x07version\x18\x02 \x01(\x04H\x01\x88\x01\x01\x12\x10\n\x03url\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x16\n\tloopCount\x18\x04 \x01(\rH\x03\x88\x01\x01\"\xb9\x01\n!AIRichResponseDynamicMetadataType\x12\x32\n.AI_RICH_RESPONSE_DYNAMIC_METADATA_TYPE_UNKNOWN\x10\x00\x12\x30\n,AI_RICH_RESPONSE_DYNAMIC_METADATA_TYPE_IMAGE\x10\x01\x12.\n*AI_RICH_RESPONSE_DYNAMIC_METADATA_TYPE_GIF\x10\x02\x42\x07\n\x05_typeB\n\n\x08_versionB\x06\n\x04_urlB\x0c\n\n_loopCount\"\x9e\x01\n\x1f\x41IRichResponseGridImageMetadata\x12\x38\n\x0cgridImageUrl\x18\x01 \x01(\x0b\x32\x1d.proto.AIRichResponseImageURLH\x00\x88\x01\x01\x12\x30\n\timageUrls\x18\x02 \x03(\x0b\x32\x1d.proto.AIRichResponseImageURLB\x0f\n\r_gridImageUrl\"\xa2\x01\n\x16\x41IRichResponseImageURL\x12\x1c\n\x0fimagePreviewUrl\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x1c\n\x0fimageHighResUrl\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x16\n\tsourceUrl\x18\x03 \x01(\tH\x02\x88\x01\x01\x42\x12\n\x10_imagePreviewUrlB\x12\n\x10_imageHighResUrlB\x0c\n\n_sourceUrl\"\xdb\x03\n!AIRichResponseInlineImageMetadata\x12\x34\n\x08imageUrl\x18\x01 \x01(\x0b\x32\x1d.proto.AIRichResponseImageURLH\x00\x88\x01\x01\x12\x16\n\timageText\x18\x02 \x01(\tH\x01\x88\x01\x01\x12]\n\talignment\x18\x03 \x01(\x0e\x32\x45.proto.AIRichResponseInlineImageMetadata.AIRichResponseImageAlignmentH\x02\x88\x01\x01\x12\x17\n\ntapLinkUrl\x18\x04 \x01(\tH\x03\x88\x01\x01\"\xb7\x01\n\x1c\x41IRichResponseImageAlignment\x12\x31\n-AI_RICH_RESPONSE_IMAGE_LAYOUT_LEADING_ALIGNED\x10\x00\x12\x32\n.AI_RICH_RESPONSE_IMAGE_LAYOUT_TRAILING_ALIGNED\x10\x01\x12\x30\n,AI_RICH_RESPONSE_IMAGE_LAYOUT_CENTER_ALIGNED\x10\x02\x42\x0b\n\t_imageUrlB\x0c\n\n_imageTextB\x0c\n\n_alignmentB\r\n\x0b_tapLinkUrl\"\xc4\x04\n\x1b\x41IRichResponseLatexMetadata\x12\x11\n\x04text\x18\x01 \x01(\tH\x00\x88\x01\x01\x12U\n\x0b\x65xpressions\x18\x02 \x03(\x0b\x32@.proto.AIRichResponseLatexMetadata.AIRichResponseLatexExpression\x1a\xb1\x03\n\x1d\x41IRichResponseLatexExpression\x12\x1c\n\x0flatexExpression\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x10\n\x03url\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x12\n\x05width\x18\x03 \x01(\x01H\x02\x88\x01\x01\x12\x13\n\x06height\x18\x04 \x01(\x01H\x03\x88\x01\x01\x12\x17\n\nfontHeight\x18\x05 \x01(\x01H\x04\x88\x01\x01\x12\x1c\n\x0fimageTopPadding\x18\x06 \x01(\x01H\x05\x88\x01\x01\x12 \n\x13imageLeadingPadding\x18\x07 \x01(\x01H\x06\x88\x01\x01\x12\x1f\n\x12imageBottomPadding\x18\x08 \x01(\x01H\x07\x88\x01\x01\x12!\n\x14imageTrailingPadding\x18\t \x01(\x01H\x08\x88\x01\x01\x42\x12\n\x10_latexExpressionB\x06\n\x04_urlB\x08\n\x06_widthB\t\n\x07_heightB\r\n\x0b_fontHeightB\x12\n\x10_imageTopPaddingB\x16\n\x14_imageLeadingPaddingB\x15\n\x13_imageBottomPaddingB\x17\n\x15_imageTrailingPaddingB\x07\n\x05_text\"\xb2\x04\n\x19\x41IRichResponseMapMetadata\x12\x1b\n\x0e\x63\x65nterLatitude\x18\x01 \x01(\x01H\x00\x88\x01\x01\x12\x1c\n\x0f\x63\x65nterLongitude\x18\x02 \x01(\x01H\x01\x88\x01\x01\x12\x1a\n\rlatitudeDelta\x18\x03 \x01(\x01H\x02\x88\x01\x01\x12\x1b\n\x0elongitudeDelta\x18\x04 \x01(\x01H\x03\x88\x01\x01\x12Q\n\x0b\x61nnotations\x18\x05 \x03(\x0b\x32<.proto.AIRichResponseMapMetadata.AIRichResponseMapAnnotation\x12\x19\n\x0cshowInfoList\x18\x06 \x01(\x08H\x04\x88\x01\x01\x1a\xd5\x01\n\x1b\x41IRichResponseMapAnnotation\x12\x1d\n\x10\x61nnotationNumber\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x15\n\x08latitude\x18\x02 \x01(\x01H\x01\x88\x01\x01\x12\x16\n\tlongitude\x18\x03 \x01(\x01H\x02\x88\x01\x01\x12\x12\n\x05title\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x11\n\x04\x62ody\x18\x05 \x01(\tH\x04\x88\x01\x01\x42\x13\n\x11_annotationNumberB\x0b\n\t_latitudeB\x0c\n\n_longitudeB\x08\n\x06_titleB\x07\n\x05_bodyB\x11\n\x0f_centerLatitudeB\x12\n\x10_centerLongitudeB\x10\n\x0e_latitudeDeltaB\x11\n\x0f_longitudeDeltaB\x0f\n\r_showInfoList\"\xaf\x02\n\x15\x41IRichResponseMessage\x12:\n\x0bmessageType\x18\x01 \x01(\x0e\x32 .proto.AIRichResponseMessageTypeH\x00\x88\x01\x01\x12\x34\n\x0bsubmessages\x18\x02 \x03(\x0b\x32\x1f.proto.AIRichResponseSubMessage\x12\x42\n\x0funifiedResponse\x18\x03 \x01(\x0b\x32$.proto.AIRichResponseUnifiedResponseH\x01\x88\x01\x01\x12,\n\x0b\x63ontextInfo\x18\x04 \x01(\x0b\x32\x12.proto.ContextInfoH\x02\x88\x01\x01\x42\x0e\n\x0c_messageTypeB\x12\n\x10_unifiedResponseB\x0e\n\x0c_contextInfo\"\xc7\x06\n\x18\x41IRichResponseSubMessage\x12=\n\x0bmessageType\x18\x01 \x01(\x0e\x32#.proto.AIRichResponseSubMessageTypeH\x00\x88\x01\x01\x12\x46\n\x11gridImageMetadata\x18\x02 \x01(\x0b\x32&.proto.AIRichResponseGridImageMetadataH\x01\x88\x01\x01\x12\x18\n\x0bmessageText\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x44\n\rimageMetadata\x18\x04 \x01(\x0b\x32(.proto.AIRichResponseInlineImageMetadataH\x03\x88\x01\x01\x12<\n\x0c\x63odeMetadata\x18\x05 \x01(\x0b\x32!.proto.AIRichResponseCodeMetadataH\x04\x88\x01\x01\x12>\n\rtableMetadata\x18\x06 \x01(\x0b\x32\".proto.AIRichResponseTableMetadataH\x05\x88\x01\x01\x12\x42\n\x0f\x64ynamicMetadata\x18\x07 \x01(\x0b\x32$.proto.AIRichResponseDynamicMetadataH\x06\x88\x01\x01\x12>\n\rlatexMetadata\x18\x08 \x01(\x0b\x32\".proto.AIRichResponseLatexMetadataH\x07\x88\x01\x01\x12:\n\x0bmapMetadata\x18\t \x01(\x0b\x32 .proto.AIRichResponseMapMetadataH\x08\x88\x01\x01\x12L\n\x14\x63ontentItemsMetadata\x18\n \x01(\x0b\x32).proto.AIRichResponseContentItemsMetadataH\t\x88\x01\x01\x42\x0e\n\x0c_messageTypeB\x14\n\x12_gridImageMetadataB\x0e\n\x0c_messageTextB\x10\n\x0e_imageMetadataB\x0f\n\r_codeMetadataB\x10\n\x0e_tableMetadataB\x12\n\x10_dynamicMetadataB\x10\n\x0e_latexMetadataB\x0e\n\x0c_mapMetadataB\x17\n\x15_contentItemsMetadata\"\xd3\x01\n\x1b\x41IRichResponseTableMetadata\x12G\n\x04rows\x18\x01 \x03(\x0b\x32\x39.proto.AIRichResponseTableMetadata.AIRichResponseTableRow\x12\x12\n\x05title\x18\x02 \x01(\tH\x00\x88\x01\x01\x1aM\n\x16\x41IRichResponseTableRow\x12\r\n\x05items\x18\x01 \x03(\t\x12\x16\n\tisHeading\x18\x02 \x01(\x08H\x00\x88\x01\x01\x42\x0c\n\n_isHeadingB\x08\n\x06_title\";\n\x1d\x41IRichResponseUnifiedResponse\x12\x11\n\x04\x64\x61ta\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x42\x07\n\x05_data\"\x83\x03\n\x0c\x41IThreadInfo\x12?\n\nserverInfo\x18\x01 \x01(\x0b\x32&.proto.AIThreadInfo.AIThreadServerInfoH\x00\x88\x01\x01\x12?\n\nclientInfo\x18\x02 \x01(\x0b\x32&.proto.AIThreadInfo.AIThreadClientInfoH\x01\x88\x01\x01\x1a\x9e\x01\n\x12\x41IThreadClientInfo\x12\x46\n\x04type\x18\x01 \x01(\x0e\x32\x33.proto.AIThreadInfo.AIThreadClientInfo.AIThreadTypeH\x00\x88\x01\x01\"7\n\x0c\x41IThreadType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x01\x12\r\n\tINCOGNITO\x10\x02\x42\x07\n\x05_type\x1a\x32\n\x12\x41IThreadServerInfo\x12\x12\n\x05title\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x08\n\x06_titleB\r\n\x0b_serverInfoB\r\n\x0b_clientInfo\"\xa7\x01\n\x07\x41\x63\x63ount\x12\x10\n\x03lid\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x15\n\x08username\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x18\n\x0b\x63ountryCode\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x1e\n\x11isUsernameDeleted\x18\x04 \x01(\x08H\x03\x88\x01\x01\x42\x06\n\x04_lidB\x0b\n\t_usernameB\x0e\n\x0c_countryCodeB\x14\n\x12_isUsernameDeleted\"P\n\nActionLink\x12\x10\n\x03url\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x18\n\x0b\x62uttonTitle\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x06\n\x04_urlB\x0e\n\x0c_buttonTitle\"\xd8\x01\n\x14\x41utoDownloadSettings\x12\x1b\n\x0e\x64ownloadImages\x18\x01 \x01(\x08H\x00\x88\x01\x01\x12\x1a\n\rdownloadAudio\x18\x02 \x01(\x08H\x01\x88\x01\x01\x12\x1a\n\rdownloadVideo\x18\x03 \x01(\x08H\x02\x88\x01\x01\x12\x1e\n\x11\x64ownloadDocuments\x18\x04 \x01(\x08H\x03\x88\x01\x01\x42\x11\n\x0f_downloadImagesB\x10\n\x0e_downloadAudioB\x10\n\x0e_downloadVideoB\x14\n\x12_downloadDocuments\"T\n\x12\x41vatarUserSettings\x12\x11\n\x04\x66\x62id\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x15\n\x08password\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x07\n\x05_fbidB\x0b\n\t_password\"\xa2\x03\n\x12\x42izAccountLinkInfo\x12 \n\x13whatsappBizAcctFbid\x18\x01 \x01(\x04H\x00\x88\x01\x01\x12\x1f\n\x12whatsappAcctNumber\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x16\n\tissueTime\x18\x03 \x01(\x04H\x02\x88\x01\x01\x12\x43\n\x0bhostStorage\x18\x04 \x01(\x0e\x32).proto.BizAccountLinkInfo.HostStorageTypeH\x03\x88\x01\x01\x12?\n\x0b\x61\x63\x63ountType\x18\x05 \x01(\x0e\x32%.proto.BizAccountLinkInfo.AccountTypeH\x04\x88\x01\x01\"\x1d\n\x0b\x41\x63\x63ountType\x12\x0e\n\nENTERPRISE\x10\x00\"/\n\x0fHostStorageType\x12\x0e\n\nON_PREMISE\x10\x00\x12\x0c\n\x08\x46\x41\x43\x45\x42OOK\x10\x01\x42\x16\n\x14_whatsappBizAcctFbidB\x15\n\x13_whatsappAcctNumberB\x0c\n\n_issueTimeB\x0e\n\x0c_hostStorageB\x0e\n\x0c_accountType\"\x8b\x01\n\x11\x42izAccountPayload\x12\x36\n\tvnameCert\x18\x01 \x01(\x0b\x32\x1e.proto.VerifiedNameCertificateH\x00\x88\x01\x01\x12\x1c\n\x0f\x62izAcctLinkInfo\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x42\x0c\n\n_vnameCertB\x12\n\x10_bizAcctLinkInfo\"\xf9\x04\n\x0f\x42izIdentityInfo\x12>\n\x06vlevel\x18\x01 \x01(\x0e\x32).proto.BizIdentityInfo.VerifiedLevelValueH\x00\x88\x01\x01\x12\x36\n\tvnameCert\x18\x02 \x01(\x0b\x32\x1e.proto.VerifiedNameCertificateH\x01\x88\x01\x01\x12\x13\n\x06signed\x18\x03 \x01(\x08H\x02\x88\x01\x01\x12\x14\n\x07revoked\x18\x04 \x01(\x08H\x03\x88\x01\x01\x12@\n\x0bhostStorage\x18\x05 \x01(\x0e\x32&.proto.BizIdentityInfo.HostStorageTypeH\x04\x88\x01\x01\x12\x42\n\x0c\x61\x63tualActors\x18\x06 \x01(\x0e\x32\'.proto.BizIdentityInfo.ActualActorsTypeH\x05\x88\x01\x01\x12\x1a\n\rprivacyModeTs\x18\x07 \x01(\x04H\x06\x88\x01\x01\x12\x1c\n\x0f\x66\x65\x61tureControls\x18\x08 \x01(\x04H\x07\x88\x01\x01\"%\n\x10\x41\x63tualActorsType\x12\x08\n\x04SELF\x10\x00\x12\x07\n\x03\x42SP\x10\x01\"/\n\x0fHostStorageType\x12\x0e\n\nON_PREMISE\x10\x00\x12\x0c\n\x08\x46\x41\x43\x45\x42OOK\x10\x01\"4\n\x12VerifiedLevelValue\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x07\n\x03LOW\x10\x01\x12\x08\n\x04HIGH\x10\x02\x42\t\n\x07_vlevelB\x0c\n\n_vnameCertB\t\n\x07_signedB\n\n\x08_revokedB\x0e\n\x0c_hostStorageB\x0f\n\r_actualActorsB\x10\n\x0e_privacyModeTsB\x12\n\x10_featureControls\"\xca\x02\n\x18\x42otAgeCollectionMetadata\x12\"\n\x15\x61geCollectionEligible\x18\x01 \x01(\x08H\x00\x88\x01\x01\x12/\n\"shouldTriggerAgeCollectionOnClient\x18\x02 \x01(\x08H\x01\x88\x01\x01\x12Q\n\x11\x61geCollectionType\x18\x03 \x01(\x0e\x32\x31.proto.BotAgeCollectionMetadata.AgeCollectionTypeH\x02\x88\x01\x01\"/\n\x11\x41geCollectionType\x12\x0e\n\nO18_BINARY\x10\x00\x12\n\n\x06WAFFLE\x10\x01\x42\x18\n\x16_ageCollectionEligibleB%\n#_shouldTriggerAgeCollectionOnClientB\x14\n\x12_ageCollectionType\"\xd3\x01\n\x11\x42otAvatarMetadata\x12\x16\n\tsentiment\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x1a\n\rbehaviorGraph\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x13\n\x06\x61\x63tion\x18\x03 \x01(\rH\x02\x88\x01\x01\x12\x16\n\tintensity\x18\x04 \x01(\rH\x03\x88\x01\x01\x12\x16\n\twordCount\x18\x05 \x01(\rH\x04\x88\x01\x01\x42\x0c\n\n_sentimentB\x10\n\x0e_behaviorGraphB\t\n\x07_actionB\x0c\n\n_intensityB\x0c\n\n_wordCount\"\xb4\r\n\x15\x42otCapabilityMetadata\x12\x44\n\x0c\x63\x61pabilities\x18\x01 \x03(\x0e\x32..proto.BotCapabilityMetadata.BotCapabilityType\"\xd4\x0c\n\x11\x42otCapabilityType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x16\n\x12PROGRESS_INDICATOR\x10\x01\x12\x19\n\x15RICH_RESPONSE_HEADING\x10\x02\x12\x1d\n\x19RICH_RESPONSE_NESTED_LIST\x10\x03\x12\r\n\tAI_MEMORY\x10\x04\x12 \n\x1cRICH_RESPONSE_THREAD_SURFING\x10\x05\x12\x17\n\x13RICH_RESPONSE_TABLE\x10\x06\x12\x16\n\x12RICH_RESPONSE_CODE\x10\x07\x12%\n!RICH_RESPONSE_STRUCTURED_RESPONSE\x10\x08\x12\x1e\n\x1aRICH_RESPONSE_INLINE_IMAGE\x10\t\x12#\n\x1fWA_IG_1P_PLUGIN_RANKING_CONTROL\x10\n\x12$\n WA_IG_1P_PLUGIN_RANKING_UPDATE_1\x10\x0b\x12$\n WA_IG_1P_PLUGIN_RANKING_UPDATE_2\x10\x0c\x12$\n WA_IG_1P_PLUGIN_RANKING_UPDATE_3\x10\r\x12$\n WA_IG_1P_PLUGIN_RANKING_UPDATE_4\x10\x0e\x12$\n WA_IG_1P_PLUGIN_RANKING_UPDATE_5\x10\x0f\x12$\n WA_IG_1P_PLUGIN_RANKING_UPDATE_6\x10\x10\x12$\n WA_IG_1P_PLUGIN_RANKING_UPDATE_7\x10\x11\x12$\n WA_IG_1P_PLUGIN_RANKING_UPDATE_8\x10\x12\x12$\n WA_IG_1P_PLUGIN_RANKING_UPDATE_9\x10\x13\x12%\n!WA_IG_1P_PLUGIN_RANKING_UPDATE_10\x10\x14\x12\x1d\n\x19RICH_RESPONSE_SUB_HEADING\x10\x15\x12\x1c\n\x18RICH_RESPONSE_GRID_IMAGE\x10\x16\x12\x18\n\x14\x41I_STUDIO_UGC_MEMORY\x10\x17\x12\x17\n\x13RICH_RESPONSE_LATEX\x10\x18\x12\x16\n\x12RICH_RESPONSE_MAPS\x10\x19\x12\x1e\n\x1aRICH_RESPONSE_INLINE_REELS\x10\x1a\x12\x14\n\x10\x41GENTIC_PLANNING\x10\x1b\x12\x13\n\x0f\x41\x43\x43OUNT_LINKING\x10\x1c\x12\x1c\n\x18STREAMING_DISAGGREGATION\x10\x1d\x12\x1f\n\x1bRICH_RESPONSE_GRID_IMAGE_3P\x10\x1e\x12\x1e\n\x1aRICH_RESPONSE_LATEX_INLINE\x10\x1f\x12\x0e\n\nQUERY_PLAN\x10 \x12\x15\n\x11PROACTIVE_MESSAGE\x10!\x12\"\n\x1eRICH_RESPONSE_UNIFIED_RESPONSE\x10\"\x12\x15\n\x11PROMOTION_MESSAGE\x10#\x12\x1b\n\x17SIMPLIFIED_PROFILE_PAGE\x10$\x12$\n RICH_RESPONSE_SOURCES_IN_MESSAGE\x10%\x12%\n!RICH_RESPONSE_SIDE_BY_SIDE_SURVEY\x10&\x12(\n$RICH_RESPONSE_UNIFIED_TEXT_COMPONENT\x10\'\x12\x14\n\x10\x41I_SHARED_MEMORY\x10(\x12!\n\x1dRICH_RESPONSE_UNIFIED_SOURCES\x10)\x12*\n&RICH_RESPONSE_UNIFIED_DOMAIN_CITATIONS\x10*\x12)\n%RICH_RESPONSE_UR_INLINE_REELS_ENABLED\x10+\x12\'\n#RICH_RESPONSE_UR_MEDIA_GRID_ENABLED\x10,\x12*\n&RICH_RESPONSE_UR_TIMESTAMP_PLACEHOLDER\x10-\x12\x1f\n\x1bRICH_RESPONSE_IN_APP_SURVEY\x10.\x12\x1e\n\x1a\x41I_RESPONSE_MODEL_BRANDING\x10/\x12\'\n#SESSION_TRANSPARENCY_SYSTEM_MESSAGE\x10\x30\x12\x1e\n\x1aRICH_RESPONSE_UR_REASONING\x10\x31\"\xa8!\n\x12\x42otFeedbackMessage\x12*\n\nmessageKey\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x12<\n\x04kind\x18\x02 \x01(\x0e\x32).proto.BotFeedbackMessage.BotFeedbackKindH\x01\x88\x01\x01\x12\x11\n\x04text\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x19\n\x0ckindNegative\x18\x04 \x01(\x04H\x03\x88\x01\x01\x12\x19\n\x0ckindPositive\x18\x05 \x01(\x04H\x04\x88\x01\x01\x12=\n\nkindReport\x18\x06 \x01(\x0e\x32$.proto.BotFeedbackMessage.ReportKindH\x05\x88\x01\x01\x12Y\n\x18sideBySideSurveyMetadata\x18\x07 \x01(\x0b\x32\x32.proto.BotFeedbackMessage.SideBySideSurveyMetadataH\x06\x88\x01\x01\x1a\xce\x13\n\x18SideBySideSurveyMetadata\x12\x1e\n\x11selectedRequestId\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x15\n\x08surveyId\x18\x02 \x01(\rH\x01\x88\x01\x01\x12\x1d\n\x10simonSessionFbid\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x19\n\x0cresponseOtid\x18\x04 \x01(\tH\x03\x88\x01\x01\x12&\n\x19responseTimestampMsString\x18\x05 \x01(\tH\x04\x88\x01\x01\x12&\n\x19isSelectedResponsePrimary\x18\x06 \x01(\x08H\x05\x88\x01\x01\x12\x1c\n\x0fmessageIdToEdit\x18\x07 \x01(\tH\x06\x88\x01\x01\x12l\n\ranalyticsData\x18\x08 \x01(\x0b\x32P.proto.BotFeedbackMessage.SideBySideSurveyMetadata.SideBySideSurveyAnalyticsDataH\x07\x88\x01\x01\x12x\n\x13metaAiAnalyticsData\x18\t \x01(\x0b\x32V.proto.BotFeedbackMessage.SideBySideSurveyMetadata.SidebySideSurveyMetaAiAnalyticsDataH\x08\x88\x01\x01\x1a\xaf\x01\n\x1dSideBySideSurveyAnalyticsData\x12\x17\n\ntessaEvent\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x1d\n\x10tessaSessionFbid\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x1d\n\x10simonSessionFbid\x18\x03 \x01(\tH\x02\x88\x01\x01\x42\r\n\x0b_tessaEventB\x13\n\x11_tessaSessionFbidB\x13\n\x11_simonSessionFbid\x1a\xf5\x0c\n#SidebySideSurveyMetaAiAnalyticsData\x12\x15\n\x08surveyId\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x1e\n\x11primaryResponseId\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x18\n\x0btestArmName\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x1e\n\x11timestampMsString\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x9e\x01\n\x12\x63taImpressionEvent\x18\x05 \x01(\x0b\x32}.proto.BotFeedbackMessage.SideBySideSurveyMetadata.SidebySideSurveyMetaAiAnalyticsData.SideBySideSurveyCTAImpressionEventDataH\x04\x88\x01\x01\x12\x94\x01\n\rctaClickEvent\x18\x06 \x01(\x0b\x32x.proto.BotFeedbackMessage.SideBySideSurveyMetadata.SidebySideSurveyMetaAiAnalyticsData.SideBySideSurveyCTAClickEventDataH\x05\x88\x01\x01\x12\xa0\x01\n\x13\x63\x61rdImpressionEvent\x18\x07 \x01(\x0b\x32~.proto.BotFeedbackMessage.SideBySideSurveyMetadata.SidebySideSurveyMetaAiAnalyticsData.SideBySideSurveyCardImpressionEventDataH\x06\x88\x01\x01\x12\x94\x01\n\rresponseEvent\x18\x08 \x01(\x0b\x32x.proto.BotFeedbackMessage.SideBySideSurveyMetadata.SidebySideSurveyMetaAiAnalyticsData.SideBySideSurveyResponseEventDataH\x07\x88\x01\x01\x12\x92\x01\n\x0c\x61\x62\x61ndonEvent\x18\t \x01(\x0b\x32w.proto.BotFeedbackMessage.SideBySideSurveyMetadata.SidebySideSurveyMetaAiAnalyticsData.SideBySideSurveyAbandonEventDataH\x08\x88\x01\x01\x1a\x66\n SideBySideSurveyAbandonEventData\x12%\n\x18\x61\x62\x61ndonDwellTimeMsString\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x1b\n\x19_abandonDwellTimeMsString\x1a\x95\x01\n!SideBySideSurveyCTAClickEventData\x12\x1c\n\x0fisSurveyExpired\x18\x01 \x01(\x08H\x00\x88\x01\x01\x12#\n\x16\x63lickDwellTimeMsString\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x12\n\x10_isSurveyExpiredB\x19\n\x17_clickDwellTimeMsString\x1aZ\n&SideBySideSurveyCTAImpressionEventData\x12\x1c\n\x0fisSurveyExpired\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\x12\n\x10_isSurveyExpired\x1a)\n\'SideBySideSurveyCardImpressionEventData\x1a\xa1\x01\n!SideBySideSurveyResponseEventData\x12&\n\x19responseDwellTimeMsString\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x1f\n\x12selectedResponseId\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x1c\n\x1a_responseDwellTimeMsStringB\x15\n\x13_selectedResponseIdB\x0b\n\t_surveyIdB\x14\n\x12_primaryResponseIdB\x0e\n\x0c_testArmNameB\x14\n\x12_timestampMsStringB\x15\n\x13_ctaImpressionEventB\x10\n\x0e_ctaClickEventB\x16\n\x14_cardImpressionEventB\x10\n\x0e_responseEventB\x0f\n\r_abandonEventB\x14\n\x12_selectedRequestIdB\x0b\n\t_surveyIdB\x13\n\x11_simonSessionFbidB\x0f\n\r_responseOtidB\x1c\n\x1a_responseTimestampMsStringB\x1c\n\x1a_isSelectedResponsePrimaryB\x12\n\x10_messageIdToEditB\x10\n\x0e_analyticsDataB\x16\n\x14_metaAiAnalyticsData\"\xd7\x04\n\x0f\x42otFeedbackKind\x12\x19\n\x15\x42OT_FEEDBACK_POSITIVE\x10\x00\x12!\n\x1d\x42OT_FEEDBACK_NEGATIVE_GENERIC\x10\x01\x12!\n\x1d\x42OT_FEEDBACK_NEGATIVE_HELPFUL\x10\x02\x12%\n!BOT_FEEDBACK_NEGATIVE_INTERESTING\x10\x03\x12\"\n\x1e\x42OT_FEEDBACK_NEGATIVE_ACCURATE\x10\x04\x12\x1e\n\x1a\x42OT_FEEDBACK_NEGATIVE_SAFE\x10\x05\x12\x1f\n\x1b\x42OT_FEEDBACK_NEGATIVE_OTHER\x10\x06\x12!\n\x1d\x42OT_FEEDBACK_NEGATIVE_REFUSED\x10\x07\x12\x30\n,BOT_FEEDBACK_NEGATIVE_NOT_VISUALLY_APPEALING\x10\x08\x12.\n*BOT_FEEDBACK_NEGATIVE_NOT_RELEVANT_TO_TEXT\x10\t\x12&\n\"BOT_FEEDBACK_NEGATIVE_PERSONALIZED\x10\n\x12!\n\x1d\x42OT_FEEDBACK_NEGATIVE_CLARITY\x10\x0b\x12\x35\n1BOT_FEEDBACK_NEGATIVE_DOESNT_LOOK_LIKE_THE_PERSON\x10\x0c\x12\x35\n1BOT_FEEDBACK_NEGATIVE_HALLUCINATION_INTERNAL_ONLY\x10\r\x12\x19\n\x15\x42OT_FEEDBACK_NEGATIVE\x10\x0e\"\x80\x04\n\x1f\x42otFeedbackKindMultipleNegative\x12\x33\n/BOT_FEEDBACK_KIND_MULTIPLE_NEGATIVE_UNSPECIFIED\x10\x00\x12*\n&BOT_FEEDBACK_MULTIPLE_NEGATIVE_GENERIC\x10\x01\x12*\n&BOT_FEEDBACK_MULTIPLE_NEGATIVE_HELPFUL\x10\x02\x12.\n*BOT_FEEDBACK_MULTIPLE_NEGATIVE_INTERESTING\x10\x04\x12+\n\'BOT_FEEDBACK_MULTIPLE_NEGATIVE_ACCURATE\x10\x08\x12\'\n#BOT_FEEDBACK_MULTIPLE_NEGATIVE_SAFE\x10\x10\x12(\n$BOT_FEEDBACK_MULTIPLE_NEGATIVE_OTHER\x10 \x12*\n&BOT_FEEDBACK_MULTIPLE_NEGATIVE_REFUSED\x10@\x12:\n5BOT_FEEDBACK_MULTIPLE_NEGATIVE_NOT_VISUALLY_APPEALING\x10\x80\x01\x12\x38\n3BOT_FEEDBACK_MULTIPLE_NEGATIVE_NOT_RELEVANT_TO_TEXT\x10\x80\x02\"\x82\x01\n\x1f\x42otFeedbackKindMultiplePositive\x12\x33\n/BOT_FEEDBACK_KIND_MULTIPLE_POSITIVE_UNSPECIFIED\x10\x00\x12*\n&BOT_FEEDBACK_MULTIPLE_POSITIVE_GENERIC\x10\x01\"#\n\nReportKind\x12\x08\n\x04NONE\x10\x00\x12\x0b\n\x07GENERIC\x10\x01\x42\r\n\x0b_messageKeyB\x07\n\x05_kindB\x07\n\x05_textB\x0f\n\r_kindNegativeB\x0f\n\r_kindPositiveB\r\n\x0b_kindReportB\x1b\n\x19_sideBySideSurveyMetadata\"\xad\x01\n\x12\x42otImagineMetadata\x12?\n\x0bimagineType\x18\x01 \x01(\x0e\x32%.proto.BotImagineMetadata.ImagineTypeH\x00\x88\x01\x01\"F\n\x0bImagineType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07IMAGINE\x10\x01\x12\x08\n\x04MEMU\x10\x02\x12\t\n\x05\x46LASH\x10\x03\x12\x08\n\x04\x45\x44IT\x10\x04\x42\x0e\n\x0c_imagineType\"\x94\x01\n\x10\x42otLinkedAccount\x12?\n\x04type\x18\x01 \x01(\x0e\x32,.proto.BotLinkedAccount.BotLinkedAccountTypeH\x00\x88\x01\x01\"6\n\x14\x42otLinkedAccountType\x12\x1e\n\x1a\x42OT_LINKED_ACCOUNT_TYPE_1P\x10\x00\x42\x07\n\x05_type\"\x9c\x01\n\x19\x42otLinkedAccountsMetadata\x12)\n\x08\x61\x63\x63ounts\x18\x01 \x03(\x0b\x32\x17.proto.BotLinkedAccount\x12\x19\n\x0c\x61\x63\x41uthTokens\x18\x02 \x01(\x0cH\x00\x88\x01\x01\x12\x18\n\x0b\x61\x63\x45rrorCode\x18\x03 \x01(\x05H\x01\x88\x01\x01\x42\x0f\n\r_acAuthTokensB\x0e\n\x0c_acErrorCode\"\xbf\x03\n\x10\x42otMediaMetadata\x12\x17\n\nfileSha256\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x15\n\x08mediaKey\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x1a\n\rfileEncSha256\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x17\n\ndirectPath\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x1e\n\x11mediaKeyTimestamp\x18\x05 \x01(\x03H\x04\x88\x01\x01\x12\x15\n\x08mimetype\x18\x06 \x01(\tH\x05\x88\x01\x01\x12\x45\n\x0forientationType\x18\x07 \x01(\x0e\x32\'.proto.BotMediaMetadata.OrientationTypeH\x06\x88\x01\x01\"T\n\x0fOrientationType\x12 \n\x1cORIENTATION_TYPE_UNSPECIFIED\x10\x00\x12\n\n\x06\x43\x45NTER\x10\x01\x12\x08\n\x04LEFT\x10\x02\x12\t\n\x05RIGHT\x10\x03\x42\r\n\x0b_fileSha256B\x0b\n\t_mediaKeyB\x10\n\x0e_fileEncSha256B\r\n\x0b_directPathB\x14\n\x12_mediaKeyTimestampB\x0b\n\t_mimetypeB\x12\n\x10_orientationType\"K\n\rBotMemoryFact\x12\x11\n\x04\x66\x61\x63t\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x13\n\x06\x66\x61\x63tId\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x07\n\x05_factB\t\n\x07_factId\"\x91\x01\n\x11\x42otMemoryMetadata\x12(\n\naddedFacts\x18\x01 \x03(\x0b\x32\x14.proto.BotMemoryFact\x12*\n\x0cremovedFacts\x18\x02 \x03(\x0b\x32\x14.proto.BotMemoryFact\x12\x17\n\ndisclaimer\x18\x03 \x01(\tH\x00\x88\x01\x01\x42\r\n\x0b_disclaimer\">\n\x0f\x42otMemuMetadata\x12+\n\nfaceImages\x18\x01 \x03(\x0b\x32\x17.proto.BotMediaMetadata\"\x9e\x01\n\x10\x42otMessageOrigin\x12?\n\x04type\x18\x01 \x01(\x0e\x32,.proto.BotMessageOrigin.BotMessageOriginTypeH\x00\x88\x01\x01\"@\n\x14\x42otMessageOriginType\x12(\n$BOT_MESSAGE_ORIGIN_TYPE_AI_INITIATED\x10\x00\x42\x07\n\x05_type\"D\n\x18\x42otMessageOriginMetadata\x12(\n\x07origins\x18\x01 \x03(\x0b\x32\x17.proto.BotMessageOrigin\"\x9a\x01\n\x15\x42otMessageSharingInfo\x12=\n\x13\x62otEntryPointOrigin\x18\x01 \x01(\x0e\x32\x1b.proto.BotMetricsEntryPointH\x00\x88\x01\x01\x12\x19\n\x0c\x66orwardScore\x18\x02 \x01(\rH\x01\x88\x01\x01\x42\x16\n\x14_botEntryPointOriginB\x0f\n\r_forwardScore\"\xad\x15\n\x0b\x42otMetadata\x12\x35\n\x0e\x61vatarMetadata\x18\x01 \x01(\x0b\x32\x18.proto.BotAvatarMetadataH\x00\x88\x01\x01\x12\x16\n\tpersonaId\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x35\n\x0epluginMetadata\x18\x03 \x01(\x0b\x32\x18.proto.BotPluginMetadataH\x02\x88\x01\x01\x12G\n\x17suggestedPromptMetadata\x18\x04 \x01(\x0b\x32!.proto.BotSuggestedPromptMetadataH\x03\x88\x01\x01\x12\x17\n\ninvokerJid\x18\x05 \x01(\tH\x04\x88\x01\x01\x12\x37\n\x0fsessionMetadata\x18\x06 \x01(\x0b\x32\x19.proto.BotSessionMetadataH\x05\x88\x01\x01\x12\x31\n\x0cmemuMetadata\x18\x07 \x01(\x0b\x32\x16.proto.BotMemuMetadataH\x06\x88\x01\x01\x12\x15\n\x08timezone\x18\x08 \x01(\tH\x07\x88\x01\x01\x12\x39\n\x10reminderMetadata\x18\t \x01(\x0b\x32\x1a.proto.BotReminderMetadataH\x08\x88\x01\x01\x12\x33\n\rmodelMetadata\x18\n \x01(\x0b\x32\x17.proto.BotModelMetadataH\t\x88\x01\x01\x12\"\n\x15messageDisclaimerText\x18\x0b \x01(\tH\n\x88\x01\x01\x12K\n\x19progressIndicatorMetadata\x18\x0c \x01(\x0b\x32#.proto.BotProgressIndicatorMetadataH\x0b\x88\x01\x01\x12=\n\x12\x63\x61pabilityMetadata\x18\r \x01(\x0b\x32\x1c.proto.BotCapabilityMetadataH\x0c\x88\x01\x01\x12\x37\n\x0fimagineMetadata\x18\x0e \x01(\x0b\x32\x19.proto.BotImagineMetadataH\r\x88\x01\x01\x12\x35\n\x0ememoryMetadata\x18\x0f \x01(\x0b\x32\x18.proto.BotMemoryMetadataH\x0e\x88\x01\x01\x12;\n\x11renderingMetadata\x18\x10 \x01(\x0b\x32\x1b.proto.BotRenderingMetadataH\x0f\x88\x01\x01\x12:\n\x12\x62otMetricsMetadata\x18\x11 \x01(\x0b\x32\x19.proto.BotMetricsMetadataH\x10\x88\x01\x01\x12H\n\x19\x62otLinkedAccountsMetadata\x18\x12 \x01(\x0b\x32 .proto.BotLinkedAccountsMetadataH\x11\x88\x01\x01\x12\x43\n\x1brichResponseSourcesMetadata\x18\x13 \x01(\x0b\x32\x19.proto.BotSourcesMetadataH\x12\x88\x01\x01\x12\"\n\x15\x61iConversationContext\x18\x14 \x01(\x0cH\x13\x88\x01\x01\x12L\n\x1b\x62otPromotionMessageMetadata\x18\x15 \x01(\x0b\x32\".proto.BotPromotionMessageMetadataH\x14\x88\x01\x01\x12\x46\n\x18\x62otModeSelectionMetadata\x18\x16 \x01(\x0b\x32\x1f.proto.BotModeSelectionMetadataH\x15\x88\x01\x01\x12\x36\n\x10\x62otQuotaMetadata\x18\x17 \x01(\x0b\x32\x17.proto.BotQuotaMetadataH\x16\x88\x01\x01\x12\x46\n\x18\x62otAgeCollectionMetadata\x18\x18 \x01(\x0b\x32\x1f.proto.BotAgeCollectionMetadataH\x17\x88\x01\x01\x12(\n\x1b\x63onversationStarterPromptId\x18\x19 \x01(\tH\x18\x88\x01\x01\x12\x1a\n\rbotResponseId\x18\x1a \x01(\tH\x19\x88\x01\x01\x12J\n\x14verificationMetadata\x18\x1b \x01(\x0b\x32\'.proto.BotSignatureVerificationMetadataH\x1a\x88\x01\x01\x12G\n\x17unifiedResponseMutation\x18\x1c \x01(\x0b\x32!.proto.BotUnifiedResponseMutationH\x1b\x88\x01\x01\x12\x46\n\x18\x62otMessageOriginMetadata\x18\x1d \x01(\x0b\x32\x1f.proto.BotMessageOriginMetadataH\x1c\x88\x01\x01\x12\x42\n\x16inThreadSurveyMetadata\x18\x1e \x01(\x0b\x32\x1d.proto.InThreadSurveyMetadataH\x1d\x88\x01\x01\x12/\n\rbotThreadInfo\x18\x1f \x01(\x0b\x32\x13.proto.AIThreadInfoH\x1e\x88\x01\x01\x12<\n\x12regenerateMetadata\x18  \x01(\x0b\x32\x1b.proto.AIRegenerateMetadataH\x1f\x88\x01\x01\x12L\n\x1bsessionTransparencyMetadata\x18! \x01(\x0b\x32\".proto.SessionTransparencyMetadataH \x88\x01\x01\x12\x1e\n\x10internalMetadata\x18\xe7\x07 \x01(\x0cH!\x88\x01\x01\x42\x11\n\x0f_avatarMetadataB\x0c\n\n_personaIdB\x11\n\x0f_pluginMetadataB\x1a\n\x18_suggestedPromptMetadataB\r\n\x0b_invokerJidB\x12\n\x10_sessionMetadataB\x0f\n\r_memuMetadataB\x0b\n\t_timezoneB\x13\n\x11_reminderMetadataB\x10\n\x0e_modelMetadataB\x18\n\x16_messageDisclaimerTextB\x1c\n\x1a_progressIndicatorMetadataB\x15\n\x13_capabilityMetadataB\x12\n\x10_imagineMetadataB\x11\n\x0f_memoryMetadataB\x14\n\x12_renderingMetadataB\x15\n\x13_botMetricsMetadataB\x1c\n\x1a_botLinkedAccountsMetadataB\x1e\n\x1c_richResponseSourcesMetadataB\x18\n\x16_aiConversationContextB\x1e\n\x1c_botPromotionMessageMetadataB\x1b\n\x19_botModeSelectionMetadataB\x13\n\x11_botQuotaMetadataB\x1b\n\x19_botAgeCollectionMetadataB\x1e\n\x1c_conversationStarterPromptIdB\x10\n\x0e_botResponseIdB\x17\n\x15_verificationMetadataB\x1a\n\x18_unifiedResponseMutationB\x1b\n\x19_botMessageOriginMetadataB\x19\n\x17_inThreadSurveyMetadataB\x10\n\x0e_botThreadInfoB\x15\n\x13_regenerateMetadataB\x1e\n\x1c_sessionTransparencyMetadataB\x13\n\x11_internalMetadata\"\xec\x01\n\x12\x42otMetricsMetadata\x12\x1a\n\rdestinationId\x18\x01 \x01(\tH\x00\x88\x01\x01\x12?\n\x15\x64\x65stinationEntryPoint\x18\x02 \x01(\x0e\x32\x1b.proto.BotMetricsEntryPointH\x01\x88\x01\x01\x12<\n\x0cthreadOrigin\x18\x03 \x01(\x0e\x32!.proto.BotMetricsThreadEntryPointH\x02\x88\x01\x01\x42\x10\n\x0e_destinationIdB\x18\n\x16_destinationEntryPointB\x0f\n\r_threadOrigin\"\x9c\x01\n\x18\x42otModeSelectionMetadata\x12\x42\n\x04mode\x18\x01 \x03(\x0e\x32\x34.proto.BotModeSelectionMetadata.BotUserSelectionMode\"<\n\x14\x42otUserSelectionMode\x12\x10\n\x0cUNKNOWN_MODE\x10\x00\x12\x12\n\x0eREASONING_MODE\x10\x01\"\x8d\x03\n\x10\x42otModelMetadata\x12\x39\n\tmodelType\x18\x01 \x01(\x0e\x32!.proto.BotModelMetadata.ModelTypeH\x00\x88\x01\x01\x12K\n\x12premiumModelStatus\x18\x02 \x01(\x0e\x32*.proto.BotModelMetadata.PremiumModelStatusH\x01\x88\x01\x01\x12\x1e\n\x11modelNameOverride\x18\x03 \x01(\tH\x02\x88\x01\x01\"E\n\tModelType\x12\x10\n\x0cUNKNOWN_TYPE\x10\x00\x12\x0e\n\nLLAMA_PROD\x10\x01\x12\x16\n\x12LLAMA_PROD_PREMIUM\x10\x02\"O\n\x12PremiumModelStatus\x12\x12\n\x0eUNKNOWN_STATUS\x10\x00\x12\r\n\tAVAILABLE\x10\x01\x12\x16\n\x12QUOTA_EXCEED_LIMIT\x10\x02\x42\x0c\n\n_modelTypeB\x15\n\x13_premiumModelStatusB\x14\n\x12_modelNameOverride\"\x8b\x07\n\x11\x42otPluginMetadata\x12>\n\x08provider\x18\x01 \x01(\x0e\x32\'.proto.BotPluginMetadata.SearchProviderH\x00\x88\x01\x01\x12<\n\npluginType\x18\x02 \x01(\x0e\x32#.proto.BotPluginMetadata.PluginTypeH\x01\x88\x01\x01\x12\x1c\n\x0fthumbnailCdnUrl\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x1f\n\x12profilePhotoCdnUrl\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x1e\n\x11searchProviderUrl\x18\x05 \x01(\tH\x04\x88\x01\x01\x12\x1b\n\x0ereferenceIndex\x18\x06 \x01(\rH\x05\x88\x01\x01\x12\x1f\n\x12\x65xpectedLinksCount\x18\x07 \x01(\rH\x06\x88\x01\x01\x12\x18\n\x0bsearchQuery\x18\t \x01(\tH\x07\x88\x01\x01\x12\x36\n\x16parentPluginMessageKey\x18\n \x01(\x0b\x32\x11.proto.MessageKeyH\x08\x88\x01\x01\x12\x41\n\x0f\x64\x65precatedField\x18\x0b \x01(\x0e\x32#.proto.BotPluginMetadata.PluginTypeH\t\x88\x01\x01\x12\x42\n\x10parentPluginType\x18\x0c \x01(\x0e\x32#.proto.BotPluginMetadata.PluginTypeH\n\x88\x01\x01\x12\x1a\n\rfaviconCdnUrl\x18\r \x01(\tH\x0b\x88\x01\x01\"7\n\nPluginType\x12\x12\n\x0eUNKNOWN_PLUGIN\x10\x00\x12\t\n\x05REELS\x10\x01\x12\n\n\x06SEARCH\x10\x02\"@\n\x0eSearchProvider\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x08\n\x04\x42ING\x10\x01\x12\n\n\x06GOOGLE\x10\x02\x12\x0b\n\x07SUPPORT\x10\x03\x42\x0b\n\t_providerB\r\n\x0b_pluginTypeB\x12\n\x10_thumbnailCdnUrlB\x15\n\x13_profilePhotoCdnUrlB\x14\n\x12_searchProviderUrlB\x11\n\x0f_referenceIndexB\x15\n\x13_expectedLinksCountB\x0e\n\x0c_searchQueryB\x19\n\x17_parentPluginMessageKeyB\x12\n\x10_deprecatedFieldB\x13\n\x11_parentPluginTypeB\x10\n\x0e_faviconCdnUrl\"\xcd\r\n\x1c\x42otProgressIndicatorMetadata\x12 \n\x13progressDescription\x18\x01 \x01(\tH\x00\x88\x01\x01\x12R\n\rstepsMetadata\x18\x02 \x03(\x0b\x32;.proto.BotProgressIndicatorMetadata.BotPlanningStepMetadata\x1a\x9e\x0c\n\x17\x42otPlanningStepMetadata\x12\x18\n\x0bstatusTitle\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x17\n\nstatusBody\x18\x02 \x01(\tH\x01\x88\x01\x01\x12u\n\x0fsourcesMetadata\x18\x03 \x03(\x0b\x32\\.proto.BotProgressIndicatorMetadata.BotPlanningStepMetadata.BotPlanningSearchSourcesMetadata\x12\x63\n\x06status\x18\x04 \x01(\x0e\x32N.proto.BotProgressIndicatorMetadata.BotPlanningStepMetadata.PlanningStepStatusH\x02\x88\x01\x01\x12\x18\n\x0bisReasoning\x18\x05 \x01(\x08H\x03\x88\x01\x01\x12\x1d\n\x10isEnhancedSearch\x18\x06 \x01(\x08H\x04\x88\x01\x01\x12l\n\x08sections\x18\x07 \x03(\x0b\x32Z.proto.BotProgressIndicatorMetadata.BotPlanningStepMetadata.BotPlanningStepSectionMetadata\x1a\x86\x02\n\x1f\x42otPlanningSearchSourceMetadata\x12\x12\n\x05title\x18\x01 \x01(\tH\x00\x88\x01\x01\x12j\n\x08provider\x18\x02 \x01(\x0e\x32S.proto.BotProgressIndicatorMetadata.BotPlanningStepMetadata.BotSearchSourceProviderH\x01\x88\x01\x01\x12\x16\n\tsourceUrl\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x17\n\nfavIconUrl\x18\x04 \x01(\tH\x03\x88\x01\x01\x42\x08\n\x06_titleB\x0b\n\t_providerB\x0c\n\n_sourceUrlB\r\n\x0b_favIconUrl\x1a\xe6\x02\n BotPlanningSearchSourcesMetadata\x12\x18\n\x0bsourceTitle\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x93\x01\n\x08provider\x18\x02 \x01(\x0e\x32|.proto.BotProgressIndicatorMetadata.BotPlanningStepMetadata.BotPlanningSearchSourcesMetadata.BotPlanningSearchSourceProviderH\x01\x88\x01\x01\x12\x16\n\tsourceUrl\x18\x03 \x01(\tH\x02\x88\x01\x01\"O\n\x1f\x42otPlanningSearchSourceProvider\x12\x0b\n\x07UNKNOWN\x10\x00\x12\t\n\x05OTHER\x10\x01\x12\n\n\x06GOOGLE\x10\x02\x12\x08\n\x04\x42ING\x10\x03\x42\x0e\n\x0c_sourceTitleB\x0b\n\t_providerB\x0c\n\n_sourceUrl\x1a\xec\x01\n\x1e\x42otPlanningStepSectionMetadata\x12\x19\n\x0csectionTitle\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x18\n\x0bsectionBody\x18\x02 \x01(\tH\x01\x88\x01\x01\x12t\n\x0fsourcesMetadata\x18\x03 \x03(\x0b\x32[.proto.BotProgressIndicatorMetadata.BotPlanningStepMetadata.BotPlanningSearchSourceMetadataB\x0f\n\r_sectionTitleB\x0e\n\x0c_sectionBody\"P\n\x17\x42otSearchSourceProvider\x12\x14\n\x10UNKNOWN_PROVIDER\x10\x00\x12\t\n\x05OTHER\x10\x01\x12\n\n\x06GOOGLE\x10\x02\x12\x08\n\x04\x42ING\x10\x03\"K\n\x12PlanningStepStatus\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07PLANNED\x10\x01\x12\r\n\tEXECUTING\x10\x02\x12\x0c\n\x08\x46INISHED\x10\x03\x42\x0e\n\x0c_statusTitleB\r\n\x0b_statusBodyB\t\n\x07_statusB\x0e\n\x0c_isReasoningB\x13\n\x11_isEnhancedSearchB\x16\n\x14_progressDescription\"\xee\x01\n\x1b\x42otPromotionMessageMetadata\x12O\n\rpromotionType\x18\x01 \x01(\x0e\x32\x33.proto.BotPromotionMessageMetadata.BotPromotionTypeH\x00\x88\x01\x01\x12\x18\n\x0b\x62uttonTitle\x18\x02 \x01(\tH\x01\x88\x01\x01\"B\n\x10\x42otPromotionType\x12\x10\n\x0cUNKNOWN_TYPE\x10\x00\x12\x07\n\x03\x43\x35\x30\x10\x01\x12\x13\n\x0fSURVEY_PLATFORM\x10\x02\x42\x10\n\x0e_promotionTypeB\x0e\n\x0c_buttonTitle\"Y\n\x13\x42otPromptSuggestion\x12\x13\n\x06prompt\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x15\n\x08promptId\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\t\n\x07_promptB\x0b\n\t_promptId\"G\n\x14\x42otPromptSuggestions\x12/\n\x0bsuggestions\x18\x01 \x03(\x0b\x32\x1a.proto.BotPromptSuggestion\"\x92\x03\n\x10\x42otQuotaMetadata\x12P\n\x17\x62otFeatureQuotaMetadata\x18\x01 \x03(\x0b\x32/.proto.BotQuotaMetadata.BotFeatureQuotaMetadata\x1a\xab\x02\n\x17\x42otFeatureQuotaMetadata\x12X\n\x0b\x66\x65\x61tureType\x18\x01 \x01(\x0e\x32>.proto.BotQuotaMetadata.BotFeatureQuotaMetadata.BotFeatureTypeH\x00\x88\x01\x01\x12\x1b\n\x0eremainingQuota\x18\x02 \x01(\rH\x01\x88\x01\x01\x12 \n\x13\x65xpirationTimestamp\x18\x03 \x01(\x04H\x02\x88\x01\x01\"<\n\x0e\x42otFeatureType\x12\x13\n\x0fUNKNOWN_FEATURE\x10\x00\x12\x15\n\x11REASONING_FEATURE\x10\x01\x42\x0e\n\x0c_featureTypeB\x11\n\x0f_remainingQuotaB\x16\n\x14_expirationTimestamp\"\xad\x04\n\x13\x42otReminderMetadata\x12\x31\n\x11requestMessageKey\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x12>\n\x06\x61\x63tion\x18\x02 \x01(\x0e\x32).proto.BotReminderMetadata.ReminderActionH\x01\x88\x01\x01\x12\x11\n\x04name\x18\x03 \x01(\tH\x02\x88\x01\x01\x12!\n\x14nextTriggerTimestamp\x18\x04 \x01(\x04H\x03\x88\x01\x01\x12\x44\n\tfrequency\x18\x05 \x01(\x0e\x32,.proto.BotReminderMetadata.ReminderFrequencyH\x04\x88\x01\x01\"a\n\x0eReminderAction\x12\x1f\n\x1bREMINDER_ACTION_UNSPECIFIED\x10\x00\x12\n\n\x06NOTIFY\x10\x01\x12\n\n\x06\x43REATE\x10\x02\x12\n\n\x06\x44\x45LETE\x10\x03\x12\n\n\x06UPDATE\x10\x04\"s\n\x11ReminderFrequency\x12\"\n\x1eREMINDER_FREQUENCY_UNSPECIFIED\x10\x00\x12\x08\n\x04ONCE\x10\x01\x12\t\n\x05\x44\x41ILY\x10\x02\x12\n\n\x06WEEKLY\x10\x03\x12\x0c\n\x08\x42IWEEKLY\x10\x04\x12\x0b\n\x07MONTHLY\x10\x05\x42\x14\n\x12_requestMessageKeyB\t\n\x07_actionB\x07\n\x05_nameB\x17\n\x15_nextTriggerTimestampB\x0c\n\n_frequency\"\x91\x01\n\x14\x42otRenderingMetadata\x12\x35\n\x08keywords\x18\x01 \x03(\x0b\x32#.proto.BotRenderingMetadata.Keyword\x1a\x42\n\x07Keyword\x12\x12\n\x05value\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x19\n\x11\x61ssociatedPrompts\x18\x02 \x03(\tB\x08\n\x06_value\"\x81\x01\n\x12\x42otSessionMetadata\x12\x16\n\tsessionId\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x33\n\rsessionSource\x18\x02 \x01(\x0e\x32\x17.proto.BotSessionSourceH\x01\x88\x01\x01\x42\x0c\n\n_sessionIdB\x10\n\x0e_sessionSource\"_\n BotSignatureVerificationMetadata\x12;\n\x06proofs\x18\x01 \x03(\x0b\x32+.proto.BotSignatureVerificationUseCaseProof\"\xa3\x02\n$BotSignatureVerificationUseCaseProof\x12\x14\n\x07version\x18\x01 \x01(\x05H\x00\x88\x01\x01\x12U\n\x07useCase\x18\x02 \x01(\x0e\x32?.proto.BotSignatureVerificationUseCaseProof.BotSignatureUseCaseH\x01\x88\x01\x01\x12\x16\n\tsignature\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x18\n\x10\x63\x65rtificateChain\x18\x04 \x03(\x0c\"6\n\x13\x42otSignatureUseCase\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\x0e\n\nWA_BOT_MSG\x10\x01\x42\n\n\x08_versionB\n\n\x08_useCaseB\x0c\n\n_signature\"\xa3\x04\n\x12\x42otSourcesMetadata\x12\x38\n\x07sources\x18\x01 \x03(\x0b\x32\'.proto.BotSourcesMetadata.BotSourceItem\x1a\xd2\x03\n\rBotSourceItem\x12M\n\x08provider\x18\x01 \x01(\x0e\x32\x36.proto.BotSourcesMetadata.BotSourceItem.SourceProviderH\x00\x88\x01\x01\x12\x1c\n\x0fthumbnailCdnUrl\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x1e\n\x11sourceProviderUrl\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x18\n\x0bsourceQuery\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x1a\n\rfaviconCdnUrl\x18\x05 \x01(\tH\x04\x88\x01\x01\x12\x1b\n\x0e\x63itationNumber\x18\x06 \x01(\rH\x05\x88\x01\x01\x12\x18\n\x0bsourceTitle\x18\x07 \x01(\tH\x06\x88\x01\x01\"K\n\x0eSourceProvider\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x08\n\x04\x42ING\x10\x01\x12\n\n\x06GOOGLE\x10\x02\x12\x0b\n\x07SUPPORT\x10\x03\x12\t\n\x05OTHER\x10\x04\x42\x0b\n\t_providerB\x12\n\x10_thumbnailCdnUrlB\x14\n\x12_sourceProviderUrlB\x0e\n\x0c_sourceQueryB\x10\n\x0e_faviconCdnUrlB\x11\n\x0f_citationNumberB\x0e\n\x0c_sourceTitle\"\xf7\x01\n\x1a\x42otSuggestedPromptMetadata\x12\x18\n\x10suggestedPrompts\x18\x01 \x03(\t\x12 \n\x13selectedPromptIndex\x18\x02 \x01(\rH\x00\x88\x01\x01\x12;\n\x11promptSuggestions\x18\x03 \x01(\x0b\x32\x1b.proto.BotPromptSuggestionsH\x01\x88\x01\x01\x12\x1d\n\x10selectedPromptId\x18\x04 \x01(\tH\x02\x88\x01\x01\x42\x16\n\x14_selectedPromptIndexB\x14\n\x12_promptSuggestionsB\x13\n\x11_selectedPromptId\"\x9a\x04\n\x1a\x42otUnifiedResponseMutation\x12N\n\x0bsbsMetadata\x18\x01 \x01(\x0b\x32\x34.proto.BotUnifiedResponseMutation.SideBySideMetadataH\x00\x88\x01\x01\x12X\n\x18mediaDetailsMetadataList\x18\x02 \x03(\x0b\x32\x36.proto.BotUnifiedResponseMutation.MediaDetailsMetadata\x1a\xb8\x01\n\x14MediaDetailsMetadata\x12\x0f\n\x02id\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x32\n\x0chighResMedia\x18\x02 \x01(\x0b\x32\x17.proto.BotMediaMetadataH\x01\x88\x01\x01\x12\x32\n\x0cpreviewMedia\x18\x03 \x01(\x0b\x32\x17.proto.BotMediaMetadataH\x02\x88\x01\x01\x42\x05\n\x03_idB\x0f\n\r_highResMediaB\x0f\n\r_previewMedia\x1a\x86\x01\n\x12SideBySideMetadata\x12\x1e\n\x11primaryResponseId\x18\x01 \x01(\tH\x00\x88\x01\x01\x12!\n\x14surveyCtaHasRendered\x18\x02 \x01(\x08H\x01\x88\x01\x01\x42\x14\n\x12_primaryResponseIdB\x17\n\x15_surveyCtaHasRenderedB\x0e\n\x0c_sbsMetadata\"\x94\t\n\rCallLogRecord\x12\x38\n\ncallResult\x18\x01 \x01(\x0e\x32\x1f.proto.CallLogRecord.CallResultH\x00\x88\x01\x01\x12\x16\n\tisDndMode\x18\x02 \x01(\x08H\x01\x88\x01\x01\x12>\n\rsilenceReason\x18\x03 \x01(\x0e\x32\".proto.CallLogRecord.SilenceReasonH\x02\x88\x01\x01\x12\x15\n\x08\x64uration\x18\x04 \x01(\x03H\x03\x88\x01\x01\x12\x16\n\tstartTime\x18\x05 \x01(\x03H\x04\x88\x01\x01\x12\x17\n\nisIncoming\x18\x06 \x01(\x08H\x05\x88\x01\x01\x12\x14\n\x07isVideo\x18\x07 \x01(\x08H\x06\x88\x01\x01\x12\x17\n\nisCallLink\x18\x08 \x01(\x08H\x07\x88\x01\x01\x12\x1a\n\rcallLinkToken\x18\t \x01(\tH\x08\x88\x01\x01\x12\x1c\n\x0fscheduledCallId\x18\n \x01(\tH\t\x88\x01\x01\x12\x13\n\x06\x63\x61llId\x18\x0b \x01(\tH\n\x88\x01\x01\x12\x1b\n\x0e\x63\x61llCreatorJid\x18\x0c \x01(\tH\x0b\x88\x01\x01\x12\x15\n\x08groupJid\x18\r \x01(\tH\x0c\x88\x01\x01\x12:\n\x0cparticipants\x18\x0e \x03(\x0b\x32$.proto.CallLogRecord.ParticipantInfo\x12\x34\n\x08\x63\x61llType\x18\x0f \x01(\x0e\x32\x1d.proto.CallLogRecord.CallTypeH\r\x88\x01\x01\x1a|\n\x0fParticipantInfo\x12\x14\n\x07userJid\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x38\n\ncallResult\x18\x02 \x01(\x0e\x32\x1f.proto.CallLogRecord.CallResultH\x01\x88\x01\x01\x42\n\n\x08_userJidB\r\n\x0b_callResult\"\xaf\x01\n\nCallResult\x12\r\n\tCONNECTED\x10\x00\x12\x0c\n\x08REJECTED\x10\x01\x12\r\n\tCANCELLED\x10\x02\x12\x15\n\x11\x41\x43\x43\x45PTEDELSEWHERE\x10\x03\x12\n\n\x06MISSED\x10\x04\x12\x0b\n\x07INVALID\x10\x05\x12\x0f\n\x0bUNAVAILABLE\x10\x06\x12\x0c\n\x08UPCOMING\x10\x07\x12\n\n\x06\x46\x41ILED\x10\x08\x12\r\n\tABANDONED\x10\t\x12\x0b\n\x07ONGOING\x10\n\";\n\x08\x43\x61llType\x12\x0b\n\x07REGULAR\x10\x00\x12\x12\n\x0eSCHEDULED_CALL\x10\x01\x12\x0e\n\nVOICE_CHAT\x10\x02\"F\n\rSilenceReason\x12\x08\n\x04NONE\x10\x00\x12\r\n\tSCHEDULED\x10\x01\x12\x0b\n\x07PRIVACY\x10\x02\x12\x0f\n\x0bLIGHTWEIGHT\x10\x03\x42\r\n\x0b_callResultB\x0c\n\n_isDndModeB\x10\n\x0e_silenceReasonB\x0b\n\t_durationB\x0c\n\n_startTimeB\r\n\x0b_isIncomingB\n\n\x08_isVideoB\r\n\x0b_isCallLinkB\x10\n\x0e_callLinkTokenB\x12\n\x10_scheduledCallIdB\t\n\x07_callIdB\x11\n\x0f_callCreatorJidB\x0b\n\t_groupJidB\x0b\n\t_callType\"\xb2\x03\n\tCertChain\x12\x34\n\x04leaf\x18\x01 \x01(\x0b\x32!.proto.CertChain.NoiseCertificateH\x00\x88\x01\x01\x12<\n\x0cintermediate\x18\x02 \x01(\x0b\x32!.proto.CertChain.NoiseCertificateH\x01\x88\x01\x01\x1a\x96\x02\n\x10NoiseCertificate\x12\x14\n\x07\x64\x65tails\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x16\n\tsignature\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x1a\xb9\x01\n\x07\x44\x65tails\x12\x13\n\x06serial\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x19\n\x0cissuerSerial\x18\x02 \x01(\rH\x01\x88\x01\x01\x12\x10\n\x03key\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x16\n\tnotBefore\x18\x04 \x01(\x04H\x03\x88\x01\x01\x12\x15\n\x08notAfter\x18\x05 \x01(\x04H\x04\x88\x01\x01\x42\t\n\x07_serialB\x0f\n\r_issuerSerialB\x06\n\x04_keyB\x0c\n\n_notBeforeB\x0b\n\t_notAfterB\n\n\x08_detailsB\x0c\n\n_signatureB\x07\n\x05_leafB\x0f\n\r_intermediate\"\x81\x01\n\x10\x43hatLockSettings\x12\x1c\n\x0fhideLockedChats\x18\x01 \x01(\x08H\x00\x88\x01\x01\x12,\n\nsecretCode\x18\x02 \x01(\x0b\x32\x13.proto.UserPasswordH\x01\x88\x01\x01\x42\x12\n\x10_hideLockedChatsB\r\n\x0b_secretCode\"\x89\n\n\x11\x43hatRowOpaqueData\x12@\n\x0c\x64raftMessage\x18\x01 \x01(\x0b\x32%.proto.ChatRowOpaqueData.DraftMessageH\x00\x88\x01\x01\x1a\xa0\t\n\x0c\x44raftMessage\x12\x11\n\x04text\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x17\n\nomittedUrl\x18\x02 \x01(\tH\x01\x88\x01\x01\x12[\n\x13\x63twaContextLinkData\x18\x03 \x01(\x0b\x32\x39.proto.ChatRowOpaqueData.DraftMessage.CtwaContextLinkDataH\x02\x88\x01\x01\x12O\n\x0b\x63twaContext\x18\x04 \x01(\x0b\x32\x35.proto.ChatRowOpaqueData.DraftMessage.CtwaContextDataH\x03\x88\x01\x01\x12\x16\n\ttimestamp\x18\x05 \x01(\x03H\x04\x88\x01\x01\x1a\xa9\x05\n\x0f\x43twaContextData\x12\x1d\n\x10\x63onversionSource\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x1b\n\x0e\x63onversionData\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x16\n\tsourceUrl\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x15\n\x08sourceId\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x17\n\nsourceType\x18\x05 \x01(\tH\x04\x88\x01\x01\x12\x12\n\x05title\x18\x06 \x01(\tH\x05\x88\x01\x01\x12\x18\n\x0b\x64\x65scription\x18\x07 \x01(\tH\x06\x88\x01\x01\x12\x16\n\tthumbnail\x18\x08 \x01(\tH\x07\x88\x01\x01\x12\x19\n\x0cthumbnailUrl\x18\t \x01(\tH\x08\x88\x01\x01\x12u\n\tmediaType\x18\n \x01(\x0e\x32].proto.ChatRowOpaqueData.DraftMessage.CtwaContextData.ContextInfoExternalAdReplyInfoMediaTypeH\t\x88\x01\x01\x12\x15\n\x08mediaUrl\x18\x0b \x01(\tH\n\x88\x01\x01\x12\x1d\n\x10isSuspiciousLink\x18\x0c \x01(\x08H\x0b\x88\x01\x01\"I\n\'ContextInfoExternalAdReplyInfoMediaType\x12\x08\n\x04NONE\x10\x00\x12\t\n\x05IMAGE\x10\x01\x12\t\n\x05VIDEO\x10\x02\x42\x13\n\x11_conversionSourceB\x11\n\x0f_conversionDataB\x0c\n\n_sourceUrlB\x0b\n\t_sourceIdB\r\n\x0b_sourceTypeB\x08\n\x06_titleB\x0e\n\x0c_descriptionB\x0c\n\n_thumbnailB\x0f\n\r_thumbnailUrlB\x0c\n\n_mediaTypeB\x0b\n\t_mediaUrlB\x13\n\x11_isSuspiciousLink\x1a\xa3\x01\n\x13\x43twaContextLinkData\x12\x14\n\x07\x63ontext\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x16\n\tsourceUrl\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x17\n\nicebreaker\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x12\n\x05phone\x18\x04 \x01(\tH\x03\x88\x01\x01\x42\n\n\x08_contextB\x0c\n\n_sourceUrlB\r\n\x0b_icebreakerB\x08\n\x06_phoneB\x07\n\x05_textB\r\n\x0b_omittedUrlB\x16\n\x14_ctwaContextLinkDataB\x0e\n\x0c_ctwaContextB\x0c\n\n_timestampB\x0f\n\r_draftMessage\"L\n\x08\x43itation\x12\r\n\x05title\x18\x01 \x01(\t\x12\x10\n\x08subtitle\x18\x02 \x01(\t\x12\r\n\x05\x63msId\x18\x03 \x01(\t\x12\x10\n\x08imageUrl\x18\x04 \x01(\t\"\xdc\x01\n\x12\x43lientPairingProps\x12 \n\x13isChatDbLidMigrated\x18\x01 \x01(\x08H\x00\x88\x01\x01\x12\"\n\x15isSyncdPureLidSession\x18\x02 \x01(\x08H\x01\x88\x01\x01\x12+\n\x1eisSyncdSnapshotRecoveryEnabled\x18\x03 \x01(\x08H\x02\x88\x01\x01\x42\x16\n\x14_isChatDbLidMigratedB\x18\n\x16_isSyncdPureLidSessionB!\n\x1f_isSyncdSnapshotRecoveryEnabled\"\x97.\n\rClientPayload\x12\x15\n\x08username\x18\x01 \x01(\x04H\x00\x88\x01\x01\x12\x14\n\x07passive\x18\x03 \x01(\x08H\x01\x88\x01\x01\x12\x36\n\tuserAgent\x18\x05 \x01(\x0b\x32\x1e.proto.ClientPayload.UserAgentH\x02\x88\x01\x01\x12\x32\n\x07webInfo\x18\x06 \x01(\x0b\x32\x1c.proto.ClientPayload.WebInfoH\x03\x88\x01\x01\x12\x15\n\x08pushName\x18\x07 \x01(\tH\x04\x88\x01\x01\x12\x16\n\tsessionId\x18\t \x01(\x0fH\x05\x88\x01\x01\x12\x19\n\x0cshortConnect\x18\n \x01(\x08H\x06\x88\x01\x01\x12:\n\x0b\x63onnectType\x18\x0c \x01(\x0e\x32 .proto.ClientPayload.ConnectTypeH\x07\x88\x01\x01\x12>\n\rconnectReason\x18\r \x01(\x0e\x32\".proto.ClientPayload.ConnectReasonH\x08\x88\x01\x01\x12\x0e\n\x06shards\x18\x0e \x03(\x05\x12\x36\n\tdnsSource\x18\x0f \x01(\x0b\x32\x1e.proto.ClientPayload.DNSSourceH\t\x88\x01\x01\x12 \n\x13\x63onnectAttemptCount\x18\x10 \x01(\rH\n\x88\x01\x01\x12\x13\n\x06\x64\x65vice\x18\x12 \x01(\rH\x0b\x88\x01\x01\x12R\n\x11\x64\x65vicePairingData\x18\x13 \x01(\x0b\x32\x32.proto.ClientPayload.DevicePairingRegistrationDataH\x0c\x88\x01\x01\x12\x32\n\x07product\x18\x14 \x01(\x0e\x32\x1c.proto.ClientPayload.ProductH\r\x88\x01\x01\x12\x12\n\x05\x66\x62\x43\x61t\x18\x15 \x01(\x0cH\x0e\x88\x01\x01\x12\x18\n\x0b\x66\x62UserAgent\x18\x16 \x01(\x0cH\x0f\x88\x01\x01\x12\x0f\n\x02oc\x18\x17 \x01(\x08H\x10\x88\x01\x01\x12\x0f\n\x02lc\x18\x18 \x01(\x05H\x11\x88\x01\x01\x12\x42\n\x0fiosAppExtension\x18\x1e \x01(\x0e\x32$.proto.ClientPayload.IOSAppExtensionH\x12\x88\x01\x01\x12\x14\n\x07\x66\x62\x41ppId\x18\x1f \x01(\x04H\x13\x88\x01\x01\x12\x17\n\nfbDeviceId\x18  \x01(\x0cH\x14\x88\x01\x01\x12\x11\n\x04pull\x18! \x01(\x08H\x15\x88\x01\x01\x12\x19\n\x0cpaddingBytes\x18\" \x01(\x0cH\x16\x88\x01\x01\x12\x16\n\tyearClass\x18$ \x01(\x05H\x17\x88\x01\x01\x12\x15\n\x08memClass\x18% \x01(\x05H\x18\x88\x01\x01\x12:\n\x0binteropData\x18& \x01(\x0b\x32 .proto.ClientPayload.InteropDataH\x19\x88\x01\x01\x12L\n\x14trafficAnonymization\x18( \x01(\x0e\x32).proto.ClientPayload.TrafficAnonymizationH\x1a\x88\x01\x01\x12\x1a\n\rlidDbMigrated\x18) \x01(\x08H\x1b\x88\x01\x01\x12:\n\x0b\x61\x63\x63ountType\x18* \x01(\x0e\x32 .proto.ClientPayload.AccountTypeH\x1c\x88\x01\x01\x12#\n\x16\x63onnectionSequenceInfo\x18+ \x01(\x0fH\x1d\x88\x01\x01\x12\x14\n\x07paaLink\x18, \x01(\x08H\x1e\x88\x01\x01\x12\x19\n\x0cpreacksCount\x18- \x01(\x05H\x1f\x88\x01\x01\x12 \n\x13processingQueueSize\x18. \x01(\x05H \x88\x01\x01\x1a\xee\x01\n\tDNSSource\x12J\n\tdnsMethod\x18\x0f \x01(\x0e\x32\x32.proto.ClientPayload.DNSSource.DNSResolutionMethodH\x00\x88\x01\x01\x12\x16\n\tappCached\x18\x10 \x01(\x08H\x01\x88\x01\x01\"a\n\x13\x44NSResolutionMethod\x12\n\n\x06SYSTEM\x10\x00\x12\n\n\x06GOOGLE\x10\x01\x12\r\n\tHARDCODED\x10\x02\x12\x0c\n\x08OVERRIDE\x10\x03\x12\x0c\n\x08\x46\x41LLBACK\x10\x04\x12\x07\n\x03MNS\x10\x05\x42\x0c\n\n_dnsMethodB\x0c\n\n_appCached\x1a\xbd\x02\n\x1d\x44\x65vicePairingRegistrationData\x12\x13\n\x06\x65Regid\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x15\n\x08\x65Keytype\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x13\n\x06\x65Ident\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x14\n\x07\x65SkeyId\x18\x04 \x01(\x0cH\x03\x88\x01\x01\x12\x15\n\x08\x65SkeyVal\x18\x05 \x01(\x0cH\x04\x88\x01\x01\x12\x15\n\x08\x65SkeySig\x18\x06 \x01(\x0cH\x05\x88\x01\x01\x12\x16\n\tbuildHash\x18\x07 \x01(\x0cH\x06\x88\x01\x01\x12\x18\n\x0b\x64\x65viceProps\x18\x08 \x01(\x0cH\x07\x88\x01\x01\x42\t\n\x07_eRegidB\x0b\n\t_eKeytypeB\t\n\x07_eIdentB\n\n\x08_eSkeyIdB\x0b\n\t_eSkeyValB\x0b\n\t_eSkeySigB\x0c\n\n_buildHashB\x0e\n\x0c_deviceProps\x1a\x89\x01\n\x0bInteropData\x12\x16\n\taccountId\x18\x01 \x01(\x04H\x00\x88\x01\x01\x12\x12\n\x05token\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x1f\n\x12\x65nableReadReceipts\x18\x03 \x01(\x08H\x02\x88\x01\x01\x42\x0c\n\n_accountIdB\x08\n\x06_tokenB\x15\n\x13_enableReadReceipts\x1a\xd5\r\n\tUserAgent\x12>\n\x08platform\x18\x01 \x01(\x0e\x32\'.proto.ClientPayload.UserAgent.PlatformH\x00\x88\x01\x01\x12\x42\n\nappVersion\x18\x02 \x01(\x0b\x32).proto.ClientPayload.UserAgent.AppVersionH\x01\x88\x01\x01\x12\x10\n\x03mcc\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x10\n\x03mnc\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x16\n\tosVersion\x18\x05 \x01(\tH\x04\x88\x01\x01\x12\x19\n\x0cmanufacturer\x18\x06 \x01(\tH\x05\x88\x01\x01\x12\x13\n\x06\x64\x65vice\x18\x07 \x01(\tH\x06\x88\x01\x01\x12\x1a\n\rosBuildNumber\x18\x08 \x01(\tH\x07\x88\x01\x01\x12\x14\n\x07phoneId\x18\t \x01(\tH\x08\x88\x01\x01\x12J\n\x0ereleaseChannel\x18\n \x01(\x0e\x32-.proto.ClientPayload.UserAgent.ReleaseChannelH\t\x88\x01\x01\x12\"\n\x15localeLanguageIso6391\x18\x0b \x01(\tH\n\x88\x01\x01\x12(\n\x1blocaleCountryIso31661Alpha2\x18\x0c \x01(\tH\x0b\x88\x01\x01\x12\x18\n\x0b\x64\x65viceBoard\x18\r \x01(\tH\x0c\x88\x01\x01\x12\x18\n\x0b\x64\x65viceExpId\x18\x0e \x01(\tH\r\x88\x01\x01\x12\x42\n\ndeviceType\x18\x0f \x01(\x0e\x32).proto.ClientPayload.UserAgent.DeviceTypeH\x0e\x88\x01\x01\x12\x1c\n\x0f\x64\x65viceModelType\x18\x10 \x01(\tH\x0f\x88\x01\x01\x1a\xc2\x01\n\nAppVersion\x12\x14\n\x07primary\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x16\n\tsecondary\x18\x02 \x01(\rH\x01\x88\x01\x01\x12\x15\n\x08tertiary\x18\x03 \x01(\rH\x02\x88\x01\x01\x12\x17\n\nquaternary\x18\x04 \x01(\rH\x03\x88\x01\x01\x12\x14\n\x07quinary\x18\x05 \x01(\rH\x04\x88\x01\x01\x42\n\n\x08_primaryB\x0c\n\n_secondaryB\x0b\n\t_tertiaryB\r\n\x0b_quaternaryB\n\n\x08_quinary\"F\n\nDeviceType\x12\t\n\x05PHONE\x10\x00\x12\n\n\x06TABLET\x10\x01\x12\x0b\n\x07\x44\x45SKTOP\x10\x02\x12\x0c\n\x08WEARABLE\x10\x03\x12\x06\n\x02VR\x10\x04\"\xa5\x04\n\x08Platform\x12\x0b\n\x07\x41NDROID\x10\x00\x12\x07\n\x03IOS\x10\x01\x12\x11\n\rWINDOWS_PHONE\x10\x02\x12\x0e\n\nBLACKBERRY\x10\x03\x12\x0f\n\x0b\x42LACKBERRYX\x10\x04\x12\x07\n\x03S40\x10\x05\x12\x07\n\x03S60\x10\x06\x12\x11\n\rPYTHON_CLIENT\x10\x07\x12\t\n\x05TIZEN\x10\x08\x12\x0e\n\nENTERPRISE\x10\t\x12\x0f\n\x0bSMB_ANDROID\x10\n\x12\t\n\x05KAIOS\x10\x0b\x12\x0b\n\x07SMB_IOS\x10\x0c\x12\x0b\n\x07WINDOWS\x10\r\x12\x07\n\x03WEB\x10\x0e\x12\n\n\x06PORTAL\x10\x0f\x12\x11\n\rGREEN_ANDROID\x10\x10\x12\x10\n\x0cGREEN_IPHONE\x10\x11\x12\x10\n\x0c\x42LUE_ANDROID\x10\x12\x12\x0f\n\x0b\x42LUE_IPHONE\x10\x13\x12\x12\n\x0e\x46\x42LITE_ANDROID\x10\x14\x12\x11\n\rMLITE_ANDROID\x10\x15\x12\x12\n\x0eIGLITE_ANDROID\x10\x16\x12\x08\n\x04PAGE\x10\x17\x12\t\n\x05MACOS\x10\x18\x12\x0e\n\nOCULUS_MSG\x10\x19\x12\x0f\n\x0bOCULUS_CALL\x10\x1a\x12\t\n\x05MILAN\x10\x1b\x12\x08\n\x04\x43\x41PI\x10\x1c\x12\n\n\x06WEAROS\x10\x1d\x12\x0c\n\x08\x41RDEVICE\x10\x1e\x12\x0c\n\x08VRDEVICE\x10\x1f\x12\x0c\n\x08\x42LUE_WEB\x10 \x12\x08\n\x04IPAD\x10!\x12\x08\n\x04TEST\x10\"\x12\x11\n\rSMART_GLASSES\x10#\x12\x0b\n\x07\x42LUE_VR\x10$\x12\x0c\n\x08\x41R_WRIST\x10%\"=\n\x0eReleaseChannel\x12\x0b\n\x07RELEASE\x10\x00\x12\x08\n\x04\x42\x45TA\x10\x01\x12\t\n\x05\x41LPHA\x10\x02\x12\t\n\x05\x44\x45\x42UG\x10\x03\x42\x0b\n\t_platformB\r\n\x0b_appVersionB\x06\n\x04_mccB\x06\n\x04_mncB\x0c\n\n_osVersionB\x0f\n\r_manufacturerB\t\n\x07_deviceB\x10\n\x0e_osBuildNumberB\n\n\x08_phoneIdB\x11\n\x0f_releaseChannelB\x18\n\x16_localeLanguageIso6391B\x1e\n\x1c_localeCountryIso31661Alpha2B\x0e\n\x0c_deviceBoardB\x0e\n\x0c_deviceExpIdB\r\n\x0b_deviceTypeB\x12\n\x10_deviceModelType\x1a\xd4\x07\n\x07WebInfo\x12\x15\n\x08refToken\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x14\n\x07version\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x42\n\x0bwebdPayload\x18\x03 \x01(\x0b\x32(.proto.ClientPayload.WebInfo.WebdPayloadH\x02\x88\x01\x01\x12H\n\x0ewebSubPlatform\x18\x04 \x01(\x0e\x32+.proto.ClientPayload.WebInfo.WebSubPlatformH\x03\x88\x01\x01\x1a\xe9\x04\n\x0bWebdPayload\x12!\n\x14usesParticipantInKey\x18\x01 \x01(\x08H\x00\x88\x01\x01\x12$\n\x17supportsStarredMessages\x18\x02 \x01(\x08H\x01\x88\x01\x01\x12%\n\x18supportsDocumentMessages\x18\x03 \x01(\x08H\x02\x88\x01\x01\x12 \n\x13supportsUrlMessages\x18\x04 \x01(\x08H\x03\x88\x01\x01\x12\x1f\n\x12supportsMediaRetry\x18\x05 \x01(\x08H\x04\x88\x01\x01\x12\x1d\n\x10supportsE2EImage\x18\x06 \x01(\x08H\x05\x88\x01\x01\x12\x1d\n\x10supportsE2EVideo\x18\x07 \x01(\x08H\x06\x88\x01\x01\x12\x1d\n\x10supportsE2EAudio\x18\x08 \x01(\x08H\x07\x88\x01\x01\x12 \n\x13supportsE2EDocument\x18\t \x01(\x08H\x08\x88\x01\x01\x12\x1a\n\rdocumentTypes\x18\n \x01(\tH\t\x88\x01\x01\x12\x15\n\x08\x66\x65\x61tures\x18\x0b \x01(\x0cH\n\x88\x01\x01\x42\x17\n\x15_usesParticipantInKeyB\x1a\n\x18_supportsStarredMessagesB\x1b\n\x19_supportsDocumentMessagesB\x16\n\x14_supportsUrlMessagesB\x15\n\x13_supportsMediaRetryB\x13\n\x11_supportsE2EImageB\x13\n\x11_supportsE2EVideoB\x13\n\x11_supportsE2EAudioB\x16\n\x14_supportsE2EDocumentB\x10\n\x0e_documentTypesB\x0b\n\t_features\"f\n\x0eWebSubPlatform\x12\x0f\n\x0bWEB_BROWSER\x10\x00\x12\r\n\tAPP_STORE\x10\x01\x12\r\n\tWIN_STORE\x10\x02\x12\n\n\x06\x44\x41RWIN\x10\x03\x12\t\n\x05WIN32\x10\x04\x12\x0e\n\nWIN_HYBRID\x10\x05\x42\x0b\n\t_refTokenB\n\n\x08_versionB\x0e\n\x0c_webdPayloadB\x11\n\x0f_webSubPlatform\"%\n\x0b\x41\x63\x63ountType\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x00\x12\t\n\x05GUEST\x10\x01\"\x86\x01\n\rConnectReason\x12\x08\n\x04PUSH\x10\x00\x12\x12\n\x0eUSER_ACTIVATED\x10\x01\x12\r\n\tSCHEDULED\x10\x02\x12\x13\n\x0f\x45RROR_RECONNECT\x10\x03\x12\x12\n\x0eNETWORK_SWITCH\x10\x04\x12\x12\n\x0ePING_RECONNECT\x10\x05\x12\x0b\n\x07UNKNOWN\x10\x06\"\xb0\x02\n\x0b\x43onnectType\x12\x14\n\x10\x43\x45LLULAR_UNKNOWN\x10\x00\x12\x10\n\x0cWIFI_UNKNOWN\x10\x01\x12\x11\n\rCELLULAR_EDGE\x10\x64\x12\x11\n\rCELLULAR_IDEN\x10\x65\x12\x11\n\rCELLULAR_UMTS\x10\x66\x12\x11\n\rCELLULAR_EVDO\x10g\x12\x11\n\rCELLULAR_GPRS\x10h\x12\x12\n\x0e\x43\x45LLULAR_HSDPA\x10i\x12\x12\n\x0e\x43\x45LLULAR_HSUPA\x10j\x12\x11\n\rCELLULAR_HSPA\x10k\x12\x11\n\rCELLULAR_CDMA\x10l\x12\x12\n\x0e\x43\x45LLULAR_1XRTT\x10m\x12\x12\n\x0e\x43\x45LLULAR_EHRPD\x10n\x12\x10\n\x0c\x43\x45LLULAR_LTE\x10o\x12\x12\n\x0e\x43\x45LLULAR_HSPAP\x10p\"T\n\x0fIOSAppExtension\x12\x13\n\x0fSHARE_EXTENSION\x10\x00\x12\x15\n\x11SERVICE_EXTENSION\x10\x01\x12\x15\n\x11INTENTS_EXTENSION\x10\x02\"W\n\x07Product\x12\x0c\n\x08WHATSAPP\x10\x00\x12\r\n\tMESSENGER\x10\x01\x12\x0b\n\x07INTEROP\x10\x02\x12\x10\n\x0cINTEROP_MSGR\x10\x03\x12\x10\n\x0cWHATSAPP_LID\x10\x04\"-\n\x14TrafficAnonymization\x12\x07\n\x03OFF\x10\x00\x12\x0c\n\x08STANDARD\x10\x01\x42\x0b\n\t_usernameB\n\n\x08_passiveB\x0c\n\n_userAgentB\n\n\x08_webInfoB\x0b\n\t_pushNameB\x0c\n\n_sessionIdB\x0f\n\r_shortConnectB\x0e\n\x0c_connectTypeB\x10\n\x0e_connectReasonB\x0c\n\n_dnsSourceB\x16\n\x14_connectAttemptCountB\t\n\x07_deviceB\x14\n\x12_devicePairingDataB\n\n\x08_productB\x08\n\x06_fbCatB\x0e\n\x0c_fbUserAgentB\x05\n\x03_ocB\x05\n\x03_lcB\x12\n\x10_iosAppExtensionB\n\n\x08_fbAppIdB\r\n\x0b_fbDeviceIdB\x07\n\x05_pullB\x0f\n\r_paddingBytesB\x0c\n\n_yearClassB\x0b\n\t_memClassB\x0e\n\x0c_interopDataB\x17\n\x15_trafficAnonymizationB\x10\n\x0e_lidDbMigratedB\x0e\n\x0c_accountTypeB\x19\n\x17_connectionSequenceInfoB\n\n\x08_paaLinkB\x0f\n\r_preacksCountB\x16\n\x14_processingQueueSize\"\x80\x01\n\x0f\x43ommentMetadata\x12\x30\n\x10\x63ommentParentKey\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x12\x17\n\nreplyCount\x18\x02 \x01(\rH\x01\x88\x01\x01\x42\x13\n\x11_commentParentKeyB\r\n\x0b_replyCount\"1\n\x13\x43ompanionCommitment\x12\x11\n\x04hash\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x42\x07\n\x05_hash\"\xa5\x01\n\x1a\x43ompanionEphemeralIdentity\x12\x16\n\tpublicKey\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x38\n\ndeviceType\x18\x02 \x01(\x0e\x32\x1f.proto.DeviceProps.PlatformTypeH\x01\x88\x01\x01\x12\x10\n\x03ref\x18\x03 \x01(\tH\x02\x88\x01\x01\x42\x0c\n\n_publicKeyB\r\n\x0b_deviceTypeB\x06\n\x04_ref\"\x8f\x01\n\x06\x43onfig\x12\'\n\x05\x66ield\x18\x01 \x03(\x0b\x32\x18.proto.Config.FieldEntry\x12\x14\n\x07version\x18\x02 \x01(\rH\x00\x88\x01\x01\x1a:\n\nFieldEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x12\x1b\n\x05value\x18\x02 \x01(\x0b\x32\x0c.proto.Field:\x02\x38\x01\x42\n\n\x08_version\"\xc3=\n\x0b\x43ontextInfo\x12\x15\n\x08stanzaId\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x18\n\x0bparticipant\x18\x02 \x01(\tH\x01\x88\x01\x01\x12*\n\rquotedMessage\x18\x03 \x01(\x0b\x32\x0e.proto.MessageH\x02\x88\x01\x01\x12\x16\n\tremoteJid\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x14\n\x0cmentionedJid\x18\x0f \x03(\t\x12\x1d\n\x10\x63onversionSource\x18\x12 \x01(\tH\x04\x88\x01\x01\x12\x1b\n\x0e\x63onversionData\x18\x13 \x01(\x0cH\x05\x88\x01\x01\x12#\n\x16\x63onversionDelaySeconds\x18\x14 \x01(\rH\x06\x88\x01\x01\x12\x1c\n\x0f\x66orwardingScore\x18\x15 \x01(\rH\x07\x88\x01\x01\x12\x18\n\x0bisForwarded\x18\x16 \x01(\x08H\x08\x88\x01\x01\x12\x35\n\x08quotedAd\x18\x17 \x01(\x0b\x32\x1e.proto.ContextInfo.AdReplyInfoH\t\x88\x01\x01\x12.\n\x0eplaceholderKey\x18\x18 \x01(\x0b\x32\x11.proto.MessageKeyH\n\x88\x01\x01\x12\x17\n\nexpiration\x18\x19 \x01(\rH\x0b\x88\x01\x01\x12&\n\x19\x65phemeralSettingTimestamp\x18\x1a \x01(\x03H\x0c\x88\x01\x01\x12\"\n\x15\x65phemeralSharedSecret\x18\x1b \x01(\x0cH\r\x88\x01\x01\x12\x44\n\x0f\x65xternalAdReply\x18\x1c \x01(\x0b\x32&.proto.ContextInfo.ExternalAdReplyInfoH\x0e\x88\x01\x01\x12\'\n\x1a\x65ntryPointConversionSource\x18\x1d \x01(\tH\x0f\x88\x01\x01\x12$\n\x17\x65ntryPointConversionApp\x18\x1e \x01(\tH\x10\x88\x01\x01\x12-\n entryPointConversionDelaySeconds\x18\x1f \x01(\rH\x11\x88\x01\x01\x12\x36\n\x10\x64isappearingMode\x18  \x01(\x0b\x32\x17.proto.DisappearingModeH\x12\x88\x01\x01\x12*\n\nactionLink\x18! \x01(\x0b\x32\x11.proto.ActionLinkH\x13\x88\x01\x01\x12\x19\n\x0cgroupSubject\x18\" \x01(\tH\x14\x88\x01\x01\x12\x1b\n\x0eparentGroupJid\x18# \x01(\tH\x15\x88\x01\x01\x12\x1c\n\x0ftrustBannerType\x18% \x01(\tH\x16\x88\x01\x01\x12\x1e\n\x11trustBannerAction\x18& \x01(\rH\x17\x88\x01\x01\x12\x16\n\tisSampled\x18\' \x01(\x08H\x18\x88\x01\x01\x12*\n\rgroupMentions\x18( \x03(\x0b\x32\x13.proto.GroupMention\x12,\n\x03utm\x18) \x01(\x0b\x32\x1a.proto.ContextInfo.UTMInfoH\x19\x88\x01\x01\x12^\n\x1e\x66orwardedNewsletterMessageInfo\x18+ \x01(\x0b\x32\x31.proto.ContextInfo.ForwardedNewsletterMessageInfoH\x1a\x88\x01\x01\x12V\n\x1a\x62usinessMessageForwardInfo\x18, \x01(\x0b\x32-.proto.ContextInfo.BusinessMessageForwardInfoH\x1b\x88\x01\x01\x12 \n\x13smbClientCampaignId\x18- \x01(\tH\x1c\x88\x01\x01\x12 \n\x13smbServerCampaignId\x18. \x01(\tH\x1d\x88\x01\x01\x12\x46\n\x12\x64\x61taSharingContext\x18/ \x01(\x0b\x32%.proto.ContextInfo.DataSharingContextH\x1e\x88\x01\x01\x12$\n\x17\x61lwaysShowAdAttribution\x18\x30 \x01(\x08H\x1f\x88\x01\x01\x12J\n\x14\x66\x65\x61tureEligibilities\x18\x31 \x01(\x0b\x32\'.proto.ContextInfo.FeatureEligibilitiesH \x88\x01\x01\x12/\n\"entryPointConversionExternalSource\x18\x32 \x01(\tH!\x88\x01\x01\x12/\n\"entryPointConversionExternalMedium\x18\x33 \x01(\tH\"\x88\x01\x01\x12\x18\n\x0b\x63twaSignals\x18\x36 \x01(\tH#\x88\x01\x01\x12\x18\n\x0b\x63twaPayload\x18\x37 \x01(\x0cH$\x88\x01\x01\x12H\n\x19\x66orwardedAiBotMessageInfo\x18\x38 \x01(\x0b\x32 .proto.ForwardedAIBotMessageInfoH%\x88\x01\x01\x12L\n\x15statusAttributionType\x18\x39 \x01(\x0e\x32(.proto.ContextInfo.StatusAttributionTypeH&\x88\x01\x01\x12\x32\n\x0eurlTrackingMap\x18: \x01(\x0b\x32\x15.proto.UrlTrackingMapH\'\x88\x01\x01\x12@\n\x0fpairedMediaType\x18; \x01(\x0e\x32\".proto.ContextInfo.PairedMediaTypeH(\x88\x01\x01\x12\x1b\n\x0erankingVersion\x18< \x01(\rH)\x88\x01\x01\x12,\n\x0bmemberLabel\x18> \x01(\x0b\x32\x12.proto.MemberLabelH*\x88\x01\x01\x12\x17\n\nisQuestion\x18? \x01(\x08H+\x88\x01\x01\x12\x42\n\x10statusSourceType\x18@ \x01(\x0e\x32#.proto.ContextInfo.StatusSourceTypeH,\x88\x01\x01\x12\x34\n\x12statusAttributions\x18\x41 \x03(\x0b\x32\x18.proto.StatusAttribution\x12\x1a\n\risGroupStatus\x18\x42 \x01(\x08H-\x88\x01\x01\x12<\n\rforwardOrigin\x18\x43 \x01(\x0e\x32 .proto.ContextInfo.ForwardOriginH.\x88\x01\x01\x12V\n\x1aquestionReplyQuotedMessage\x18\x44 \x01(\x0b\x32-.proto.ContextInfo.QuestionReplyQuotedMessageH/\x88\x01\x01\x12N\n\x16statusAudienceMetadata\x18\x45 \x01(\x0b\x32).proto.ContextInfo.StatusAudienceMetadataH0\x88\x01\x01\x12\x1b\n\x0enonJidMentions\x18\x46 \x01(\rH1\x88\x01\x01\x12\x36\n\nquotedType\x18G \x01(\x0e\x32\x1d.proto.ContextInfo.QuotedTypeH2\x88\x01\x01\x12@\n\x15\x62otMessageSharingInfo\x18H \x01(\x0b\x32\x1c.proto.BotMessageSharingInfoH3\x88\x01\x01\x1a\x8a\x02\n\x0b\x41\x64ReplyInfo\x12\x1b\n\x0e\x61\x64vertiserName\x18\x01 \x01(\tH\x00\x88\x01\x01\x12@\n\tmediaType\x18\x02 \x01(\x0e\x32(.proto.ContextInfo.AdReplyInfo.MediaTypeH\x01\x88\x01\x01\x12\x1a\n\rjpegThumbnail\x18\x10 \x01(\x0cH\x02\x88\x01\x01\x12\x14\n\x07\x63\x61ption\x18\x11 \x01(\tH\x03\x88\x01\x01\"+\n\tMediaType\x12\x08\n\x04NONE\x10\x00\x12\t\n\x05IMAGE\x10\x01\x12\t\n\x05VIDEO\x10\x02\x42\x11\n\x0f_advertiserNameB\x0c\n\n_mediaTypeB\x10\n\x0e_jpegThumbnailB\n\n\x08_caption\x1aP\n\x1a\x42usinessMessageForwardInfo\x12\x1d\n\x10\x62usinessOwnerJid\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x13\n\x11_businessOwnerJid\x1a\xf8\x04\n\x12\x44\x61taSharingContext\x12\x1d\n\x10showMmDisclosure\x18\x01 \x01(\x08H\x00\x88\x01\x01\x12*\n\x1d\x65ncryptedSignalTokenConsented\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x44\n\nparameters\x18\x03 \x03(\x0b\x32\x30.proto.ContextInfo.DataSharingContext.Parameters\x12\x1d\n\x10\x64\x61taSharingFlags\x18\x04 \x01(\x05H\x02\x88\x01\x01\x1a\xec\x01\n\nParameters\x12\x10\n\x03key\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x17\n\nstringData\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x14\n\x07intData\x18\x03 \x01(\x03H\x02\x88\x01\x01\x12\x16\n\tfloatData\x18\x04 \x01(\x02H\x03\x88\x01\x01\x12G\n\x08\x63ontents\x18\x05 \x01(\x0b\x32\x30.proto.ContextInfo.DataSharingContext.ParametersH\x04\x88\x01\x01\x42\x06\n\x04_keyB\r\n\x0b_stringDataB\n\n\x08_intDataB\x0c\n\n_floatDataB\x0b\n\t_contents\"w\n\x10\x44\x61taSharingFlags\x12\"\n\x1e\x44\x41TA_SHARING_FLAGS_UNSPECIFIED\x10\x00\x12\x1f\n\x1bSHOW_MM_DISCLOSURE_ON_CLICK\x10\x01\x12\x1e\n\x1aSHOW_MM_DISCLOSURE_ON_READ\x10\x02\x42\x13\n\x11_showMmDisclosureB \n\x1e_encryptedSignalTokenConsentedB\x13\n\x11_dataSharingFlags\x1a\xa1\x0b\n\x13\x45xternalAdReplyInfo\x12\x12\n\x05title\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x11\n\x04\x62ody\x18\x02 \x01(\tH\x01\x88\x01\x01\x12H\n\tmediaType\x18\x03 \x01(\x0e\x32\x30.proto.ContextInfo.ExternalAdReplyInfo.MediaTypeH\x02\x88\x01\x01\x12\x19\n\x0cthumbnailUrl\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x15\n\x08mediaUrl\x18\x05 \x01(\tH\x04\x88\x01\x01\x12\x16\n\tthumbnail\x18\x06 \x01(\x0cH\x05\x88\x01\x01\x12\x17\n\nsourceType\x18\x07 \x01(\tH\x06\x88\x01\x01\x12\x15\n\x08sourceId\x18\x08 \x01(\tH\x07\x88\x01\x01\x12\x16\n\tsourceUrl\x18\t \x01(\tH\x08\x88\x01\x01\x12\x1e\n\x11\x63ontainsAutoReply\x18\n \x01(\x08H\t\x88\x01\x01\x12\"\n\x15renderLargerThumbnail\x18\x0b \x01(\x08H\n\x88\x01\x01\x12\x1e\n\x11showAdAttribution\x18\x0c \x01(\x08H\x0b\x88\x01\x01\x12\x15\n\x08\x63twaClid\x18\r \x01(\tH\x0c\x88\x01\x01\x12\x10\n\x03ref\x18\x0e \x01(\tH\r\x88\x01\x01\x12 \n\x13\x63lickToWhatsappCall\x18\x0f \x01(\x08H\x0e\x88\x01\x01\x12&\n\x19\x61\x64\x43ontextPreviewDismissed\x18\x10 \x01(\x08H\x0f\x88\x01\x01\x12\x16\n\tsourceApp\x18\x11 \x01(\tH\x10\x88\x01\x01\x12*\n\x1d\x61utomatedGreetingMessageShown\x18\x12 \x01(\x08H\x11\x88\x01\x01\x12 \n\x13greetingMessageBody\x18\x13 \x01(\tH\x12\x88\x01\x01\x12\x17\n\nctaPayload\x18\x14 \x01(\tH\x13\x88\x01\x01\x12\x19\n\x0c\x64isableNudge\x18\x15 \x01(\x08H\x14\x88\x01\x01\x12\x1d\n\x10originalImageUrl\x18\x16 \x01(\tH\x15\x88\x01\x01\x12,\n\x1f\x61utomatedGreetingMessageCtaType\x18\x17 \x01(\tH\x16\x88\x01\x01\x12\x19\n\x0cwtwaAdFormat\x18\x18 \x01(\x08H\x17\x88\x01\x01\x12\x42\n\x06\x61\x64Type\x18\x19 \x01(\x0e\x32-.proto.ContextInfo.ExternalAdReplyInfo.AdTypeH\x18\x88\x01\x01\x12\x1b\n\x0ewtwaWebsiteUrl\x18\x1a \x01(\tH\x19\x88\x01\x01\x12\x19\n\x0c\x61\x64PreviewUrl\x18\x1b \x01(\tH\x1a\x88\x01\x01\"\x1c\n\x06\x41\x64Type\x12\x08\n\x04\x43TWA\x10\x00\x12\x08\n\x04\x43\x41WC\x10\x01\"+\n\tMediaType\x12\x08\n\x04NONE\x10\x00\x12\t\n\x05IMAGE\x10\x01\x12\t\n\x05VIDEO\x10\x02\x42\x08\n\x06_titleB\x07\n\x05_bodyB\x0c\n\n_mediaTypeB\x0f\n\r_thumbnailUrlB\x0b\n\t_mediaUrlB\x0c\n\n_thumbnailB\r\n\x0b_sourceTypeB\x0b\n\t_sourceIdB\x0c\n\n_sourceUrlB\x14\n\x12_containsAutoReplyB\x18\n\x16_renderLargerThumbnailB\x14\n\x12_showAdAttributionB\x0b\n\t_ctwaClidB\x06\n\x04_refB\x16\n\x14_clickToWhatsappCallB\x1c\n\x1a_adContextPreviewDismissedB\x0c\n\n_sourceAppB \n\x1e_automatedGreetingMessageShownB\x16\n\x14_greetingMessageBodyB\r\n\x0b_ctaPayloadB\x0f\n\r_disableNudgeB\x13\n\x11_originalImageUrlB\"\n _automatedGreetingMessageCtaTypeB\x0f\n\r_wtwaAdFormatB\t\n\x07_adTypeB\x11\n\x0f_wtwaWebsiteUrlB\x0f\n\r_adPreviewUrl\x1a\x9e\x02\n\x14\x46\x65\x61tureEligibilities\x12\x1e\n\x11\x63\x61nnotBeReactedTo\x18\x01 \x01(\x08H\x00\x88\x01\x01\x12\x1b\n\x0e\x63\x61nnotBeRanked\x18\x02 \x01(\x08H\x01\x88\x01\x01\x12\x1f\n\x12\x63\x61nRequestFeedback\x18\x03 \x01(\x08H\x02\x88\x01\x01\x12\x1a\n\rcanBeReshared\x18\x04 \x01(\x08H\x03\x88\x01\x01\x12!\n\x14\x63\x61nReceiveMultiReact\x18\x05 \x01(\x08H\x04\x88\x01\x01\x42\x14\n\x12_cannotBeReactedToB\x11\n\x0f_cannotBeRankedB\x15\n\x13_canRequestFeedbackB\x10\n\x0e_canBeResharedB\x17\n\x15_canReceiveMultiReact\x1a\xa8\x03\n\x1e\x46orwardedNewsletterMessageInfo\x12\x1a\n\rnewsletterJid\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x1c\n\x0fserverMessageId\x18\x02 \x01(\x05H\x01\x88\x01\x01\x12\x1b\n\x0enewsletterName\x18\x03 \x01(\tH\x02\x88\x01\x01\x12W\n\x0b\x63ontentType\x18\x04 \x01(\x0e\x32=.proto.ContextInfo.ForwardedNewsletterMessageInfo.ContentTypeH\x03\x88\x01\x01\x12\x1e\n\x11\x61\x63\x63\x65ssibilityText\x18\x05 \x01(\tH\x04\x88\x01\x01\"W\n\x0b\x43ontentType\x12\x1c\n\x18\x43ONTENT_TYPE_UNSPECIFIED\x10\x00\x12\n\n\x06UPDATE\x10\x01\x12\x0f\n\x0bUPDATE_CARD\x10\x02\x12\r\n\tLINK_CARD\x10\x03\x42\x10\n\x0e_newsletterJidB\x12\n\x10_serverMessageIdB\x11\n\x0f_newsletterNameB\x0e\n\x0c_contentTypeB\x14\n\x12_accessibilityText\x1a\xd0\x01\n\x1aQuestionReplyQuotedMessage\x12\x1d\n\x10serverQuestionId\x18\x01 \x01(\x05H\x00\x88\x01\x01\x12+\n\x0equotedQuestion\x18\x02 \x01(\x0b\x32\x0e.proto.MessageH\x01\x88\x01\x01\x12+\n\x0equotedResponse\x18\x03 \x01(\x0b\x32\x0e.proto.MessageH\x02\x88\x01\x01\x42\x13\n\x11_serverQuestionIdB\x11\n\x0f_quotedQuestionB\x11\n\x0f_quotedResponse\x1a\xac\x01\n\x16StatusAudienceMetadata\x12Q\n\x0c\x61udienceType\x18\x01 \x01(\x0e\x32\x36.proto.ContextInfo.StatusAudienceMetadata.AudienceTypeH\x00\x88\x01\x01\".\n\x0c\x41udienceType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x11\n\rCLOSE_FRIENDS\x10\x01\x42\x0f\n\r_audienceType\x1aY\n\x07UTMInfo\x12\x16\n\tutmSource\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x18\n\x0butmCampaign\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x0c\n\n_utmSourceB\x0e\n\x0c_utmCampaign\"V\n\rForwardOrigin\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x08\n\x04\x43HAT\x10\x01\x12\n\n\x06STATUS\x10\x02\x12\x0c\n\x08\x43HANNELS\x10\x03\x12\x0b\n\x07META_AI\x10\x04\x12\x07\n\x03UGC\x10\x05\"\xd7\x01\n\x0fPairedMediaType\x12\x14\n\x10NOT_PAIRED_MEDIA\x10\x00\x12\x13\n\x0fSD_VIDEO_PARENT\x10\x01\x12\x12\n\x0eHD_VIDEO_CHILD\x10\x02\x12\x13\n\x0fSD_IMAGE_PARENT\x10\x03\x12\x12\n\x0eHD_IMAGE_CHILD\x10\x04\x12\x17\n\x13MOTION_PHOTO_PARENT\x10\x05\x12\x16\n\x12MOTION_PHOTO_CHILD\x10\x06\x12\x15\n\x11HEVC_VIDEO_PARENT\x10\x07\x12\x14\n\x10HEVC_VIDEO_CHILD\x10\x08\"$\n\nQuotedType\x12\x0c\n\x08\x45XPLICIT\x10\x00\x12\x08\n\x04\x41UTO\x10\x01\"\x92\x01\n\x15StatusAttributionType\x12\x08\n\x04NONE\x10\x00\x12\x19\n\x15RESHARED_FROM_MENTION\x10\x01\x12\x16\n\x12RESHARED_FROM_POST\x10\x02\x12!\n\x1dRESHARED_FROM_POST_MANY_TIMES\x10\x03\x12\x19\n\x15\x46ORWARDED_FROM_STATUS\x10\x04\"\\\n\x10StatusSourceType\x12\t\n\x05IMAGE\x10\x00\x12\t\n\x05VIDEO\x10\x01\x12\x07\n\x03GIF\x10\x02\x12\t\n\x05\x41UDIO\x10\x03\x12\x08\n\x04TEXT\x10\x04\x12\x14\n\x10MUSIC_STANDALONE\x10\x05\x42\x0b\n\t_stanzaIdB\x0e\n\x0c_participantB\x10\n\x0e_quotedMessageB\x0c\n\n_remoteJidB\x13\n\x11_conversionSourceB\x11\n\x0f_conversionDataB\x19\n\x17_conversionDelaySecondsB\x12\n\x10_forwardingScoreB\x0e\n\x0c_isForwardedB\x0b\n\t_quotedAdB\x11\n\x0f_placeholderKeyB\r\n\x0b_expirationB\x1c\n\x1a_ephemeralSettingTimestampB\x18\n\x16_ephemeralSharedSecretB\x12\n\x10_externalAdReplyB\x1d\n\x1b_entryPointConversionSourceB\x1a\n\x18_entryPointConversionAppB#\n!_entryPointConversionDelaySecondsB\x13\n\x11_disappearingModeB\r\n\x0b_actionLinkB\x0f\n\r_groupSubjectB\x11\n\x0f_parentGroupJidB\x12\n\x10_trustBannerTypeB\x14\n\x12_trustBannerActionB\x0c\n\n_isSampledB\x06\n\x04_utmB!\n\x1f_forwardedNewsletterMessageInfoB\x1d\n\x1b_businessMessageForwardInfoB\x16\n\x14_smbClientCampaignIdB\x16\n\x14_smbServerCampaignIdB\x15\n\x13_dataSharingContextB\x1a\n\x18_alwaysShowAdAttributionB\x17\n\x15_featureEligibilitiesB%\n#_entryPointConversionExternalSourceB%\n#_entryPointConversionExternalMediumB\x0e\n\x0c_ctwaSignalsB\x0e\n\x0c_ctwaPayloadB\x1c\n\x1a_forwardedAiBotMessageInfoB\x18\n\x16_statusAttributionTypeB\x11\n\x0f_urlTrackingMapB\x12\n\x10_pairedMediaTypeB\x11\n\x0f_rankingVersionB\x0e\n\x0c_memberLabelB\r\n\x0b_isQuestionB\x13\n\x11_statusSourceTypeB\x10\n\x0e_isGroupStatusB\x10\n\x0e_forwardOriginB\x1d\n\x1b_questionReplyQuotedMessageB\x19\n\x17_statusAudienceMetadataB\x11\n\x0f_nonJidMentionsB\r\n\x0b_quotedTypeB\x18\n\x16_botMessageSharingInfo\"\xb5\x16\n\x0c\x43onversation\x12\n\n\x02id\x18\x01 \x01(\t\x12\'\n\x08messages\x18\x02 \x03(\x0b\x32\x15.proto.HistorySyncMsg\x12\x13\n\x06newJid\x18\x03 \x01(\tH\x00\x88\x01\x01\x12\x13\n\x06oldJid\x18\x04 \x01(\tH\x01\x88\x01\x01\x12\x1d\n\x10lastMsgTimestamp\x18\x05 \x01(\x04H\x02\x88\x01\x01\x12\x18\n\x0bunreadCount\x18\x06 \x01(\rH\x03\x88\x01\x01\x12\x15\n\x08readOnly\x18\x07 \x01(\x08H\x04\x88\x01\x01\x12!\n\x14\x65ndOfHistoryTransfer\x18\x08 \x01(\x08H\x05\x88\x01\x01\x12 \n\x13\x65phemeralExpiration\x18\t \x01(\rH\x06\x88\x01\x01\x12&\n\x19\x65phemeralSettingTimestamp\x18\n \x01(\x03H\x07\x88\x01\x01\x12S\n\x18\x65ndOfHistoryTransferType\x18\x0b \x01(\x0e\x32,.proto.Conversation.EndOfHistoryTransferTypeH\x08\x88\x01\x01\x12\"\n\x15\x63onversationTimestamp\x18\x0c \x01(\x04H\t\x88\x01\x01\x12\x11\n\x04name\x18\r \x01(\tH\n\x88\x01\x01\x12\x12\n\x05pHash\x18\x0e \x01(\tH\x0b\x88\x01\x01\x12\x14\n\x07notSpam\x18\x0f \x01(\x08H\x0c\x88\x01\x01\x12\x15\n\x08\x61rchived\x18\x10 \x01(\x08H\r\x88\x01\x01\x12\x36\n\x10\x64isappearingMode\x18\x11 \x01(\x0b\x32\x17.proto.DisappearingModeH\x0e\x88\x01\x01\x12\x1f\n\x12unreadMentionCount\x18\x12 \x01(\rH\x0f\x88\x01\x01\x12\x1b\n\x0emarkedAsUnread\x18\x13 \x01(\x08H\x10\x88\x01\x01\x12,\n\x0bparticipant\x18\x14 \x03(\x0b\x32\x17.proto.GroupParticipant\x12\x14\n\x07tcToken\x18\x15 \x01(\x0cH\x11\x88\x01\x01\x12\x1d\n\x10tcTokenTimestamp\x18\x16 \x01(\x04H\x12\x88\x01\x01\x12&\n\x19\x63ontactPrimaryIdentityKey\x18\x17 \x01(\x0cH\x13\x88\x01\x01\x12\x13\n\x06pinned\x18\x18 \x01(\rH\x14\x88\x01\x01\x12\x18\n\x0bmuteEndTime\x18\x19 \x01(\x04H\x15\x88\x01\x01\x12\x30\n\twallpaper\x18\x1a \x01(\x0b\x32\x18.proto.WallpaperSettingsH\x16\x88\x01\x01\x12\x34\n\x0fmediaVisibility\x18\x1b \x01(\x0e\x32\x16.proto.MediaVisibilityH\x17\x88\x01\x01\x12#\n\x16tcTokenSenderTimestamp\x18\x1c \x01(\x04H\x18\x88\x01\x01\x12\x16\n\tsuspended\x18\x1d \x01(\x08H\x19\x88\x01\x01\x12\x17\n\nterminated\x18\x1e \x01(\x08H\x1a\x88\x01\x01\x12\x16\n\tcreatedAt\x18\x1f \x01(\x04H\x1b\x88\x01\x01\x12\x16\n\tcreatedBy\x18  \x01(\tH\x1c\x88\x01\x01\x12\x18\n\x0b\x64\x65scription\x18! \x01(\tH\x1d\x88\x01\x01\x12\x14\n\x07support\x18\" \x01(\x08H\x1e\x88\x01\x01\x12\x1a\n\risParentGroup\x18# \x01(\x08H\x1f\x88\x01\x01\x12\x1a\n\rparentGroupId\x18% \x01(\tH \x88\x01\x01\x12\x1e\n\x11isDefaultSubgroup\x18$ \x01(\x08H!\x88\x01\x01\x12\x18\n\x0b\x64isplayName\x18& \x01(\tH\"\x88\x01\x01\x12\x12\n\x05pnJid\x18\' \x01(\tH#\x88\x01\x01\x12\x17\n\nshareOwnPn\x18( \x01(\x08H$\x88\x01\x01\x12\"\n\x15pnhDuplicateLidThread\x18) \x01(\x08H%\x88\x01\x01\x12\x13\n\x06lidJid\x18* \x01(\tH&\x88\x01\x01\x12\x15\n\x08username\x18+ \x01(\tH\'\x88\x01\x01\x12\x1a\n\rlidOriginType\x18, \x01(\tH(\x88\x01\x01\x12\x1a\n\rcommentsCount\x18- \x01(\rH)\x88\x01\x01\x12\x13\n\x06locked\x18. \x01(\x08H*\x88\x01\x01\x12?\n\x15systemMessageToInsert\x18/ \x01(\x0e\x32\x1b.proto.PrivacySystemMessageH+\x88\x01\x01\x12\x1d\n\x10\x63\x61piCreatedGroup\x18\x30 \x01(\x08H,\x88\x01\x01\x12\x17\n\naccountLid\x18\x31 \x01(\tH-\x88\x01\x01\x12\x19\n\x0climitSharing\x18\x32 \x01(\x08H.\x88\x01\x01\x12)\n\x1climitSharingSettingTimestamp\x18\x33 \x01(\x03H/\x88\x01\x01\x12\x41\n\x13limitSharingTrigger\x18\x34 \x01(\x0e\x32\x1f.proto.LimitSharing.TriggerTypeH0\x88\x01\x01\x12&\n\x19limitSharingInitiatedByMe\x18\x35 \x01(\x08H1\x88\x01\x01\x12!\n\x14maibaAiThreadEnabled\x18\x36 \x01(\x08H2\x88\x01\x01\"\xbc\x01\n\x18\x45ndOfHistoryTransferType\x12\x30\n,COMPLETE_BUT_MORE_MESSAGES_REMAIN_ON_PRIMARY\x10\x00\x12\x32\n.COMPLETE_AND_NO_MORE_MESSAGE_REMAIN_ON_PRIMARY\x10\x01\x12:\n6COMPLETE_ON_DEMAND_SYNC_BUT_MORE_MSG_REMAIN_ON_PRIMARY\x10\x02\x42\t\n\x07_newJidB\t\n\x07_oldJidB\x13\n\x11_lastMsgTimestampB\x0e\n\x0c_unreadCountB\x0b\n\t_readOnlyB\x17\n\x15_endOfHistoryTransferB\x16\n\x14_ephemeralExpirationB\x1c\n\x1a_ephemeralSettingTimestampB\x1b\n\x19_endOfHistoryTransferTypeB\x18\n\x16_conversationTimestampB\x07\n\x05_nameB\x08\n\x06_pHashB\n\n\x08_notSpamB\x0b\n\t_archivedB\x13\n\x11_disappearingModeB\x15\n\x13_unreadMentionCountB\x11\n\x0f_markedAsUnreadB\n\n\x08_tcTokenB\x13\n\x11_tcTokenTimestampB\x1c\n\x1a_contactPrimaryIdentityKeyB\t\n\x07_pinnedB\x0e\n\x0c_muteEndTimeB\x0c\n\n_wallpaperB\x12\n\x10_mediaVisibilityB\x19\n\x17_tcTokenSenderTimestampB\x0c\n\n_suspendedB\r\n\x0b_terminatedB\x0c\n\n_createdAtB\x0c\n\n_createdByB\x0e\n\x0c_descriptionB\n\n\x08_supportB\x10\n\x0e_isParentGroupB\x10\n\x0e_parentGroupIdB\x14\n\x12_isDefaultSubgroupB\x0e\n\x0c_displayNameB\x08\n\x06_pnJidB\r\n\x0b_shareOwnPnB\x18\n\x16_pnhDuplicateLidThreadB\t\n\x07_lidJidB\x0b\n\t_usernameB\x10\n\x0e_lidOriginTypeB\x10\n\x0e_commentsCountB\t\n\x07_lockedB\x18\n\x16_systemMessageToInsertB\x13\n\x11_capiCreatedGroupB\r\n\x0b_accountLidB\x0f\n\r_limitSharingB\x1f\n\x1d_limitSharingSettingTimestampB\x16\n\x14_limitSharingTriggerB\x1c\n\x1a_limitSharingInitiatedByMeB\x17\n\x15_maibaAiThreadEnabled\"\xfd\x06\n\x12\x44\x65viceCapabilities\x12Q\n\x14\x63hatLockSupportLevel\x18\x01 \x01(\x0e\x32..proto.DeviceCapabilities.ChatLockSupportLevelH\x00\x88\x01\x01\x12\x41\n\x0clidMigration\x18\x02 \x01(\x0b\x32&.proto.DeviceCapabilities.LIDMigrationH\x01\x88\x01\x01\x12K\n\x11\x62usinessBroadcast\x18\x03 \x01(\x0b\x32+.proto.DeviceCapabilities.BusinessBroadcastH\x02\x88\x01\x01\x12\x43\n\ruserHasAvatar\x18\x04 \x01(\x0b\x32\'.proto.DeviceCapabilities.UserHasAvatarH\x03\x88\x01\x01\x12_\n\x1bmemberNameTagPrimarySupport\x18\x05 \x01(\x0e\x32\x35.proto.DeviceCapabilities.MemberNameTagPrimarySupportH\x04\x88\x01\x01\x1aI\n\x11\x42usinessBroadcast\x12\x1e\n\x11importListEnabled\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\x14\n\x12_importListEnabled\x1aR\n\x0cLIDMigration\x12%\n\x18\x63hatDbMigrationTimestamp\x18\x01 \x01(\x04H\x00\x88\x01\x01\x42\x1b\n\x19_chatDbMigrationTimestamp\x1a=\n\rUserHasAvatar\x12\x1a\n\ruserHasAvatar\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\x10\n\x0e_userHasAvatar\"7\n\x14\x43hatLockSupportLevel\x12\x08\n\x04NONE\x10\x00\x12\x0b\n\x07MINIMAL\x10\x01\x12\x08\n\x04\x46ULL\x10\x02\"U\n\x1bMemberNameTagPrimarySupport\x12\x0c\n\x08\x44ISABLED\x10\x00\x12\x14\n\x10RECEIVER_ENABLED\x10\x01\x12\x12\n\x0eSENDER_ENABLED\x10\x02\x42\x17\n\x15_chatLockSupportLevelB\x0f\n\r_lidMigrationB\x14\n\x12_businessBroadcastB\x10\n\x0e_userHasAvatarB\x1e\n\x1c_memberNameTagPrimarySupport\"l\n\x1c\x44\x65viceConsistencyCodeMessage\x12\x17\n\ngeneration\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x16\n\tsignature\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x42\r\n\x0b_generationB\x0c\n\n_signature\"\xc3\x03\n\x12\x44\x65viceListMetadata\x12\x1a\n\rsenderKeyHash\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x1c\n\x0fsenderTimestamp\x18\x02 \x01(\x04H\x01\x88\x01\x01\x12\x1c\n\x10senderKeyIndexes\x18\x03 \x03(\rB\x02\x10\x01\x12\x38\n\x11senderAccountType\x18\x04 \x01(\x0e\x32\x18.proto.ADVEncryptionTypeH\x02\x88\x01\x01\x12:\n\x13receiverAccountType\x18\x05 \x01(\x0e\x32\x18.proto.ADVEncryptionTypeH\x03\x88\x01\x01\x12\x1d\n\x10recipientKeyHash\x18\x08 \x01(\x0cH\x04\x88\x01\x01\x12\x1f\n\x12recipientTimestamp\x18\t \x01(\x04H\x05\x88\x01\x01\x12\x1f\n\x13recipientKeyIndexes\x18\n \x03(\rB\x02\x10\x01\x42\x10\n\x0e_senderKeyHashB\x12\n\x10_senderTimestampB\x14\n\x12_senderAccountTypeB\x16\n\x14_receiverAccountTypeB\x13\n\x11_recipientKeyHashB\x15\n\x13_recipientTimestamp\"\xd4\x10\n\x0b\x44\x65viceProps\x12\x0f\n\x02os\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x33\n\x07version\x18\x02 \x01(\x0b\x32\x1d.proto.DeviceProps.AppVersionH\x01\x88\x01\x01\x12:\n\x0cplatformType\x18\x03 \x01(\x0e\x32\x1f.proto.DeviceProps.PlatformTypeH\x02\x88\x01\x01\x12\x1c\n\x0frequireFullSync\x18\x04 \x01(\x08H\x03\x88\x01\x01\x12\x44\n\x11historySyncConfig\x18\x05 \x01(\x0b\x32$.proto.DeviceProps.HistorySyncConfigH\x04\x88\x01\x01\x1a\xc2\x01\n\nAppVersion\x12\x14\n\x07primary\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x16\n\tsecondary\x18\x02 \x01(\rH\x01\x88\x01\x01\x12\x15\n\x08tertiary\x18\x03 \x01(\rH\x02\x88\x01\x01\x12\x17\n\nquaternary\x18\x04 \x01(\rH\x03\x88\x01\x01\x12\x14\n\x07quinary\x18\x05 \x01(\rH\x04\x88\x01\x01\x42\n\n\x08_primaryB\x0c\n\n_secondaryB\x0b\n\t_tertiaryB\r\n\x0b_quaternaryB\n\n\x08_quinary\x1a\xe9\t\n\x11HistorySyncConfig\x12\x1e\n\x11\x66ullSyncDaysLimit\x18\x01 \x01(\rH\x00\x88\x01\x01\x12 \n\x13\x66ullSyncSizeMbLimit\x18\x02 \x01(\rH\x01\x88\x01\x01\x12\x1b\n\x0estorageQuotaMb\x18\x03 \x01(\rH\x02\x88\x01\x01\x12*\n\x1dinlineInitialPayloadInE2EeMsg\x18\x04 \x01(\x08H\x03\x88\x01\x01\x12 \n\x13recentSyncDaysLimit\x18\x05 \x01(\rH\x04\x88\x01\x01\x12\"\n\x15supportCallLogHistory\x18\x06 \x01(\x08H\x05\x88\x01\x01\x12+\n\x1esupportBotUserAgentChatHistory\x18\x07 \x01(\x08H\x06\x88\x01\x01\x12(\n\x1bsupportCagReactionsAndPolls\x18\x08 \x01(\x08H\x07\x88\x01\x01\x12 \n\x13supportBizHostedMsg\x18\t \x01(\x08H\x08\x88\x01\x01\x12\x35\n(supportRecentSyncChunkMessageCountTuning\x18\n \x01(\x08H\t\x88\x01\x01\x12\"\n\x15supportHostedGroupMsg\x18\x0b \x01(\x08H\n\x88\x01\x01\x12&\n\x19supportFbidBotChatHistory\x18\x0c \x01(\x08H\x0b\x88\x01\x01\x12-\n supportAddOnHistorySyncMigration\x18\r \x01(\x08H\x0c\x88\x01\x01\x12&\n\x19supportMessageAssociation\x18\x0e \x01(\x08H\r\x88\x01\x01\x12 \n\x13supportGroupHistory\x18\x0f \x01(\x08H\x0e\x88\x01\x01\x12\x1a\n\ronDemandReady\x18\x10 \x01(\x08H\x0f\x88\x01\x01\x12\x1d\n\x10supportGuestChat\x18\x11 \x01(\x08H\x10\x88\x01\x01\x12\"\n\x15\x63ompleteOnDemandReady\x18\x12 \x01(\x08H\x11\x88\x01\x01\x12#\n\x16thumbnailSyncDaysLimit\x18\x13 \x01(\rH\x12\x88\x01\x01\x42\x14\n\x12_fullSyncDaysLimitB\x16\n\x14_fullSyncSizeMbLimitB\x11\n\x0f_storageQuotaMbB \n\x1e_inlineInitialPayloadInE2EeMsgB\x16\n\x14_recentSyncDaysLimitB\x18\n\x16_supportCallLogHistoryB!\n\x1f_supportBotUserAgentChatHistoryB\x1e\n\x1c_supportCagReactionsAndPollsB\x16\n\x14_supportBizHostedMsgB+\n)_supportRecentSyncChunkMessageCountTuningB\x18\n\x16_supportHostedGroupMsgB\x1c\n\x1a_supportFbidBotChatHistoryB#\n!_supportAddOnHistorySyncMigrationB\x1c\n\x1a_supportMessageAssociationB\x16\n\x14_supportGroupHistoryB\x10\n\x0e_onDemandReadyB\x13\n\x11_supportGuestChatB\x18\n\x16_completeOnDemandReadyB\x19\n\x17_thumbnailSyncDaysLimit\"\xdf\x02\n\x0cPlatformType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\n\n\x06\x43HROME\x10\x01\x12\x0b\n\x07\x46IREFOX\x10\x02\x12\x06\n\x02IE\x10\x03\x12\t\n\x05OPERA\x10\x04\x12\n\n\x06SAFARI\x10\x05\x12\x08\n\x04\x45\x44GE\x10\x06\x12\x0b\n\x07\x44\x45SKTOP\x10\x07\x12\x08\n\x04IPAD\x10\x08\x12\x12\n\x0e\x41NDROID_TABLET\x10\t\x12\t\n\x05OHANA\x10\n\x12\t\n\x05\x41LOHA\x10\x0b\x12\x0c\n\x08\x43\x41TALINA\x10\x0c\x12\n\n\x06TCL_TV\x10\r\x12\r\n\tIOS_PHONE\x10\x0e\x12\x10\n\x0cIOS_CATALYST\x10\x0f\x12\x11\n\rANDROID_PHONE\x10\x10\x12\x15\n\x11\x41NDROID_AMBIGUOUS\x10\x11\x12\x0b\n\x07WEAR_OS\x10\x12\x12\x0c\n\x08\x41R_WRIST\x10\x13\x12\r\n\tAR_DEVICE\x10\x14\x12\x07\n\x03UWP\x10\x15\x12\x06\n\x02VR\x10\x16\x12\r\n\tCLOUD_API\x10\x17\x12\x10\n\x0cSMARTGLASSES\x10\x18\x42\x05\n\x03_osB\n\n\x08_versionB\x0f\n\r_platformTypeB\x12\n\x10_requireFullSyncB\x14\n\x12_historySyncConfig\"\xf0\x03\n\x10\x44isappearingMode\x12\x39\n\tinitiator\x18\x01 \x01(\x0e\x32!.proto.DisappearingMode.InitiatorH\x00\x88\x01\x01\x12\x35\n\x07trigger\x18\x02 \x01(\x0e\x32\x1f.proto.DisappearingMode.TriggerH\x01\x88\x01\x01\x12\x1f\n\x12initiatorDeviceJid\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x1a\n\rinitiatedByMe\x18\x04 \x01(\x08H\x03\x88\x01\x01\"i\n\tInitiator\x12\x13\n\x0f\x43HANGED_IN_CHAT\x10\x00\x12\x13\n\x0fINITIATED_BY_ME\x10\x01\x12\x16\n\x12INITIATED_BY_OTHER\x10\x02\x12\x1a\n\x16\x42IZ_UPGRADE_FB_HOSTING\x10\x03\"\x7f\n\x07Trigger\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x10\n\x0c\x43HAT_SETTING\x10\x01\x12\x13\n\x0f\x41\x43\x43OUNT_SETTING\x10\x02\x12\x0f\n\x0b\x42ULK_CHANGE\x10\x03\x12\x1b\n\x17\x42IZ_SUPPORTS_FB_HOSTING\x10\x04\x12\x12\n\x0eUNKNOWN_GROUPS\x10\x05\x42\x0c\n\n_initiatorB\n\n\x08_triggerB\x15\n\x13_initiatorDeviceJidB\x10\n\x0e_initiatedByMe\"~\n\x0f\x45mbeddedContent\x12\x31\n\x0f\x65mbeddedMessage\x18\x01 \x01(\x0b\x32\x16.proto.EmbeddedMessageH\x00\x12-\n\rembeddedMusic\x18\x02 \x01(\x0b\x32\x14.proto.EmbeddedMusicH\x00\x42\t\n\x07\x63ontent\"g\n\x0f\x45mbeddedMessage\x12\x15\n\x08stanzaId\x18\x01 \x01(\tH\x00\x88\x01\x01\x12$\n\x07message\x18\x02 \x01(\x0b\x32\x0e.proto.MessageH\x01\x88\x01\x01\x42\x0b\n\t_stanzaIdB\n\n\x08_message\"\xc7\x05\n\rEmbeddedMusic\x12 \n\x13musicContentMediaId\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x13\n\x06songId\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x13\n\x06\x61uthor\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x12\n\x05title\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x1e\n\x11\x61rtworkDirectPath\x18\x05 \x01(\tH\x04\x88\x01\x01\x12\x1a\n\rartworkSha256\x18\x06 \x01(\x0cH\x05\x88\x01\x01\x12\x1d\n\x10\x61rtworkEncSha256\x18\x07 \x01(\x0cH\x06\x88\x01\x01\x12\x1e\n\x11\x61rtistAttribution\x18\x08 \x01(\tH\x07\x88\x01\x01\x12\x1d\n\x10\x63ountryBlocklist\x18\t \x01(\x0cH\x08\x88\x01\x01\x12\x17\n\nisExplicit\x18\n \x01(\x08H\t\x88\x01\x01\x12\x1c\n\x0f\x61rtworkMediaKey\x18\x0b \x01(\x0cH\n\x88\x01\x01\x12#\n\x16musicSongStartTimeInMs\x18\x0c \x01(\x03H\x0b\x88\x01\x01\x12(\n\x1b\x64\x65rivedContentStartTimeInMs\x18\r \x01(\x03H\x0c\x88\x01\x01\x12 \n\x13overlapDurationInMs\x18\x0e \x01(\x03H\r\x88\x01\x01\x42\x16\n\x14_musicContentMediaIdB\t\n\x07_songIdB\t\n\x07_authorB\x08\n\x06_titleB\x14\n\x12_artworkDirectPathB\x10\n\x0e_artworkSha256B\x13\n\x11_artworkEncSha256B\x14\n\x12_artistAttributionB\x13\n\x11_countryBlocklistB\r\n\x0b_isExplicitB\x12\n\x10_artworkMediaKeyB\x19\n\x17_musicSongStartTimeInMsB\x1e\n\x1c_derivedContentStartTimeInMsB\x16\n\x14_overlapDurationInMs\"e\n\x17\x45ncryptedPairingRequest\x12\x1d\n\x10\x65ncryptedPayload\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x0f\n\x02iv\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x42\x13\n\x11_encryptedPayloadB\x05\n\x03_iv\"\\\n\x10\x45phemeralSetting\x12\x15\n\x08\x64uration\x18\x01 \x01(\x0fH\x00\x88\x01\x01\x12\x16\n\ttimestamp\x18\x02 \x01(\x10H\x01\x88\x01\x01\x42\x0b\n\t_durationB\x0c\n\n_timestamp\";\n\x17\x45ventAdditionalMetadata\x12\x14\n\x07isStale\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\n\n\x08_isStale\"\x8f\x02\n\rEventResponse\x12\x37\n\x17\x65ventResponseMessageKey\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x12\x18\n\x0btimestampMs\x18\x02 \x01(\x03H\x01\x88\x01\x01\x12\x46\n\x14\x65ventResponseMessage\x18\x03 \x01(\x0b\x32#.proto.Message.EventResponseMessageH\x02\x88\x01\x01\x12\x13\n\x06unread\x18\x04 \x01(\x08H\x03\x88\x01\x01\x42\x1a\n\x18_eventResponseMessageKeyB\x0e\n\x0c_timestampMsB\x17\n\x15_eventResponseMessageB\t\n\x07_unread\"B\n\x08\x45xitCode\x12\x11\n\x04\x63ode\x18\x01 \x01(\x04H\x00\x88\x01\x01\x12\x11\n\x04text\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x07\n\x05_codeB\x07\n\x05_text\"\x87\x02\n\x15\x45xternalBlobReference\x12\x15\n\x08mediaKey\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x17\n\ndirectPath\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x13\n\x06handle\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x1a\n\rfileSizeBytes\x18\x04 \x01(\x04H\x03\x88\x01\x01\x12\x17\n\nfileSha256\x18\x05 \x01(\x0cH\x04\x88\x01\x01\x12\x1a\n\rfileEncSha256\x18\x06 \x01(\x0cH\x05\x88\x01\x01\x42\x0b\n\t_mediaKeyB\r\n\x0b_directPathB\t\n\x07_handleB\x10\n\x0e_fileSizeBytesB\r\n\x0b_fileSha256B\x10\n\x0e_fileEncSha256\"\xac\x02\n\x05\x46ield\x12\x17\n\nminVersion\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x17\n\nmaxVersion\x18\x02 \x01(\rH\x01\x88\x01\x01\x12$\n\x17notReportableMinVersion\x18\x03 \x01(\rH\x02\x88\x01\x01\x12\x16\n\tisMessage\x18\x04 \x01(\x08H\x03\x88\x01\x01\x12,\n\x08subfield\x18\x05 \x03(\x0b\x32\x1a.proto.Field.SubfieldEntry\x1a=\n\rSubfieldEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x12\x1b\n\x05value\x18\x02 \x01(\x0b\x32\x0c.proto.Field:\x02\x38\x01\x42\r\n\x0b_minVersionB\r\n\x0b_maxVersionB\x1a\n\x18_notReportableMinVersionB\x0c\n\n_isMessage\"\x87\x01\n\x19\x46orwardedAIBotMessageInfo\x12\x14\n\x07\x62otName\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x13\n\x06\x62otJid\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x18\n\x0b\x63reatorName\x18\x03 \x01(\tH\x02\x88\x01\x01\x42\n\n\x08_botNameB\t\n\x07_botJidB\x0e\n\x0c_creatorName\"\xed\x0b\n\x0eGlobalSettings\x12:\n\x13lightThemeWallpaper\x18\x01 \x01(\x0b\x32\x18.proto.WallpaperSettingsH\x00\x88\x01\x01\x12\x34\n\x0fmediaVisibility\x18\x02 \x01(\x0e\x32\x16.proto.MediaVisibilityH\x01\x88\x01\x01\x12\x39\n\x12\x64\x61rkThemeWallpaper\x18\x03 \x01(\x0b\x32\x18.proto.WallpaperSettingsH\x02\x88\x01\x01\x12:\n\x10\x61utoDownloadWiFi\x18\x04 \x01(\x0b\x32\x1b.proto.AutoDownloadSettingsH\x03\x88\x01\x01\x12>\n\x14\x61utoDownloadCellular\x18\x05 \x01(\x0b\x32\x1b.proto.AutoDownloadSettingsH\x04\x88\x01\x01\x12=\n\x13\x61utoDownloadRoaming\x18\x06 \x01(\x0b\x32\x1b.proto.AutoDownloadSettingsH\x05\x88\x01\x01\x12/\n\"showIndividualNotificationsPreview\x18\x07 \x01(\x08H\x06\x88\x01\x01\x12*\n\x1dshowGroupNotificationsPreview\x18\x08 \x01(\x08H\x07\x88\x01\x01\x12%\n\x18\x64isappearingModeDuration\x18\t \x01(\x05H\x08\x88\x01\x01\x12&\n\x19\x64isappearingModeTimestamp\x18\n \x01(\x03H\t\x88\x01\x01\x12:\n\x12\x61vatarUserSettings\x18\x0b \x01(\x0b\x32\x19.proto.AvatarUserSettingsH\n\x88\x01\x01\x12\x15\n\x08\x66ontSize\x18\x0c \x01(\x05H\x0b\x88\x01\x01\x12\"\n\x15securityNotifications\x18\r \x01(\x08H\x0c\x88\x01\x01\x12\x1f\n\x12\x61utoUnarchiveChats\x18\x0e \x01(\x08H\r\x88\x01\x01\x12\x1d\n\x10videoQualityMode\x18\x0f \x01(\x05H\x0e\x88\x01\x01\x12\x1d\n\x10photoQualityMode\x18\x10 \x01(\x05H\x0f\x88\x01\x01\x12H\n\x1eindividualNotificationSettings\x18\x11 \x01(\x0b\x32\x1b.proto.NotificationSettingsH\x10\x88\x01\x01\x12\x43\n\x19groupNotificationSettings\x18\x12 \x01(\x0b\x32\x1b.proto.NotificationSettingsH\x11\x88\x01\x01\x12\x36\n\x10\x63hatLockSettings\x18\x13 \x01(\x0b\x32\x17.proto.ChatLockSettingsH\x12\x88\x01\x01\x12(\n\x1b\x63hatDbLidMigrationTimestamp\x18\x14 \x01(\x03H\x13\x88\x01\x01\x42\x16\n\x14_lightThemeWallpaperB\x12\n\x10_mediaVisibilityB\x15\n\x13_darkThemeWallpaperB\x13\n\x11_autoDownloadWiFiB\x17\n\x15_autoDownloadCellularB\x16\n\x14_autoDownloadRoamingB%\n#_showIndividualNotificationsPreviewB \n\x1e_showGroupNotificationsPreviewB\x1b\n\x19_disappearingModeDurationB\x1c\n\x1a_disappearingModeTimestampB\x15\n\x13_avatarUserSettingsB\x0b\n\t_fontSizeB\x18\n\x16_securityNotificationsB\x15\n\x13_autoUnarchiveChatsB\x13\n\x11_videoQualityModeB\x13\n\x11_photoQualityModeB!\n\x1f_individualNotificationSettingsB\x1c\n\x1a_groupNotificationSettingsB\x13\n\x11_chatLockSettingsB\x1e\n\x1c_chatDbLidMigrationTimestamp\"\xe0\x02\n\x16GroupHistoryBundleInfo\x12P\n\x1e\x64\x65precatedMessageHistoryBundle\x18\x01 \x01(\x0b\x32#.proto.Message.MessageHistoryBundleH\x00\x88\x01\x01\x12\x45\n\x0cprocessState\x18\x02 \x01(\x0e\x32*.proto.GroupHistoryBundleInfo.ProcessStateH\x01\x88\x01\x01\"y\n\x0cProcessState\x12\x10\n\x0cNOT_INJECTED\x10\x00\x12\x0c\n\x08INJECTED\x10\x01\x12\x14\n\x10INJECTED_PARTIAL\x10\x02\x12\x14\n\x10INJECTION_FAILED\x10\x03\x12\x1d\n\x19INJECTION_FAILED_NO_RETRY\x10\x04\x42!\n\x1f_deprecatedMessageHistoryBundleB\x0f\n\r_processState\"\xb6\x01\n!GroupHistoryIndividualMessageInfo\x12\x30\n\x10\x62undleMessageKey\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x12)\n\x1c\x65\x64itedAfterReceivedAsHistory\x18\x02 \x01(\x08H\x01\x88\x01\x01\x42\x13\n\x11_bundleMessageKeyB\x1f\n\x1d_editedAfterReceivedAsHistory\"^\n\x0cGroupMention\x12\x15\n\x08groupJid\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x19\n\x0cgroupSubject\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x0b\n\t_groupJidB\x0f\n\r_groupSubject\"\xcb\x01\n\x10GroupParticipant\x12\x0f\n\x07userJid\x18\x01 \x01(\t\x12/\n\x04rank\x18\x02 \x01(\x0e\x32\x1c.proto.GroupParticipant.RankH\x00\x88\x01\x01\x12,\n\x0bmemberLabel\x18\x03 \x01(\x0b\x32\x12.proto.MemberLabelH\x01\x88\x01\x01\".\n\x04Rank\x12\x0b\n\x07REGULAR\x10\x00\x12\t\n\x05\x41\x44MIN\x10\x01\x12\x0e\n\nSUPERADMIN\x10\x02\x42\x07\n\x05_rankB\x0e\n\x0c_memberLabel\"\x8f\x06\n\x10HandshakeMessage\x12=\n\x0b\x63lientHello\x18\x02 \x01(\x0b\x32#.proto.HandshakeMessage.ClientHelloH\x00\x88\x01\x01\x12=\n\x0bserverHello\x18\x03 \x01(\x0b\x32#.proto.HandshakeMessage.ServerHelloH\x01\x88\x01\x01\x12?\n\x0c\x63lientFinish\x18\x04 \x01(\x0b\x32$.proto.HandshakeMessage.ClientFinishH\x02\x88\x01\x01\x1a\x88\x01\n\x0c\x43lientFinish\x12\x13\n\x06static\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x14\n\x07payload\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x1f\n\x12\x65xtendedCiphertext\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x42\t\n\x07_staticB\n\n\x08_payloadB\x15\n\x13_extendedCiphertext\x1a\xd7\x01\n\x0b\x43lientHello\x12\x16\n\tephemeral\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x13\n\x06static\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x14\n\x07payload\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x18\n\x0buseExtended\x18\x04 \x01(\x08H\x03\x88\x01\x01\x12\x1f\n\x12\x65xtendedCiphertext\x18\x05 \x01(\x0cH\x04\x88\x01\x01\x42\x0c\n\n_ephemeralB\t\n\x07_staticB\n\n\x08_payloadB\x0e\n\x0c_useExtendedB\x15\n\x13_extendedCiphertext\x1a\xa5\x01\n\x0bServerHello\x12\x16\n\tephemeral\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x13\n\x06static\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x14\n\x07payload\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x1b\n\x0e\x65xtendedStatic\x18\x04 \x01(\x0cH\x03\x88\x01\x01\x42\x0c\n\n_ephemeralB\t\n\x07_staticB\n\n\x08_payloadB\x11\n\x0f_extendedStaticB\x0e\n\x0c_clientHelloB\x0e\n\x0c_serverHelloB\x0f\n\r_clientFinish\"\xf9\x08\n\x0bHistorySync\x12\x34\n\x08syncType\x18\x01 \x01(\x0e\x32\".proto.HistorySync.HistorySyncType\x12*\n\rconversations\x18\x02 \x03(\x0b\x32\x13.proto.Conversation\x12/\n\x10statusV3Messages\x18\x03 \x03(\x0b\x32\x15.proto.WebMessageInfo\x12\x17\n\nchunkOrder\x18\x05 \x01(\rH\x00\x88\x01\x01\x12\x15\n\x08progress\x18\x06 \x01(\rH\x01\x88\x01\x01\x12\"\n\tpushnames\x18\x07 \x03(\x0b\x32\x0f.proto.Pushname\x12\x32\n\x0eglobalSettings\x18\x08 \x01(\x0b\x32\x15.proto.GlobalSettingsH\x02\x88\x01\x01\x12\x1f\n\x12threadIdUserSecret\x18\t \x01(\x0cH\x03\x88\x01\x01\x12$\n\x17threadDsTimeframeOffset\x18\n \x01(\rH\x04\x88\x01\x01\x12.\n\x0erecentStickers\x18\x0b \x03(\x0b\x32\x16.proto.StickerMetadata\x12\x31\n\x10pastParticipants\x18\x0c \x03(\x0b\x32\x17.proto.PastParticipants\x12,\n\x0e\x63\x61llLogRecords\x18\r \x03(\x0b\x32\x14.proto.CallLogRecord\x12\x43\n\x0f\x61iWaitListState\x18\x0e \x01(\x0e\x32%.proto.HistorySync.BotAIWaitListStateH\x05\x88\x01\x01\x12@\n\x18phoneNumberToLidMappings\x18\x0f \x03(\x0b\x32\x1e.proto.PhoneNumberToLIDMapping\x12\x1f\n\x12\x63ompanionMetaNonce\x18\x10 \x01(\tH\x06\x88\x01\x01\x12\x31\n$shareableChatIdentifierEncryptionKey\x18\x11 \x01(\x0cH\x07\x88\x01\x01\x12 \n\x08\x61\x63\x63ounts\x18\x12 \x03(\x0b\x32\x0e.proto.Account\"7\n\x12\x42otAIWaitListState\x12\x0f\n\x0bIN_WAITLIST\x10\x00\x12\x10\n\x0c\x41I_AVAILABLE\x10\x01\"\x8a\x01\n\x0fHistorySyncType\x12\x15\n\x11INITIAL_BOOTSTRAP\x10\x00\x12\x15\n\x11INITIAL_STATUS_V3\x10\x01\x12\x08\n\x04\x46ULL\x10\x02\x12\n\n\x06RECENT\x10\x03\x12\r\n\tPUSH_NAME\x10\x04\x12\x15\n\x11NON_BLOCKING_DATA\x10\x05\x12\r\n\tON_DEMAND\x10\x06\x42\r\n\x0b_chunkOrderB\x0b\n\t_progressB\x11\n\x0f_globalSettingsB\x15\n\x13_threadIdUserSecretB\x1a\n\x18_threadDsTimeframeOffsetB\x12\n\x10_aiWaitListStateB\x15\n\x13_companionMetaNonceB\'\n%_shareableChatIdentifierEncryptionKey\"q\n\x0eHistorySyncMsg\x12+\n\x07message\x18\x01 \x01(\x0b\x32\x15.proto.WebMessageInfoH\x00\x88\x01\x01\x12\x17\n\nmsgOrderId\x18\x02 \x01(\x04H\x01\x88\x01\x01\x42\n\n\x08_messageB\r\n\x0b_msgOrderId\"\xec\x06\n\x16HydratedTemplateButton\x12\x12\n\x05index\x18\x04 \x01(\rH\x01\x88\x01\x01\x12R\n\x10quickReplyButton\x18\x01 \x01(\x0b\x32\x36.proto.HydratedTemplateButton.HydratedQuickReplyButtonH\x00\x12\x44\n\turlButton\x18\x02 \x01(\x0b\x32/.proto.HydratedTemplateButton.HydratedURLButtonH\x00\x12\x46\n\ncallButton\x18\x03 \x01(\x0b\x32\x30.proto.HydratedTemplateButton.HydratedCallButtonH\x00\x1ah\n\x12HydratedCallButton\x12\x18\n\x0b\x64isplayText\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x18\n\x0bphoneNumber\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x0e\n\x0c_displayTextB\x0e\n\x0c_phoneNumber\x1a\\\n\x18HydratedQuickReplyButton\x12\x18\n\x0b\x64isplayText\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x0f\n\x02id\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x0e\n\x0c_displayTextB\x05\n\x03_id\x1a\xf7\x02\n\x11HydratedURLButton\x12\x18\n\x0b\x64isplayText\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x10\n\x03url\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x1e\n\x11\x63onsentedUsersUrl\x18\x03 \x01(\tH\x02\x88\x01\x01\x12i\n\x13webviewPresentation\x18\x04 \x01(\x0e\x32G.proto.HydratedTemplateButton.HydratedURLButton.WebviewPresentationTypeH\x03\x88\x01\x01\"e\n\x17WebviewPresentationType\x12)\n%WEBVIEW_PRESENTATION_TYPE_UNSPECIFIED\x10\x00\x12\x08\n\x04\x46ULL\x10\x01\x12\x08\n\x04TALL\x10\x02\x12\x0b\n\x07\x43OMPACT\x10\x03\x42\x0e\n\x0c_displayTextB\x06\n\x04_urlB\x14\n\x12_consentedUsersUrlB\x16\n\x14_webviewPresentationB\x10\n\x0ehydratedButtonB\x08\n\x06_index\"h\n\x18IdentityKeyPairStructure\x12\x16\n\tpublicKey\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x17\n\nprivateKey\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x42\x0c\n\n_publicKeyB\r\n\x0b_privateKey\"\xfd\n\n\x16InThreadSurveyMetadata\x12\x1b\n\x0etessaSessionId\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x1b\n\x0esimonSessionId\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x1a\n\rsimonSurveyId\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x18\n\x0btessaRootId\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x16\n\trequestId\x18\x05 \x01(\tH\x04\x88\x01\x01\x12\x17\n\ntessaEvent\x18\x06 \x01(\tH\x05\x88\x01\x01\x12!\n\x14invitationHeaderText\x18\x07 \x01(\tH\x06\x88\x01\x01\x12\x1f\n\x12invitationBodyText\x18\x08 \x01(\tH\x07\x88\x01\x01\x12\x1e\n\x11invitationCtaText\x18\t \x01(\tH\x08\x88\x01\x01\x12\x1d\n\x10invitationCtaUrl\x18\n \x01(\tH\t\x88\x01\x01\x12\x18\n\x0bsurveyTitle\x18\x0b \x01(\tH\n\x88\x01\x01\x12G\n\tquestions\x18\x0c \x03(\x0b\x32\x34.proto.InThreadSurveyMetadata.InThreadSurveyQuestion\x12%\n\x18surveyContinueButtonText\x18\r \x01(\tH\x0b\x88\x01\x01\x12#\n\x16surveySubmitButtonText\x18\x0e \x01(\tH\x0c\x88\x01\x01\x12!\n\x14privacyStatementFull\x18\x0f \x01(\tH\r\x88\x01\x01\x12_\n\x15privacyStatementParts\x18\x10 \x03(\x0b\x32@.proto.InThreadSurveyMetadata.InThreadSurveyPrivacyStatementPart\x12\x1e\n\x11\x66\x65\x65\x64\x62\x61\x63kToastText\x18\x11 \x01(\tH\x0e\x88\x01\x01\x1a\x9c\x01\n\x14InThreadSurveyOption\x12\x18\n\x0bstringValue\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x19\n\x0cnumericValue\x18\x02 \x01(\rH\x01\x88\x01\x01\x12\x1b\n\x0etextTranslated\x18\x03 \x01(\tH\x02\x88\x01\x01\x42\x0e\n\x0c_stringValueB\x0f\n\r_numericValueB\x11\n\x0f_textTranslated\x1aZ\n\"InThreadSurveyPrivacyStatementPart\x12\x11\n\x04text\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x10\n\x03url\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x07\n\x05_textB\x06\n\x04_url\x1a\xb9\x01\n\x16InThreadSurveyQuestion\x12\x19\n\x0cquestionText\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x17\n\nquestionId\x18\x02 \x01(\tH\x01\x88\x01\x01\x12K\n\x0fquestionOptions\x18\x03 \x03(\x0b\x32\x32.proto.InThreadSurveyMetadata.InThreadSurveyOptionB\x0f\n\r_questionTextB\r\n\x0b_questionIdB\x11\n\x0f_tessaSessionIdB\x11\n\x0f_simonSessionIdB\x10\n\x0e_simonSurveyIdB\x0e\n\x0c_tessaRootIdB\x0c\n\n_requestIdB\r\n\x0b_tessaEventB\x17\n\x15_invitationHeaderTextB\x15\n\x13_invitationBodyTextB\x14\n\x12_invitationCtaTextB\x13\n\x11_invitationCtaUrlB\x0e\n\x0c_surveyTitleB\x1b\n\x19_surveyContinueButtonTextB\x19\n\x17_surveySubmitButtonTextB\x17\n\x15_privacyStatementFullB\x14\n\x12_feedbackToastText\"\xf1\x04\n\x15InteractiveAnnotation\x12%\n\x0fpolygonVertices\x18\x01 \x03(\x0b\x32\x0c.proto.Point\x12#\n\x16shouldSkipConfirmation\x18\x04 \x01(\x08H\x01\x88\x01\x01\x12\x34\n\x0f\x65mbeddedContent\x18\x05 \x01(\x0b\x32\x16.proto.EmbeddedContentH\x02\x88\x01\x01\x12H\n\x0estatusLinkType\x18\x08 \x01(\x0e\x32+.proto.InteractiveAnnotation.StatusLinkTypeH\x03\x88\x01\x01\x12#\n\x08location\x18\x02 \x01(\x0b\x32\x0f.proto.LocationH\x00\x12G\n\nnewsletter\x18\x03 \x01(\x0b\x32\x31.proto.ContextInfo.ForwardedNewsletterMessageInfoH\x00\x12\x18\n\x0e\x65mbeddedAction\x18\x06 \x01(\x08H\x00\x12)\n\ttapAction\x18\x07 \x01(\x0b\x32\x14.proto.TapLinkActionH\x00\"\x8c\x01\n\x0eStatusLinkType\x12 \n\x1cSTATUS_LINK_TYPE_UNSPECIFIED\x10\x00\x12\x1b\n\x17RASTERIZED_LINK_PREVIEW\x10\x01\x12\x1d\n\x19RASTERIZED_LINK_TRUNCATED\x10\x02\x12\x1c\n\x18RASTERIZED_LINK_FULL_URL\x10\x03\x42\x08\n\x06\x61\x63tionB\x19\n\x17_shouldSkipConfirmationB\x12\n\x10_embeddedContentB\x11\n\x0f_statusLinkType\"d\n$InteractiveMessageAdditionalMetadata\x12\"\n\x15isGalaxyFlowCompleted\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\x18\n\x16_isGalaxyFlowCompleted\"\xb2\x02\n\nKeepInChat\x12&\n\x08keepType\x18\x01 \x01(\x0e\x32\x0f.proto.KeepTypeH\x00\x88\x01\x01\x12\x1c\n\x0fserverTimestamp\x18\x02 \x01(\x03H\x01\x88\x01\x01\x12#\n\x03key\x18\x03 \x01(\x0b\x32\x11.proto.MessageKeyH\x02\x88\x01\x01\x12\x16\n\tdeviceJid\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x1e\n\x11\x63lientTimestampMs\x18\x05 \x01(\x03H\x04\x88\x01\x01\x12\x1e\n\x11serverTimestampMs\x18\x06 \x01(\x03H\x05\x88\x01\x01\x42\x0b\n\t_keepTypeB\x12\n\x10_serverTimestampB\x06\n\x04_keyB\x0c\n\n_deviceJidB\x14\n\x12_clientTimestampMsB\x14\n\x12_serverTimestampMs\"\xd4\x01\n\x12KeyExchangeMessage\x12\x0f\n\x02id\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x14\n\x07\x62\x61seKey\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x17\n\nratchetKey\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x18\n\x0bidentityKey\x18\x04 \x01(\x0cH\x03\x88\x01\x01\x12\x1d\n\x10\x62\x61seKeySignature\x18\x05 \x01(\x0cH\x04\x88\x01\x01\x42\x05\n\x03_idB\n\n\x08_baseKeyB\r\n\x0b_ratchetKeyB\x0e\n\x0c_identityKeyB\x13\n\x11_baseKeySignature\"\x1f\n\x05KeyId\x12\x0f\n\x02id\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x42\x05\n\x03_id\"\\\n\x13LIDMigrationMapping\x12\n\n\x02pn\x18\x01 \x01(\x04\x12\x13\n\x0b\x61ssignedLid\x18\x02 \x01(\x04\x12\x16\n\tlatestLid\x18\x03 \x01(\x04H\x00\x88\x01\x01\x42\x0c\n\n_latestLid\"^\n\x1eLIDMigrationMappingSyncMessage\x12\"\n\x15\x65ncodedMappingPayload\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x42\x18\n\x16_encodedMappingPayload\"\x99\x01\n\x1eLIDMigrationMappingSyncPayload\x12\x33\n\x0fpnToLidMappings\x18\x01 \x03(\x0b\x32\x1a.proto.LIDMigrationMapping\x12%\n\x18\x63hatDbMigrationTimestamp\x18\x02 \x01(\x04H\x00\x88\x01\x01\x42\x1b\n\x19_chatDbMigrationTimestamp\"\xb4\x01\n\rLegacyMessage\x12\x46\n\x14\x65ventResponseMessage\x18\x01 \x01(\x0b\x32#.proto.Message.EventResponseMessageH\x00\x88\x01\x01\x12\x35\n\x08pollVote\x18\x02 \x01(\x0b\x32\x1e.proto.Message.PollVoteMessageH\x01\x88\x01\x01\x42\x17\n\x15_eventResponseMessageB\x0b\n\t_pollVote\"\xd9\x02\n\x0cLimitSharing\x12\x1b\n\x0esharingLimited\x18\x01 \x01(\x08H\x00\x88\x01\x01\x12\x35\n\x07trigger\x18\x02 \x01(\x0e\x32\x1f.proto.LimitSharing.TriggerTypeH\x01\x88\x01\x01\x12)\n\x1climitSharingSettingTimestamp\x18\x03 \x01(\x03H\x02\x88\x01\x01\x12\x1a\n\rinitiatedByMe\x18\x04 \x01(\x08H\x03\x88\x01\x01\"\\\n\x0bTriggerType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x10\n\x0c\x43HAT_SETTING\x10\x01\x12\x1b\n\x17\x42IZ_SUPPORTS_FB_HOSTING\x10\x02\x12\x11\n\rUNKNOWN_GROUP\x10\x03\x42\x11\n\x0f_sharingLimitedB\n\n\x08_triggerB\x1f\n\x1d_limitSharingSettingTimestampB\x10\n\x0e_initiatedByMe\"k\n\rLocalizedName\x12\x0f\n\x02lg\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x0f\n\x02lc\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x19\n\x0cverifiedName\x18\x03 \x01(\tH\x02\x88\x01\x01\x42\x05\n\x03_lgB\x05\n\x03_lcB\x0f\n\r_verifiedName\"\x8c\x01\n\x08Location\x12\x1c\n\x0f\x64\x65greesLatitude\x18\x01 \x01(\x01H\x00\x88\x01\x01\x12\x1d\n\x10\x64\x65greesLongitude\x18\x02 \x01(\x01H\x01\x88\x01\x01\x12\x11\n\x04name\x18\x03 \x01(\tH\x02\x88\x01\x01\x42\x12\n\x10_degreesLatitudeB\x13\n\x11_degreesLongitudeB\x07\n\x05_name\"1\n\tMediaData\x12\x16\n\tlocalPath\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x0c\n\n_localPath\"\x9a\x01\n\x12MediaNotifyMessage\x12\x1b\n\x0e\x65xpressPathUrl\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x1a\n\rfileEncSha256\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x17\n\nfileLength\x18\x03 \x01(\x04H\x02\x88\x01\x01\x42\x11\n\x0f_expressPathUrlB\x10\n\x0e_fileEncSha256B\r\n\x0b_fileLength\"\xaf\x02\n\x16MediaRetryNotification\x12\x15\n\x08stanzaId\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x17\n\ndirectPath\x18\x02 \x01(\tH\x01\x88\x01\x01\x12=\n\x06result\x18\x03 \x01(\x0e\x32(.proto.MediaRetryNotification.ResultTypeH\x02\x88\x01\x01\x12\x1a\n\rmessageSecret\x18\x04 \x01(\x0cH\x03\x88\x01\x01\"Q\n\nResultType\x12\x11\n\rGENERAL_ERROR\x10\x00\x12\x0b\n\x07SUCCESS\x10\x01\x12\r\n\tNOT_FOUND\x10\x02\x12\x14\n\x10\x44\x45\x43RYPTION_ERROR\x10\x03\x42\x0b\n\t_stanzaIdB\r\n\x0b_directPathB\t\n\x07_resultB\x10\n\x0e_messageSecret\"[\n\x0bMemberLabel\x12\x12\n\x05label\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x1b\n\x0elabelTimestamp\x18\x02 \x01(\x03H\x01\x88\x01\x01\x42\x08\n\x06_labelB\x11\n\x0f_labelTimestamp\"\xcb\xd1\x03\n\x07Message\x12\x19\n\x0c\x63onversation\x18\x01 \x01(\tH\x00\x88\x01\x01\x12V\n\x1csenderKeyDistributionMessage\x18\x02 \x01(\x0b\x32+.proto.Message.SenderKeyDistributionMessageH\x01\x88\x01\x01\x12\x36\n\x0cimageMessage\x18\x03 \x01(\x0b\x32\x1b.proto.Message.ImageMessageH\x02\x88\x01\x01\x12:\n\x0e\x63ontactMessage\x18\x04 \x01(\x0b\x32\x1d.proto.Message.ContactMessageH\x03\x88\x01\x01\x12<\n\x0flocationMessage\x18\x05 \x01(\x0b\x32\x1e.proto.Message.LocationMessageH\x04\x88\x01\x01\x12\x44\n\x13\x65xtendedTextMessage\x18\x06 \x01(\x0b\x32\".proto.Message.ExtendedTextMessageH\x05\x88\x01\x01\x12<\n\x0f\x64ocumentMessage\x18\x07 \x01(\x0b\x32\x1e.proto.Message.DocumentMessageH\x06\x88\x01\x01\x12\x36\n\x0c\x61udioMessage\x18\x08 \x01(\x0b\x32\x1b.proto.Message.AudioMessageH\x07\x88\x01\x01\x12\x36\n\x0cvideoMessage\x18\t \x01(\x0b\x32\x1b.proto.Message.VideoMessageH\x08\x88\x01\x01\x12&\n\x04\x63\x61ll\x18\n \x01(\x0b\x32\x13.proto.Message.CallH\t\x88\x01\x01\x12&\n\x04\x63hat\x18\x0b \x01(\x0b\x32\x13.proto.Message.ChatH\n\x88\x01\x01\x12<\n\x0fprotocolMessage\x18\x0c \x01(\x0b\x32\x1e.proto.Message.ProtocolMessageH\x0b\x88\x01\x01\x12\x46\n\x14\x63ontactsArrayMessage\x18\r \x01(\x0b\x32#.proto.Message.ContactsArrayMessageH\x0c\x88\x01\x01\x12L\n\x17highlyStructuredMessage\x18\x0e \x01(\x0b\x32&.proto.Message.HighlyStructuredMessageH\r\x88\x01\x01\x12\x64\n*fastRatchetKeySenderKeyDistributionMessage\x18\x0f \x01(\x0b\x32+.proto.Message.SenderKeyDistributionMessageH\x0e\x88\x01\x01\x12\x42\n\x12sendPaymentMessage\x18\x10 \x01(\x0b\x32!.proto.Message.SendPaymentMessageH\x0f\x88\x01\x01\x12\x44\n\x13liveLocationMessage\x18\x12 \x01(\x0b\x32\".proto.Message.LiveLocationMessageH\x10\x88\x01\x01\x12H\n\x15requestPaymentMessage\x18\x16 \x01(\x0b\x32$.proto.Message.RequestPaymentMessageH\x11\x88\x01\x01\x12V\n\x1c\x64\x65\x63linePaymentRequestMessage\x18\x17 \x01(\x0b\x32+.proto.Message.DeclinePaymentRequestMessageH\x12\x88\x01\x01\x12T\n\x1b\x63\x61ncelPaymentRequestMessage\x18\x18 \x01(\x0b\x32*.proto.Message.CancelPaymentRequestMessageH\x13\x88\x01\x01\x12<\n\x0ftemplateMessage\x18\x19 \x01(\x0b\x32\x1e.proto.Message.TemplateMessageH\x14\x88\x01\x01\x12:\n\x0estickerMessage\x18\x1a \x01(\x0b\x32\x1d.proto.Message.StickerMessageH\x15\x88\x01\x01\x12\x42\n\x12groupInviteMessage\x18\x1c \x01(\x0b\x32!.proto.Message.GroupInviteMessageH\x16\x88\x01\x01\x12R\n\x1atemplateButtonReplyMessage\x18\x1d \x01(\x0b\x32).proto.Message.TemplateButtonReplyMessageH\x17\x88\x01\x01\x12:\n\x0eproductMessage\x18\x1e \x01(\x0b\x32\x1d.proto.Message.ProductMessageH\x18\x88\x01\x01\x12@\n\x11\x64\x65viceSentMessage\x18\x1f \x01(\x0b\x32 .proto.Message.DeviceSentMessageH\x19\x88\x01\x01\x12:\n\x12messageContextInfo\x18# \x01(\x0b\x32\x19.proto.MessageContextInfoH\x1a\x88\x01\x01\x12\x34\n\x0blistMessage\x18$ \x01(\x0b\x32\x1a.proto.Message.ListMessageH\x1b\x88\x01\x01\x12?\n\x0fviewOnceMessage\x18% \x01(\x0b\x32!.proto.Message.FutureProofMessageH\x1c\x88\x01\x01\x12\x36\n\x0corderMessage\x18& \x01(\x0b\x32\x1b.proto.Message.OrderMessageH\x1d\x88\x01\x01\x12\x44\n\x13listResponseMessage\x18\' \x01(\x0b\x32\".proto.Message.ListResponseMessageH\x1e\x88\x01\x01\x12@\n\x10\x65phemeralMessage\x18( \x01(\x0b\x32!.proto.Message.FutureProofMessageH\x1f\x88\x01\x01\x12:\n\x0einvoiceMessage\x18) \x01(\x0b\x32\x1d.proto.Message.InvoiceMessageH \x88\x01\x01\x12:\n\x0e\x62uttonsMessage\x18* \x01(\x0b\x32\x1d.proto.Message.ButtonsMessageH!\x88\x01\x01\x12J\n\x16\x62uttonsResponseMessage\x18+ \x01(\x0b\x32%.proto.Message.ButtonsResponseMessageH\"\x88\x01\x01\x12\x46\n\x14paymentInviteMessage\x18, \x01(\x0b\x32#.proto.Message.PaymentInviteMessageH#\x88\x01\x01\x12\x42\n\x12interactiveMessage\x18- \x01(\x0b\x32!.proto.Message.InteractiveMessageH$\x88\x01\x01\x12<\n\x0freactionMessage\x18. \x01(\x0b\x32\x1e.proto.Message.ReactionMessageH%\x88\x01\x01\x12H\n\x15stickerSyncRmrMessage\x18/ \x01(\x0b\x32$.proto.Message.StickerSyncRMRMessageH&\x88\x01\x01\x12R\n\x1ainteractiveResponseMessage\x18\x30 \x01(\x0b\x32).proto.Message.InteractiveResponseMessageH\'\x88\x01\x01\x12\x44\n\x13pollCreationMessage\x18\x31 \x01(\x0b\x32\".proto.Message.PollCreationMessageH(\x88\x01\x01\x12@\n\x11pollUpdateMessage\x18\x32 \x01(\x0b\x32 .proto.Message.PollUpdateMessageH)\x88\x01\x01\x12@\n\x11keepInChatMessage\x18\x33 \x01(\x0b\x32 .proto.Message.KeepInChatMessageH*\x88\x01\x01\x12J\n\x1a\x64ocumentWithCaptionMessage\x18\x35 \x01(\x0b\x32!.proto.Message.FutureProofMessageH+\x88\x01\x01\x12P\n\x19requestPhoneNumberMessage\x18\x36 \x01(\x0b\x32(.proto.Message.RequestPhoneNumberMessageH,\x88\x01\x01\x12\x41\n\x11viewOnceMessageV2\x18\x37 \x01(\x0b\x32!.proto.Message.FutureProofMessageH-\x88\x01\x01\x12\x42\n\x12\x65ncReactionMessage\x18\x38 \x01(\x0b\x32!.proto.Message.EncReactionMessageH.\x88\x01\x01\x12=\n\reditedMessage\x18: \x01(\x0b\x32!.proto.Message.FutureProofMessageH/\x88\x01\x01\x12J\n\x1aviewOnceMessageV2Extension\x18; \x01(\x0b\x32!.proto.Message.FutureProofMessageH0\x88\x01\x01\x12\x46\n\x15pollCreationMessageV2\x18< \x01(\x0b\x32\".proto.Message.PollCreationMessageH1\x88\x01\x01\x12V\n\x1cscheduledCallCreationMessage\x18= \x01(\x0b\x32+.proto.Message.ScheduledCallCreationMessageH2\x88\x01\x01\x12\x45\n\x15groupMentionedMessage\x18> \x01(\x0b\x32!.proto.Message.FutureProofMessageH3\x88\x01\x01\x12>\n\x10pinInChatMessage\x18? \x01(\x0b\x32\x1f.proto.Message.PinInChatMessageH4\x88\x01\x01\x12\x46\n\x15pollCreationMessageV3\x18@ \x01(\x0b\x32\".proto.Message.PollCreationMessageH5\x88\x01\x01\x12N\n\x18scheduledCallEditMessage\x18\x41 \x01(\x0b\x32\'.proto.Message.ScheduledCallEditMessageH6\x88\x01\x01\x12\x34\n\nptvMessage\x18\x42 \x01(\x0b\x32\x1b.proto.Message.VideoMessageH7\x88\x01\x01\x12@\n\x10\x62otInvokeMessage\x18\x43 \x01(\x0b\x32!.proto.Message.FutureProofMessageH8\x88\x01\x01\x12;\n\x0f\x63\x61llLogMesssage\x18\x45 \x01(\x0b\x32\x1d.proto.Message.CallLogMessageH9\x88\x01\x01\x12\x46\n\x14messageHistoryBundle\x18\x46 \x01(\x0b\x32#.proto.Message.MessageHistoryBundleH:\x88\x01\x01\x12@\n\x11\x65ncCommentMessage\x18G \x01(\x0b\x32 .proto.Message.EncCommentMessageH;\x88\x01\x01\x12\x36\n\x0c\x62\x63\x61llMessage\x18H \x01(\x0b\x32\x1b.proto.Message.BCallMessageH<\x88\x01\x01\x12\x44\n\x14lottieStickerMessage\x18J \x01(\x0b\x32!.proto.Message.FutureProofMessageH=\x88\x01\x01\x12\x36\n\x0c\x65ventMessage\x18K \x01(\x0b\x32\x1b.proto.Message.EventMessageH>\x88\x01\x01\x12L\n\x17\x65ncEventResponseMessage\x18L \x01(\x0b\x32&.proto.Message.EncEventResponseMessageH?\x88\x01\x01\x12:\n\x0e\x63ommentMessage\x18M \x01(\x0b\x32\x1d.proto.Message.CommentMessageH@\x88\x01\x01\x12V\n\x1cnewsletterAdminInviteMessage\x18N \x01(\x0b\x32+.proto.Message.NewsletterAdminInviteMessageHA\x88\x01\x01\x12\x42\n\x12placeholderMessage\x18P \x01(\x0b\x32!.proto.Message.PlaceholderMessageHB\x88\x01\x01\x12J\n\x16secretEncryptedMessage\x18R \x01(\x0b\x32%.proto.Message.SecretEncryptedMessageHC\x88\x01\x01\x12\x36\n\x0c\x61lbumMessage\x18S \x01(\x0b\x32\x1b.proto.Message.AlbumMessageHD\x88\x01\x01\x12?\n\x0f\x65ventCoverImage\x18U \x01(\x0b\x32!.proto.Message.FutureProofMessageHE\x88\x01\x01\x12\x42\n\x12stickerPackMessage\x18V \x01(\x0b\x32!.proto.Message.StickerPackMessageHF\x88\x01\x01\x12\x44\n\x14statusMentionMessage\x18W \x01(\x0b\x32!.proto.Message.FutureProofMessageHG\x88\x01\x01\x12P\n\x19pollResultSnapshotMessage\x18X \x01(\x0b\x32(.proto.Message.PollResultSnapshotMessageHH\x88\x01\x01\x12N\n\x1epollCreationOptionImageMessage\x18Z \x01(\x0b\x32!.proto.Message.FutureProofMessageHI\x88\x01\x01\x12\x46\n\x16\x61ssociatedChildMessage\x18[ \x01(\x0b\x32!.proto.Message.FutureProofMessageHJ\x88\x01\x01\x12I\n\x19groupStatusMentionMessage\x18\\ \x01(\x0b\x32!.proto.Message.FutureProofMessageHK\x88\x01\x01\x12\x45\n\x15pollCreationMessageV4\x18] \x01(\x0b\x32!.proto.Message.FutureProofMessageHL\x88\x01\x01\x12>\n\x0estatusAddYours\x18_ \x01(\x0b\x32!.proto.Message.FutureProofMessageHM\x88\x01\x01\x12\x42\n\x12groupStatusMessage\x18` \x01(\x0b\x32!.proto.Message.FutureProofMessageHN\x88\x01\x01\x12>\n\x13richResponseMessage\x18\x61 \x01(\x0b\x32\x1c.proto.AIRichResponseMessageHO\x88\x01\x01\x12P\n\x19statusNotificationMessage\x18\x62 \x01(\x0b\x32(.proto.Message.StatusNotificationMessageHP\x88\x01\x01\x12\x43\n\x13limitSharingMessage\x18\x63 \x01(\x0b\x32!.proto.Message.FutureProofMessageHQ\x88\x01\x01\x12>\n\x0e\x62otTaskMessage\x18\x64 \x01(\x0b\x32!.proto.Message.FutureProofMessageHR\x88\x01\x01\x12?\n\x0fquestionMessage\x18\x65 \x01(\x0b\x32!.proto.Message.FutureProofMessageHS\x88\x01\x01\x12\x46\n\x14messageHistoryNotice\x18\x66 \x01(\x0b\x32#.proto.Message.MessageHistoryNoticeHT\x88\x01\x01\x12\x44\n\x14groupStatusMessageV2\x18g \x01(\x0b\x32!.proto.Message.FutureProofMessageHU\x88\x01\x01\x12\x43\n\x13\x62otForwardedMessage\x18h \x01(\x0b\x32!.proto.Message.FutureProofMessageHV\x88\x01\x01\x12T\n\x1bstatusQuestionAnswerMessage\x18i \x01(\x0b\x32*.proto.Message.StatusQuestionAnswerMessageHW\x88\x01\x01\x12\x44\n\x14questionReplyMessage\x18j \x01(\x0b\x32!.proto.Message.FutureProofMessageHX\x88\x01\x01\x12L\n\x17questionResponseMessage\x18k \x01(\x0b\x32&.proto.Message.QuestionResponseMessageHY\x88\x01\x01\x12\x44\n\x13statusQuotedMessage\x18m \x01(\x0b\x32\".proto.Message.StatusQuotedMessageHZ\x88\x01\x01\x12\\\n\x1fstatusStickerInteractionMessage\x18n \x01(\x0b\x32..proto.Message.StatusStickerInteractionMessageH[\x88\x01\x01\x12\x46\n\x15pollCreationMessageV5\x18o \x01(\x0b\x32\".proto.Message.PollCreationMessageH\\\x88\x01\x01\x12^\n!newsletterFollowerInviteMessageV2\x18q \x01(\x0b\x32..proto.Message.NewsletterFollowerInviteMessageH]\x88\x01\x01\x12R\n\x1bpollResultSnapshotMessageV3\x18r \x01(\x0b\x32(.proto.Message.PollResultSnapshotMessageH^\x88\x01\x01\x1a\xbc\x01\n\x0c\x41lbumMessage\x12\x1f\n\x12\x65xpectedImageCount\x18\x02 \x01(\rH\x00\x88\x01\x01\x12\x1f\n\x12\x65xpectedVideoCount\x18\x03 \x01(\rH\x01\x88\x01\x01\x12,\n\x0b\x63ontextInfo\x18\x11 \x01(\x0b\x32\x12.proto.ContextInfoH\x02\x88\x01\x01\x42\x15\n\x13_expectedImageCountB\x15\n\x13_expectedVideoCountB\x0e\n\x0c_contextInfo\x1a\x63\n\"AppStateFatalExceptionNotification\x12\x17\n\x0f\x63ollectionNames\x18\x01 \x03(\t\x12\x16\n\ttimestamp\x18\x02 \x01(\x03H\x00\x88\x01\x01\x42\x0c\n\n_timestamp\x1a\x97\x01\n\x0f\x41ppStateSyncKey\x12\x34\n\x05keyId\x18\x01 \x01(\x0b\x32 .proto.Message.AppStateSyncKeyIdH\x00\x88\x01\x01\x12\x38\n\x07keyData\x18\x02 \x01(\x0b\x32\".proto.Message.AppStateSyncKeyDataH\x01\x88\x01\x01\x42\x08\n\x06_keyIdB\n\n\x08_keyData\x1a\xb2\x01\n\x13\x41ppStateSyncKeyData\x12\x14\n\x07keyData\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x43\n\x0b\x66ingerprint\x18\x02 \x01(\x0b\x32).proto.Message.AppStateSyncKeyFingerprintH\x01\x88\x01\x01\x12\x16\n\ttimestamp\x18\x03 \x01(\x03H\x02\x88\x01\x01\x42\n\n\x08_keyDataB\x0e\n\x0c_fingerprintB\x0c\n\n_timestamp\x1a\x81\x01\n\x1a\x41ppStateSyncKeyFingerprint\x12\x12\n\x05rawId\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x19\n\x0c\x63urrentIndex\x18\x02 \x01(\rH\x01\x88\x01\x01\x12\x19\n\rdeviceIndexes\x18\x03 \x03(\rB\x02\x10\x01\x42\x08\n\x06_rawIdB\x0f\n\r_currentIndex\x1a\x31\n\x11\x41ppStateSyncKeyId\x12\x12\n\x05keyId\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x42\x08\n\x06_keyId\x1aJ\n\x16\x41ppStateSyncKeyRequest\x12\x30\n\x06keyIds\x18\x01 \x03(\x0b\x32 .proto.Message.AppStateSyncKeyId\x1a\x44\n\x14\x41ppStateSyncKeyShare\x12,\n\x04keys\x18\x01 \x03(\x0b\x32\x1e.proto.Message.AppStateSyncKey\x1a\xf9\x05\n\x0c\x41udioMessage\x12\x10\n\x03url\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x15\n\x08mimetype\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x17\n\nfileSha256\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x17\n\nfileLength\x18\x04 \x01(\x04H\x03\x88\x01\x01\x12\x14\n\x07seconds\x18\x05 \x01(\rH\x04\x88\x01\x01\x12\x10\n\x03ptt\x18\x06 \x01(\x08H\x05\x88\x01\x01\x12\x15\n\x08mediaKey\x18\x07 \x01(\x0cH\x06\x88\x01\x01\x12\x1a\n\rfileEncSha256\x18\x08 \x01(\x0cH\x07\x88\x01\x01\x12\x17\n\ndirectPath\x18\t \x01(\tH\x08\x88\x01\x01\x12\x1e\n\x11mediaKeyTimestamp\x18\n \x01(\x03H\t\x88\x01\x01\x12,\n\x0b\x63ontextInfo\x18\x11 \x01(\x0b\x32\x12.proto.ContextInfoH\n\x88\x01\x01\x12\x1d\n\x10streamingSidecar\x18\x12 \x01(\x0cH\x0b\x88\x01\x01\x12\x15\n\x08waveform\x18\x13 \x01(\x0cH\x0c\x88\x01\x01\x12\x1b\n\x0e\x62\x61\x63kgroundArgb\x18\x14 \x01(\x07H\r\x88\x01\x01\x12\x15\n\x08viewOnce\x18\x15 \x01(\x08H\x0e\x88\x01\x01\x12\x1f\n\x12\x61\x63\x63\x65ssibilityLabel\x18\x16 \x01(\tH\x0f\x88\x01\x01\x12:\n\x0emediaKeyDomain\x18\x17 \x01(\x0e\x32\x1d.proto.Message.MediaKeyDomainH\x10\x88\x01\x01\x42\x06\n\x04_urlB\x0b\n\t_mimetypeB\r\n\x0b_fileSha256B\r\n\x0b_fileLengthB\n\n\x08_secondsB\x06\n\x04_pttB\x0b\n\t_mediaKeyB\x10\n\x0e_fileEncSha256B\r\n\x0b_directPathB\x14\n\x12_mediaKeyTimestampB\x0e\n\x0c_contextInfoB\x13\n\x11_streamingSidecarB\x0b\n\t_waveformB\x11\n\x0f_backgroundArgbB\x0b\n\t_viewOnceB\x15\n\x13_accessibilityLabelB\x11\n\x0f_mediaKeyDomain\x1a\xf9\x01\n\x0c\x42\x43\x61llMessage\x12\x16\n\tsessionId\x18\x01 \x01(\tH\x00\x88\x01\x01\x12=\n\tmediaType\x18\x02 \x01(\x0e\x32%.proto.Message.BCallMessage.MediaTypeH\x01\x88\x01\x01\x12\x16\n\tmasterKey\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x14\n\x07\x63\x61ption\x18\x04 \x01(\tH\x03\x88\x01\x01\".\n\tMediaType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\t\n\x05\x41UDIO\x10\x01\x12\t\n\x05VIDEO\x10\x02\x42\x0c\n\n_sessionIdB\x0c\n\n_mediaTypeB\x0c\n\n_masterKeyB\n\n\x08_caption\x1a\xfb\x08\n\x0e\x42uttonsMessage\x12\x18\n\x0b\x63ontentText\x18\x06 \x01(\tH\x01\x88\x01\x01\x12\x17\n\nfooterText\x18\x07 \x01(\tH\x02\x88\x01\x01\x12,\n\x0b\x63ontextInfo\x18\x08 \x01(\x0b\x32\x12.proto.ContextInfoH\x03\x88\x01\x01\x12\x35\n\x07\x62uttons\x18\t \x03(\x0b\x32$.proto.Message.ButtonsMessage.Button\x12\x41\n\nheaderType\x18\n \x01(\x0e\x32(.proto.Message.ButtonsMessage.HeaderTypeH\x04\x88\x01\x01\x12\x0e\n\x04text\x18\x01 \x01(\tH\x00\x12\x39\n\x0f\x64ocumentMessage\x18\x02 \x01(\x0b\x32\x1e.proto.Message.DocumentMessageH\x00\x12\x33\n\x0cimageMessage\x18\x03 \x01(\x0b\x32\x1b.proto.Message.ImageMessageH\x00\x12\x33\n\x0cvideoMessage\x18\x04 \x01(\x0b\x32\x1b.proto.Message.VideoMessageH\x00\x12\x39\n\x0flocationMessage\x18\x05 \x01(\x0b\x32\x1e.proto.Message.LocationMessageH\x00\x1a\xf3\x03\n\x06\x42utton\x12\x15\n\x08\x62uttonId\x18\x01 \x01(\tH\x00\x88\x01\x01\x12H\n\nbuttonText\x18\x02 \x01(\x0b\x32/.proto.Message.ButtonsMessage.Button.ButtonTextH\x01\x88\x01\x01\x12<\n\x04type\x18\x03 \x01(\x0e\x32).proto.Message.ButtonsMessage.Button.TypeH\x02\x88\x01\x01\x12P\n\x0enativeFlowInfo\x18\x04 \x01(\x0b\x32\x33.proto.Message.ButtonsMessage.Button.NativeFlowInfoH\x03\x88\x01\x01\x1a\x36\n\nButtonText\x12\x18\n\x0b\x64isplayText\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x0e\n\x0c_displayText\x1aT\n\x0eNativeFlowInfo\x12\x11\n\x04name\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x17\n\nparamsJson\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x07\n\x05_nameB\r\n\x0b_paramsJson\"2\n\x04Type\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0c\n\x08RESPONSE\x10\x01\x12\x0f\n\x0bNATIVE_FLOW\x10\x02\x42\x0b\n\t_buttonIdB\r\n\x0b_buttonTextB\x07\n\x05_typeB\x11\n\x0f_nativeFlowInfo\"`\n\nHeaderType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\t\n\x05\x45MPTY\x10\x01\x12\x08\n\x04TEXT\x10\x02\x12\x0c\n\x08\x44OCUMENT\x10\x03\x12\t\n\x05IMAGE\x10\x04\x12\t\n\x05VIDEO\x10\x05\x12\x0c\n\x08LOCATION\x10\x06\x42\x08\n\x06headerB\x0e\n\x0c_contentTextB\r\n\x0b_footerTextB\x0e\n\x0c_contextInfoB\r\n\x0b_headerType\x1a\xa4\x02\n\x16\x42uttonsResponseMessage\x12\x1d\n\x10selectedButtonId\x18\x01 \x01(\tH\x01\x88\x01\x01\x12,\n\x0b\x63ontextInfo\x18\x03 \x01(\x0b\x32\x12.proto.ContextInfoH\x02\x88\x01\x01\x12=\n\x04type\x18\x04 \x01(\x0e\x32*.proto.Message.ButtonsResponseMessage.TypeH\x03\x88\x01\x01\x12\x1d\n\x13selectedDisplayText\x18\x02 \x01(\tH\x00\"%\n\x04Type\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x10\n\x0c\x44ISPLAY_TEXT\x10\x01\x42\n\n\x08responseB\x13\n\x11_selectedButtonIdB\x0e\n\x0c_contextInfoB\x07\n\x05_type\x1a\xda\x03\n\x04\x43\x61ll\x12\x14\n\x07\x63\x61llKey\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x1d\n\x10\x63onversionSource\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x1b\n\x0e\x63onversionData\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12#\n\x16\x63onversionDelaySeconds\x18\x04 \x01(\rH\x03\x88\x01\x01\x12\x18\n\x0b\x63twaSignals\x18\x05 \x01(\tH\x04\x88\x01\x01\x12\x18\n\x0b\x63twaPayload\x18\x06 \x01(\x0cH\x05\x88\x01\x01\x12,\n\x0b\x63ontextInfo\x18\x07 \x01(\x0b\x32\x12.proto.ContextInfoH\x06\x88\x01\x01\x12(\n\x1bnativeFlowCallButtonPayload\x18\x08 \x01(\tH\x07\x88\x01\x01\x12\x1c\n\x0f\x64\x65\x65plinkPayload\x18\t \x01(\tH\x08\x88\x01\x01\x42\n\n\x08_callKeyB\x13\n\x11_conversionSourceB\x11\n\x0f_conversionDataB\x19\n\x17_conversionDelaySecondsB\x0e\n\x0c_ctwaSignalsB\x0e\n\x0c_ctwaPayloadB\x0e\n\x0c_contextInfoB\x1e\n\x1c_nativeFlowCallButtonPayloadB\x12\n\x10_deeplinkPayload\x1a\xa0\x05\n\x0e\x43\x61llLogMessage\x12\x14\n\x07isVideo\x18\x01 \x01(\x08H\x00\x88\x01\x01\x12\x43\n\x0b\x63\x61llOutcome\x18\x02 \x01(\x0e\x32).proto.Message.CallLogMessage.CallOutcomeH\x01\x88\x01\x01\x12\x19\n\x0c\x64urationSecs\x18\x03 \x01(\x03H\x02\x88\x01\x01\x12=\n\x08\x63\x61llType\x18\x04 \x01(\x0e\x32&.proto.Message.CallLogMessage.CallTypeH\x03\x88\x01\x01\x12\x43\n\x0cparticipants\x18\x05 \x03(\x0b\x32-.proto.Message.CallLogMessage.CallParticipant\x1a\x80\x01\n\x0f\x43\x61llParticipant\x12\x10\n\x03jid\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x43\n\x0b\x63\x61llOutcome\x18\x02 \x01(\x0e\x32).proto.Message.CallLogMessage.CallOutcomeH\x01\x88\x01\x01\x42\x06\n\x04_jidB\x0e\n\x0c_callOutcome\"\x99\x01\n\x0b\x43\x61llOutcome\x12\r\n\tCONNECTED\x10\x00\x12\n\n\x06MISSED\x10\x01\x12\n\n\x06\x46\x41ILED\x10\x02\x12\x0c\n\x08REJECTED\x10\x03\x12\x16\n\x12\x41\x43\x43\x45PTED_ELSEWHERE\x10\x04\x12\x0b\n\x07ONGOING\x10\x05\x12\x13\n\x0fSILENCED_BY_DND\x10\x06\x12\x1b\n\x17SILENCED_UNKNOWN_CALLER\x10\x07\";\n\x08\x43\x61llType\x12\x0b\n\x07REGULAR\x10\x00\x12\x12\n\x0eSCHEDULED_CALL\x10\x01\x12\x0e\n\nVOICE_CHAT\x10\x02\x42\n\n\x08_isVideoB\x0e\n\x0c_callOutcomeB\x0f\n\r_durationSecsB\x0b\n\t_callType\x1aJ\n\x1b\x43\x61ncelPaymentRequestMessage\x12#\n\x03key\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x42\x06\n\x04_key\x1aH\n\x04\x43hat\x12\x18\n\x0b\x64isplayName\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x0f\n\x02id\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x0e\n\x0c_displayNameB\x05\n\x03_id\x1a\xfc\x05\n!CloudAPIThreadControlNotification\x12[\n\x06status\x18\x01 \x01(\x0e\x32\x46.proto.Message.CloudAPIThreadControlNotification.CloudAPIThreadControlH\x00\x88\x01\x01\x12*\n\x1dsenderNotificationTimestampMs\x18\x02 \x01(\x03H\x01\x88\x01\x01\x12\x18\n\x0b\x63onsumerLid\x18\x03 \x01(\tH\x02\x88\x01\x01\x12 \n\x13\x63onsumerPhoneNumber\x18\x04 \x01(\tH\x03\x88\x01\x01\x12{\n\x13notificationContent\x18\x05 \x01(\x0b\x32Y.proto.Message.CloudAPIThreadControlNotification.CloudAPIThreadControlNotificationContentH\x04\x88\x01\x01\x12\'\n\x1ashouldSuppressNotification\x18\x06 \x01(\x08H\x05\x88\x01\x01\x1a\x92\x01\n(CloudAPIThreadControlNotificationContent\x12$\n\x17handoffNotificationText\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x16\n\textraJson\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x1a\n\x18_handoffNotificationTextB\x0c\n\n_extraJson\"K\n\x15\x43loudAPIThreadControl\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x12\n\x0e\x43ONTROL_PASSED\x10\x01\x12\x11\n\rCONTROL_TAKEN\x10\x02\x42\t\n\x07_statusB \n\x1e_senderNotificationTimestampMsB\x0e\n\x0c_consumerLidB\x16\n\x14_consumerPhoneNumberB\x16\n\x14_notificationContentB\x1d\n\x1b_shouldSuppressNotification\x1a\x89\x01\n\x0e\x43ommentMessage\x12$\n\x07message\x18\x01 \x01(\x0b\x32\x0e.proto.MessageH\x00\x88\x01\x01\x12\x30\n\x10targetMessageKey\x18\x02 \x01(\x0b\x32\x11.proto.MessageKeyH\x01\x88\x01\x01\x42\n\n\x08_messageB\x13\n\x11_targetMessageKey\x1a\x96\x01\n\x0e\x43ontactMessage\x12\x18\n\x0b\x64isplayName\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x12\n\x05vcard\x18\x10 \x01(\tH\x01\x88\x01\x01\x12,\n\x0b\x63ontextInfo\x18\x11 \x01(\x0b\x32\x12.proto.ContextInfoH\x02\x88\x01\x01\x42\x0e\n\x0c_displayNameB\x08\n\x06_vcardB\x0e\n\x0c_contextInfo\x1a\xaf\x01\n\x14\x43ontactsArrayMessage\x12\x18\n\x0b\x64isplayName\x18\x01 \x01(\tH\x00\x88\x01\x01\x12/\n\x08\x63ontacts\x18\x02 \x03(\x0b\x32\x1d.proto.Message.ContactMessage\x12,\n\x0b\x63ontextInfo\x18\x11 \x01(\x0b\x32\x12.proto.ContextInfoH\x01\x88\x01\x01\x42\x0e\n\x0c_displayNameB\x0e\n\x0c_contextInfo\x1aK\n\x1c\x44\x65\x63linePaymentRequestMessage\x12#\n\x03key\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x42\x06\n\x04_key\x1a\x93\x01\n\x11\x44\x65viceSentMessage\x12\x1b\n\x0e\x64\x65stinationJid\x18\x01 \x01(\tH\x00\x88\x01\x01\x12$\n\x07message\x18\x02 \x01(\x0b\x32\x0e.proto.MessageH\x01\x88\x01\x01\x12\x12\n\x05phash\x18\x03 \x01(\tH\x02\x88\x01\x01\x42\x11\n\x0f_destinationJidB\n\n\x08_messageB\x08\n\x06_phash\x1a\xfe\x07\n\x0f\x44ocumentMessage\x12\x10\n\x03url\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x15\n\x08mimetype\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x12\n\x05title\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x17\n\nfileSha256\x18\x04 \x01(\x0cH\x03\x88\x01\x01\x12\x17\n\nfileLength\x18\x05 \x01(\x04H\x04\x88\x01\x01\x12\x16\n\tpageCount\x18\x06 \x01(\rH\x05\x88\x01\x01\x12\x15\n\x08mediaKey\x18\x07 \x01(\x0cH\x06\x88\x01\x01\x12\x15\n\x08\x66ileName\x18\x08 \x01(\tH\x07\x88\x01\x01\x12\x1a\n\rfileEncSha256\x18\t \x01(\x0cH\x08\x88\x01\x01\x12\x17\n\ndirectPath\x18\n \x01(\tH\t\x88\x01\x01\x12\x1e\n\x11mediaKeyTimestamp\x18\x0b \x01(\x03H\n\x88\x01\x01\x12\x19\n\x0c\x63ontactVcard\x18\x0c \x01(\x08H\x0b\x88\x01\x01\x12 \n\x13thumbnailDirectPath\x18\r \x01(\tH\x0c\x88\x01\x01\x12\x1c\n\x0fthumbnailSha256\x18\x0e \x01(\x0cH\r\x88\x01\x01\x12\x1f\n\x12thumbnailEncSha256\x18\x0f \x01(\x0cH\x0e\x88\x01\x01\x12\x1a\n\rjpegThumbnail\x18\x10 \x01(\x0cH\x0f\x88\x01\x01\x12,\n\x0b\x63ontextInfo\x18\x11 \x01(\x0b\x32\x12.proto.ContextInfoH\x10\x88\x01\x01\x12\x1c\n\x0fthumbnailHeight\x18\x12 \x01(\rH\x11\x88\x01\x01\x12\x1b\n\x0ethumbnailWidth\x18\x13 \x01(\rH\x12\x88\x01\x01\x12\x14\n\x07\x63\x61ption\x18\x14 \x01(\tH\x13\x88\x01\x01\x12\x1f\n\x12\x61\x63\x63\x65ssibilityLabel\x18\x15 \x01(\tH\x14\x88\x01\x01\x12:\n\x0emediaKeyDomain\x18\x16 \x01(\x0e\x32\x1d.proto.Message.MediaKeyDomainH\x15\x88\x01\x01\x42\x06\n\x04_urlB\x0b\n\t_mimetypeB\x08\n\x06_titleB\r\n\x0b_fileSha256B\r\n\x0b_fileLengthB\x0c\n\n_pageCountB\x0b\n\t_mediaKeyB\x0b\n\t_fileNameB\x10\n\x0e_fileEncSha256B\r\n\x0b_directPathB\x14\n\x12_mediaKeyTimestampB\x0f\n\r_contactVcardB\x16\n\x14_thumbnailDirectPathB\x12\n\x10_thumbnailSha256B\x15\n\x13_thumbnailEncSha256B\x10\n\x0e_jpegThumbnailB\x0e\n\x0c_contextInfoB\x12\n\x10_thumbnailHeightB\x11\n\x0f_thumbnailWidthB\n\n\x08_captionB\x15\n\x13_accessibilityLabelB\x11\n\x0f_mediaKeyDomain\x1a\xa0\x01\n\x11\x45ncCommentMessage\x12\x30\n\x10targetMessageKey\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x12\x17\n\nencPayload\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x12\n\x05\x65ncIv\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x42\x13\n\x11_targetMessageKeyB\r\n\x0b_encPayloadB\x08\n\x06_encIv\x1a\xb4\x01\n\x17\x45ncEventResponseMessage\x12\x37\n\x17\x65ventCreationMessageKey\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x12\x17\n\nencPayload\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x12\n\x05\x65ncIv\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x42\x1a\n\x18_eventCreationMessageKeyB\r\n\x0b_encPayloadB\x08\n\x06_encIv\x1a\xa1\x01\n\x12\x45ncReactionMessage\x12\x30\n\x10targetMessageKey\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x12\x17\n\nencPayload\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x12\n\x05\x65ncIv\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x42\x13\n\x11_targetMessageKeyB\r\n\x0b_encPayloadB\x08\n\x06_encIv\x1a\xb2\x04\n\x0c\x45ventMessage\x12,\n\x0b\x63ontextInfo\x18\x01 \x01(\x0b\x32\x12.proto.ContextInfoH\x00\x88\x01\x01\x12\x17\n\nisCanceled\x18\x02 \x01(\x08H\x01\x88\x01\x01\x12\x11\n\x04name\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x18\n\x0b\x64\x65scription\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x35\n\x08location\x18\x05 \x01(\x0b\x32\x1e.proto.Message.LocationMessageH\x04\x88\x01\x01\x12\x15\n\x08joinLink\x18\x06 \x01(\tH\x05\x88\x01\x01\x12\x16\n\tstartTime\x18\x07 \x01(\x03H\x06\x88\x01\x01\x12\x14\n\x07\x65ndTime\x18\x08 \x01(\x03H\x07\x88\x01\x01\x12\x1f\n\x12\x65xtraGuestsAllowed\x18\t \x01(\x08H\x08\x88\x01\x01\x12\x1b\n\x0eisScheduleCall\x18\n \x01(\x08H\t\x88\x01\x01\x12\x18\n\x0bhasReminder\x18\x0b \x01(\x08H\n\x88\x01\x01\x12\x1e\n\x11reminderOffsetSec\x18\x0c \x01(\x03H\x0b\x88\x01\x01\x42\x0e\n\x0c_contextInfoB\r\n\x0b_isCanceledB\x07\n\x05_nameB\x0e\n\x0c_descriptionB\x0b\n\t_locationB\x0b\n\t_joinLinkB\x0c\n\n_startTimeB\n\n\x08_endTimeB\x15\n\x13_extraGuestsAllowedB\x11\n\x0f_isScheduleCallB\x0e\n\x0c_hasReminderB\x14\n\x12_reminderOffsetSec\x1a\x94\x02\n\x14\x45ventResponseMessage\x12L\n\x08response\x18\x01 \x01(\x0e\x32\x35.proto.Message.EventResponseMessage.EventResponseTypeH\x00\x88\x01\x01\x12\x18\n\x0btimestampMs\x18\x02 \x01(\x03H\x01\x88\x01\x01\x12\x1c\n\x0f\x65xtraGuestCount\x18\x03 \x01(\x05H\x02\x88\x01\x01\"E\n\x11\x45ventResponseType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\t\n\x05GOING\x10\x01\x12\r\n\tNOT_GOING\x10\x02\x12\t\n\x05MAYBE\x10\x03\x42\x0b\n\t_responseB\x0e\n\x0c_timestampMsB\x12\n\x10_extraGuestCount\x1a\x97\x12\n\x13\x45xtendedTextMessage\x12\x11\n\x04text\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x18\n\x0bmatchedText\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x18\n\x0b\x64\x65scription\x18\x05 \x01(\tH\x02\x88\x01\x01\x12\x12\n\x05title\x18\x06 \x01(\tH\x03\x88\x01\x01\x12\x15\n\x08textArgb\x18\x07 \x01(\x07H\x04\x88\x01\x01\x12\x1b\n\x0e\x62\x61\x63kgroundArgb\x18\x08 \x01(\x07H\x05\x88\x01\x01\x12>\n\x04\x66ont\x18\t \x01(\x0e\x32+.proto.Message.ExtendedTextMessage.FontTypeH\x06\x88\x01\x01\x12H\n\x0bpreviewType\x18\n \x01(\x0e\x32..proto.Message.ExtendedTextMessage.PreviewTypeH\x07\x88\x01\x01\x12\x1a\n\rjpegThumbnail\x18\x10 \x01(\x0cH\x08\x88\x01\x01\x12,\n\x0b\x63ontextInfo\x18\x11 \x01(\x0b\x32\x12.proto.ContextInfoH\t\x88\x01\x01\x12\x1c\n\x0f\x64oNotPlayInline\x18\x12 \x01(\x08H\n\x88\x01\x01\x12 \n\x13thumbnailDirectPath\x18\x13 \x01(\tH\x0b\x88\x01\x01\x12\x1c\n\x0fthumbnailSha256\x18\x14 \x01(\x0cH\x0c\x88\x01\x01\x12\x1f\n\x12thumbnailEncSha256\x18\x15 \x01(\x0cH\r\x88\x01\x01\x12\x15\n\x08mediaKey\x18\x16 \x01(\x0cH\x0e\x88\x01\x01\x12\x1e\n\x11mediaKeyTimestamp\x18\x17 \x01(\x03H\x0f\x88\x01\x01\x12\x1c\n\x0fthumbnailHeight\x18\x18 \x01(\rH\x10\x88\x01\x01\x12\x1b\n\x0ethumbnailWidth\x18\x19 \x01(\rH\x11\x88\x01\x01\x12X\n\x13inviteLinkGroupType\x18\x1a \x01(\x0e\x32\x36.proto.Message.ExtendedTextMessage.InviteLinkGroupTypeH\x12\x88\x01\x01\x12+\n\x1einviteLinkParentGroupSubjectV2\x18\x1b \x01(\tH\x13\x88\x01\x01\x12-\n inviteLinkParentGroupThumbnailV2\x18\x1c \x01(\x0cH\x14\x88\x01\x01\x12Z\n\x15inviteLinkGroupTypeV2\x18\x1d \x01(\x0e\x32\x36.proto.Message.ExtendedTextMessage.InviteLinkGroupTypeH\x15\x88\x01\x01\x12\x15\n\x08viewOnce\x18\x1e \x01(\x08H\x16\x88\x01\x01\x12\x18\n\x0bvideoHeight\x18\x1f \x01(\rH\x17\x88\x01\x01\x12\x17\n\nvideoWidth\x18  \x01(\rH\x18\x88\x01\x01\x12\x44\n\x12\x66\x61viconMMSMetadata\x18! \x01(\x0b\x32#.proto.Message.MMSThumbnailMetadataH\x19\x88\x01\x01\x12\x44\n\x13linkPreviewMetadata\x18\" \x01(\x0b\x32\".proto.Message.LinkPreviewMetadataH\x1a\x88\x01\x01\x12\x44\n\x13paymentLinkMetadata\x18# \x01(\x0b\x32\".proto.Message.PaymentLinkMetadataH\x1b\x88\x01\x01\x12\x31\n\x0c\x65ndCardTiles\x18$ \x03(\x0b\x32\x1b.proto.Message.VideoEndCard\x12\x1c\n\x0fvideoContentUrl\x18% \x01(\tH\x1c\x88\x01\x01\x12\x30\n\rmusicMetadata\x18& \x01(\x0b\x32\x14.proto.EmbeddedMusicH\x1d\x88\x01\x01\x12L\n\x17paymentExtendedMetadata\x18\' \x01(\x0b\x32&.proto.Message.PaymentExtendedMetadataH\x1e\x88\x01\x01\"\xa4\x01\n\x08\x46ontType\x12\n\n\x06SYSTEM\x10\x00\x12\x0f\n\x0bSYSTEM_TEXT\x10\x01\x12\r\n\tFB_SCRIPT\x10\x02\x12\x0f\n\x0bSYSTEM_BOLD\x10\x06\x12\x19\n\x15MORNINGBREEZE_REGULAR\x10\x07\x12\x15\n\x11\x43\x41LISTOGA_REGULAR\x10\x08\x12\x12\n\x0e\x45XO2_EXTRABOLD\x10\t\x12\x15\n\x11\x43OURIERPRIME_BOLD\x10\n\"H\n\x13InviteLinkGroupType\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x00\x12\n\n\x06PARENT\x10\x01\x12\x07\n\x03SUB\x10\x02\x12\x0f\n\x0b\x44\x45\x46\x41ULT_SUB\x10\x03\"^\n\x0bPreviewType\x12\x08\n\x04NONE\x10\x00\x12\t\n\x05VIDEO\x10\x01\x12\x0f\n\x0bPLACEHOLDER\x10\x04\x12\t\n\x05IMAGE\x10\x05\x12\x11\n\rPAYMENT_LINKS\x10\x06\x12\x0b\n\x07PROFILE\x10\x07\x42\x07\n\x05_textB\x0e\n\x0c_matchedTextB\x0e\n\x0c_descriptionB\x08\n\x06_titleB\x0b\n\t_textArgbB\x11\n\x0f_backgroundArgbB\x07\n\x05_fontB\x0e\n\x0c_previewTypeB\x10\n\x0e_jpegThumbnailB\x0e\n\x0c_contextInfoB\x12\n\x10_doNotPlayInlineB\x16\n\x14_thumbnailDirectPathB\x12\n\x10_thumbnailSha256B\x15\n\x13_thumbnailEncSha256B\x0b\n\t_mediaKeyB\x14\n\x12_mediaKeyTimestampB\x12\n\x10_thumbnailHeightB\x11\n\x0f_thumbnailWidthB\x16\n\x14_inviteLinkGroupTypeB!\n\x1f_inviteLinkParentGroupSubjectV2B#\n!_inviteLinkParentGroupThumbnailV2B\x18\n\x16_inviteLinkGroupTypeV2B\x0b\n\t_viewOnceB\x0e\n\x0c_videoHeightB\r\n\x0b_videoWidthB\x15\n\x13_faviconMMSMetadataB\x16\n\x14_linkPreviewMetadataB\x16\n\x14_paymentLinkMetadataB\x12\n\x10_videoContentUrlB\x10\n\x0e_musicMetadataB\x1a\n\x18_paymentExtendedMetadata\x1aN\n&FullHistorySyncOnDemandRequestMetadata\x12\x16\n\trequestId\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x0c\n\n_requestId\x1a\x46\n\x12\x46utureProofMessage\x12$\n\x07message\x18\x01 \x01(\x0b\x32\x0e.proto.MessageH\x00\x88\x01\x01\x42\n\n\x08_message\x1a\xc1\x03\n\x12GroupInviteMessage\x12\x15\n\x08groupJid\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x17\n\ninviteCode\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x1d\n\x10inviteExpiration\x18\x03 \x01(\x03H\x02\x88\x01\x01\x12\x16\n\tgroupName\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x1a\n\rjpegThumbnail\x18\x05 \x01(\x0cH\x04\x88\x01\x01\x12\x14\n\x07\x63\x61ption\x18\x06 \x01(\tH\x05\x88\x01\x01\x12,\n\x0b\x63ontextInfo\x18\x07 \x01(\x0b\x32\x12.proto.ContextInfoH\x06\x88\x01\x01\x12\x43\n\tgroupType\x18\x08 \x01(\x0e\x32+.proto.Message.GroupInviteMessage.GroupTypeH\x07\x88\x01\x01\"$\n\tGroupType\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x00\x12\n\n\x06PARENT\x10\x01\x42\x0b\n\t_groupJidB\r\n\x0b_inviteCodeB\x13\n\x11_inviteExpirationB\x0c\n\n_groupNameB\x10\n\x0e_jpegThumbnailB\n\n\x08_captionB\x0e\n\x0c_contextInfoB\x0c\n\n_groupType\x1a\xcb\x0e\n\x17HighlyStructuredMessage\x12\x16\n\tnamespace\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x18\n\x0b\x65lementName\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x0e\n\x06params\x18\x03 \x03(\t\x12\x17\n\nfallbackLg\x18\x04 \x01(\tH\x02\x88\x01\x01\x12\x17\n\nfallbackLc\x18\x05 \x01(\tH\x03\x88\x01\x01\x12Y\n\x11localizableParams\x18\x06 \x03(\x0b\x32>.proto.Message.HighlyStructuredMessage.HSMLocalizableParameter\x12\x1c\n\x0f\x64\x65terministicLg\x18\x07 \x01(\tH\x04\x88\x01\x01\x12\x1c\n\x0f\x64\x65terministicLc\x18\x08 \x01(\tH\x05\x88\x01\x01\x12\x38\n\x0bhydratedHsm\x18\t \x01(\x0b\x32\x1e.proto.Message.TemplateMessageH\x06\x88\x01\x01\x1a\xf6\n\n\x17HSMLocalizableParameter\x12\x14\n\x07\x64\x65\x66\x61ult\x18\x01 \x01(\tH\x01\x88\x01\x01\x12^\n\x08\x63urrency\x18\x02 \x01(\x0b\x32J.proto.Message.HighlyStructuredMessage.HSMLocalizableParameter.HSMCurrencyH\x00\x12^\n\x08\x64\x61teTime\x18\x03 \x01(\x0b\x32J.proto.Message.HighlyStructuredMessage.HSMLocalizableParameter.HSMDateTimeH\x00\x1a\x61\n\x0bHSMCurrency\x12\x19\n\x0c\x63urrencyCode\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x17\n\namount1000\x18\x02 \x01(\x03H\x01\x88\x01\x01\x42\x0f\n\r_currencyCodeB\r\n\x0b_amount1000\x1a\x87\x08\n\x0bHSMDateTime\x12t\n\tcomponent\x18\x01 \x01(\x0b\x32_.proto.Message.HighlyStructuredMessage.HSMLocalizableParameter.HSMDateTime.HSMDateTimeComponentH\x00\x12t\n\tunixEpoch\x18\x02 \x01(\x0b\x32_.proto.Message.HighlyStructuredMessage.HSMLocalizableParameter.HSMDateTime.HSMDateTimeUnixEpochH\x00\x1a\xbc\x05\n\x14HSMDateTimeComponent\x12\x85\x01\n\tdayOfWeek\x18\x01 \x01(\x0e\x32m.proto.Message.HighlyStructuredMessage.HSMLocalizableParameter.HSMDateTime.HSMDateTimeComponent.DayOfWeekTypeH\x00\x88\x01\x01\x12\x11\n\x04year\x18\x02 \x01(\rH\x01\x88\x01\x01\x12\x12\n\x05month\x18\x03 \x01(\rH\x02\x88\x01\x01\x12\x17\n\ndayOfMonth\x18\x04 \x01(\rH\x03\x88\x01\x01\x12\x11\n\x04hour\x18\x05 \x01(\rH\x04\x88\x01\x01\x12\x13\n\x06minute\x18\x06 \x01(\rH\x05\x88\x01\x01\x12\x83\x01\n\x08\x63\x61lendar\x18\x07 \x01(\x0e\x32l.proto.Message.HighlyStructuredMessage.HSMLocalizableParameter.HSMDateTime.HSMDateTimeComponent.CalendarTypeH\x06\x88\x01\x01\"M\n\x0c\x43\x61lendarType\x12\x1d\n\x19\x43\x41LENDAR_TYPE_UNSPECIFIED\x10\x00\x12\r\n\tGREGORIAN\x10\x01\x12\x0f\n\x0bSOLAR_HIJRI\x10\x02\"\x8d\x01\n\rDayOfWeekType\x12 \n\x1c\x44\x41Y_OF_WEEK_TYPE_UNSPECIFIED\x10\x00\x12\n\n\x06MONDAY\x10\x01\x12\x0b\n\x07TUESDAY\x10\x02\x12\r\n\tWEDNESDAY\x10\x03\x12\x0c\n\x08THURSDAY\x10\x04\x12\n\n\x06\x46RIDAY\x10\x05\x12\x0c\n\x08SATURDAY\x10\x06\x12\n\n\x06SUNDAY\x10\x07\x42\x0c\n\n_dayOfWeekB\x07\n\x05_yearB\x08\n\x06_monthB\r\n\x0b_dayOfMonthB\x07\n\x05_hourB\t\n\x07_minuteB\x0b\n\t_calendar\x1a<\n\x14HSMDateTimeUnixEpoch\x12\x16\n\ttimestamp\x18\x01 \x01(\x03H\x00\x88\x01\x01\x42\x0c\n\n_timestampB\x0f\n\rdatetimeOneofB\x0c\n\nparamOneofB\n\n\x08_defaultB\x0c\n\n_namespaceB\x0e\n\x0c_elementNameB\r\n\x0b_fallbackLgB\r\n\x0b_fallbackLcB\x12\n\x10_deterministicLgB\x12\n\x10_deterministicLcB\x0e\n\x0c_hydratedHsm\x1a^\n\x1eHistorySyncMessageAccessStatus\x12\"\n\x15\x63ompleteAccessGranted\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\x18\n\x16_completeAccessGranted\x1a\xb5\x07\n\x17HistorySyncNotification\x12\x17\n\nfileSha256\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x17\n\nfileLength\x18\x02 \x01(\x04H\x01\x88\x01\x01\x12\x15\n\x08mediaKey\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x1a\n\rfileEncSha256\x18\x04 \x01(\x0cH\x03\x88\x01\x01\x12\x17\n\ndirectPath\x18\x05 \x01(\tH\x04\x88\x01\x01\x12\x35\n\x08syncType\x18\x06 \x01(\x0e\x32\x1e.proto.Message.HistorySyncTypeH\x05\x88\x01\x01\x12\x17\n\nchunkOrder\x18\x07 \x01(\rH\x06\x88\x01\x01\x12\x1e\n\x11originalMessageId\x18\x08 \x01(\tH\x07\x88\x01\x01\x12\x15\n\x08progress\x18\t \x01(\rH\x08\x88\x01\x01\x12)\n\x1coldestMsgInChunkTimestampSec\x18\n \x01(\x03H\t\x88\x01\x01\x12.\n!initialHistBootstrapInlinePayload\x18\x0b \x01(\x0cH\n\x88\x01\x01\x12%\n\x18peerDataRequestSessionId\x18\x0c \x01(\tH\x0b\x88\x01\x01\x12j\n&fullHistorySyncOnDemandRequestMetadata\x18\r \x01(\x0b\x32\x35.proto.Message.FullHistorySyncOnDemandRequestMetadataH\x0c\x88\x01\x01\x12\x16\n\tencHandle\x18\x0e \x01(\tH\r\x88\x01\x01\x12O\n\x13messageAccessStatus\x18\x0f \x01(\x0b\x32-.proto.Message.HistorySyncMessageAccessStatusH\x0e\x88\x01\x01\x42\r\n\x0b_fileSha256B\r\n\x0b_fileLengthB\x0b\n\t_mediaKeyB\x10\n\x0e_fileEncSha256B\r\n\x0b_directPathB\x0b\n\t_syncTypeB\r\n\x0b_chunkOrderB\x14\n\x12_originalMessageIdB\x0b\n\t_progressB\x1f\n\x1d_oldestMsgInChunkTimestampSecB$\n\"_initialHistBootstrapInlinePayloadB\x1b\n\x19_peerDataRequestSessionIdB)\n\'_fullHistorySyncOnDemandRequestMetadataB\x0c\n\n_encHandleB\x16\n\x14_messageAccessStatus\x1a\xb8\x0c\n\x0cImageMessage\x12\x10\n\x03url\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x15\n\x08mimetype\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x14\n\x07\x63\x61ption\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x17\n\nfileSha256\x18\x04 \x01(\x0cH\x03\x88\x01\x01\x12\x17\n\nfileLength\x18\x05 \x01(\x04H\x04\x88\x01\x01\x12\x13\n\x06height\x18\x06 \x01(\rH\x05\x88\x01\x01\x12\x12\n\x05width\x18\x07 \x01(\rH\x06\x88\x01\x01\x12\x15\n\x08mediaKey\x18\x08 \x01(\x0cH\x07\x88\x01\x01\x12\x1a\n\rfileEncSha256\x18\t \x01(\x0cH\x08\x88\x01\x01\x12<\n\x16interactiveAnnotations\x18\n \x03(\x0b\x32\x1c.proto.InteractiveAnnotation\x12\x17\n\ndirectPath\x18\x0b \x01(\tH\t\x88\x01\x01\x12\x1e\n\x11mediaKeyTimestamp\x18\x0c \x01(\x03H\n\x88\x01\x01\x12\x1a\n\rjpegThumbnail\x18\x10 \x01(\x0cH\x0b\x88\x01\x01\x12,\n\x0b\x63ontextInfo\x18\x11 \x01(\x0b\x32\x12.proto.ContextInfoH\x0c\x88\x01\x01\x12\x1d\n\x10\x66irstScanSidecar\x18\x12 \x01(\x0cH\r\x88\x01\x01\x12\x1c\n\x0f\x66irstScanLength\x18\x13 \x01(\rH\x0e\x88\x01\x01\x12\x1e\n\x11\x65xperimentGroupId\x18\x14 \x01(\rH\x0f\x88\x01\x01\x12\x19\n\x0cscansSidecar\x18\x15 \x01(\x0cH\x10\x88\x01\x01\x12\x13\n\x0bscanLengths\x18\x16 \x03(\r\x12!\n\x14midQualityFileSha256\x18\x17 \x01(\x0cH\x11\x88\x01\x01\x12$\n\x17midQualityFileEncSha256\x18\x18 \x01(\x0cH\x12\x88\x01\x01\x12\x15\n\x08viewOnce\x18\x19 \x01(\x08H\x13\x88\x01\x01\x12 \n\x13thumbnailDirectPath\x18\x1a \x01(\tH\x14\x88\x01\x01\x12\x1c\n\x0fthumbnailSha256\x18\x1b \x01(\x0cH\x15\x88\x01\x01\x12\x1f\n\x12thumbnailEncSha256\x18\x1c \x01(\x0cH\x16\x88\x01\x01\x12\x16\n\tstaticUrl\x18\x1d \x01(\tH\x17\x88\x01\x01\x12\x31\n\x0b\x61nnotations\x18\x1e \x03(\x0b\x32\x1c.proto.InteractiveAnnotation\x12I\n\x0fimageSourceType\x18\x1f \x01(\x0e\x32+.proto.Message.ImageMessage.ImageSourceTypeH\x18\x88\x01\x01\x12\x1f\n\x12\x61\x63\x63\x65ssibilityLabel\x18  \x01(\tH\x19\x88\x01\x01\x12:\n\x0emediaKeyDomain\x18! \x01(\x0e\x32\x1d.proto.Message.MediaKeyDomainH\x1a\x88\x01\x01\x12\x12\n\x05qrUrl\x18\" \x01(\tH\x1b\x88\x01\x01\"`\n\x0fImageSourceType\x12\x0e\n\nUSER_IMAGE\x10\x00\x12\x10\n\x0c\x41I_GENERATED\x10\x01\x12\x0f\n\x0b\x41I_MODIFIED\x10\x02\x12\x1a\n\x16RASTERIZED_TEXT_STATUS\x10\x03\x42\x06\n\x04_urlB\x0b\n\t_mimetypeB\n\n\x08_captionB\r\n\x0b_fileSha256B\r\n\x0b_fileLengthB\t\n\x07_heightB\x08\n\x06_widthB\x0b\n\t_mediaKeyB\x10\n\x0e_fileEncSha256B\r\n\x0b_directPathB\x14\n\x12_mediaKeyTimestampB\x10\n\x0e_jpegThumbnailB\x0e\n\x0c_contextInfoB\x13\n\x11_firstScanSidecarB\x12\n\x10_firstScanLengthB\x14\n\x12_experimentGroupIdB\x0f\n\r_scansSidecarB\x17\n\x15_midQualityFileSha256B\x1a\n\x18_midQualityFileEncSha256B\x0b\n\t_viewOnceB\x16\n\x14_thumbnailDirectPathB\x12\n\x10_thumbnailSha256B\x15\n\x13_thumbnailEncSha256B\x0c\n\n_staticUrlB\x12\n\x10_imageSourceTypeB\x15\n\x13_accessibilityLabelB\x11\n\x0f_mediaKeyDomainB\x08\n\x06_qrUrl\x1ar\n&InitialSecurityNotificationSettingSync\x12(\n\x1bsecurityNotificationEnabled\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\x1e\n\x1c_securityNotificationEnabled\x1a\x9a\x12\n\x12InteractiveMessage\x12=\n\x06header\x18\x01 \x01(\x0b\x32(.proto.Message.InteractiveMessage.HeaderH\x01\x88\x01\x01\x12\x39\n\x04\x62ody\x18\x02 \x01(\x0b\x32&.proto.Message.InteractiveMessage.BodyH\x02\x88\x01\x01\x12=\n\x06\x66ooter\x18\x03 \x01(\x0b\x32(.proto.Message.InteractiveMessage.FooterH\x03\x88\x01\x01\x12,\n\x0b\x63ontextInfo\x18\x0f \x01(\x0b\x32\x12.proto.ContextInfoH\x04\x88\x01\x01\x12\x32\n\x0eurlTrackingMap\x18\x10 \x01(\x0b\x32\x15.proto.UrlTrackingMapH\x05\x88\x01\x01\x12N\n\x15shopStorefrontMessage\x18\x04 \x01(\x0b\x32-.proto.Message.InteractiveMessage.ShopMessageH\x00\x12P\n\x11\x63ollectionMessage\x18\x05 \x01(\x0b\x32\x33.proto.Message.InteractiveMessage.CollectionMessageH\x00\x12P\n\x11nativeFlowMessage\x18\x06 \x01(\x0b\x32\x33.proto.Message.InteractiveMessage.NativeFlowMessageH\x00\x12L\n\x0f\x63\x61rouselMessage\x18\x07 \x01(\x0b\x32\x31.proto.Message.InteractiveMessage.CarouselMessageH\x00\x1a\"\n\x04\x42ody\x12\x11\n\x04text\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x07\n\x05_text\x1a\xb0\x02\n\x0f\x43\x61rouselMessage\x12\x30\n\x05\x63\x61rds\x18\x01 \x03(\x0b\x32!.proto.Message.InteractiveMessage\x12\x1b\n\x0emessageVersion\x18\x02 \x01(\x05H\x00\x88\x01\x01\x12\x61\n\x10\x63\x61rouselCardType\x18\x03 \x01(\x0e\x32\x42.proto.Message.InteractiveMessage.CarouselMessage.CarouselCardTypeH\x01\x88\x01\x01\"C\n\x10\x43\x61rouselCardType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x11\n\rHSCROLL_CARDS\x10\x01\x12\x0f\n\x0b\x41LBUM_IMAGE\x10\x02\x42\x11\n\x0f_messageVersionB\x13\n\x11_carouselCardType\x1a{\n\x11\x43ollectionMessage\x12\x13\n\x06\x62izJid\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x0f\n\x02id\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x1b\n\x0emessageVersion\x18\x03 \x01(\x05H\x02\x88\x01\x01\x42\t\n\x07_bizJidB\x05\n\x03_idB\x11\n\x0f_messageVersion\x1a\x9a\x01\n\x06\x46ooter\x12\x11\n\x04text\x18\x01 \x01(\tH\x01\x88\x01\x01\x12\x1f\n\x12hasMediaAttachment\x18\x03 \x01(\x08H\x02\x88\x01\x01\x12\x33\n\x0c\x61udioMessage\x18\x02 \x01(\x0b\x32\x1b.proto.Message.AudioMessageH\x00\x42\x07\n\x05mediaB\x07\n\x05_textB\x15\n\x13_hasMediaAttachment\x1a\xbd\x03\n\x06Header\x12\x12\n\x05title\x18\x01 \x01(\tH\x01\x88\x01\x01\x12\x15\n\x08subtitle\x18\x02 \x01(\tH\x02\x88\x01\x01\x12\x1f\n\x12hasMediaAttachment\x18\x05 \x01(\x08H\x03\x88\x01\x01\x12\x39\n\x0f\x64ocumentMessage\x18\x03 \x01(\x0b\x32\x1e.proto.Message.DocumentMessageH\x00\x12\x33\n\x0cimageMessage\x18\x04 \x01(\x0b\x32\x1b.proto.Message.ImageMessageH\x00\x12\x17\n\rjpegThumbnail\x18\x06 \x01(\x0cH\x00\x12\x33\n\x0cvideoMessage\x18\x07 \x01(\x0b\x32\x1b.proto.Message.VideoMessageH\x00\x12\x39\n\x0flocationMessage\x18\x08 \x01(\x0b\x32\x1e.proto.Message.LocationMessageH\x00\x12\x37\n\x0eproductMessage\x18\t \x01(\x0b\x32\x1d.proto.Message.ProductMessageH\x00\x42\x07\n\x05mediaB\x08\n\x06_titleB\x0b\n\t_subtitleB\x15\n\x13_hasMediaAttachment\x1a\xb4\x02\n\x11NativeFlowMessage\x12U\n\x07\x62uttons\x18\x01 \x03(\x0b\x32\x44.proto.Message.InteractiveMessage.NativeFlowMessage.NativeFlowButton\x12\x1e\n\x11messageParamsJson\x18\x02 \x01(\tH\x00\x88\x01\x01\x12\x1b\n\x0emessageVersion\x18\x03 \x01(\x05H\x01\x88\x01\x01\x1a\x62\n\x10NativeFlowButton\x12\x11\n\x04name\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x1d\n\x10\x62uttonParamsJson\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x07\n\x05_nameB\x13\n\x11_buttonParamsJsonB\x14\n\x12_messageParamsJsonB\x11\n\x0f_messageVersion\x1a\xe6\x01\n\x0bShopMessage\x12\x0f\n\x02id\x18\x01 \x01(\tH\x00\x88\x01\x01\x12K\n\x07surface\x18\x02 \x01(\x0e\x32\x35.proto.Message.InteractiveMessage.ShopMessage.SurfaceH\x01\x88\x01\x01\x12\x1b\n\x0emessageVersion\x18\x03 \x01(\x05H\x02\x88\x01\x01\"6\n\x07Surface\x12\x13\n\x0fUNKNOWN_SURFACE\x10\x00\x12\x06\n\x02\x46\x42\x10\x01\x12\x06\n\x02IG\x10\x02\x12\x06\n\x02WA\x10\x03\x42\x05\n\x03_idB\n\n\x08_surfaceB\x11\n\x0f_messageVersionB\x14\n\x12interactiveMessageB\t\n\x07_headerB\x07\n\x05_bodyB\t\n\x07_footerB\x0e\n\x0c_contextInfoB\x11\n\x0f_urlTrackingMap\x1a\xd7\x04\n\x1aInteractiveResponseMessage\x12\x41\n\x04\x62ody\x18\x01 \x01(\x0b\x32..proto.Message.InteractiveResponseMessage.BodyH\x01\x88\x01\x01\x12,\n\x0b\x63ontextInfo\x18\x0f \x01(\x0b\x32\x12.proto.ContextInfoH\x02\x88\x01\x01\x12h\n\x19nativeFlowResponseMessage\x18\x02 \x01(\x0b\x32\x43.proto.Message.InteractiveResponseMessage.NativeFlowResponseMessageH\x00\x1a\xa2\x01\n\x04\x42ody\x12\x11\n\x04text\x18\x01 \x01(\tH\x00\x88\x01\x01\x12J\n\x06\x66ormat\x18\x02 \x01(\x0e\x32\x35.proto.Message.InteractiveResponseMessage.Body.FormatH\x01\x88\x01\x01\"\'\n\x06\x46ormat\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x00\x12\x10\n\x0c\x45XTENSIONS_1\x10\x01\x42\x07\n\x05_textB\t\n\x07_format\x1a\x81\x01\n\x19NativeFlowResponseMessage\x12\x11\n\x04name\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x17\n\nparamsJson\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x14\n\x07version\x18\x03 \x01(\x05H\x02\x88\x01\x01\x42\x07\n\x05_nameB\r\n\x0b_paramsJsonB\n\n\x08_versionB\x1c\n\x1ainteractiveResponseMessageB\x07\n\x05_bodyB\x0e\n\x0c_contextInfo\x1a\x84\x05\n\x0eInvoiceMessage\x12\x11\n\x04note\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x12\n\x05token\x18\x02 \x01(\tH\x01\x88\x01\x01\x12I\n\x0e\x61ttachmentType\x18\x03 \x01(\x0e\x32,.proto.Message.InvoiceMessage.AttachmentTypeH\x02\x88\x01\x01\x12\x1f\n\x12\x61ttachmentMimetype\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x1f\n\x12\x61ttachmentMediaKey\x18\x05 \x01(\x0cH\x04\x88\x01\x01\x12(\n\x1b\x61ttachmentMediaKeyTimestamp\x18\x06 \x01(\x03H\x05\x88\x01\x01\x12!\n\x14\x61ttachmentFileSha256\x18\x07 \x01(\x0cH\x06\x88\x01\x01\x12$\n\x17\x61ttachmentFileEncSha256\x18\x08 \x01(\x0cH\x07\x88\x01\x01\x12!\n\x14\x61ttachmentDirectPath\x18\t \x01(\tH\x08\x88\x01\x01\x12$\n\x17\x61ttachmentJpegThumbnail\x18\n \x01(\x0cH\t\x88\x01\x01\"$\n\x0e\x41ttachmentType\x12\t\n\x05IMAGE\x10\x00\x12\x07\n\x03PDF\x10\x01\x42\x07\n\x05_noteB\x08\n\x06_tokenB\x11\n\x0f_attachmentTypeB\x15\n\x13_attachmentMimetypeB\x15\n\x13_attachmentMediaKeyB\x1e\n\x1c_attachmentMediaKeyTimestampB\x17\n\x15_attachmentFileSha256B\x1a\n\x18_attachmentFileEncSha256B\x17\n\x15_attachmentDirectPathB\x1a\n\x18_attachmentJpegThumbnail\x1a\x9f\x01\n\x11KeepInChatMessage\x12#\n\x03key\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x12&\n\x08keepType\x18\x02 \x01(\x0e\x32\x0f.proto.KeepTypeH\x01\x88\x01\x01\x12\x18\n\x0btimestampMs\x18\x03 \x01(\x03H\x02\x88\x01\x01\x42\x06\n\x04_keyB\x0b\n\t_keepTypeB\x0e\n\x0c_timestampMs\x1a\xe8\x05\n\x13LinkPreviewMetadata\x12\x44\n\x13paymentLinkMetadata\x18\x01 \x01(\x0b\x32\".proto.Message.PaymentLinkMetadataH\x00\x88\x01\x01\x12\x34\n\x0burlMetadata\x18\x02 \x01(\x0b\x32\x1a.proto.Message.URLMetadataH\x01\x88\x01\x01\x12\x1b\n\x0e\x66\x62\x45xperimentId\x18\x03 \x01(\rH\x02\x88\x01\x01\x12\x1e\n\x11linkMediaDuration\x18\x04 \x01(\rH\x03\x88\x01\x01\x12X\n\x13socialMediaPostType\x18\x05 \x01(\x0e\x32\x36.proto.Message.LinkPreviewMetadata.SocialMediaPostTypeH\x04\x88\x01\x01\x12!\n\x14linkInlineVideoMuted\x18\x06 \x01(\x08H\x05\x88\x01\x01\x12\x1c\n\x0fvideoContentUrl\x18\x07 \x01(\tH\x06\x88\x01\x01\x12\x30\n\rmusicMetadata\x18\x08 \x01(\x0b\x32\x14.proto.EmbeddedMusicH\x07\x88\x01\x01\x12 \n\x13videoContentCaption\x18\t \x01(\tH\x08\x88\x01\x01\"i\n\x13SocialMediaPostType\x12\x08\n\x04NONE\x10\x00\x12\x08\n\x04REEL\x10\x01\x12\x0e\n\nLIVE_VIDEO\x10\x02\x12\x0e\n\nLONG_VIDEO\x10\x03\x12\x10\n\x0cSINGLE_IMAGE\x10\x04\x12\x0c\n\x08\x43\x41ROUSEL\x10\x05\x42\x16\n\x14_paymentLinkMetadataB\x0e\n\x0c_urlMetadataB\x11\n\x0f_fbExperimentIdB\x14\n\x12_linkMediaDurationB\x16\n\x14_socialMediaPostTypeB\x17\n\x15_linkInlineVideoMutedB\x12\n\x10_videoContentUrlB\x10\n\x0e_musicMetadataB\x16\n\x14_videoContentCaption\x1a\xb0\t\n\x0bListMessage\x12\x12\n\x05title\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x18\n\x0b\x64\x65scription\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x17\n\nbuttonText\x18\x03 \x01(\tH\x02\x88\x01\x01\x12:\n\x08listType\x18\x04 \x01(\x0e\x32#.proto.Message.ListMessage.ListTypeH\x03\x88\x01\x01\x12\x34\n\x08sections\x18\x05 \x03(\x0b\x32\".proto.Message.ListMessage.Section\x12H\n\x0fproductListInfo\x18\x06 \x01(\x0b\x32*.proto.Message.ListMessage.ProductListInfoH\x04\x88\x01\x01\x12\x17\n\nfooterText\x18\x07 \x01(\tH\x05\x88\x01\x01\x12,\n\x0b\x63ontextInfo\x18\x08 \x01(\x0b\x32\x12.proto.ContextInfoH\x06\x88\x01\x01\x1a/\n\x07Product\x12\x16\n\tproductId\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x0c\n\n_productId\x1al\n\x16ProductListHeaderImage\x12\x16\n\tproductId\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x1a\n\rjpegThumbnail\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x42\x0c\n\n_productIdB\x10\n\x0e_jpegThumbnail\x1a\xe6\x01\n\x0fProductListInfo\x12\x42\n\x0fproductSections\x18\x01 \x03(\x0b\x32).proto.Message.ListMessage.ProductSection\x12K\n\x0bheaderImage\x18\x02 \x01(\x0b\x32\x31.proto.Message.ListMessage.ProductListHeaderImageH\x00\x88\x01\x01\x12\x1d\n\x10\x62usinessOwnerJid\x18\x03 \x01(\tH\x01\x88\x01\x01\x42\x0e\n\x0c_headerImageB\x13\n\x11_businessOwnerJid\x1a\x64\n\x0eProductSection\x12\x12\n\x05title\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x34\n\x08products\x18\x02 \x03(\x0b\x32\".proto.Message.ListMessage.ProductB\x08\n\x06_title\x1ak\n\x03Row\x12\x12\n\x05title\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x18\n\x0b\x64\x65scription\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x12\n\x05rowId\x18\x03 \x01(\tH\x02\x88\x01\x01\x42\x08\n\x06_titleB\x0e\n\x0c_descriptionB\x08\n\x06_rowId\x1aU\n\x07Section\x12\x12\n\x05title\x18\x01 \x01(\tH\x00\x88\x01\x01\x12,\n\x04rows\x18\x02 \x03(\x0b\x32\x1e.proto.Message.ListMessage.RowB\x08\n\x06_title\"<\n\x08ListType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x11\n\rSINGLE_SELECT\x10\x01\x12\x10\n\x0cPRODUCT_LIST\x10\x02\x42\x08\n\x06_titleB\x0e\n\x0c_descriptionB\r\n\x0b_buttonTextB\x0b\n\t_listTypeB\x12\n\x10_productListInfoB\r\n\x0b_footerTextB\x0e\n\x0c_contextInfo\x1a\xc7\x03\n\x13ListResponseMessage\x12\x12\n\x05title\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x42\n\x08listType\x18\x02 \x01(\x0e\x32+.proto.Message.ListResponseMessage.ListTypeH\x01\x88\x01\x01\x12T\n\x11singleSelectReply\x18\x03 \x01(\x0b\x32\x34.proto.Message.ListResponseMessage.SingleSelectReplyH\x02\x88\x01\x01\x12,\n\x0b\x63ontextInfo\x18\x04 \x01(\x0b\x32\x12.proto.ContextInfoH\x03\x88\x01\x01\x12\x18\n\x0b\x64\x65scription\x18\x05 \x01(\tH\x04\x88\x01\x01\x1a\x41\n\x11SingleSelectReply\x12\x1a\n\rselectedRowId\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x10\n\x0e_selectedRowId\"*\n\x08ListType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x11\n\rSINGLE_SELECT\x10\x01\x42\x08\n\x06_titleB\x0b\n\t_listTypeB\x14\n\x12_singleSelectReplyB\x0e\n\x0c_contextInfoB\x0e\n\x0c_description\x1a\x93\x04\n\x13LiveLocationMessage\x12\x1c\n\x0f\x64\x65greesLatitude\x18\x01 \x01(\x01H\x00\x88\x01\x01\x12\x1d\n\x10\x64\x65greesLongitude\x18\x02 \x01(\x01H\x01\x88\x01\x01\x12\x1d\n\x10\x61\x63\x63uracyInMeters\x18\x03 \x01(\rH\x02\x88\x01\x01\x12\x17\n\nspeedInMps\x18\x04 \x01(\x02H\x03\x88\x01\x01\x12.\n!degreesClockwiseFromMagneticNorth\x18\x05 \x01(\rH\x04\x88\x01\x01\x12\x14\n\x07\x63\x61ption\x18\x06 \x01(\tH\x05\x88\x01\x01\x12\x1b\n\x0esequenceNumber\x18\x07 \x01(\x03H\x06\x88\x01\x01\x12\x17\n\ntimeOffset\x18\x08 \x01(\rH\x07\x88\x01\x01\x12\x1a\n\rjpegThumbnail\x18\x10 \x01(\x0cH\x08\x88\x01\x01\x12,\n\x0b\x63ontextInfo\x18\x11 \x01(\x0b\x32\x12.proto.ContextInfoH\t\x88\x01\x01\x42\x12\n\x10_degreesLatitudeB\x13\n\x11_degreesLongitudeB\x13\n\x11_accuracyInMetersB\r\n\x0b_speedInMpsB$\n\"_degreesClockwiseFromMagneticNorthB\n\n\x08_captionB\x11\n\x0f_sequenceNumberB\r\n\x0b_timeOffsetB\x10\n\x0e_jpegThumbnailB\x0e\n\x0c_contextInfo\x1a\xaf\x04\n\x0fLocationMessage\x12\x1c\n\x0f\x64\x65greesLatitude\x18\x01 \x01(\x01H\x00\x88\x01\x01\x12\x1d\n\x10\x64\x65greesLongitude\x18\x02 \x01(\x01H\x01\x88\x01\x01\x12\x11\n\x04name\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x14\n\x07\x61\x64\x64ress\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x10\n\x03url\x18\x05 \x01(\tH\x04\x88\x01\x01\x12\x13\n\x06isLive\x18\x06 \x01(\x08H\x05\x88\x01\x01\x12\x1d\n\x10\x61\x63\x63uracyInMeters\x18\x07 \x01(\rH\x06\x88\x01\x01\x12\x17\n\nspeedInMps\x18\x08 \x01(\x02H\x07\x88\x01\x01\x12.\n!degreesClockwiseFromMagneticNorth\x18\t \x01(\rH\x08\x88\x01\x01\x12\x14\n\x07\x63omment\x18\x0b \x01(\tH\t\x88\x01\x01\x12\x1a\n\rjpegThumbnail\x18\x10 \x01(\x0cH\n\x88\x01\x01\x12,\n\x0b\x63ontextInfo\x18\x11 \x01(\x0b\x32\x12.proto.ContextInfoH\x0b\x88\x01\x01\x42\x12\n\x10_degreesLatitudeB\x13\n\x11_degreesLongitudeB\x07\n\x05_nameB\n\n\x08_addressB\x06\n\x04_urlB\t\n\x07_isLiveB\x13\n\x11_accuracyInMetersB\r\n\x0b_speedInMpsB$\n\"_degreesClockwiseFromMagneticNorthB\n\n\x08_commentB\x10\n\x0e_jpegThumbnailB\x0e\n\x0c_contextInfo\x1a\xc5\x03\n\x14MMSThumbnailMetadata\x12 \n\x13thumbnailDirectPath\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x1c\n\x0fthumbnailSha256\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x1f\n\x12thumbnailEncSha256\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x15\n\x08mediaKey\x18\x04 \x01(\x0cH\x03\x88\x01\x01\x12\x1e\n\x11mediaKeyTimestamp\x18\x05 \x01(\x03H\x04\x88\x01\x01\x12\x1c\n\x0fthumbnailHeight\x18\x06 \x01(\rH\x05\x88\x01\x01\x12\x1b\n\x0ethumbnailWidth\x18\x07 \x01(\rH\x06\x88\x01\x01\x12:\n\x0emediaKeyDomain\x18\x08 \x01(\x0e\x32\x1d.proto.Message.MediaKeyDomainH\x07\x88\x01\x01\x42\x16\n\x14_thumbnailDirectPathB\x12\n\x10_thumbnailSha256B\x15\n\x13_thumbnailEncSha256B\x0b\n\t_mediaKeyB\x14\n\x12_mediaKeyTimestampB\x12\n\x10_thumbnailHeightB\x11\n\x0f_thumbnailWidthB\x11\n\x0f_mediaKeyDomain\x1a\xb7\x03\n\x14MessageHistoryBundle\x12\x15\n\x08mimetype\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x17\n\nfileSha256\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x15\n\x08mediaKey\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x1a\n\rfileEncSha256\x18\x04 \x01(\x0cH\x03\x88\x01\x01\x12\x17\n\ndirectPath\x18\x05 \x01(\tH\x04\x88\x01\x01\x12\x1e\n\x11mediaKeyTimestamp\x18\x06 \x01(\x03H\x05\x88\x01\x01\x12,\n\x0b\x63ontextInfo\x18\x07 \x01(\x0b\x32\x12.proto.ContextInfoH\x06\x88\x01\x01\x12J\n\x16messageHistoryMetadata\x18\x08 \x01(\x0b\x32%.proto.Message.MessageHistoryMetadataH\x07\x88\x01\x01\x42\x0b\n\t_mimetypeB\r\n\x0b_fileSha256B\x0b\n\t_mediaKeyB\x10\n\x0e_fileEncSha256B\r\n\x0b_directPathB\x14\n\x12_mediaKeyTimestampB\x0e\n\x0c_contextInfoB\x19\n\x17_messageHistoryMetadata\x1a\x9e\x01\n\x16MessageHistoryMetadata\x12\x18\n\x10historyReceivers\x18\x01 \x03(\t\x12#\n\x16oldestMessageTimestamp\x18\x02 \x01(\x03H\x00\x88\x01\x01\x12\x19\n\x0cmessageCount\x18\x03 \x01(\x03H\x01\x88\x01\x01\x42\x19\n\x17_oldestMessageTimestampB\x0f\n\r_messageCount\x1a\xbb\x01\n\x14MessageHistoryNotice\x12,\n\x0b\x63ontextInfo\x18\x01 \x01(\x0b\x32\x12.proto.ContextInfoH\x00\x88\x01\x01\x12J\n\x16messageHistoryMetadata\x18\x02 \x01(\x0b\x32%.proto.Message.MessageHistoryMetadataH\x01\x88\x01\x01\x42\x0e\n\x0c_contextInfoB\x19\n\x17_messageHistoryMetadata\x1a\xbe\x02\n\x1cNewsletterAdminInviteMessage\x12\x1a\n\rnewsletterJid\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x1b\n\x0enewsletterName\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x1a\n\rjpegThumbnail\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x14\n\x07\x63\x61ption\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x1d\n\x10inviteExpiration\x18\x05 \x01(\x03H\x04\x88\x01\x01\x12,\n\x0b\x63ontextInfo\x18\x06 \x01(\x0b\x32\x12.proto.ContextInfoH\x05\x88\x01\x01\x42\x10\n\x0e_newsletterJidB\x11\n\x0f_newsletterNameB\x10\n\x0e_jpegThumbnailB\n\n\x08_captionB\x13\n\x11_inviteExpirationB\x0e\n\x0c_contextInfo\x1a\x8d\x02\n\x1fNewsletterFollowerInviteMessage\x12\x1a\n\rnewsletterJid\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x1b\n\x0enewsletterName\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x1a\n\rjpegThumbnail\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x14\n\x07\x63\x61ption\x18\x04 \x01(\tH\x03\x88\x01\x01\x12,\n\x0b\x63ontextInfo\x18\x05 \x01(\x0b\x32\x12.proto.ContextInfoH\x04\x88\x01\x01\x42\x10\n\x0e_newsletterJidB\x11\n\x0f_newsletterNameB\x10\n\x0e_jpegThumbnailB\n\n\x08_captionB\x0e\n\x0c_contextInfo\x1a\x82\x07\n\x0cOrderMessage\x12\x14\n\x07orderId\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x16\n\tthumbnail\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x16\n\titemCount\x18\x03 \x01(\x05H\x02\x88\x01\x01\x12<\n\x06status\x18\x04 \x01(\x0e\x32\'.proto.Message.OrderMessage.OrderStatusH\x03\x88\x01\x01\x12>\n\x07surface\x18\x05 \x01(\x0e\x32(.proto.Message.OrderMessage.OrderSurfaceH\x04\x88\x01\x01\x12\x14\n\x07message\x18\x06 \x01(\tH\x05\x88\x01\x01\x12\x17\n\norderTitle\x18\x07 \x01(\tH\x06\x88\x01\x01\x12\x16\n\tsellerJid\x18\x08 \x01(\tH\x07\x88\x01\x01\x12\x12\n\x05token\x18\t \x01(\tH\x08\x88\x01\x01\x12\x1c\n\x0ftotalAmount1000\x18\n \x01(\x03H\t\x88\x01\x01\x12\x1e\n\x11totalCurrencyCode\x18\x0b \x01(\tH\n\x88\x01\x01\x12,\n\x0b\x63ontextInfo\x18\x11 \x01(\x0b\x32\x12.proto.ContextInfoH\x0b\x88\x01\x01\x12\x1b\n\x0emessageVersion\x18\x0c \x01(\x05H\x0c\x88\x01\x01\x12\x35\n\x15orderRequestMessageId\x18\r \x01(\x0b\x32\x11.proto.MessageKeyH\r\x88\x01\x01\x12\x18\n\x0b\x63\x61talogType\x18\x0f \x01(\tH\x0e\x88\x01\x01\"T\n\x0bOrderStatus\x12\x1c\n\x18ORDER_STATUS_UNSPECIFIED\x10\x00\x12\x0b\n\x07INQUIRY\x10\x01\x12\x0c\n\x08\x41\x43\x43\x45PTED\x10\x02\x12\x0c\n\x08\x44\x45\x43LINED\x10\x03\":\n\x0cOrderSurface\x12\x1d\n\x19ORDER_SURFACE_UNSPECIFIED\x10\x00\x12\x0b\n\x07\x43\x41TALOG\x10\x01\x42\n\n\x08_orderIdB\x0c\n\n_thumbnailB\x0c\n\n_itemCountB\t\n\x07_statusB\n\n\x08_surfaceB\n\n\x08_messageB\r\n\x0b_orderTitleB\x0c\n\n_sellerJidB\x08\n\x06_tokenB\x12\n\x10_totalAmount1000B\x14\n\x12_totalCurrencyCodeB\x0e\n\x0c_contextInfoB\x11\n\x0f_messageVersionB\x18\n\x16_orderRequestMessageIdB\x0e\n\x0c_catalogType\x1a\x8f\x01\n\x17PaymentExtendedMetadata\x12\x11\n\x04type\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x15\n\x08platform\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x1e\n\x11messageParamsJson\x18\x03 \x01(\tH\x02\x88\x01\x01\x42\x07\n\x05_typeB\x0b\n\t_platformB\x14\n\x12_messageParamsJson\x1a\xdd\x01\n\x14PaymentInviteMessage\x12I\n\x0bserviceType\x18\x01 \x01(\x0e\x32/.proto.Message.PaymentInviteMessage.ServiceTypeH\x00\x88\x01\x01\x12\x1c\n\x0f\x65xpiryTimestamp\x18\x02 \x01(\x03H\x01\x88\x01\x01\"8\n\x0bServiceType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\t\n\x05\x46\x42PAY\x10\x01\x12\x08\n\x04NOVI\x10\x02\x12\x07\n\x03UPI\x10\x03\x42\x0e\n\x0c_serviceTypeB\x12\n\x10_expiryTimestamp\x1a\xdb\x04\n\x13PaymentLinkMetadata\x12I\n\x06\x62utton\x18\x01 \x01(\x0b\x32\x34.proto.Message.PaymentLinkMetadata.PaymentLinkButtonH\x00\x88\x01\x01\x12I\n\x06header\x18\x02 \x01(\x0b\x32\x34.proto.Message.PaymentLinkMetadata.PaymentLinkHeaderH\x01\x88\x01\x01\x12M\n\x08provider\x18\x03 \x01(\x0b\x32\x36.proto.Message.PaymentLinkMetadata.PaymentLinkProviderH\x02\x88\x01\x01\x1a=\n\x11PaymentLinkButton\x12\x18\n\x0b\x64isplayText\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x0e\n\x0c_displayText\x1a\xbd\x01\n\x11PaymentLinkHeader\x12\x63\n\nheaderType\x18\x01 \x01(\x0e\x32J.proto.Message.PaymentLinkMetadata.PaymentLinkHeader.PaymentLinkHeaderTypeH\x00\x88\x01\x01\"4\n\x15PaymentLinkHeaderType\x12\x10\n\x0cLINK_PREVIEW\x10\x00\x12\t\n\x05ORDER\x10\x01\x42\r\n\x0b_headerType\x1a=\n\x13PaymentLinkProvider\x12\x17\n\nparamsJson\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\r\n\x0b_paramsJsonB\t\n\x07_buttonB\t\n\x07_headerB\x0b\n\t_provider\x1a\xff\x14\n\x1fPeerDataOperationRequestMessage\x12V\n\x1cpeerDataOperationRequestType\x18\x01 \x01(\x0e\x32+.proto.Message.PeerDataOperationRequestTypeH\x00\x88\x01\x01\x12\x65\n\x16requestStickerReupload\x18\x02 \x03(\x0b\x32\x45.proto.Message.PeerDataOperationRequestMessage.RequestStickerReupload\x12[\n\x11requestUrlPreview\x18\x03 \x03(\x0b\x32@.proto.Message.PeerDataOperationRequestMessage.RequestUrlPreview\x12r\n\x1ahistorySyncOnDemandRequest\x18\x04 \x01(\x0b\x32I.proto.Message.PeerDataOperationRequestMessage.HistorySyncOnDemandRequestH\x01\x88\x01\x01\x12w\n\x1fplaceholderMessageResendRequest\x18\x05 \x03(\x0b\x32N.proto.Message.PeerDataOperationRequestMessage.PlaceholderMessageResendRequest\x12z\n\x1e\x66ullHistorySyncOnDemandRequest\x18\x06 \x01(\x0b\x32M.proto.Message.PeerDataOperationRequestMessage.FullHistorySyncOnDemandRequestH\x02\x88\x01\x01\x12\x84\x01\n#syncdCollectionFatalRecoveryRequest\x18\x07 \x01(\x0b\x32R.proto.Message.PeerDataOperationRequestMessage.SyncDCollectionFatalRecoveryRequestH\x03\x88\x01\x01\x12v\n\x1chistorySyncChunkRetryRequest\x18\x08 \x01(\x0b\x32K.proto.Message.PeerDataOperationRequestMessage.HistorySyncChunkRetryRequestH\x04\x88\x01\x01\x12^\n\x10galaxyFlowAction\x18\t \x01(\x0b\x32?.proto.Message.PeerDataOperationRequestMessage.GalaxyFlowActionH\x05\x88\x01\x01\x1a\xe5\x01\n\x1e\x46ullHistorySyncOnDemandRequest\x12S\n\x0frequestMetadata\x18\x01 \x01(\x0b\x32\x35.proto.Message.FullHistorySyncOnDemandRequestMetadataH\x00\x88\x01\x01\x12\x44\n\x11historySyncConfig\x18\x02 \x01(\x0b\x32$.proto.DeviceProps.HistorySyncConfigH\x01\x88\x01\x01\x42\x12\n\x10_requestMetadataB\x14\n\x12_historySyncConfig\x1a\x9c\x02\n\x10GalaxyFlowAction\x12g\n\x04type\x18\x01 \x01(\x0e\x32T.proto.Message.PeerDataOperationRequestMessage.GalaxyFlowAction.GalaxyFlowActionTypeH\x00\x88\x01\x01\x12\x13\n\x06\x66lowId\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x15\n\x08stanzaId\x18\x03 \x01(\tH\x02\x88\x01\x01\"R\n\x14GalaxyFlowActionType\x12\'\n#GALAXY_FLOW_ACTION_TYPE_UNSPECIFIED\x10\x00\x12\x11\n\rNOTIFY_LAUNCH\x10\x01\x42\x07\n\x05_typeB\t\n\x07_flowIdB\x0b\n\t_stanzaId\x1a\xf6\x01\n\x1cHistorySyncChunkRetryRequest\x12\x35\n\x08syncType\x18\x01 \x01(\x0e\x32\x1e.proto.Message.HistorySyncTypeH\x00\x88\x01\x01\x12\x17\n\nchunkOrder\x18\x02 \x01(\rH\x01\x88\x01\x01\x12 \n\x13\x63hunkNotificationId\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x1c\n\x0fregenerateChunk\x18\x04 \x01(\x08H\x03\x88\x01\x01\x42\x0b\n\t_syncTypeB\r\n\x0b_chunkOrderB\x16\n\x14_chunkNotificationIdB\x12\n\x10_regenerateChunk\x1a\xb2\x02\n\x1aHistorySyncOnDemandRequest\x12\x14\n\x07\x63hatJid\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x18\n\x0boldestMsgId\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x1c\n\x0foldestMsgFromMe\x18\x03 \x01(\x08H\x02\x88\x01\x01\x12\x1d\n\x10onDemandMsgCount\x18\x04 \x01(\x05H\x03\x88\x01\x01\x12!\n\x14oldestMsgTimestampMs\x18\x05 \x01(\x03H\x04\x88\x01\x01\x12\x17\n\naccountLid\x18\x06 \x01(\tH\x05\x88\x01\x01\x42\n\n\x08_chatJidB\x0e\n\x0c_oldestMsgIdB\x12\n\x10_oldestMsgFromMeB\x13\n\x11_onDemandMsgCountB\x17\n\x15_oldestMsgTimestampMsB\r\n\x0b_accountLid\x1a\\\n\x1fPlaceholderMessageResendRequest\x12*\n\nmessageKey\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x42\r\n\x0b_messageKey\x1a@\n\x16RequestStickerReupload\x12\x17\n\nfileSha256\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\r\n\x0b_fileSha256\x1a\x65\n\x11RequestUrlPreview\x12\x10\n\x03url\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x1f\n\x12includeHqThumbnail\x18\x02 \x01(\x08H\x01\x88\x01\x01\x42\x06\n\x04_urlB\x15\n\x13_includeHqThumbnail\x1a{\n#SyncDCollectionFatalRecoveryRequest\x12\x1b\n\x0e\x63ollectionName\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x16\n\ttimestamp\x18\x02 \x01(\x03H\x01\x88\x01\x01\x42\x11\n\x0f_collectionNameB\x0c\n\n_timestampB\x1f\n\x1d_peerDataOperationRequestTypeB\x1d\n\x1b_historySyncOnDemandRequestB!\n\x1f_fullHistorySyncOnDemandRequestB&\n$_syncdCollectionFatalRecoveryRequestB\x1f\n\x1d_historySyncChunkRetryRequestB\x13\n\x11_galaxyFlowAction\x1a\x8f&\n\'PeerDataOperationRequestResponseMessage\x12V\n\x1cpeerDataOperationRequestType\x18\x01 \x01(\x0e\x32+.proto.Message.PeerDataOperationRequestTypeH\x00\x88\x01\x01\x12\x15\n\x08stanzaId\x18\x02 \x01(\tH\x01\x88\x01\x01\x12o\n\x17peerDataOperationResult\x18\x03 \x03(\x0b\x32N.proto.Message.PeerDataOperationRequestResponseMessage.PeerDataOperationResult\x1a\xd5#\n\x17PeerDataOperationResult\x12H\n\x11mediaUploadResult\x18\x01 \x01(\x0e\x32(.proto.MediaRetryNotification.ResultTypeH\x00\x88\x01\x01\x12:\n\x0estickerMessage\x18\x02 \x01(\x0b\x32\x1d.proto.Message.StickerMessageH\x01\x88\x01\x01\x12\x84\x01\n\x13linkPreviewResponse\x18\x03 \x01(\x0b\x32\x62.proto.Message.PeerDataOperationRequestResponseMessage.PeerDataOperationResult.LinkPreviewResponseH\x02\x88\x01\x01\x12\x9e\x01\n placeholderMessageResendResponse\x18\x04 \x01(\x0b\x32o.proto.Message.PeerDataOperationRequestResponseMessage.PeerDataOperationResult.PlaceholderMessageResendResponseH\x03\x88\x01\x01\x12\x95\x01\n\x1fwaffleNonceFetchRequestResponse\x18\x05 \x01(\x0b\x32g.proto.Message.PeerDataOperationRequestResponseMessage.PeerDataOperationResult.WaffleNonceFetchResponseH\x04\x88\x01\x01\x12\xaa\x01\n&fullHistorySyncOnDemandRequestResponse\x18\x06 \x01(\x0b\x32u.proto.Message.PeerDataOperationRequestResponseMessage.PeerDataOperationResult.FullHistorySyncOnDemandRequestResponseH\x05\x88\x01\x01\x12\xa3\x01\n&companionMetaNonceFetchRequestResponse\x18\x07 \x01(\x0b\x32n.proto.Message.PeerDataOperationRequestResponseMessage.PeerDataOperationResult.CompanionMetaNonceFetchResponseH\x06\x88\x01\x01\x12\xa2\x01\n\"syncdSnapshotFatalRecoveryResponse\x18\x08 \x01(\x0b\x32q.proto.Message.PeerDataOperationRequestResponseMessage.PeerDataOperationResult.SyncDSnapshotFatalRecoveryResponseH\x07\x88\x01\x01\x12\xb5\x01\n/companionCanonicalUserNonceFetchRequestResponse\x18\t \x01(\x0b\x32w.proto.Message.PeerDataOperationRequestResponseMessage.PeerDataOperationResult.CompanionCanonicalUserNonceFetchResponseH\x08\x88\x01\x01\x12\x98\x01\n\x1dhistorySyncChunkRetryResponse\x18\n \x01(\x0b\x32l.proto.Message.PeerDataOperationRequestResponseMessage.PeerDataOperationResult.HistorySyncChunkRetryResponseH\t\x88\x01\x01\x1a\x94\x01\n(CompanionCanonicalUserNonceFetchResponse\x12\x12\n\x05nonce\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x13\n\x06waFbid\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x19\n\x0c\x66orceRefresh\x18\x03 \x01(\x08H\x02\x88\x01\x01\x42\x08\n\x06_nonceB\t\n\x07_waFbidB\x0f\n\r_forceRefresh\x1a?\n\x1f\x43ompanionMetaNonceFetchResponse\x12\x12\n\x05nonce\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x08\n\x06_nonce\x1a\xb2\x02\n&FullHistorySyncOnDemandRequestResponse\x12S\n\x0frequestMetadata\x18\x01 \x01(\x0b\x32\x35.proto.Message.FullHistorySyncOnDemandRequestMetadataH\x00\x88\x01\x01\x12\x8d\x01\n\x0cresponseCode\x18\x02 \x01(\x0e\x32r.proto.Message.PeerDataOperationRequestResponseMessage.PeerDataOperationResult.FullHistorySyncOnDemandResponseCodeH\x01\x88\x01\x01\x42\x12\n\x10_requestMetadataB\x0f\n\r_responseCode\x1a\xf8\x02\n\x1dHistorySyncChunkRetryResponse\x12\x35\n\x08syncType\x18\x01 \x01(\x0e\x32\x1e.proto.Message.HistorySyncTypeH\x00\x88\x01\x01\x12\x17\n\nchunkOrder\x18\x02 \x01(\rH\x01\x88\x01\x01\x12\x16\n\trequestId\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x8b\x01\n\x0cresponseCode\x18\x04 \x01(\x0e\x32p.proto.Message.PeerDataOperationRequestResponseMessage.PeerDataOperationResult.HistorySyncChunkRetryResponseCodeH\x03\x88\x01\x01\x12\x17\n\ncanRecover\x18\x05 \x01(\x08H\x04\x88\x01\x01\x42\x0b\n\t_syncTypeB\r\n\x0b_chunkOrderB\x0c\n\n_requestIdB\x0f\n\r_responseCodeB\r\n\x0b_canRecover\x1a\xa0\x08\n\x13LinkPreviewResponse\x12\x10\n\x03url\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x12\n\x05title\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x18\n\x0b\x64\x65scription\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x16\n\tthumbData\x18\x04 \x01(\x0cH\x03\x88\x01\x01\x12\x16\n\tmatchText\x18\x06 \x01(\tH\x04\x88\x01\x01\x12\x18\n\x0bpreviewType\x18\x07 \x01(\tH\x05\x88\x01\x01\x12\x9d\x01\n\x0bhqThumbnail\x18\x08 \x01(\x0b\x32\x82\x01.proto.Message.PeerDataOperationRequestResponseMessage.PeerDataOperationResult.LinkPreviewResponse.LinkPreviewHighQualityThumbnailH\x06\x88\x01\x01\x12\x9b\x01\n\x0fpreviewMetadata\x18\t \x01(\x0b\x32}.proto.Message.PeerDataOperationRequestResponseMessage.PeerDataOperationResult.LinkPreviewResponse.PaymentLinkPreviewMetadataH\x07\x88\x01\x01\x1a\xcb\x02\n\x1fLinkPreviewHighQualityThumbnail\x12\x17\n\ndirectPath\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x16\n\tthumbHash\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x19\n\x0c\x65ncThumbHash\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x15\n\x08mediaKey\x18\x04 \x01(\x0cH\x03\x88\x01\x01\x12 \n\x13mediaKeyTimestampMs\x18\x05 \x01(\x03H\x04\x88\x01\x01\x12\x17\n\nthumbWidth\x18\x06 \x01(\x05H\x05\x88\x01\x01\x12\x18\n\x0bthumbHeight\x18\x07 \x01(\x05H\x06\x88\x01\x01\x42\r\n\x0b_directPathB\x0c\n\n_thumbHashB\x0f\n\r_encThumbHashB\x0b\n\t_mediaKeyB\x16\n\x14_mediaKeyTimestampMsB\r\n\x0b_thumbWidthB\x0e\n\x0c_thumbHeight\x1a\x80\x01\n\x1aPaymentLinkPreviewMetadata\x12\x1f\n\x12isBusinessVerified\x18\x01 \x01(\x08H\x00\x88\x01\x01\x12\x19\n\x0cproviderName\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x15\n\x13_isBusinessVerifiedB\x0f\n\r_providerNameB\x06\n\x04_urlB\x08\n\x06_titleB\x0e\n\x0c_descriptionB\x0c\n\n_thumbDataB\x0c\n\n_matchTextB\x0e\n\x0c_previewTypeB\x0e\n\x0c_hqThumbnailB\x12\n\x10_previewMetadata\x1a\\\n PlaceholderMessageResendResponse\x12 \n\x13webMessageInfoBytes\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x42\x16\n\x14_webMessageInfoBytes\x1a\x88\x01\n\"SyncDSnapshotFatalRecoveryResponse\x12\x1f\n\x12\x63ollectionSnapshot\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x19\n\x0cisCompressed\x18\x02 \x01(\x08H\x01\x88\x01\x01\x42\x15\n\x13_collectionSnapshotB\x0f\n\r_isCompressed\x1a^\n\x18WaffleNonceFetchResponse\x12\x12\n\x05nonce\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x16\n\twaEntFbid\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x08\n\x06_nonceB\x0c\n\n_waEntFbid\"\xfe\x01\n#FullHistorySyncOnDemandResponseCode\x12\x13\n\x0fREQUEST_SUCCESS\x10\x00\x12\x18\n\x14REQUEST_TIME_EXPIRED\x10\x01\x12\x1c\n\x18\x44\x45\x43LINED_SHARING_HISTORY\x10\x02\x12\x11\n\rGENERIC_ERROR\x10\x03\x12$\n ERROR_REQUEST_ON_NON_SMB_PRIMARY\x10\x04\x12%\n!ERROR_HOSTED_DEVICE_NOT_CONNECTED\x10\x05\x12*\n&ERROR_HOSTED_DEVICE_LOGIN_TIME_NOT_SET\x10\x06\"\xd6\x01\n!HistorySyncChunkRetryResponseCode\x12\x36\n2HISTORY_SYNC_CHUNK_RETRY_RESPONSE_CODE_UNSPECIFIED\x10\x00\x12\x14\n\x10GENERATION_ERROR\x10\x01\x12\x12\n\x0e\x43HUNK_CONSUMED\x10\x02\x12\x0b\n\x07TIMEOUT\x10\x03\x12\x15\n\x11SESSION_EXHAUSTED\x10\x04\x12\x13\n\x0f\x43HUNK_EXHAUSTED\x10\x05\x12\x16\n\x12\x44UPLICATED_REQUEST\x10\x06\x42\x14\n\x12_mediaUploadResultB\x11\n\x0f_stickerMessageB\x16\n\x14_linkPreviewResponseB#\n!_placeholderMessageResendResponseB\"\n _waffleNonceFetchRequestResponseB)\n\'_fullHistorySyncOnDemandRequestResponseB)\n\'_companionMetaNonceFetchRequestResponseB%\n#_syncdSnapshotFatalRecoveryResponseB2\n0_companionCanonicalUserNonceFetchRequestResponseB \n\x1e_historySyncChunkRetryResponseB\x1f\n\x1d_peerDataOperationRequestTypeB\x0b\n\t_stanzaId\x1a\xf5\x01\n\x10PinInChatMessage\x12#\n\x03key\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x12\x37\n\x04type\x18\x02 \x01(\x0e\x32$.proto.Message.PinInChatMessage.TypeH\x01\x88\x01\x01\x12\x1e\n\x11senderTimestampMs\x18\x03 \x01(\x03H\x02\x88\x01\x01\"<\n\x04Type\x12\x10\n\x0cUNKNOWN_TYPE\x10\x00\x12\x0f\n\x0bPIN_FOR_ALL\x10\x01\x12\x11\n\rUNPIN_FOR_ALL\x10\x02\x42\x06\n\x04_keyB\x07\n\x05_typeB\x14\n\x12_senderTimestampMs\x1a\x8f\x01\n\x12PlaceholderMessage\x12\x44\n\x04type\x18\x01 \x01(\x0e\x32\x31.proto.Message.PlaceholderMessage.PlaceholderTypeH\x00\x88\x01\x01\"*\n\x0fPlaceholderType\x12\x17\n\x13MASK_LINKED_DEVICES\x10\x00\x42\x07\n\x05_type\x1a\xcd\x04\n\x13PollCreationMessage\x12\x13\n\x06\x65ncKey\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x11\n\x04name\x18\x02 \x01(\tH\x01\x88\x01\x01\x12:\n\x07options\x18\x03 \x03(\x0b\x32).proto.Message.PollCreationMessage.Option\x12#\n\x16selectableOptionsCount\x18\x04 \x01(\rH\x02\x88\x01\x01\x12,\n\x0b\x63ontextInfo\x18\x05 \x01(\x0b\x32\x12.proto.ContextInfoH\x03\x88\x01\x01\x12<\n\x0fpollContentType\x18\x06 \x01(\x0e\x32\x1e.proto.Message.PollContentTypeH\x04\x88\x01\x01\x12.\n\x08pollType\x18\x07 \x01(\x0e\x32\x17.proto.Message.PollTypeH\x05\x88\x01\x01\x12\x45\n\rcorrectAnswer\x18\x08 \x01(\x0b\x32).proto.Message.PollCreationMessage.OptionH\x06\x88\x01\x01\x1aX\n\x06Option\x12\x17\n\noptionName\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x17\n\noptionHash\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\r\n\x0b_optionNameB\r\n\x0b_optionHashB\t\n\x07_encKeyB\x07\n\x05_nameB\x19\n\x17_selectableOptionsCountB\x0e\n\x0c_contextInfoB\x12\n\x10_pollContentTypeB\x0b\n\t_pollTypeB\x10\n\x0e_correctAnswer\x1aT\n\x0cPollEncValue\x12\x17\n\nencPayload\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x12\n\x05\x65ncIv\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x42\r\n\x0b_encPayloadB\x08\n\x06_encIv\x1a\xde\x02\n\x19PollResultSnapshotMessage\x12\x11\n\x04name\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x44\n\tpollVotes\x18\x02 \x03(\x0b\x32\x31.proto.Message.PollResultSnapshotMessage.PollVote\x12,\n\x0b\x63ontextInfo\x18\x03 \x01(\x0b\x32\x12.proto.ContextInfoH\x01\x88\x01\x01\x12.\n\x08pollType\x18\x04 \x01(\x0e\x32\x17.proto.Message.PollTypeH\x02\x88\x01\x01\x1a\x64\n\x08PollVote\x12\x17\n\noptionName\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x1c\n\x0foptionVoteCount\x18\x02 \x01(\x03H\x01\x88\x01\x01\x42\r\n\x0b_optionNameB\x12\n\x10_optionVoteCountB\x07\n\x05_nameB\x0e\n\x0c_contextInfoB\x0b\n\t_pollType\x1a\xa3\x02\n\x11PollUpdateMessage\x12\x36\n\x16pollCreationMessageKey\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x12.\n\x04vote\x18\x02 \x01(\x0b\x32\x1b.proto.Message.PollEncValueH\x01\x88\x01\x01\x12?\n\x08metadata\x18\x03 \x01(\x0b\x32(.proto.Message.PollUpdateMessageMetadataH\x02\x88\x01\x01\x12\x1e\n\x11senderTimestampMs\x18\x04 \x01(\x03H\x03\x88\x01\x01\x42\x19\n\x17_pollCreationMessageKeyB\x07\n\x05_voteB\x0b\n\t_metadataB\x14\n\x12_senderTimestampMs\x1a\x1b\n\x19PollUpdateMessageMetadata\x1a*\n\x0fPollVoteMessage\x12\x17\n\x0fselectedOptions\x18\x01 \x03(\x0c\x1a\xb2\x08\n\x0eProductMessage\x12\x43\n\x07product\x18\x01 \x01(\x0b\x32-.proto.Message.ProductMessage.ProductSnapshotH\x00\x88\x01\x01\x12\x1d\n\x10\x62usinessOwnerJid\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x43\n\x07\x63\x61talog\x18\x04 \x01(\x0b\x32-.proto.Message.ProductMessage.CatalogSnapshotH\x02\x88\x01\x01\x12\x11\n\x04\x62ody\x18\x05 \x01(\tH\x03\x88\x01\x01\x12\x13\n\x06\x66ooter\x18\x06 \x01(\tH\x04\x88\x01\x01\x12,\n\x0b\x63ontextInfo\x18\x11 \x01(\x0b\x32\x12.proto.ContextInfoH\x05\x88\x01\x01\x1a\xa2\x01\n\x0f\x43\x61talogSnapshot\x12\x36\n\x0c\x63\x61talogImage\x18\x01 \x01(\x0b\x32\x1b.proto.Message.ImageMessageH\x00\x88\x01\x01\x12\x12\n\x05title\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x18\n\x0b\x64\x65scription\x18\x03 \x01(\tH\x02\x88\x01\x01\x42\x0f\n\r_catalogImageB\x08\n\x06_titleB\x0e\n\x0c_description\x1a\xaa\x04\n\x0fProductSnapshot\x12\x36\n\x0cproductImage\x18\x01 \x01(\x0b\x32\x1b.proto.Message.ImageMessageH\x00\x88\x01\x01\x12\x16\n\tproductId\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x12\n\x05title\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x18\n\x0b\x64\x65scription\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x19\n\x0c\x63urrencyCode\x18\x05 \x01(\tH\x04\x88\x01\x01\x12\x1c\n\x0fpriceAmount1000\x18\x06 \x01(\x03H\x05\x88\x01\x01\x12\x17\n\nretailerId\x18\x07 \x01(\tH\x06\x88\x01\x01\x12\x10\n\x03url\x18\x08 \x01(\tH\x07\x88\x01\x01\x12\x1e\n\x11productImageCount\x18\t \x01(\rH\x08\x88\x01\x01\x12\x19\n\x0c\x66irstImageId\x18\x0b \x01(\tH\t\x88\x01\x01\x12 \n\x13salePriceAmount1000\x18\x0c \x01(\x03H\n\x88\x01\x01\x12\x16\n\tsignedUrl\x18\r \x01(\tH\x0b\x88\x01\x01\x42\x0f\n\r_productImageB\x0c\n\n_productIdB\x08\n\x06_titleB\x0e\n\x0c_descriptionB\x0f\n\r_currencyCodeB\x12\n\x10_priceAmount1000B\r\n\x0b_retailerIdB\x06\n\x04_urlB\x14\n\x12_productImageCountB\x0f\n\r_firstImageIdB\x16\n\x14_salePriceAmount1000B\x0c\n\n_signedUrlB\n\n\x08_productB\x13\n\x11_businessOwnerJidB\n\n\x08_catalogB\x07\n\x05_bodyB\t\n\x07_footerB\x0e\n\x0c_contextInfo\x1a\xee\x16\n\x0fProtocolMessage\x12#\n\x03key\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x12\x36\n\x04type\x18\x02 \x01(\x0e\x32#.proto.Message.ProtocolMessage.TypeH\x01\x88\x01\x01\x12 \n\x13\x65phemeralExpiration\x18\x04 \x01(\rH\x02\x88\x01\x01\x12&\n\x19\x65phemeralSettingTimestamp\x18\x05 \x01(\x03H\x03\x88\x01\x01\x12L\n\x17historySyncNotification\x18\x06 \x01(\x0b\x32&.proto.Message.HistorySyncNotificationH\x04\x88\x01\x01\x12\x46\n\x14\x61ppStateSyncKeyShare\x18\x07 \x01(\x0b\x32#.proto.Message.AppStateSyncKeyShareH\x05\x88\x01\x01\x12J\n\x16\x61ppStateSyncKeyRequest\x18\x08 \x01(\x0b\x32%.proto.Message.AppStateSyncKeyRequestH\x06\x88\x01\x01\x12j\n&initialSecurityNotificationSettingSync\x18\t \x01(\x0b\x32\x35.proto.Message.InitialSecurityNotificationSettingSyncH\x07\x88\x01\x01\x12\x62\n\"appStateFatalExceptionNotification\x18\n \x01(\x0b\x32\x31.proto.Message.AppStateFatalExceptionNotificationH\x08\x88\x01\x01\x12\x36\n\x10\x64isappearingMode\x18\x0b \x01(\x0b\x32\x17.proto.DisappearingModeH\t\x88\x01\x01\x12*\n\reditedMessage\x18\x0e \x01(\x0b\x32\x0e.proto.MessageH\n\x88\x01\x01\x12\x18\n\x0btimestampMs\x18\x0f \x01(\x03H\x0b\x88\x01\x01\x12\\\n\x1fpeerDataOperationRequestMessage\x18\x10 \x01(\x0b\x32..proto.Message.PeerDataOperationRequestMessageH\x0c\x88\x01\x01\x12l\n\'peerDataOperationRequestResponseMessage\x18\x11 \x01(\x0b\x32\x36.proto.Message.PeerDataOperationRequestResponseMessageH\r\x88\x01\x01\x12:\n\x12\x62otFeedbackMessage\x18\x12 \x01(\x0b\x32\x19.proto.BotFeedbackMessageH\x0e\x88\x01\x01\x12\x17\n\ninvokerJid\x18\x13 \x01(\tH\x0f\x88\x01\x01\x12X\n\x1drequestWelcomeMessageMetadata\x18\x14 \x01(\x0b\x32,.proto.Message.RequestWelcomeMessageMetadataH\x10\x88\x01\x01\x12:\n\x12mediaNotifyMessage\x18\x15 \x01(\x0b\x32\x19.proto.MediaNotifyMessageH\x11\x88\x01\x01\x12`\n!cloudApiThreadControlNotification\x18\x16 \x01(\x0b\x32\x30.proto.Message.CloudAPIThreadControlNotificationH\x12\x88\x01\x01\x12R\n\x1elidMigrationMappingSyncMessage\x18\x17 \x01(\x0b\x32%.proto.LIDMigrationMappingSyncMessageH\x13\x88\x01\x01\x12.\n\x0climitSharing\x18\x18 \x01(\x0b\x32\x13.proto.LimitSharingH\x14\x88\x01\x01\x12\x1a\n\raiPsiMetadata\x18\x19 \x01(\x0cH\x15\x88\x01\x01\x12\x30\n\raiQueryFanout\x18\x1a \x01(\x0b\x32\x14.proto.AIQueryFanoutH\x16\x88\x01\x01\x12,\n\x0bmemberLabel\x18\x1b \x01(\x0b\x32\x12.proto.MemberLabelH\x17\x88\x01\x01\"\x8d\x06\n\x04Type\x12\n\n\x06REVOKE\x10\x00\x12\x15\n\x11\x45PHEMERAL_SETTING\x10\x03\x12\x1b\n\x17\x45PHEMERAL_SYNC_RESPONSE\x10\x04\x12\x1d\n\x19HISTORY_SYNC_NOTIFICATION\x10\x05\x12\x1c\n\x18\x41PP_STATE_SYNC_KEY_SHARE\x10\x06\x12\x1e\n\x1a\x41PP_STATE_SYNC_KEY_REQUEST\x10\x07\x12\x1f\n\x1bMSG_FANOUT_BACKFILL_REQUEST\x10\x08\x12.\n*INITIAL_SECURITY_NOTIFICATION_SETTING_SYNC\x10\t\x12*\n&APP_STATE_FATAL_EXCEPTION_NOTIFICATION\x10\n\x12\x16\n\x12SHARE_PHONE_NUMBER\x10\x0b\x12\x10\n\x0cMESSAGE_EDIT\x10\x0e\x12\'\n#PEER_DATA_OPERATION_REQUEST_MESSAGE\x10\x10\x12\x30\n,PEER_DATA_OPERATION_REQUEST_RESPONSE_MESSAGE\x10\x11\x12\x1b\n\x17REQUEST_WELCOME_MESSAGE\x10\x12\x12\x18\n\x14\x42OT_FEEDBACK_MESSAGE\x10\x13\x12\x18\n\x14MEDIA_NOTIFY_MESSAGE\x10\x14\x12)\n%CLOUD_API_THREAD_CONTROL_NOTIFICATION\x10\x15\x12\x1e\n\x1aLID_MIGRATION_MAPPING_SYNC\x10\x16\x12\x14\n\x10REMINDER_MESSAGE\x10\x17\x12\x1f\n\x1b\x42OT_MEMU_ONBOARDING_MESSAGE\x10\x18\x12\x1a\n\x16STATUS_MENTION_MESSAGE\x10\x19\x12\x1b\n\x17STOP_GENERATION_MESSAGE\x10\x1a\x12\x11\n\rLIMIT_SHARING\x10\x1b\x12\x13\n\x0f\x41I_PSI_METADATA\x10\x1c\x12\x13\n\x0f\x41I_QUERY_FANOUT\x10\x1d\x12\x1d\n\x19GROUP_MEMBER_LABEL_CHANGE\x10\x1e\x42\x06\n\x04_keyB\x07\n\x05_typeB\x16\n\x14_ephemeralExpirationB\x1c\n\x1a_ephemeralSettingTimestampB\x1a\n\x18_historySyncNotificationB\x17\n\x15_appStateSyncKeyShareB\x19\n\x17_appStateSyncKeyRequestB)\n\'_initialSecurityNotificationSettingSyncB%\n#_appStateFatalExceptionNotificationB\x13\n\x11_disappearingModeB\x10\n\x0e_editedMessageB\x0e\n\x0c_timestampMsB\"\n _peerDataOperationRequestMessageB*\n(_peerDataOperationRequestResponseMessageB\x15\n\x13_botFeedbackMessageB\r\n\x0b_invokerJidB \n\x1e_requestWelcomeMessageMetadataB\x15\n\x13_mediaNotifyMessageB$\n\"_cloudApiThreadControlNotificationB!\n\x1f_lidMigrationMappingSyncMessageB\x0f\n\r_limitSharingB\x10\n\x0e_aiPsiMetadataB\x10\n\x0e_aiQueryFanoutB\x0e\n\x0c_memberLabel\x1a\x62\n\x17QuestionResponseMessage\x12#\n\x03key\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x12\x11\n\x04text\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x06\n\x04_keyB\x07\n\x05_text\x1a\xba\x01\n\x0fReactionMessage\x12#\n\x03key\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x12\x11\n\x04text\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x18\n\x0bgroupingKey\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x1e\n\x11senderTimestampMs\x18\x04 \x01(\x03H\x03\x88\x01\x01\x42\x06\n\x04_keyB\x07\n\x05_textB\x0e\n\x0c_groupingKeyB\x14\n\x12_senderTimestampMs\x1a\xff\x02\n\x15RequestPaymentMessage\x12(\n\x0bnoteMessage\x18\x04 \x01(\x0b\x32\x0e.proto.MessageH\x00\x88\x01\x01\x12 \n\x13\x63urrencyCodeIso4217\x18\x01 \x01(\tH\x01\x88\x01\x01\x12\x17\n\namount1000\x18\x02 \x01(\x04H\x02\x88\x01\x01\x12\x18\n\x0brequestFrom\x18\x03 \x01(\tH\x03\x88\x01\x01\x12\x1c\n\x0f\x65xpiryTimestamp\x18\x05 \x01(\x03H\x04\x88\x01\x01\x12!\n\x06\x61mount\x18\x06 \x01(\x0b\x32\x0c.proto.MoneyH\x05\x88\x01\x01\x12\x31\n\nbackground\x18\x07 \x01(\x0b\x32\x18.proto.PaymentBackgroundH\x06\x88\x01\x01\x42\x0e\n\x0c_noteMessageB\x16\n\x14_currencyCodeIso4217B\r\n\x0b_amount1000B\x0e\n\x0c_requestFromB\x12\n\x10_expiryTimestampB\t\n\x07_amountB\r\n\x0b_background\x1aY\n\x19RequestPhoneNumberMessage\x12,\n\x0b\x63ontextInfo\x18\x01 \x01(\x0b\x32\x12.proto.ContextInfoH\x00\x88\x01\x01\x42\x0e\n\x0c_contextInfo\x1a\xb8\x01\n\x1dRequestWelcomeMessageMetadata\x12X\n\x0elocalChatState\x18\x01 \x01(\x0e\x32;.proto.Message.RequestWelcomeMessageMetadata.LocalChatStateH\x00\x88\x01\x01\"*\n\x0eLocalChatState\x12\t\n\x05\x45MPTY\x10\x00\x12\r\n\tNON_EMPTY\x10\x01\x42\x11\n\x0f_localChatState\x1a\x81\x02\n\x1cScheduledCallCreationMessage\x12!\n\x14scheduledTimestampMs\x18\x01 \x01(\x03H\x00\x88\x01\x01\x12K\n\x08\x63\x61llType\x18\x02 \x01(\x0e\x32\x34.proto.Message.ScheduledCallCreationMessage.CallTypeH\x01\x88\x01\x01\x12\x12\n\x05title\x18\x03 \x01(\tH\x02\x88\x01\x01\"-\n\x08\x43\x61llType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\t\n\x05VOICE\x10\x01\x12\t\n\x05VIDEO\x10\x02\x42\x17\n\x15_scheduledTimestampMsB\x0b\n\t_callTypeB\x08\n\x06_title\x1a\xc2\x01\n\x18ScheduledCallEditMessage\x12#\n\x03key\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x12G\n\x08\x65\x64itType\x18\x02 \x01(\x0e\x32\x30.proto.Message.ScheduledCallEditMessage.EditTypeH\x01\x88\x01\x01\"#\n\x08\x45\x64itType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\n\n\x06\x43\x41NCEL\x10\x01\x42\x06\n\x04_keyB\x0b\n\t_editType\x1a\xc8\x02\n\x16SecretEncryptedMessage\x12\x30\n\x10targetMessageKey\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x12\x17\n\nencPayload\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x12\n\x05\x65ncIv\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12O\n\rsecretEncType\x18\x04 \x01(\x0e\x32\x33.proto.Message.SecretEncryptedMessage.SecretEncTypeH\x03\x88\x01\x01\">\n\rSecretEncType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0e\n\nEVENT_EDIT\x10\x01\x12\x10\n\x0cMESSAGE_EDIT\x10\x02\x42\x13\n\x11_targetMessageKeyB\r\n\x0b_encPayloadB\x08\n\x06_encIvB\x10\n\x0e_secretEncType\x1a\x8b\x02\n\x12SendPaymentMessage\x12(\n\x0bnoteMessage\x18\x02 \x01(\x0b\x32\x0e.proto.MessageH\x00\x88\x01\x01\x12\x31\n\x11requestMessageKey\x18\x03 \x01(\x0b\x32\x11.proto.MessageKeyH\x01\x88\x01\x01\x12\x31\n\nbackground\x18\x04 \x01(\x0b\x32\x18.proto.PaymentBackgroundH\x02\x88\x01\x01\x12\x1c\n\x0ftransactionData\x18\x05 \x01(\tH\x03\x88\x01\x01\x42\x0e\n\x0c_noteMessageB\x14\n\x12_requestMessageKeyB\r\n\x0b_backgroundB\x12\n\x10_transactionData\x1a\x9a\x01\n\x1cSenderKeyDistributionMessage\x12\x14\n\x07groupId\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x30\n#axolotlSenderKeyDistributionMessage\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x42\n\n\x08_groupIdB&\n$_axolotlSenderKeyDistributionMessage\x1a\x83\x03\n\x19StatusNotificationMessage\x12\x32\n\x12responseMessageKey\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x12\x32\n\x12originalMessageKey\x18\x02 \x01(\x0b\x32\x11.proto.MessageKeyH\x01\x88\x01\x01\x12R\n\x04type\x18\x03 \x01(\x0e\x32?.proto.Message.StatusNotificationMessage.StatusNotificationTypeH\x02\x88\x01\x01\"s\n\x16StatusNotificationType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x14\n\x10STATUS_ADD_YOURS\x10\x01\x12\x12\n\x0eSTATUS_RESHARE\x10\x02\x12\"\n\x1eSTATUS_QUESTION_ANSWER_RESHARE\x10\x03\x42\x15\n\x13_responseMessageKeyB\x15\n\x13_originalMessageKeyB\x07\n\x05_type\x1a\x66\n\x1bStatusQuestionAnswerMessage\x12#\n\x03key\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x12\x11\n\x04text\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x06\n\x04_keyB\x07\n\x05_text\x1a\xd2\x02\n\x13StatusQuotedMessage\x12M\n\x04type\x18\x01 \x01(\x0e\x32:.proto.Message.StatusQuotedMessage.StatusQuotedMessageTypeH\x00\x88\x01\x01\x12\x11\n\x04text\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x16\n\tthumbnail\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x30\n\x10originalStatusId\x18\x04 \x01(\x0b\x32\x11.proto.MessageKeyH\x03\x88\x01\x01\"Z\n\x17StatusQuotedMessageType\x12*\n&STATUS_QUOTED_MESSAGE_TYPE_UNSPECIFIED\x10\x00\x12\x13\n\x0fQUESTION_ANSWER\x10\x01\x42\x07\n\x05_typeB\x07\n\x05_textB\x0c\n\n_thumbnailB\x13\n\x11_originalStatusId\x1a\x84\x02\n\x1fStatusStickerInteractionMessage\x12#\n\x03key\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x12\x17\n\nstickerKey\x18\x02 \x01(\tH\x01\x88\x01\x01\x12S\n\x04type\x18\x03 \x01(\x0e\x32@.proto.Message.StatusStickerInteractionMessage.StatusStickerTypeH\x02\x88\x01\x01\".\n\x11StatusStickerType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0c\n\x08REACTION\x10\x01\x42\x06\n\x04_keyB\r\n\x0b_stickerKeyB\x07\n\x05_type\x1a\xaf\x07\n\x0eStickerMessage\x12\x10\n\x03url\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x17\n\nfileSha256\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x1a\n\rfileEncSha256\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x15\n\x08mediaKey\x18\x04 \x01(\x0cH\x03\x88\x01\x01\x12\x15\n\x08mimetype\x18\x05 \x01(\tH\x04\x88\x01\x01\x12\x13\n\x06height\x18\x06 \x01(\rH\x05\x88\x01\x01\x12\x12\n\x05width\x18\x07 \x01(\rH\x06\x88\x01\x01\x12\x17\n\ndirectPath\x18\x08 \x01(\tH\x07\x88\x01\x01\x12\x17\n\nfileLength\x18\t \x01(\x04H\x08\x88\x01\x01\x12\x1e\n\x11mediaKeyTimestamp\x18\n \x01(\x03H\t\x88\x01\x01\x12\x1d\n\x10\x66irstFrameLength\x18\x0b \x01(\rH\n\x88\x01\x01\x12\x1e\n\x11\x66irstFrameSidecar\x18\x0c \x01(\x0cH\x0b\x88\x01\x01\x12\x17\n\nisAnimated\x18\r \x01(\x08H\x0c\x88\x01\x01\x12\x19\n\x0cpngThumbnail\x18\x10 \x01(\x0cH\r\x88\x01\x01\x12,\n\x0b\x63ontextInfo\x18\x11 \x01(\x0b\x32\x12.proto.ContextInfoH\x0e\x88\x01\x01\x12\x1a\n\rstickerSentTs\x18\x12 \x01(\x03H\x0f\x88\x01\x01\x12\x15\n\x08isAvatar\x18\x13 \x01(\x08H\x10\x88\x01\x01\x12\x18\n\x0bisAiSticker\x18\x14 \x01(\x08H\x11\x88\x01\x01\x12\x15\n\x08isLottie\x18\x15 \x01(\x08H\x12\x88\x01\x01\x12\x1f\n\x12\x61\x63\x63\x65ssibilityLabel\x18\x16 \x01(\tH\x13\x88\x01\x01\x12:\n\x0emediaKeyDomain\x18\x17 \x01(\x0e\x32\x1d.proto.Message.MediaKeyDomainH\x14\x88\x01\x01\x42\x06\n\x04_urlB\r\n\x0b_fileSha256B\x10\n\x0e_fileEncSha256B\x0b\n\t_mediaKeyB\x0b\n\t_mimetypeB\t\n\x07_heightB\x08\n\x06_widthB\r\n\x0b_directPathB\r\n\x0b_fileLengthB\x14\n\x12_mediaKeyTimestampB\x13\n\x11_firstFrameLengthB\x14\n\x12_firstFrameSidecarB\r\n\x0b_isAnimatedB\x0f\n\r_pngThumbnailB\x0e\n\x0c_contextInfoB\x10\n\x0e_stickerSentTsB\x0b\n\t_isAvatarB\x0e\n\x0c_isAiStickerB\x0b\n\t_isLottieB\x15\n\x13_accessibilityLabelB\x11\n\x0f_mediaKeyDomain\x1a\x89\x0b\n\x12StickerPackMessage\x12\x1a\n\rstickerPackId\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x11\n\x04name\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x16\n\tpublisher\x18\x03 \x01(\tH\x02\x88\x01\x01\x12;\n\x08stickers\x18\x04 \x03(\x0b\x32).proto.Message.StickerPackMessage.Sticker\x12\x17\n\nfileLength\x18\x05 \x01(\x04H\x03\x88\x01\x01\x12\x17\n\nfileSha256\x18\x06 \x01(\x0cH\x04\x88\x01\x01\x12\x1a\n\rfileEncSha256\x18\x07 \x01(\x0cH\x05\x88\x01\x01\x12\x15\n\x08mediaKey\x18\x08 \x01(\x0cH\x06\x88\x01\x01\x12\x17\n\ndirectPath\x18\t \x01(\tH\x07\x88\x01\x01\x12\x14\n\x07\x63\x61ption\x18\n \x01(\tH\x08\x88\x01\x01\x12,\n\x0b\x63ontextInfo\x18\x0b \x01(\x0b\x32\x12.proto.ContextInfoH\t\x88\x01\x01\x12\x1c\n\x0fpackDescription\x18\x0c \x01(\tH\n\x88\x01\x01\x12\x1e\n\x11mediaKeyTimestamp\x18\r \x01(\x03H\x0b\x88\x01\x01\x12\x1d\n\x10trayIconFileName\x18\x0e \x01(\tH\x0c\x88\x01\x01\x12 \n\x13thumbnailDirectPath\x18\x0f \x01(\tH\r\x88\x01\x01\x12\x1c\n\x0fthumbnailSha256\x18\x10 \x01(\x0cH\x0e\x88\x01\x01\x12\x1f\n\x12thumbnailEncSha256\x18\x11 \x01(\x0cH\x0f\x88\x01\x01\x12\x1c\n\x0fthumbnailHeight\x18\x12 \x01(\rH\x10\x88\x01\x01\x12\x1b\n\x0ethumbnailWidth\x18\x13 \x01(\rH\x11\x88\x01\x01\x12\x1a\n\rimageDataHash\x18\x14 \x01(\tH\x12\x88\x01\x01\x12\x1c\n\x0fstickerPackSize\x18\x15 \x01(\x04H\x13\x88\x01\x01\x12S\n\x11stickerPackOrigin\x18\x16 \x01(\x0e\x32\x33.proto.Message.StickerPackMessage.StickerPackOriginH\x14\x88\x01\x01\x1a\xe5\x01\n\x07Sticker\x12\x15\n\x08\x66ileName\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x17\n\nisAnimated\x18\x02 \x01(\x08H\x01\x88\x01\x01\x12\x0e\n\x06\x65mojis\x18\x03 \x03(\t\x12\x1f\n\x12\x61\x63\x63\x65ssibilityLabel\x18\x04 \x01(\tH\x02\x88\x01\x01\x12\x15\n\x08isLottie\x18\x05 \x01(\x08H\x03\x88\x01\x01\x12\x15\n\x08mimetype\x18\x06 \x01(\tH\x04\x88\x01\x01\x42\x0b\n\t_fileNameB\r\n\x0b_isAnimatedB\x15\n\x13_accessibilityLabelB\x0b\n\t_isLottieB\x0b\n\t_mimetype\"G\n\x11StickerPackOrigin\x12\x0f\n\x0b\x46IRST_PARTY\x10\x00\x12\x0f\n\x0bTHIRD_PARTY\x10\x01\x12\x10\n\x0cUSER_CREATED\x10\x02\x42\x10\n\x0e_stickerPackIdB\x07\n\x05_nameB\x0c\n\n_publisherB\r\n\x0b_fileLengthB\r\n\x0b_fileSha256B\x10\n\x0e_fileEncSha256B\x0b\n\t_mediaKeyB\r\n\x0b_directPathB\n\n\x08_captionB\x0e\n\x0c_contextInfoB\x12\n\x10_packDescriptionB\x14\n\x12_mediaKeyTimestampB\x13\n\x11_trayIconFileNameB\x16\n\x14_thumbnailDirectPathB\x12\n\x10_thumbnailSha256B\x15\n\x13_thumbnailEncSha256B\x12\n\x10_thumbnailHeightB\x11\n\x0f_thumbnailWidthB\x10\n\x0e_imageDataHashB\x12\n\x10_stickerPackSizeB\x14\n\x12_stickerPackOrigin\x1a\x83\x01\n\x15StickerSyncRMRMessage\x12\x10\n\x08\x66ilehash\x18\x01 \x03(\t\x12\x16\n\trmrSource\x18\x02 \x01(\tH\x00\x88\x01\x01\x12\x1d\n\x10requestTimestamp\x18\x03 \x01(\x03H\x01\x88\x01\x01\x42\x0c\n\n_rmrSourceB\x13\n\x11_requestTimestamp\x1a\xb0\x02\n\x1aTemplateButtonReplyMessage\x12\x17\n\nselectedId\x18\x01 \x01(\tH\x00\x88\x01\x01\x12 \n\x13selectedDisplayText\x18\x02 \x01(\tH\x01\x88\x01\x01\x12,\n\x0b\x63ontextInfo\x18\x03 \x01(\x0b\x32\x12.proto.ContextInfoH\x02\x88\x01\x01\x12\x1a\n\rselectedIndex\x18\x04 \x01(\rH\x03\x88\x01\x01\x12&\n\x19selectedCarouselCardIndex\x18\x05 \x01(\rH\x04\x88\x01\x01\x42\r\n\x0b_selectedIdB\x16\n\x14_selectedDisplayTextB\x0e\n\x0c_contextInfoB\x10\n\x0e_selectedIndexB\x1c\n\x1a_selectedCarouselCardIndex\x1a\x88\x0c\n\x0fTemplateMessage\x12,\n\x0b\x63ontextInfo\x18\x03 \x01(\x0b\x32\x12.proto.ContextInfoH\x01\x88\x01\x01\x12U\n\x10hydratedTemplate\x18\x04 \x01(\x0b\x32\x36.proto.Message.TemplateMessage.HydratedFourRowTemplateH\x02\x88\x01\x01\x12\x17\n\ntemplateId\x18\t \x01(\tH\x03\x88\x01\x01\x12I\n\x0f\x66ourRowTemplate\x18\x01 \x01(\x0b\x32..proto.Message.TemplateMessage.FourRowTemplateH\x00\x12Y\n\x17hydratedFourRowTemplate\x18\x02 \x01(\x0b\x32\x36.proto.Message.TemplateMessage.HydratedFourRowTemplateH\x00\x12G\n\x1ainteractiveMessageTemplate\x18\x05 \x01(\x0b\x32!.proto.Message.InteractiveMessageH\x00\x1a\xff\x03\n\x0f\x46ourRowTemplate\x12<\n\x07\x63ontent\x18\x06 \x01(\x0b\x32&.proto.Message.HighlyStructuredMessageH\x01\x88\x01\x01\x12;\n\x06\x66ooter\x18\x07 \x01(\x0b\x32&.proto.Message.HighlyStructuredMessageH\x02\x88\x01\x01\x12&\n\x07\x62uttons\x18\x08 \x03(\x0b\x32\x15.proto.TemplateButton\x12\x39\n\x0f\x64ocumentMessage\x18\x01 \x01(\x0b\x32\x1e.proto.Message.DocumentMessageH\x00\x12I\n\x17highlyStructuredMessage\x18\x02 \x01(\x0b\x32&.proto.Message.HighlyStructuredMessageH\x00\x12\x33\n\x0cimageMessage\x18\x03 \x01(\x0b\x32\x1b.proto.Message.ImageMessageH\x00\x12\x33\n\x0cvideoMessage\x18\x04 \x01(\x0b\x32\x1b.proto.Message.VideoMessageH\x00\x12\x39\n\x0flocationMessage\x18\x05 \x01(\x0b\x32\x1e.proto.Message.LocationMessageH\x00\x42\x07\n\x05titleB\n\n\x08_contentB\t\n\x07_footer\x1a\xa7\x04\n\x17HydratedFourRowTemplate\x12 \n\x13hydratedContentText\x18\x06 \x01(\tH\x01\x88\x01\x01\x12\x1f\n\x12hydratedFooterText\x18\x07 \x01(\tH\x02\x88\x01\x01\x12\x36\n\x0fhydratedButtons\x18\x08 \x03(\x0b\x32\x1d.proto.HydratedTemplateButton\x12\x17\n\ntemplateId\x18\t \x01(\tH\x03\x88\x01\x01\x12\x1e\n\x11maskLinkedDevices\x18\n \x01(\x08H\x04\x88\x01\x01\x12\x39\n\x0f\x64ocumentMessage\x18\x01 \x01(\x0b\x32\x1e.proto.Message.DocumentMessageH\x00\x12\x1b\n\x11hydratedTitleText\x18\x02 \x01(\tH\x00\x12\x33\n\x0cimageMessage\x18\x03 \x01(\x0b\x32\x1b.proto.Message.ImageMessageH\x00\x12\x33\n\x0cvideoMessage\x18\x04 \x01(\x0b\x32\x1b.proto.Message.VideoMessageH\x00\x12\x39\n\x0flocationMessage\x18\x05 \x01(\x0b\x32\x1e.proto.Message.LocationMessageH\x00\x42\x07\n\x05titleB\x16\n\x14_hydratedContentTextB\x15\n\x13_hydratedFooterTextB\r\n\x0b_templateIdB\x14\n\x12_maskLinkedDevicesB\x08\n\x06\x66ormatB\x0e\n\x0c_contextInfoB\x13\n\x11_hydratedTemplateB\r\n\x0b_templateId\x1a=\n\x0bURLMetadata\x12\x1b\n\x0e\x66\x62\x45xperimentId\x18\x01 \x01(\rH\x00\x88\x01\x01\x42\x11\n\x0f_fbExperimentId\x1ag\n\x0cVideoEndCard\x12\x10\n\x08username\x18\x01 \x01(\t\x12\x0f\n\x07\x63\x61ption\x18\x02 \x01(\t\x12\x19\n\x11thumbnailImageUrl\x18\x03 \x01(\t\x12\x19\n\x11profilePictureUrl\x18\x04 \x01(\t\x1a\xb3\r\n\x0cVideoMessage\x12\x10\n\x03url\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x15\n\x08mimetype\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x17\n\nfileSha256\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x17\n\nfileLength\x18\x04 \x01(\x04H\x03\x88\x01\x01\x12\x14\n\x07seconds\x18\x05 \x01(\rH\x04\x88\x01\x01\x12\x15\n\x08mediaKey\x18\x06 \x01(\x0cH\x05\x88\x01\x01\x12\x14\n\x07\x63\x61ption\x18\x07 \x01(\tH\x06\x88\x01\x01\x12\x18\n\x0bgifPlayback\x18\x08 \x01(\x08H\x07\x88\x01\x01\x12\x13\n\x06height\x18\t \x01(\rH\x08\x88\x01\x01\x12\x12\n\x05width\x18\n \x01(\rH\t\x88\x01\x01\x12\x1a\n\rfileEncSha256\x18\x0b \x01(\x0cH\n\x88\x01\x01\x12<\n\x16interactiveAnnotations\x18\x0c \x03(\x0b\x32\x1c.proto.InteractiveAnnotation\x12\x17\n\ndirectPath\x18\r \x01(\tH\x0b\x88\x01\x01\x12\x1e\n\x11mediaKeyTimestamp\x18\x0e \x01(\x03H\x0c\x88\x01\x01\x12\x1a\n\rjpegThumbnail\x18\x10 \x01(\x0cH\r\x88\x01\x01\x12,\n\x0b\x63ontextInfo\x18\x11 \x01(\x0b\x32\x12.proto.ContextInfoH\x0e\x88\x01\x01\x12\x1d\n\x10streamingSidecar\x18\x12 \x01(\x0cH\x0f\x88\x01\x01\x12\x44\n\x0egifAttribution\x18\x13 \x01(\x0e\x32\'.proto.Message.VideoMessage.AttributionH\x10\x88\x01\x01\x12\x15\n\x08viewOnce\x18\x14 \x01(\x08H\x11\x88\x01\x01\x12 \n\x13thumbnailDirectPath\x18\x15 \x01(\tH\x12\x88\x01\x01\x12\x1c\n\x0fthumbnailSha256\x18\x16 \x01(\x0cH\x13\x88\x01\x01\x12\x1f\n\x12thumbnailEncSha256\x18\x17 \x01(\x0cH\x14\x88\x01\x01\x12\x16\n\tstaticUrl\x18\x18 \x01(\tH\x15\x88\x01\x01\x12\x31\n\x0b\x61nnotations\x18\x19 \x03(\x0b\x32\x1c.proto.InteractiveAnnotation\x12\x1f\n\x12\x61\x63\x63\x65ssibilityLabel\x18\x1a \x01(\tH\x16\x88\x01\x01\x12.\n\x0fprocessedVideos\x18\x1b \x03(\x0b\x32\x15.proto.ProcessedVideo\x12\x34\n\'externalShareFullVideoDurationInSeconds\x18\x1c \x01(\rH\x17\x88\x01\x01\x12,\n\x1fmotionPhotoPresentationOffsetMs\x18\x1d \x01(\x04H\x18\x88\x01\x01\x12\x18\n\x0bmetadataUrl\x18\x1e \x01(\tH\x19\x88\x01\x01\x12I\n\x0fvideoSourceType\x18\x1f \x01(\x0e\x32+.proto.Message.VideoMessage.VideoSourceTypeH\x1a\x88\x01\x01\x12:\n\x0emediaKeyDomain\x18  \x01(\x0e\x32\x1d.proto.Message.MediaKeyDomainH\x1b\x88\x01\x01\"8\n\x0b\x41ttribution\x12\x08\n\x04NONE\x10\x00\x12\t\n\x05GIPHY\x10\x01\x12\t\n\x05TENOR\x10\x02\x12\t\n\x05KLIPY\x10\x03\"3\n\x0fVideoSourceType\x12\x0e\n\nUSER_VIDEO\x10\x00\x12\x10\n\x0c\x41I_GENERATED\x10\x01\x42\x06\n\x04_urlB\x0b\n\t_mimetypeB\r\n\x0b_fileSha256B\r\n\x0b_fileLengthB\n\n\x08_secondsB\x0b\n\t_mediaKeyB\n\n\x08_captionB\x0e\n\x0c_gifPlaybackB\t\n\x07_heightB\x08\n\x06_widthB\x10\n\x0e_fileEncSha256B\r\n\x0b_directPathB\x14\n\x12_mediaKeyTimestampB\x10\n\x0e_jpegThumbnailB\x0e\n\x0c_contextInfoB\x13\n\x11_streamingSidecarB\x11\n\x0f_gifAttributionB\x0b\n\t_viewOnceB\x16\n\x14_thumbnailDirectPathB\x12\n\x10_thumbnailSha256B\x15\n\x13_thumbnailEncSha256B\x0c\n\n_staticUrlB\x15\n\x13_accessibilityLabelB*\n(_externalShareFullVideoDurationInSecondsB\"\n _motionPhotoPresentationOffsetMsB\x0e\n\x0c_metadataUrlB\x12\n\x10_videoSourceTypeB\x11\n\x0f_mediaKeyDomain\"\xb5\x01\n\x0fHistorySyncType\x12\x15\n\x11INITIAL_BOOTSTRAP\x10\x00\x12\x15\n\x11INITIAL_STATUS_V3\x10\x01\x12\x08\n\x04\x46ULL\x10\x02\x12\n\n\x06RECENT\x10\x03\x12\r\n\tPUSH_NAME\x10\x04\x12\x15\n\x11NON_BLOCKING_DATA\x10\x05\x12\r\n\tON_DEMAND\x10\x06\x12\x0e\n\nNO_HISTORY\x10\x07\x12\x19\n\x15MESSAGE_ACCESS_STATUS\x10\x08\"I\n\x0eMediaKeyDomain\x12\t\n\x05UNSET\x10\x00\x12\r\n\tE2EE_CHAT\x10\x01\x12\n\n\x06STATUS\x10\x02\x12\x08\n\x04\x43\x41PI\x10\x03\x12\x07\n\x03\x42OT\x10\x04\"\x9a\x03\n\x1cPeerDataOperationRequestType\x12\x12\n\x0eUPLOAD_STICKER\x10\x00\x12!\n\x1dSEND_RECENT_STICKER_BOOTSTRAP\x10\x01\x12\x19\n\x15GENERATE_LINK_PREVIEW\x10\x02\x12\x1a\n\x16HISTORY_SYNC_ON_DEMAND\x10\x03\x12\x1e\n\x1aPLACEHOLDER_MESSAGE_RESEND\x10\x04\x12\x1e\n\x1aWAFFLE_LINKING_NONCE_FETCH\x10\x05\x12\x1f\n\x1b\x46ULL_HISTORY_SYNC_ON_DEMAND\x10\x06\x12\x1e\n\x1a\x43OMPANION_META_NONCE_FETCH\x10\x07\x12+\n\'COMPANION_SYNCD_SNAPSHOT_FATAL_RECOVERY\x10\x08\x12(\n$COMPANION_CANONICAL_USER_NONCE_FETCH\x10\t\x12\x1c\n\x18HISTORY_SYNC_CHUNK_RETRY\x10\n\x12\x16\n\x12GALAXY_FLOW_ACTION\x10\x0b\"3\n\x0fPollContentType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x08\n\x04TEXT\x10\x01\x12\t\n\x05IMAGE\x10\x02\"\x1e\n\x08PollType\x12\x08\n\x04POLL\x10\x00\x12\x08\n\x04QUIZ\x10\x01\x42\x0f\n\r_conversationB\x1f\n\x1d_senderKeyDistributionMessageB\x0f\n\r_imageMessageB\x11\n\x0f_contactMessageB\x12\n\x10_locationMessageB\x16\n\x14_extendedTextMessageB\x12\n\x10_documentMessageB\x0f\n\r_audioMessageB\x0f\n\r_videoMessageB\x07\n\x05_callB\x07\n\x05_chatB\x12\n\x10_protocolMessageB\x17\n\x15_contactsArrayMessageB\x1a\n\x18_highlyStructuredMessageB-\n+_fastRatchetKeySenderKeyDistributionMessageB\x15\n\x13_sendPaymentMessageB\x16\n\x14_liveLocationMessageB\x18\n\x16_requestPaymentMessageB\x1f\n\x1d_declinePaymentRequestMessageB\x1e\n\x1c_cancelPaymentRequestMessageB\x12\n\x10_templateMessageB\x11\n\x0f_stickerMessageB\x15\n\x13_groupInviteMessageB\x1d\n\x1b_templateButtonReplyMessageB\x11\n\x0f_productMessageB\x14\n\x12_deviceSentMessageB\x15\n\x13_messageContextInfoB\x0e\n\x0c_listMessageB\x12\n\x10_viewOnceMessageB\x0f\n\r_orderMessageB\x16\n\x14_listResponseMessageB\x13\n\x11_ephemeralMessageB\x11\n\x0f_invoiceMessageB\x11\n\x0f_buttonsMessageB\x19\n\x17_buttonsResponseMessageB\x17\n\x15_paymentInviteMessageB\x15\n\x13_interactiveMessageB\x12\n\x10_reactionMessageB\x18\n\x16_stickerSyncRmrMessageB\x1d\n\x1b_interactiveResponseMessageB\x16\n\x14_pollCreationMessageB\x14\n\x12_pollUpdateMessageB\x14\n\x12_keepInChatMessageB\x1d\n\x1b_documentWithCaptionMessageB\x1c\n\x1a_requestPhoneNumberMessageB\x14\n\x12_viewOnceMessageV2B\x15\n\x13_encReactionMessageB\x10\n\x0e_editedMessageB\x1d\n\x1b_viewOnceMessageV2ExtensionB\x18\n\x16_pollCreationMessageV2B\x1f\n\x1d_scheduledCallCreationMessageB\x18\n\x16_groupMentionedMessageB\x13\n\x11_pinInChatMessageB\x18\n\x16_pollCreationMessageV3B\x1b\n\x19_scheduledCallEditMessageB\r\n\x0b_ptvMessageB\x13\n\x11_botInvokeMessageB\x12\n\x10_callLogMesssageB\x17\n\x15_messageHistoryBundleB\x14\n\x12_encCommentMessageB\x0f\n\r_bcallMessageB\x17\n\x15_lottieStickerMessageB\x0f\n\r_eventMessageB\x1a\n\x18_encEventResponseMessageB\x11\n\x0f_commentMessageB\x1f\n\x1d_newsletterAdminInviteMessageB\x15\n\x13_placeholderMessageB\x19\n\x17_secretEncryptedMessageB\x0f\n\r_albumMessageB\x12\n\x10_eventCoverImageB\x15\n\x13_stickerPackMessageB\x17\n\x15_statusMentionMessageB\x1c\n\x1a_pollResultSnapshotMessageB!\n\x1f_pollCreationOptionImageMessageB\x19\n\x17_associatedChildMessageB\x1c\n\x1a_groupStatusMentionMessageB\x18\n\x16_pollCreationMessageV4B\x11\n\x0f_statusAddYoursB\x15\n\x13_groupStatusMessageB\x16\n\x14_richResponseMessageB\x1c\n\x1a_statusNotificationMessageB\x16\n\x14_limitSharingMessageB\x11\n\x0f_botTaskMessageB\x12\n\x10_questionMessageB\x17\n\x15_messageHistoryNoticeB\x17\n\x15_groupStatusMessageV2B\x16\n\x14_botForwardedMessageB\x1e\n\x1c_statusQuestionAnswerMessageB\x17\n\x15_questionReplyMessageB\x1a\n\x18_questionResponseMessageB\x16\n\x14_statusQuotedMessageB\"\n _statusStickerInteractionMessageB\x18\n\x16_pollCreationMessageV5B$\n\"_newsletterFollowerInviteMessageV2B\x1e\n\x1c_pollResultSnapshotMessageV3\"\x92\x05\n\x0cMessageAddOn\x12\x43\n\x10messageAddOnType\x18\x01 \x01(\x0e\x32$.proto.MessageAddOn.MessageAddOnTypeH\x00\x88\x01\x01\x12)\n\x0cmessageAddOn\x18\x02 \x01(\x0b\x32\x0e.proto.MessageH\x01\x88\x01\x01\x12\x1e\n\x11senderTimestampMs\x18\x03 \x01(\x03H\x02\x88\x01\x01\x12\x1e\n\x11serverTimestampMs\x18\x04 \x01(\x03H\x03\x88\x01\x01\x12\x31\n\x06status\x18\x05 \x01(\x0e\x32\x1c.proto.WebMessageInfo.StatusH\x04\x88\x01\x01\x12=\n\x10\x61\x64\x64OnContextInfo\x18\x06 \x01(\x0b\x32\x1e.proto.MessageAddOnContextInfoH\x05\x88\x01\x01\x12/\n\x0fmessageAddOnKey\x18\x07 \x01(\x0b\x32\x11.proto.MessageKeyH\x06\x88\x01\x01\x12\x30\n\rlegacyMessage\x18\x08 \x01(\x0b\x32\x14.proto.LegacyMessageH\x07\x88\x01\x01\"e\n\x10MessageAddOnType\x12\r\n\tUNDEFINED\x10\x00\x12\x0c\n\x08REACTION\x10\x01\x12\x12\n\x0e\x45VENT_RESPONSE\x10\x02\x12\x0f\n\x0bPOLL_UPDATE\x10\x03\x12\x0f\n\x0bPIN_IN_CHAT\x10\x04\x42\x13\n\x11_messageAddOnTypeB\x0f\n\r_messageAddOnB\x14\n\x12_senderTimestampMsB\x14\n\x12_serverTimestampMsB\t\n\x07_statusB\x13\n\x11_addOnContextInfoB\x12\n\x10_messageAddOnKeyB\x10\n\x0e_legacyMessage\"\xd3\x01\n\x17MessageAddOnContextInfo\x12\'\n\x1amessageAddOnDurationInSecs\x18\x01 \x01(\rH\x00\x88\x01\x01\x12U\n\x16messageAddOnExpiryType\x18\x02 \x01(\x0e\x32\x30.proto.MessageContextInfo.MessageAddonExpiryTypeH\x01\x88\x01\x01\x42\x1d\n\x1b_messageAddOnDurationInSecsB\x19\n\x17_messageAddOnExpiryType\"\xba\x05\n\x12MessageAssociation\x12G\n\x0f\x61ssociationType\x18\x01 \x01(\x0e\x32).proto.MessageAssociation.AssociationTypeH\x00\x88\x01\x01\x12\x30\n\x10parentMessageKey\x18\x02 \x01(\x0b\x32\x11.proto.MessageKeyH\x01\x88\x01\x01\x12\x19\n\x0cmessageIndex\x18\x03 \x01(\x05H\x02\x88\x01\x01\"\xd3\x03\n\x0f\x41ssociationType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0f\n\x0bMEDIA_ALBUM\x10\x01\x12\x0e\n\nBOT_PLUGIN\x10\x02\x12\x15\n\x11\x45VENT_COVER_IMAGE\x10\x03\x12\x0f\n\x0bSTATUS_POLL\x10\x04\x12\x18\n\x14HD_VIDEO_DUAL_UPLOAD\x10\x05\x12\x1b\n\x17STATUS_EXTERNAL_RESHARE\x10\x06\x12\x0e\n\nMEDIA_POLL\x10\x07\x12\x14\n\x10STATUS_ADD_YOURS\x10\x08\x12\x17\n\x13STATUS_NOTIFICATION\x10\t\x12\x18\n\x14HD_IMAGE_DUAL_UPLOAD\x10\n\x12\x16\n\x12STICKER_ANNOTATION\x10\x0b\x12\x10\n\x0cMOTION_PHOTO\x10\x0c\x12\x16\n\x12STATUS_LINK_ACTION\x10\r\x12\x14\n\x10VIEW_ALL_REPLIES\x10\x0e\x12\x1f\n\x1bSTATUS_ADD_YOURS_AI_IMAGINE\x10\x0f\x12\x13\n\x0fSTATUS_QUESTION\x10\x10\x12\x1b\n\x17STATUS_ADD_YOURS_DIWALI\x10\x11\x12\x13\n\x0fSTATUS_REACTION\x10\x12\x12\x1a\n\x16HEVC_VIDEO_DUAL_UPLOAD\x10\x13\x42\x12\n\x10_associationTypeB\x13\n\x11_parentMessageKeyB\x0f\n\r_messageIndex\"\x91\t\n\x12MessageContextInfo\x12:\n\x12\x64\x65viceListMetadata\x18\x01 \x01(\x0b\x32\x19.proto.DeviceListMetadataH\x00\x88\x01\x01\x12&\n\x19\x64\x65viceListMetadataVersion\x18\x02 \x01(\x05H\x01\x88\x01\x01\x12\x1a\n\rmessageSecret\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x19\n\x0cpaddingBytes\x18\x04 \x01(\x0cH\x03\x88\x01\x01\x12\'\n\x1amessageAddOnDurationInSecs\x18\x05 \x01(\rH\x04\x88\x01\x01\x12\x1d\n\x10\x62otMessageSecret\x18\x06 \x01(\x0cH\x05\x88\x01\x01\x12,\n\x0b\x62otMetadata\x18\x07 \x01(\x0b\x32\x12.proto.BotMetadataH\x06\x88\x01\x01\x12\"\n\x15reportingTokenVersion\x18\x08 \x01(\x05H\x07\x88\x01\x01\x12U\n\x16messageAddOnExpiryType\x18\t \x01(\x0e\x32\x30.proto.MessageContextInfo.MessageAddonExpiryTypeH\x08\x88\x01\x01\x12:\n\x12messageAssociation\x18\n \x01(\x0b\x32\x19.proto.MessageAssociationH\t\x88\x01\x01\x12\x1d\n\x10\x63\x61piCreatedGroup\x18\x0b \x01(\x08H\n\x88\x01\x01\x12\x1b\n\x0esupportPayload\x18\x0c \x01(\tH\x0b\x88\x01\x01\x12.\n\x0climitSharing\x18\r \x01(\x0b\x32\x13.proto.LimitSharingH\x0c\x88\x01\x01\x12\x30\n\x0elimitSharingV2\x18\x0e \x01(\x0b\x32\x13.proto.LimitSharingH\r\x88\x01\x01\x12!\n\x08threadId\x18\x0f \x03(\x0b\x32\x0f.proto.ThreadID\x12<\n\x13weblinkRenderConfig\x18\x10 \x01(\x0e\x32\x1a.proto.WebLinkRenderConfigH\x0e\x88\x01\x01\"h\n\x16MessageAddonExpiryType\x12)\n%MESSAGE_ADDON_EXPIRY_TYPE_UNSPECIFIED\x10\x00\x12\n\n\x06STATIC\x10\x01\x12\x17\n\x13\x44\x45PENDENT_ON_PARENT\x10\x02\x42\x15\n\x13_deviceListMetadataB\x1c\n\x1a_deviceListMetadataVersionB\x10\n\x0e_messageSecretB\x0f\n\r_paddingBytesB\x1d\n\x1b_messageAddOnDurationInSecsB\x13\n\x11_botMessageSecretB\x0e\n\x0c_botMetadataB\x18\n\x16_reportingTokenVersionB\x19\n\x17_messageAddOnExpiryTypeB\x15\n\x13_messageAssociationB\x13\n\x11_capiCreatedGroupB\x11\n\x0f_supportPayloadB\x0f\n\r_limitSharingB\x11\n\x0f_limitSharingV2B\x16\n\x14_weblinkRenderConfig\"\x94\x01\n\nMessageKey\x12\x16\n\tremoteJid\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x13\n\x06\x66romMe\x18\x02 \x01(\x08H\x01\x88\x01\x01\x12\x0f\n\x02id\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x18\n\x0bparticipant\x18\x04 \x01(\tH\x03\x88\x01\x01\x42\x0c\n\n_remoteJidB\t\n\x07_fromMeB\x05\n\x03_idB\x0e\n\x0c_participant\"~\n\x14MessageSecretMessage\x12\x14\n\x07version\x18\x01 \x01(\x0fH\x00\x88\x01\x01\x12\x12\n\x05\x65ncIv\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x17\n\nencPayload\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x42\n\n\x08_versionB\x08\n\x06_encIvB\r\n\x0b_encPayload\"q\n\x05Money\x12\x12\n\x05value\x18\x01 \x01(\x03H\x00\x88\x01\x01\x12\x13\n\x06offset\x18\x02 \x01(\rH\x01\x88\x01\x01\x12\x19\n\x0c\x63urrencyCode\x18\x03 \x01(\tH\x02\x88\x01\x01\x42\x08\n\x06_valueB\t\n\x07_offsetB\x0f\n\r_currencyCode\"\x95\x16\n\rMsgOpaqueData\x12\x11\n\x04\x62ody\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x14\n\x07\x63\x61ption\x18\x03 \x01(\tH\x01\x88\x01\x01\x12\x10\n\x03lng\x18\x05 \x01(\x01H\x02\x88\x01\x01\x12\x13\n\x06isLive\x18\x06 \x01(\x08H\x03\x88\x01\x01\x12\x10\n\x03lat\x18\x07 \x01(\x01H\x04\x88\x01\x01\x12\x1e\n\x11paymentAmount1000\x18\x08 \x01(\x05H\x05\x88\x01\x01\x12\x1f\n\x12paymentNoteMsgBody\x18\t \x01(\tH\x06\x88\x01\x01\x12\x18\n\x0bmatchedText\x18\x0b \x01(\tH\x07\x88\x01\x01\x12\x12\n\x05title\x18\x0c \x01(\tH\x08\x88\x01\x01\x12\x18\n\x0b\x64\x65scription\x18\r \x01(\tH\t\x88\x01\x01\x12\x1e\n\x11\x66utureproofBuffer\x18\x0e \x01(\x0cH\n\x88\x01\x01\x12\x16\n\tclientUrl\x18\x0f \x01(\tH\x0b\x88\x01\x01\x12\x10\n\x03loc\x18\x10 \x01(\tH\x0c\x88\x01\x01\x12\x15\n\x08pollName\x18\x11 \x01(\tH\r\x88\x01\x01\x12\x34\n\x0bpollOptions\x18\x12 \x03(\x0b\x32\x1f.proto.MsgOpaqueData.PollOption\x12\'\n\x1apollSelectableOptionsCount\x18\x14 \x01(\rH\x0e\x88\x01\x01\x12\x1a\n\rmessageSecret\x18\x15 \x01(\x0cH\x0f\x88\x01\x01\x12\x1f\n\x12originalSelfAuthor\x18\x33 \x01(\tH\x10\x88\x01\x01\x12\x1e\n\x11senderTimestampMs\x18\x16 \x01(\x03H\x11\x88\x01\x01\x12 \n\x13pollUpdateParentKey\x18\x17 \x01(\tH\x12\x88\x01\x01\x12-\n\x0b\x65ncPollVote\x18\x18 \x01(\x0b\x32\x13.proto.PollEncValueH\x13\x88\x01\x01\x12\"\n\x15isSentCagPollCreation\x18\x1c \x01(\x08H\x14\x88\x01\x01\x12\x42\n\x0fpollContentType\x18* \x01(\x0e\x32$.proto.MsgOpaqueData.PollContentTypeH\x15\x88\x01\x01\x12\x34\n\x08pollType\x18. \x01(\x0e\x32\x1d.proto.MsgOpaqueData.PollTypeH\x16\x88\x01\x01\x12\x1f\n\x12\x63orrectOptionIndex\x18/ \x01(\x05H\x17\x88\x01\x01\x12\x46\n\x11pollVotesSnapshot\x18) \x01(\x0b\x32&.proto.MsgOpaqueData.PollVotesSnapshotH\x18\x88\x01\x01\x12(\n\x1b\x65ncReactionTargetMessageKey\x18\x19 \x01(\tH\x19\x88\x01\x01\x12\"\n\x15\x65ncReactionEncPayload\x18\x1a \x01(\x0cH\x1a\x88\x01\x01\x12\x1d\n\x10\x65ncReactionEncIv\x18\x1b \x01(\x0cH\x1b\x88\x01\x01\x12\x1d\n\x10\x62otMessageSecret\x18\x1d \x01(\x0cH\x1c\x88\x01\x01\x12\x1d\n\x10targetMessageKey\x18\x1e \x01(\tH\x1d\x88\x01\x01\x12\x17\n\nencPayload\x18\x1f \x01(\x0cH\x1e\x88\x01\x01\x12\x12\n\x05\x65ncIv\x18  \x01(\x0cH\x1f\x88\x01\x01\x12\x16\n\teventName\x18! \x01(\tH \x88\x01\x01\x12\x1c\n\x0fisEventCanceled\x18\" \x01(\x08H!\x88\x01\x01\x12\x1d\n\x10\x65ventDescription\x18# \x01(\tH\"\x88\x01\x01\x12\x1a\n\reventJoinLink\x18$ \x01(\tH#\x88\x01\x01\x12\x1b\n\x0e\x65ventStartTime\x18% \x01(\x03H$\x88\x01\x01\x12>\n\reventLocation\x18& \x01(\x0b\x32\".proto.MsgOpaqueData.EventLocationH%\x88\x01\x01\x12\x19\n\x0c\x65ventEndTime\x18( \x01(\x03H&\x88\x01\x01\x12!\n\x14\x65ventIsScheduledCall\x18, \x01(\x08H\'\x88\x01\x01\x12$\n\x17\x65ventExtraGuestsAllowed\x18- \x01(\x08H(\x88\x01\x01\x12\x1f\n\x12plainProtobufBytes\x18+ \x01(\x0cH)\x88\x01\x01\x1a\xfb\x01\n\rEventLocation\x12\x1c\n\x0f\x64\x65greesLatitude\x18\x01 \x01(\x01H\x00\x88\x01\x01\x12\x1d\n\x10\x64\x65greesLongitude\x18\x02 \x01(\x01H\x01\x88\x01\x01\x12\x11\n\x04name\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x14\n\x07\x61\x64\x64ress\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x10\n\x03url\x18\x05 \x01(\tH\x04\x88\x01\x01\x12\x1a\n\rjpegThumbnail\x18\x06 \x01(\x0cH\x05\x88\x01\x01\x42\x12\n\x10_degreesLatitudeB\x13\n\x11_degreesLongitudeB\x07\n\x05_nameB\n\n\x08_addressB\x06\n\x04_urlB\x10\n\x0e_jpegThumbnail\x1a\x44\n\nPollOption\x12\x11\n\x04name\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x11\n\x04hash\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x07\n\x05_nameB\x07\n\x05_hash\x1a\x85\x01\n\x10PollVoteSnapshot\x12\x34\n\x06option\x18\x01 \x01(\x0b\x32\x1f.proto.MsgOpaqueData.PollOptionH\x00\x88\x01\x01\x12\x1c\n\x0foptionVoteCount\x18\x02 \x01(\x05H\x01\x88\x01\x01\x42\t\n\x07_optionB\x12\n\x10_optionVoteCount\x1aM\n\x11PollVotesSnapshot\x12\x38\n\tpollVotes\x18\x01 \x03(\x0b\x32%.proto.MsgOpaqueData.PollVoteSnapshot\"3\n\x0fPollContentType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x08\n\x04TEXT\x10\x01\x12\t\n\x05IMAGE\x10\x02\"\x1e\n\x08PollType\x12\x08\n\x04POLL\x10\x00\x12\x08\n\x04QUIZ\x10\x01\x42\x07\n\x05_bodyB\n\n\x08_captionB\x06\n\x04_lngB\t\n\x07_isLiveB\x06\n\x04_latB\x14\n\x12_paymentAmount1000B\x15\n\x13_paymentNoteMsgBodyB\x0e\n\x0c_matchedTextB\x08\n\x06_titleB\x0e\n\x0c_descriptionB\x14\n\x12_futureproofBufferB\x0c\n\n_clientUrlB\x06\n\x04_locB\x0b\n\t_pollNameB\x1d\n\x1b_pollSelectableOptionsCountB\x10\n\x0e_messageSecretB\x15\n\x13_originalSelfAuthorB\x14\n\x12_senderTimestampMsB\x16\n\x14_pollUpdateParentKeyB\x0e\n\x0c_encPollVoteB\x18\n\x16_isSentCagPollCreationB\x12\n\x10_pollContentTypeB\x0b\n\t_pollTypeB\x15\n\x13_correctOptionIndexB\x14\n\x12_pollVotesSnapshotB\x1e\n\x1c_encReactionTargetMessageKeyB\x18\n\x16_encReactionEncPayloadB\x13\n\x11_encReactionEncIvB\x13\n\x11_botMessageSecretB\x13\n\x11_targetMessageKeyB\r\n\x0b_encPayloadB\x08\n\x06_encIvB\x0c\n\n_eventNameB\x12\n\x10_isEventCanceledB\x13\n\x11_eventDescriptionB\x10\n\x0e_eventJoinLinkB\x11\n\x0f_eventStartTimeB\x10\n\x0e_eventLocationB\x0f\n\r_eventEndTimeB\x17\n\x15_eventIsScheduledCallB\x1a\n\x18_eventExtraGuestsAllowedB\x15\n\x13_plainProtobufBytes\"\x8c\x01\n\x10MsgRowOpaqueData\x12-\n\ncurrentMsg\x18\x01 \x01(\x0b\x32\x14.proto.MsgOpaqueDataH\x00\x88\x01\x01\x12,\n\tquotedMsg\x18\x02 \x01(\x0b\x32\x14.proto.MsgOpaqueDataH\x01\x88\x01\x01\x42\r\n\x0b_currentMsgB\x0c\n\n_quotedMsg\"\x84\x02\n\x10NoiseCertificate\x12\x14\n\x07\x64\x65tails\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x16\n\tsignature\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x1a\xa7\x01\n\x07\x44\x65tails\x12\x13\n\x06serial\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x13\n\x06issuer\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x14\n\x07\x65xpires\x18\x03 \x01(\x04H\x02\x88\x01\x01\x12\x14\n\x07subject\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x10\n\x03key\x18\x05 \x01(\x0cH\x04\x88\x01\x01\x42\t\n\x07_serialB\t\n\x07_issuerB\n\n\x08_expiresB\n\n\x08_subjectB\x06\n\x04_keyB\n\n\x08_detailsB\x0c\n\n_signature\"\xd6\x01\n\x17NotificationMessageInfo\x12#\n\x03key\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x12$\n\x07message\x18\x02 \x01(\x0b\x32\x0e.proto.MessageH\x01\x88\x01\x01\x12\x1d\n\x10messageTimestamp\x18\x03 \x01(\x04H\x02\x88\x01\x01\x12\x18\n\x0bparticipant\x18\x04 \x01(\tH\x03\x88\x01\x01\x42\x06\n\x04_keyB\n\n\x08_messageB\x13\n\x11_messageTimestampB\x0e\n\x0c_participant\"\xbc\x02\n\x14NotificationSettings\x12\x1b\n\x0emessageVibrate\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x19\n\x0cmessagePopup\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x19\n\x0cmessageLight\x18\x03 \x01(\tH\x02\x88\x01\x01\x12%\n\x18lowPriorityNotifications\x18\x04 \x01(\x08H\x03\x88\x01\x01\x12\x1b\n\x0ereactionsMuted\x18\x05 \x01(\x08H\x04\x88\x01\x01\x12\x18\n\x0b\x63\x61llVibrate\x18\x06 \x01(\tH\x05\x88\x01\x01\x42\x11\n\x0f_messageVibrateB\x0f\n\r_messagePopupB\x0f\n\r_messageLightB\x1b\n\x19_lowPriorityNotificationsB\x11\n\x0f_reactionsMutedB\x0e\n\x0c_callVibrate\"\xaa\x01\n\x0ePairingRequest\x12\x1f\n\x12\x63ompanionPublicKey\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12!\n\x14\x63ompanionIdentityKey\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x16\n\tadvSecret\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x42\x15\n\x13_companionPublicKeyB\x17\n\x15_companionIdentityKeyB\x0c\n\n_advSecret\"\xc9\x01\n\x0fPastParticipant\x12\x14\n\x07userJid\x18\x01 \x01(\tH\x00\x88\x01\x01\x12<\n\x0bleaveReason\x18\x02 \x01(\x0e\x32\".proto.PastParticipant.LeaveReasonH\x01\x88\x01\x01\x12\x14\n\x07leaveTs\x18\x03 \x01(\x04H\x02\x88\x01\x01\"$\n\x0bLeaveReason\x12\x08\n\x04LEFT\x10\x00\x12\x0b\n\x07REMOVED\x10\x01\x42\n\n\x08_userJidB\x0e\n\x0c_leaveReasonB\n\n\x08_leaveTs\"h\n\x10PastParticipants\x12\x15\n\x08groupJid\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x30\n\x10pastParticipants\x18\x02 \x03(\x0b\x32\x16.proto.PastParticipantB\x0b\n\t_groupJid\"\xe9\x05\n\x0ePatchDebugData\x12\x1a\n\rcurrentLthash\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x16\n\tnewLthash\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x19\n\x0cpatchVersion\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x1b\n\x0e\x63ollectionName\x18\x04 \x01(\x0cH\x03\x88\x01\x01\x12\x34\n\'firstFourBytesFromAHashOfSnapshotMacKey\x18\x05 \x01(\x0cH\x04\x88\x01\x01\x12\x1e\n\x11newLthashSubtract\x18\x06 \x01(\x0cH\x05\x88\x01\x01\x12\x16\n\tnumberAdd\x18\x07 \x01(\x05H\x06\x88\x01\x01\x12\x19\n\x0cnumberRemove\x18\x08 \x01(\x05H\x07\x88\x01\x01\x12\x1b\n\x0enumberOverride\x18\t \x01(\x05H\x08\x88\x01\x01\x12;\n\x0esenderPlatform\x18\n \x01(\x0e\x32\x1e.proto.PatchDebugData.PlatformH\t\x88\x01\x01\x12\x1c\n\x0fisSenderPrimary\x18\x0b \x01(\x08H\n\x88\x01\x01\"\x8a\x01\n\x08Platform\x12\x0b\n\x07\x41NDROID\x10\x00\x12\x08\n\x04SMBA\x10\x01\x12\n\n\x06IPHONE\x10\x02\x12\x08\n\x04SMBI\x10\x03\x12\x07\n\x03WEB\x10\x04\x12\x07\n\x03UWP\x10\x05\x12\n\n\x06\x44\x41RWIN\x10\x06\x12\x08\n\x04IPAD\x10\x07\x12\n\n\x06WEAROS\x10\x08\x12\x08\n\x04WASG\x10\t\x12\t\n\x05WEARM\x10\n\x12\x08\n\x04\x43\x41PI\x10\x0b\x42\x10\n\x0e_currentLthashB\x0c\n\n_newLthashB\x0f\n\r_patchVersionB\x11\n\x0f_collectionNameB*\n(_firstFourBytesFromAHashOfSnapshotMacKeyB\x14\n\x12_newLthashSubtractB\x0c\n\n_numberAddB\x0f\n\r_numberRemoveB\x11\n\x0f_numberOverrideB\x11\n\x0f_senderPlatformB\x12\n\x10_isSenderPrimary\"\xc2\x05\n\x11PaymentBackground\x12\x0f\n\x02id\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x17\n\nfileLength\x18\x02 \x01(\x04H\x01\x88\x01\x01\x12\x12\n\x05width\x18\x03 \x01(\rH\x02\x88\x01\x01\x12\x13\n\x06height\x18\x04 \x01(\rH\x03\x88\x01\x01\x12\x15\n\x08mimetype\x18\x05 \x01(\tH\x04\x88\x01\x01\x12\x1c\n\x0fplaceholderArgb\x18\x06 \x01(\x07H\x05\x88\x01\x01\x12\x15\n\x08textArgb\x18\x07 \x01(\x07H\x06\x88\x01\x01\x12\x18\n\x0bsubtextArgb\x18\x08 \x01(\x07H\x07\x88\x01\x01\x12:\n\tmediaData\x18\t \x01(\x0b\x32\".proto.PaymentBackground.MediaDataH\x08\x88\x01\x01\x12\x30\n\x04type\x18\n \x01(\x0e\x32\x1d.proto.PaymentBackground.TypeH\t\x88\x01\x01\x1a\xe3\x01\n\tMediaData\x12\x15\n\x08mediaKey\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x1e\n\x11mediaKeyTimestamp\x18\x02 \x01(\x03H\x01\x88\x01\x01\x12\x17\n\nfileSha256\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x1a\n\rfileEncSha256\x18\x04 \x01(\x0cH\x03\x88\x01\x01\x12\x17\n\ndirectPath\x18\x05 \x01(\tH\x04\x88\x01\x01\x42\x0b\n\t_mediaKeyB\x14\n\x12_mediaKeyTimestampB\r\n\x0b_fileSha256B\x10\n\x0e_fileEncSha256B\r\n\x0b_directPath\" \n\x04Type\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x01\x42\x05\n\x03_idB\r\n\x0b_fileLengthB\x08\n\x06_widthB\t\n\x07_heightB\x0b\n\t_mimetypeB\x12\n\x10_placeholderArgbB\x0b\n\t_textArgbB\x0e\n\x0c_subtextArgbB\x0c\n\n_mediaDataB\x07\n\x05_type\"\x82\r\n\x0bPaymentInfo\x12<\n\x12\x63urrencyDeprecated\x18\x01 \x01(\x0e\x32\x1b.proto.PaymentInfo.CurrencyH\x00\x88\x01\x01\x12\x17\n\namount1000\x18\x02 \x01(\x04H\x01\x88\x01\x01\x12\x18\n\x0breceiverJid\x18\x03 \x01(\tH\x02\x88\x01\x01\x12.\n\x06status\x18\x04 \x01(\x0e\x32\x19.proto.PaymentInfo.StatusH\x03\x88\x01\x01\x12!\n\x14transactionTimestamp\x18\x05 \x01(\x04H\x04\x88\x01\x01\x12\x31\n\x11requestMessageKey\x18\x06 \x01(\x0b\x32\x11.proto.MessageKeyH\x05\x88\x01\x01\x12\x1c\n\x0f\x65xpiryTimestamp\x18\x07 \x01(\x04H\x06\x88\x01\x01\x12\x1a\n\rfutureproofed\x18\x08 \x01(\x08H\x07\x88\x01\x01\x12\x15\n\x08\x63urrency\x18\t \x01(\tH\x08\x88\x01\x01\x12\x34\n\ttxnStatus\x18\n \x01(\x0e\x32\x1c.proto.PaymentInfo.TxnStatusH\t\x88\x01\x01\x12\x1e\n\x11useNoviFiatFormat\x18\x0b \x01(\x08H\n\x88\x01\x01\x12(\n\rprimaryAmount\x18\x0c \x01(\x0b\x32\x0c.proto.MoneyH\x0b\x88\x01\x01\x12)\n\x0e\x65xchangeAmount\x18\r \x01(\x0b\x32\x0c.proto.MoneyH\x0c\x88\x01\x01\")\n\x08\x43urrency\x12\x14\n\x10UNKNOWN_CURRENCY\x10\x00\x12\x07\n\x03INR\x10\x01\"\xcc\x01\n\x06Status\x12\x12\n\x0eUNKNOWN_STATUS\x10\x00\x12\x0e\n\nPROCESSING\x10\x01\x12\x08\n\x04SENT\x10\x02\x12\x12\n\x0eNEED_TO_ACCEPT\x10\x03\x12\x0c\n\x08\x43OMPLETE\x10\x04\x12\x16\n\x12\x43OULD_NOT_COMPLETE\x10\x05\x12\x0c\n\x08REFUNDED\x10\x06\x12\x0b\n\x07\x45XPIRED\x10\x07\x12\x0c\n\x08REJECTED\x10\x08\x12\r\n\tCANCELLED\x10\t\x12\x15\n\x11WAITING_FOR_PAYER\x10\n\x12\x0b\n\x07WAITING\x10\x0b\"\x99\x05\n\tTxnStatus\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x11\n\rPENDING_SETUP\x10\x01\x12\x1a\n\x16PENDING_RECEIVER_SETUP\x10\x02\x12\x08\n\x04INIT\x10\x03\x12\x0b\n\x07SUCCESS\x10\x04\x12\r\n\tCOMPLETED\x10\x05\x12\n\n\x06\x46\x41ILED\x10\x06\x12\x0f\n\x0b\x46\x41ILED_RISK\x10\x07\x12\x15\n\x11\x46\x41ILED_PROCESSING\x10\x08\x12\x1e\n\x1a\x46\x41ILED_RECEIVER_PROCESSING\x10\t\x12\r\n\tFAILED_DA\x10\n\x12\x13\n\x0f\x46\x41ILED_DA_FINAL\x10\x0b\x12\x10\n\x0cREFUNDED_TXN\x10\x0c\x12\x11\n\rREFUND_FAILED\x10\r\x12\x1c\n\x18REFUND_FAILED_PROCESSING\x10\x0e\x12\x14\n\x10REFUND_FAILED_DA\x10\x0f\x12\x0f\n\x0b\x45XPIRED_TXN\x10\x10\x12\x11\n\rAUTH_CANCELED\x10\x11\x12!\n\x1d\x41UTH_CANCEL_FAILED_PROCESSING\x10\x12\x12\x16\n\x12\x41UTH_CANCEL_FAILED\x10\x13\x12\x10\n\x0c\x43OLLECT_INIT\x10\x14\x12\x13\n\x0f\x43OLLECT_SUCCESS\x10\x15\x12\x12\n\x0e\x43OLLECT_FAILED\x10\x16\x12\x17\n\x13\x43OLLECT_FAILED_RISK\x10\x17\x12\x14\n\x10\x43OLLECT_REJECTED\x10\x18\x12\x13\n\x0f\x43OLLECT_EXPIRED\x10\x19\x12\x14\n\x10\x43OLLECT_CANCELED\x10\x1a\x12\x16\n\x12\x43OLLECT_CANCELLING\x10\x1b\x12\r\n\tIN_REVIEW\x10\x1c\x12\x14\n\x10REVERSAL_SUCCESS\x10\x1d\x12\x14\n\x10REVERSAL_PENDING\x10\x1e\x12\x12\n\x0eREFUND_PENDING\x10\x1f\x42\x15\n\x13_currencyDeprecatedB\r\n\x0b_amount1000B\x0e\n\x0c_receiverJidB\t\n\x07_statusB\x17\n\x15_transactionTimestampB\x14\n\x12_requestMessageKeyB\x12\n\x10_expiryTimestampB\x10\n\x0e_futureproofedB\x0b\n\t_currencyB\x0c\n\n_txnStatusB\x14\n\x12_useNoviFiatFormatB\x10\n\x0e_primaryAmountB\x11\n\x0f_exchangeAmount\"W\n\x17PhoneNumberToLIDMapping\x12\x12\n\x05pnJid\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x13\n\x06lidJid\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x08\n\x06_pnJidB\t\n\x07_lidJid\"}\n\x0bPhotoChange\x12\x15\n\x08oldPhoto\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x15\n\x08newPhoto\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x17\n\nnewPhotoId\x18\x03 \x01(\rH\x02\x88\x01\x01\x42\x0b\n\t_oldPhotoB\x0b\n\t_newPhotoB\r\n\x0b_newPhotoId\"\xf7\x02\n\tPinInChat\x12(\n\x04type\x18\x01 \x01(\x0e\x32\x15.proto.PinInChat.TypeH\x00\x88\x01\x01\x12#\n\x03key\x18\x02 \x01(\x0b\x32\x11.proto.MessageKeyH\x01\x88\x01\x01\x12\x1e\n\x11senderTimestampMs\x18\x03 \x01(\x03H\x02\x88\x01\x01\x12\x1e\n\x11serverTimestampMs\x18\x04 \x01(\x03H\x03\x88\x01\x01\x12\x44\n\x17messageAddOnContextInfo\x18\x05 \x01(\x0b\x32\x1e.proto.MessageAddOnContextInfoH\x04\x88\x01\x01\"<\n\x04Type\x12\x10\n\x0cUNKNOWN_TYPE\x10\x00\x12\x0f\n\x0bPIN_FOR_ALL\x10\x01\x12\x11\n\rUNPIN_FOR_ALL\x10\x02\x42\x07\n\x05_typeB\x06\n\x04_keyB\x14\n\x12_senderTimestampMsB\x14\n\x12_serverTimestampMsB\x1a\n\x18_messageAddOnContextInfo\"\x87\x01\n\x05Point\x12\x18\n\x0bxDeprecated\x18\x01 \x01(\x05H\x00\x88\x01\x01\x12\x18\n\x0byDeprecated\x18\x02 \x01(\x05H\x01\x88\x01\x01\x12\x0e\n\x01x\x18\x03 \x01(\x01H\x02\x88\x01\x01\x12\x0e\n\x01y\x18\x04 \x01(\x01H\x03\x88\x01\x01\x42\x0e\n\x0c_xDeprecatedB\x0e\n\x0c_yDeprecatedB\x04\n\x02_xB\x04\n\x02_y\"J\n\x16PollAdditionalMetadata\x12\x1c\n\x0fpollInvalidated\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\x12\n\x10_pollInvalidated\"T\n\x0cPollEncValue\x12\x17\n\nencPayload\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x12\n\x05\x65ncIv\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x42\r\n\x0b_encPayloadB\x08\n\x06_encIv\"\xa3\x02\n\nPollUpdate\x12\x34\n\x14pollUpdateMessageKey\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x12\x31\n\x04vote\x18\x02 \x01(\x0b\x32\x1e.proto.Message.PollVoteMessageH\x01\x88\x01\x01\x12\x1e\n\x11senderTimestampMs\x18\x03 \x01(\x03H\x02\x88\x01\x01\x12\x1e\n\x11serverTimestampMs\x18\x04 \x01(\x03H\x03\x88\x01\x01\x12\x13\n\x06unread\x18\x05 \x01(\x08H\x04\x88\x01\x01\x42\x17\n\x15_pollUpdateMessageKeyB\x07\n\x05_voteB\x14\n\x12_senderTimestampMsB\x14\n\x12_serverTimestampMsB\t\n\x07_unread\"}\n\x15PreKeyRecordStructure\x12\x0f\n\x02id\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x16\n\tpublicKey\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x17\n\nprivateKey\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x42\x05\n\x03_idB\x0c\n\n_publicKeyB\r\n\x0b_privateKey\"\x87\x02\n\x13PreKeySignalMessage\x12\x1b\n\x0eregistrationId\x18\x05 \x01(\rH\x00\x88\x01\x01\x12\x15\n\x08preKeyId\x18\x01 \x01(\rH\x01\x88\x01\x01\x12\x1b\n\x0esignedPreKeyId\x18\x06 \x01(\rH\x02\x88\x01\x01\x12\x14\n\x07\x62\x61seKey\x18\x02 \x01(\x0cH\x03\x88\x01\x01\x12\x18\n\x0bidentityKey\x18\x03 \x01(\x0cH\x04\x88\x01\x01\x12\x14\n\x07message\x18\x04 \x01(\x0cH\x05\x88\x01\x01\x42\x11\n\x0f_registrationIdB\x0b\n\t_preKeyIdB\x11\n\x0f_signedPreKeyIdB\n\n\x08_baseKeyB\x0e\n\x0c_identityKeyB\n\n\x08_message\"H\n\x12PremiumMessageInfo\x12\x1d\n\x10serverCampaignId\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x13\n\x11_serverCampaignId\"^\n\x18PrimaryEphemeralIdentity\x12\x16\n\tpublicKey\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x12\n\x05nonce\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x42\x0c\n\n_publicKeyB\x08\n\x06_nonce\"\xff\x02\n\x0eProcessedVideo\x12\x17\n\ndirectPath\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x17\n\nfileSha256\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x13\n\x06height\x18\x03 \x01(\rH\x02\x88\x01\x01\x12\x12\n\x05width\x18\x04 \x01(\rH\x03\x88\x01\x01\x12\x17\n\nfileLength\x18\x05 \x01(\x04H\x04\x88\x01\x01\x12\x14\n\x07\x62itrate\x18\x06 \x01(\rH\x05\x88\x01\x01\x12\x38\n\x07quality\x18\x07 \x01(\x0e\x32\".proto.ProcessedVideo.VideoQualityH\x06\x88\x01\x01\x12\x14\n\x0c\x63\x61pabilities\x18\x08 \x03(\t\"9\n\x0cVideoQuality\x12\r\n\tUNDEFINED\x10\x00\x12\x07\n\x03LOW\x10\x01\x12\x07\n\x03MID\x10\x02\x12\x08\n\x04HIGH\x10\x03\x42\r\n\x0b_directPathB\r\n\x0b_fileSha256B\t\n\x07_heightB\x08\n\x06_widthB\r\n\x0b_fileLengthB\n\n\x08_bitrateB\n\n\x08_quality\"\x9d\x01\n\x0fProloguePayload\x12\'\n\x1a\x63ompanionEphemeralIdentity\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x33\n\ncommitment\x18\x02 \x01(\x0b\x32\x1a.proto.CompanionCommitmentH\x01\x88\x01\x01\x42\x1d\n\x1b_companionEphemeralIdentityB\r\n\x0b_commitment\"F\n\x08Pushname\x12\x0f\n\x02id\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x15\n\x08pushname\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x05\n\x03_idB\x0b\n\t_pushname\"n\n\x12QuarantinedMessage\x12\x19\n\x0coriginalData\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x1a\n\rextractedText\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x0f\n\r_originalDataB\x10\n\x0e_extractedText\"\xd3\x01\n\x08Reaction\x12#\n\x03key\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x12\x11\n\x04text\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x18\n\x0bgroupingKey\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x1e\n\x11senderTimestampMs\x18\x04 \x01(\x03H\x03\x88\x01\x01\x12\x13\n\x06unread\x18\x05 \x01(\x08H\x04\x88\x01\x01\x42\x06\n\x04_keyB\x07\n\x05_textB\x0e\n\x0c_groupingKeyB\x14\n\x12_senderTimestampMsB\t\n\x07_unread\"Q\n\x11RecentEmojiWeight\x12\x12\n\x05\x65moji\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x13\n\x06weight\x18\x02 \x01(\x02H\x01\x88\x01\x01\x42\x08\n\x06_emojiB\t\n\x07_weight\"\x8d\x01\n\x0fRecordStructure\x12\x34\n\x0e\x63urrentSession\x18\x01 \x01(\x0b\x32\x17.proto.SessionStructureH\x00\x88\x01\x01\x12\x31\n\x10previousSessions\x18\x02 \x03(\x0b\x32\x17.proto.SessionStructureB\x11\n\x0f_currentSession\"\xbc\x01\n\nReportable\x12\x17\n\nminVersion\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x17\n\nmaxVersion\x18\x02 \x01(\rH\x01\x88\x01\x01\x12$\n\x17notReportableMinVersion\x18\x03 \x01(\rH\x02\x88\x01\x01\x12\x12\n\x05never\x18\x04 \x01(\x08H\x03\x88\x01\x01\x42\r\n\x0b_minVersionB\r\n\x0b_maxVersionB\x1a\n\x18_notReportableMinVersionB\x08\n\x06_never\"@\n\x12ReportingTokenInfo\x12\x19\n\x0creportingTag\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x42\x0f\n\r_reportingTag\"\xa8\x01\n\x1cSenderKeyDistributionMessage\x12\x0f\n\x02id\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x16\n\titeration\x18\x02 \x01(\rH\x01\x88\x01\x01\x12\x15\n\x08\x63hainKey\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x17\n\nsigningKey\x18\x04 \x01(\x0cH\x03\x88\x01\x01\x42\x05\n\x03_idB\x0c\n\n_iterationB\x0b\n\t_chainKeyB\r\n\x0b_signingKey\"x\n\x10SenderKeyMessage\x12\x0f\n\x02id\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x16\n\titeration\x18\x02 \x01(\rH\x01\x88\x01\x01\x12\x17\n\nciphertext\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x42\x05\n\x03_idB\x0c\n\n_iterationB\r\n\x0b_ciphertext\"S\n\x18SenderKeyRecordStructure\x12\x37\n\x0fsenderKeyStates\x18\x01 \x03(\x0b\x32\x1e.proto.SenderKeyStateStructure\"\xd3\x04\n\x17SenderKeyStateStructure\x12\x18\n\x0bsenderKeyId\x18\x01 \x01(\rH\x00\x88\x01\x01\x12J\n\x0esenderChainKey\x18\x02 \x01(\x0b\x32-.proto.SenderKeyStateStructure.SenderChainKeyH\x01\x88\x01\x01\x12N\n\x10senderSigningKey\x18\x03 \x01(\x0b\x32/.proto.SenderKeyStateStructure.SenderSigningKeyH\x02\x88\x01\x01\x12J\n\x11senderMessageKeys\x18\x04 \x03(\x0b\x32/.proto.SenderKeyStateStructure.SenderMessageKey\x1aR\n\x0eSenderChainKey\x12\x16\n\titeration\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x11\n\x04seed\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x42\x0c\n\n_iterationB\x07\n\x05_seed\x1aT\n\x10SenderMessageKey\x12\x16\n\titeration\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x11\n\x04seed\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x42\x0c\n\n_iterationB\x07\n\x05_seed\x1aT\n\x10SenderSigningKey\x12\x13\n\x06public\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x14\n\x07private\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x42\t\n\x07_publicB\n\n\x08_privateB\x0e\n\x0c_senderKeyIdB\x11\n\x0f_senderChainKeyB\x13\n\x11_senderSigningKey\"8\n\x12ServerErrorReceipt\x12\x15\n\x08stanzaId\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x0b\n\t_stanzaId\"\xfe\r\n\x10SessionStructure\x12\x1b\n\x0esessionVersion\x18\x01 \x01(\rH\x00\x88\x01\x01\x12 \n\x13localIdentityPublic\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12!\n\x14remoteIdentityPublic\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x14\n\x07rootKey\x18\x04 \x01(\x0cH\x03\x88\x01\x01\x12\x1c\n\x0fpreviousCounter\x18\x05 \x01(\rH\x04\x88\x01\x01\x12\x37\n\x0bsenderChain\x18\x06 \x01(\x0b\x32\x1d.proto.SessionStructure.ChainH\x05\x88\x01\x01\x12\x35\n\x0ereceiverChains\x18\x07 \x03(\x0b\x32\x1d.proto.SessionStructure.Chain\x12K\n\x12pendingKeyExchange\x18\x08 \x01(\x0b\x32*.proto.SessionStructure.PendingKeyExchangeH\x06\x88\x01\x01\x12\x41\n\rpendingPreKey\x18\t \x01(\x0b\x32%.proto.SessionStructure.PendingPreKeyH\x07\x88\x01\x01\x12!\n\x14remoteRegistrationId\x18\n \x01(\rH\x08\x88\x01\x01\x12 \n\x13localRegistrationId\x18\x0b \x01(\rH\t\x88\x01\x01\x12\x19\n\x0cneedsRefresh\x18\x0c \x01(\x08H\n\x88\x01\x01\x12\x19\n\x0c\x61liceBaseKey\x18\r \x01(\x0cH\x0b\x88\x01\x01\x1a\xd7\x03\n\x05\x43hain\x12\x1d\n\x10senderRatchetKey\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12$\n\x17senderRatchetKeyPrivate\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12=\n\x08\x63hainKey\x18\x03 \x01(\x0b\x32&.proto.SessionStructure.Chain.ChainKeyH\x02\x88\x01\x01\x12=\n\x0bmessageKeys\x18\x04 \x03(\x0b\x32(.proto.SessionStructure.Chain.MessageKey\x1a\x42\n\x08\x43hainKey\x12\x12\n\x05index\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x10\n\x03key\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x42\x08\n\x06_indexB\x06\n\x04_key\x1a\x88\x01\n\nMessageKey\x12\x12\n\x05index\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x16\n\tcipherKey\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x13\n\x06macKey\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x0f\n\x02iv\x18\x04 \x01(\x0cH\x03\x88\x01\x01\x42\x08\n\x06_indexB\x0c\n\n_cipherKeyB\t\n\x07_macKeyB\x05\n\x03_ivB\x13\n\x11_senderRatchetKeyB\x1a\n\x18_senderRatchetKeyPrivateB\x0b\n\t_chainKey\x1a\x86\x03\n\x12PendingKeyExchange\x12\x15\n\x08sequence\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x19\n\x0clocalBaseKey\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12 \n\x13localBaseKeyPrivate\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x1c\n\x0flocalRatchetKey\x18\x04 \x01(\x0cH\x03\x88\x01\x01\x12#\n\x16localRatchetKeyPrivate\x18\x05 \x01(\x0cH\x04\x88\x01\x01\x12\x1d\n\x10localIdentityKey\x18\x07 \x01(\x0cH\x05\x88\x01\x01\x12$\n\x17localIdentityKeyPrivate\x18\x08 \x01(\x0cH\x06\x88\x01\x01\x42\x0b\n\t_sequenceB\x0f\n\r_localBaseKeyB\x16\n\x14_localBaseKeyPrivateB\x12\n\x10_localRatchetKeyB\x19\n\x17_localRatchetKeyPrivateB\x13\n\x11_localIdentityKeyB\x1a\n\x18_localIdentityKeyPrivate\x1a\x85\x01\n\rPendingPreKey\x12\x15\n\x08preKeyId\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x1b\n\x0esignedPreKeyId\x18\x03 \x01(\x05H\x01\x88\x01\x01\x12\x14\n\x07\x62\x61seKey\x18\x02 \x01(\x0cH\x02\x88\x01\x01\x42\x0b\n\t_preKeyIdB\x11\n\x0f_signedPreKeyIdB\n\n\x08_baseKeyB\x11\n\x0f_sessionVersionB\x16\n\x14_localIdentityPublicB\x17\n\x15_remoteIdentityPublicB\n\n\x08_rootKeyB\x12\n\x10_previousCounterB\x0e\n\x0c_senderChainB\x15\n\x13_pendingKeyExchangeB\x10\n\x0e_pendingPreKeyB\x17\n\x15_remoteRegistrationIdB\x16\n\x14_localRegistrationIdB\x0f\n\r_needsRefreshB\x0f\n\r_aliceBaseKey\"\xcd\x01\n\x1bSessionTransparencyMetadata\x12\x1b\n\x0e\x64isclaimerText\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x12\n\x05hcaId\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x44\n\x17sessionTransparencyType\x18\x03 \x01(\x0e\x32\x1e.proto.SessionTransparencyTypeH\x02\x88\x01\x01\x42\x11\n\x0f_disclaimerTextB\x08\n\x06_hcaIdB\x1a\n\x18_sessionTransparencyType\"\xb3\x01\n\rSignalMessage\x12\x17\n\nratchetKey\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x14\n\x07\x63ounter\x18\x02 \x01(\rH\x01\x88\x01\x01\x12\x1c\n\x0fpreviousCounter\x18\x03 \x01(\rH\x02\x88\x01\x01\x12\x17\n\nciphertext\x18\x04 \x01(\x0cH\x03\x88\x01\x01\x42\r\n\x0b_ratchetKeyB\n\n\x08_counterB\x12\n\x10_previousCounterB\r\n\x0b_ciphertext\"\xcf\x01\n\x1bSignedPreKeyRecordStructure\x12\x0f\n\x02id\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x16\n\tpublicKey\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x17\n\nprivateKey\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x16\n\tsignature\x18\x04 \x01(\x0cH\x03\x88\x01\x01\x12\x16\n\ttimestamp\x18\x05 \x01(\x06H\x04\x88\x01\x01\x42\x05\n\x03_idB\x0c\n\n_publicKeyB\r\n\x0b_privateKeyB\x0c\n\n_signatureB\x0c\n\n_timestamp\"\x8e\x11\n\x11StatusAttribution\x12\x30\n\x04type\x18\x01 \x01(\x0e\x32\x1d.proto.StatusAttribution.TypeH\x01\x88\x01\x01\x12\x16\n\tactionUrl\x18\x02 \x01(\tH\x02\x88\x01\x01\x12?\n\rstatusReshare\x18\x03 \x01(\x0b\x32&.proto.StatusAttribution.StatusReshareH\x00\x12?\n\rexternalShare\x18\x04 \x01(\x0b\x32&.proto.StatusAttribution.ExternalShareH\x00\x12/\n\x05music\x18\x05 \x01(\x0b\x32\x1e.proto.StatusAttribution.MusicH\x00\x12;\n\x0bgroupStatus\x18\x06 \x01(\x0b\x32$.proto.StatusAttribution.GroupStatusH\x00\x12?\n\rrlAttribution\x18\x07 \x01(\x0b\x32&.proto.StatusAttribution.RLAttributionH\x00\x12M\n\x14\x61iCreatedAttribution\x18\x08 \x01(\x0b\x32-.proto.StatusAttribution.AiCreatedAttributionH\x00\x1a\x97\x01\n\x14\x41iCreatedAttribution\x12I\n\x06source\x18\x01 \x01(\x0e\x32\x34.proto.StatusAttribution.AiCreatedAttribution.SourceH\x00\x88\x01\x01\")\n\x06Source\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x12\n\x0eSTATUS_MIMICRY\x10\x01\x42\t\n\x07_source\x1a\x8b\x03\n\rExternalShare\x12\x16\n\tactionUrl\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x42\n\x06source\x18\x02 \x01(\x0e\x32-.proto.StatusAttribution.ExternalShare.SourceH\x01\x88\x01\x01\x12\x15\n\x08\x64uration\x18\x03 \x01(\x05H\x02\x88\x01\x01\x12\x1e\n\x11\x61\x63tionFallbackUrl\x18\x04 \x01(\tH\x03\x88\x01\x01\"\xaa\x01\n\x06Source\x12\x0b\n\x07UNKNOWN\x10\x00\x12\r\n\tINSTAGRAM\x10\x01\x12\x0c\n\x08\x46\x41\x43\x45\x42OOK\x10\x02\x12\r\n\tMESSENGER\x10\x03\x12\x0b\n\x07SPOTIFY\x10\x04\x12\x0b\n\x07YOUTUBE\x10\x05\x12\r\n\tPINTEREST\x10\x06\x12\x0b\n\x07THREADS\x10\x07\x12\x0f\n\x0b\x41PPLE_MUSIC\x10\x08\x12\r\n\tSHARECHAT\x10\t\x12\x11\n\rGOOGLE_PHOTOS\x10\nB\x0c\n\n_actionUrlB\t\n\x07_sourceB\x0b\n\t_durationB\x14\n\x12_actionFallbackUrl\x1a\x33\n\x0bGroupStatus\x12\x16\n\tauthorJid\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x0c\n\n_authorJid\x1a\xeb\x01\n\x05Music\x12\x17\n\nauthorName\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x13\n\x06songId\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x12\n\x05title\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x13\n\x06\x61uthor\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x1e\n\x11\x61rtistAttribution\x18\x05 \x01(\tH\x04\x88\x01\x01\x12\x17\n\nisExplicit\x18\x06 \x01(\x08H\x05\x88\x01\x01\x42\r\n\x0b_authorNameB\t\n\x07_songIdB\x08\n\x06_titleB\t\n\x07_authorB\x14\n\x12_artistAttributionB\r\n\x0b_isExplicit\x1a\xbf\x01\n\rRLAttribution\x12\x42\n\x06source\x18\x01 \x01(\x0e\x32-.proto.StatusAttribution.RLAttribution.SourceH\x00\x88\x01\x01\"_\n\x06Source\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x18\n\x14RAY_BAN_META_GLASSES\x10\x01\x12\x17\n\x13OAKLEY_META_GLASSES\x10\x02\x12\x15\n\x11HYPERNOVA_GLASSES\x10\x03\x42\t\n\x07_source\x1a\xde\x03\n\rStatusReshare\x12\x42\n\x06source\x18\x01 \x01(\x0e\x32-.proto.StatusAttribution.StatusReshare.SourceH\x00\x88\x01\x01\x12\x46\n\x08metadata\x18\x02 \x01(\x0b\x32/.proto.StatusAttribution.StatusReshare.MetadataH\x01\x88\x01\x01\x1a\xc4\x01\n\x08Metadata\x12\x15\n\x08\x64uration\x18\x01 \x01(\x05H\x00\x88\x01\x01\x12\x17\n\nchannelJid\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x1d\n\x10\x63hannelMessageId\x18\x03 \x01(\x05H\x02\x88\x01\x01\x12 \n\x13hasMultipleReshares\x18\x04 \x01(\x08H\x03\x88\x01\x01\x42\x0b\n\t_durationB\r\n\x0b_channelJidB\x13\n\x11_channelMessageIdB\x16\n\x14_hasMultipleReshares\"b\n\x06Source\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x14\n\x10INTERNAL_RESHARE\x10\x01\x12\x13\n\x0fMENTION_RESHARE\x10\x02\x12\x13\n\x0f\x43HANNEL_RESHARE\x10\x03\x12\x0b\n\x07\x46ORWARD\x10\x04\x42\t\n\x07_sourceB\x0b\n\t_metadata\"\x96\x01\n\x04Type\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07RESHARE\x10\x01\x12\x12\n\x0e\x45XTERNAL_SHARE\x10\x02\x12\t\n\x05MUSIC\x10\x03\x12\x12\n\x0eSTATUS_MENTION\x10\x04\x12\x10\n\x0cGROUP_STATUS\x10\x05\x12\x12\n\x0eRL_ATTRIBUTION\x10\x06\x12\x0e\n\nAI_CREATED\x10\x07\x12\x0b\n\x07LAYOUTS\x10\x08\x42\x11\n\x0f\x61ttributionDataB\x07\n\x05_typeB\x0c\n\n_actionUrl\"R\n\x14StatusMentionMessage\x12)\n\x0cquotedStatus\x18\x01 \x01(\x0b\x32\x0e.proto.MessageH\x00\x88\x01\x01\x42\x0f\n\r_quotedStatus\"i\n\tStatusPSA\x12\x12\n\ncampaignId\x18, \x01(\x04\x12(\n\x1b\x63\x61mpaignExpirationTimestamp\x18- \x01(\x04H\x00\x88\x01\x01\x42\x1e\n\x1c_campaignExpirationTimestamp\"\xa9\x04\n\x0fStickerMetadata\x12\x10\n\x03url\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x17\n\nfileSha256\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x1a\n\rfileEncSha256\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x15\n\x08mediaKey\x18\x04 \x01(\x0cH\x03\x88\x01\x01\x12\x15\n\x08mimetype\x18\x05 \x01(\tH\x04\x88\x01\x01\x12\x13\n\x06height\x18\x06 \x01(\rH\x05\x88\x01\x01\x12\x12\n\x05width\x18\x07 \x01(\rH\x06\x88\x01\x01\x12\x17\n\ndirectPath\x18\x08 \x01(\tH\x07\x88\x01\x01\x12\x17\n\nfileLength\x18\t \x01(\x04H\x08\x88\x01\x01\x12\x13\n\x06weight\x18\n \x01(\x02H\t\x88\x01\x01\x12\x1e\n\x11lastStickerSentTs\x18\x0b \x01(\x03H\n\x88\x01\x01\x12\x15\n\x08isLottie\x18\x0c \x01(\x08H\x0b\x88\x01\x01\x12\x16\n\timageHash\x18\r \x01(\tH\x0c\x88\x01\x01\x12\x1c\n\x0fisAvatarSticker\x18\x0e \x01(\x08H\r\x88\x01\x01\x42\x06\n\x04_urlB\r\n\x0b_fileSha256B\x10\n\x0e_fileEncSha256B\x0b\n\t_mediaKeyB\x0b\n\t_mimetypeB\t\n\x07_heightB\x08\n\x06_widthB\r\n\x0b_directPathB\r\n\x0b_fileLengthB\t\n\x07_weightB\x14\n\x12_lastStickerSentTsB\x0b\n\t_isLottieB\x0c\n\n_imageHashB\x12\n\x10_isAvatarSticker\"\xa8\x01\n\x0eSyncActionData\x12\x12\n\x05index\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12*\n\x05value\x18\x02 \x01(\x0b\x32\x16.proto.SyncActionValueH\x01\x88\x01\x01\x12\x14\n\x07padding\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x14\n\x07version\x18\x04 \x01(\x05H\x03\x88\x01\x01\x42\x08\n\x06_indexB\x08\n\x06_valueB\n\n\x08_paddingB\n\n\x08_version\"\xb8\x83\x01\n\x0fSyncActionValue\x12\x16\n\ttimestamp\x18\x01 \x01(\x03H\x00\x88\x01\x01\x12:\n\nstarAction\x18\x02 \x01(\x0b\x32!.proto.SyncActionValue.StarActionH\x01\x88\x01\x01\x12@\n\rcontactAction\x18\x03 \x01(\x0b\x32$.proto.SyncActionValue.ContactActionH\x02\x88\x01\x01\x12:\n\nmuteAction\x18\x04 \x01(\x0b\x32!.proto.SyncActionValue.MuteActionH\x03\x88\x01\x01\x12\x38\n\tpinAction\x18\x05 \x01(\x0b\x32 .proto.SyncActionValue.PinActionH\x04\x88\x01\x01\x12\x44\n\x0fpushNameSetting\x18\x07 \x01(\x0b\x32&.proto.SyncActionValue.PushNameSettingH\x05\x88\x01\x01\x12\x46\n\x10quickReplyAction\x18\x08 \x01(\x0b\x32\'.proto.SyncActionValue.QuickReplyActionH\x06\x88\x01\x01\x12V\n\x18recentEmojiWeightsAction\x18\x0b \x01(\x0b\x32/.proto.SyncActionValue.RecentEmojiWeightsActionH\x07\x88\x01\x01\x12\x44\n\x0flabelEditAction\x18\x0e \x01(\x0b\x32&.proto.SyncActionValue.LabelEditActionH\x08\x88\x01\x01\x12R\n\x16labelAssociationAction\x18\x0f \x01(\x0b\x32-.proto.SyncActionValue.LabelAssociationActionH\t\x88\x01\x01\x12@\n\rlocaleSetting\x18\x10 \x01(\x0b\x32$.proto.SyncActionValue.LocaleSettingH\n\x88\x01\x01\x12H\n\x11\x61rchiveChatAction\x18\x11 \x01(\x0b\x32(.proto.SyncActionValue.ArchiveChatActionH\x0b\x88\x01\x01\x12V\n\x18\x64\x65leteMessageForMeAction\x18\x12 \x01(\x0b\x32/.proto.SyncActionValue.DeleteMessageForMeActionH\x0c\x88\x01\x01\x12@\n\rkeyExpiration\x18\x13 \x01(\x0b\x32$.proto.SyncActionValue.KeyExpirationH\r\x88\x01\x01\x12N\n\x14markChatAsReadAction\x18\x14 \x01(\x0b\x32+.proto.SyncActionValue.MarkChatAsReadActionH\x0e\x88\x01\x01\x12\x44\n\x0f\x63learChatAction\x18\x15 \x01(\x0b\x32&.proto.SyncActionValue.ClearChatActionH\x0f\x88\x01\x01\x12\x46\n\x10\x64\x65leteChatAction\x18\x16 \x01(\x0b\x32\'.proto.SyncActionValue.DeleteChatActionH\x10\x88\x01\x01\x12P\n\x15unarchiveChatsSetting\x18\x17 \x01(\x0b\x32,.proto.SyncActionValue.UnarchiveChatsSettingH\x11\x88\x01\x01\x12\x42\n\x0eprimaryFeature\x18\x18 \x01(\x0b\x32%.proto.SyncActionValue.PrimaryFeatureH\x12\x88\x01\x01\x12X\n\x19\x61ndroidUnsupportedActions\x18\x1a \x01(\x0b\x32\x30.proto.SyncActionValue.AndroidUnsupportedActionsH\x13\x88\x01\x01\x12<\n\x0b\x61gentAction\x18\x1b \x01(\x0b\x32\".proto.SyncActionValue.AgentActionH\x14\x88\x01\x01\x12J\n\x12subscriptionAction\x18\x1c \x01(\x0b\x32).proto.SyncActionValue.SubscriptionActionH\x15\x88\x01\x01\x12N\n\x14userStatusMuteAction\x18\x1d \x01(\x0b\x32+.proto.SyncActionValue.UserStatusMuteActionH\x16\x88\x01\x01\x12\x46\n\x10timeFormatAction\x18\x1e \x01(\x0b\x32\'.proto.SyncActionValue.TimeFormatActionH\x17\x88\x01\x01\x12\x38\n\tnuxAction\x18\x1f \x01(\x0b\x32 .proto.SyncActionValue.NuxActionH\x18\x88\x01\x01\x12N\n\x14primaryVersionAction\x18  \x01(\x0b\x32+.proto.SyncActionValue.PrimaryVersionActionH\x19\x88\x01\x01\x12@\n\rstickerAction\x18! \x01(\x0b\x32$.proto.SyncActionValue.StickerActionH\x1a\x88\x01\x01\x12X\n\x19removeRecentStickerAction\x18\" \x01(\x0b\x32\x30.proto.SyncActionValue.RemoveRecentStickerActionH\x1b\x88\x01\x01\x12H\n\x0e\x63hatAssignment\x18# \x01(\x0b\x32+.proto.SyncActionValue.ChatAssignmentActionH\x1c\x88\x01\x01\x12`\n\x1a\x63hatAssignmentOpenedStatus\x18$ \x01(\x0b\x32\x37.proto.SyncActionValue.ChatAssignmentOpenedStatusActionH\x1d\x88\x01\x01\x12J\n\x12pnForLidChatAction\x18% \x01(\x0b\x32).proto.SyncActionValue.PnForLidChatActionH\x1e\x88\x01\x01\x12R\n\x16marketingMessageAction\x18& \x01(\x0b\x32-.proto.SyncActionValue.MarketingMessageActionH\x1f\x88\x01\x01\x12\x64\n\x1fmarketingMessageBroadcastAction\x18\' \x01(\x0b\x32\x36.proto.SyncActionValue.MarketingMessageBroadcastActionH \x88\x01\x01\x12P\n\x15\x65xternalWebBetaAction\x18( \x01(\x0b\x32,.proto.SyncActionValue.ExternalWebBetaActionH!\x88\x01\x01\x12\\\n\x1bprivacySettingRelayAllCalls\x18) \x01(\x0b\x32\x32.proto.SyncActionValue.PrivacySettingRelayAllCallsH\"\x88\x01\x01\x12@\n\rcallLogAction\x18* \x01(\x0b\x32$.proto.SyncActionValue.CallLogActionH#\x88\x01\x01\x12\x32\n\x06ugcBot\x18+ \x01(\x0b\x32\x1d.proto.SyncActionValue.UGCBotH$\x88\x01\x01\x12\x46\n\rstatusPrivacy\x18, \x01(\x0b\x32*.proto.SyncActionValue.StatusPrivacyActionH%\x88\x01\x01\x12T\n\x17\x62otWelcomeRequestAction\x18- \x01(\x0b\x32..proto.SyncActionValue.BotWelcomeRequestActionH&\x88\x01\x01\x12Z\n\x17\x64\x65leteIndividualCallLog\x18. \x01(\x0b\x32\x34.proto.SyncActionValue.DeleteIndividualCallLogActionH\'\x88\x01\x01\x12P\n\x15labelReorderingAction\x18/ \x01(\x0b\x32,.proto.SyncActionValue.LabelReorderingActionH(\x88\x01\x01\x12H\n\x11paymentInfoAction\x18\x30 \x01(\x0b\x32(.proto.SyncActionValue.PaymentInfoActionH)\x88\x01\x01\x12Z\n\x1a\x63ustomPaymentMethodsAction\x18\x31 \x01(\x0b\x32\x31.proto.SyncActionValue.CustomPaymentMethodsActionH*\x88\x01\x01\x12\x42\n\x0elockChatAction\x18\x32 \x01(\x0b\x32%.proto.SyncActionValue.LockChatActionH+\x88\x01\x01\x12\x36\n\x10\x63hatLockSettings\x18\x33 \x01(\x0b\x32\x17.proto.ChatLockSettingsH,\x88\x01\x01\x12V\n\x18wamoUserIdentifierAction\x18\x34 \x01(\x0b\x32/.proto.SyncActionValue.WamoUserIdentifierActionH-\x88\x01\x01\x12t\n\'privacySettingDisableLinkPreviewsAction\x18\x35 \x01(\x0b\x32>.proto.SyncActionValue.PrivacySettingDisableLinkPreviewsActionH.\x88\x01\x01\x12:\n\x12\x64\x65viceCapabilities\x18\x36 \x01(\x0b\x32\x19.proto.DeviceCapabilitiesH/\x88\x01\x01\x12\x42\n\x0enoteEditAction\x18\x37 \x01(\x0b\x32%.proto.SyncActionValue.NoteEditActionH0\x88\x01\x01\x12\x44\n\x0f\x66\x61voritesAction\x18\x38 \x01(\x0b\x32&.proto.SyncActionValue.FavoritesActionH1\x88\x01\x01\x12^\n\x1cmerchantPaymentPartnerAction\x18\x39 \x01(\x0b\x32\x33.proto.SyncActionValue.MerchantPaymentPartnerActionH2\x88\x01\x01\x12^\n\x1cwaffleAccountLinkStateAction\x18: \x01(\x0b\x32\x33.proto.SyncActionValue.WaffleAccountLinkStateActionH3\x88\x01\x01\x12V\n\x15usernameChatStartMode\x18; \x01(\x0b\x32\x32.proto.SyncActionValue.UsernameChatStartModeActionH4\x88\x01\x01\x12h\n!notificationActivitySettingAction\x18< \x01(\x0b\x32\x38.proto.SyncActionValue.NotificationActivitySettingActionH5\x88\x01\x01\x12\x46\n\x10lidContactAction\x18= \x01(\x0b\x32\'.proto.SyncActionValue.LidContactActionH6\x88\x01\x01\x12\x66\n ctwaPerCustomerDataSharingAction\x18> \x01(\x0b\x32\x37.proto.SyncActionValue.CtwaPerCustomerDataSharingActionH7\x88\x01\x01\x12\x46\n\x10paymentTosAction\x18? \x01(\x0b\x32\'.proto.SyncActionValue.PaymentTosActionH8\x88\x01\x01\x12\x92\x01\n6privacySettingChannelsPersonalisedRecommendationAction\x18@ \x01(\x0b\x32M.proto.SyncActionValue.PrivacySettingChannelsPersonalisedRecommendationActionH9\x88\x01\x01\x12j\n\"businessBroadcastAssociationAction\x18\x41 \x01(\x0b\x32\x39.proto.SyncActionValue.BusinessBroadcastAssociationActionH:\x88\x01\x01\x12^\n\x1c\x64\x65tectedOutcomesStatusAction\x18\x42 \x01(\x0b\x32\x33.proto.SyncActionValue.DetectedOutcomesStatusActionH;\x88\x01\x01\x12^\n\x1cmaibaAiFeaturesControlAction\x18\x44 \x01(\x0b\x32\x33.proto.SyncActionValue.MaibaAIFeaturesControlActionH<\x88\x01\x01\x12\\\n\x1b\x62usinessBroadcastListAction\x18\x45 \x01(\x0b\x32\x32.proto.SyncActionValue.BusinessBroadcastListActionH=\x88\x01\x01\x12H\n\x11musicUserIdAction\x18\x46 \x01(\x0b\x32(.proto.SyncActionValue.MusicUserIdActionH>\x88\x01\x01\x12~\n,statusPostOptInNotificationPreferencesAction\x18G \x01(\x0b\x32\x43.proto.SyncActionValue.StatusPostOptInNotificationPreferencesActionH?\x88\x01\x01\x12L\n\x13\x61vatarUpdatedAction\x18H \x01(\x0b\x32*.proto.SyncActionValue.AvatarUpdatedActionH@\x88\x01\x01\x12\x62\n\x1eprivateProcessingSettingAction\x18J \x01(\x0b\x32\x35.proto.SyncActionValue.PrivateProcessingSettingActionHA\x88\x01\x01\x12\x62\n\x1enewsletterSavedInterestsAction\x18K \x01(\x0b\x32\x35.proto.SyncActionValue.NewsletterSavedInterestsActionHB\x88\x01\x01\x12N\n\x14\x61iThreadRenameAction\x18L \x01(\x0b\x32+.proto.SyncActionValue.AiThreadRenameActionHC\x88\x01\x01\x12V\n\x18interactiveMessageAction\x18M \x01(\x0b\x32/.proto.SyncActionValue.InteractiveMessageActionHD\x88\x01\x01\x1as\n\x0b\x41gentAction\x12\x11\n\x04name\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x15\n\x08\x64\x65viceID\x18\x02 \x01(\x05H\x01\x88\x01\x01\x12\x16\n\tisDeleted\x18\x03 \x01(\x08H\x02\x88\x01\x01\x42\x07\n\x05_nameB\x0b\n\t_deviceIDB\x0c\n\n_isDeleted\x1a:\n\x14\x41iThreadRenameAction\x12\x15\n\x08newTitle\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x0b\n\t_newTitle\x1a=\n\x19\x41ndroidUnsupportedActions\x12\x14\n\x07\x61llowed\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\n\n\x08_allowed\x1a\x92\x01\n\x11\x41rchiveChatAction\x12\x15\n\x08\x61rchived\x18\x01 \x01(\x08H\x00\x88\x01\x01\x12H\n\x0cmessageRange\x18\x02 \x01(\x0b\x32-.proto.SyncActionValue.SyncActionMessageRangeH\x01\x88\x01\x01\x42\x0b\n\t_archivedB\x0f\n\r_messageRange\x1a\xf5\x01\n\x13\x41vatarUpdatedAction\x12R\n\teventType\x18\x01 \x01(\x0e\x32:.proto.SyncActionValue.AvatarUpdatedAction.AvatarEventTypeH\x00\x88\x01\x01\x12\x42\n\x14recentAvatarStickers\x18\x02 \x03(\x0b\x32$.proto.SyncActionValue.StickerAction\"8\n\x0f\x41vatarEventType\x12\x0b\n\x07UPDATED\x10\x00\x12\x0b\n\x07\x43REATED\x10\x01\x12\x0b\n\x07\x44\x45LETED\x10\x02\x42\x0c\n\n_eventType\x1a\x39\n\x17\x42otWelcomeRequestAction\x12\x13\n\x06isSent\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\t\n\x07_isSent\x1aH\n\x18\x42roadcastListParticipant\x12\x0e\n\x06lidJid\x18\x01 \x01(\t\x12\x12\n\x05pnJid\x18\x02 \x01(\tH\x00\x88\x01\x01\x42\x08\n\x06_pnJid\x1a\x46\n\"BusinessBroadcastAssociationAction\x12\x14\n\x07\x64\x65leted\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\n\n\x08_deleted\x1a\xaa\x01\n\x1b\x42usinessBroadcastListAction\x12\x14\n\x07\x64\x65leted\x18\x01 \x01(\x08H\x00\x88\x01\x01\x12\x45\n\x0cparticipants\x18\x02 \x03(\x0b\x32/.proto.SyncActionValue.BroadcastListParticipant\x12\x15\n\x08listName\x18\x03 \x01(\tH\x01\x88\x01\x01\x42\n\n\x08_deletedB\x0b\n\t_listName\x1aS\n\rCallLogAction\x12\x30\n\rcallLogRecord\x18\x01 \x01(\x0b\x32\x14.proto.CallLogRecordH\x00\x88\x01\x01\x42\x10\n\x0e_callLogRecord\x1a\x44\n\x14\x43hatAssignmentAction\x12\x1a\n\rdeviceAgentID\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x10\n\x0e_deviceAgentID\x1aJ\n ChatAssignmentOpenedStatusAction\x12\x17\n\nchatOpened\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\r\n\x0b_chatOpened\x1al\n\x0f\x43learChatAction\x12H\n\x0cmessageRange\x18\x01 \x01(\x0b\x32-.proto.SyncActionValue.SyncActionMessageRangeH\x00\x88\x01\x01\x42\x0f\n\r_messageRange\x1a\xff\x01\n\rContactAction\x12\x15\n\x08\x66ullName\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x16\n\tfirstName\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x13\n\x06lidJid\x18\x03 \x01(\tH\x02\x88\x01\x01\x12%\n\x18saveOnPrimaryAddressbook\x18\x04 \x01(\x08H\x03\x88\x01\x01\x12\x12\n\x05pnJid\x18\x05 \x01(\tH\x04\x88\x01\x01\x12\x15\n\x08username\x18\x06 \x01(\tH\x05\x88\x01\x01\x42\x0b\n\t_fullNameB\x0c\n\n_firstNameB\t\n\x07_lidJidB\x1b\n\x19_saveOnPrimaryAddressbookB\x08\n\x06_pnJidB\x0b\n\t_username\x1a|\n CtwaPerCustomerDataSharingAction\x12\x30\n#isCtwaPerCustomerDataSharingEnabled\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42&\n$_isCtwaPerCustomerDataSharingEnabled\x1a\x90\x01\n\x13\x43ustomPaymentMethod\x12\x14\n\x0c\x63redentialId\x18\x01 \x01(\t\x12\x0f\n\x07\x63ountry\x18\x02 \x01(\t\x12\x0c\n\x04type\x18\x03 \x01(\t\x12\x44\n\x08metadata\x18\x04 \x03(\x0b\x32\x32.proto.SyncActionValue.CustomPaymentMethodMetadata\x1a\x39\n\x1b\x43ustomPaymentMethodMetadata\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\x1a\x66\n\x1a\x43ustomPaymentMethodsAction\x12H\n\x14\x63ustomPaymentMethods\x18\x01 \x03(\x0b\x32*.proto.SyncActionValue.CustomPaymentMethod\x1am\n\x10\x44\x65leteChatAction\x12H\n\x0cmessageRange\x18\x01 \x01(\x0b\x32-.proto.SyncActionValue.SyncActionMessageRangeH\x00\x88\x01\x01\x42\x0f\n\r_messageRange\x1ai\n\x1d\x44\x65leteIndividualCallLogAction\x12\x14\n\x07peerJid\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x17\n\nisIncoming\x18\x02 \x01(\x08H\x01\x88\x01\x01\x42\n\n\x08_peerJidB\r\n\x0b_isIncoming\x1ax\n\x18\x44\x65leteMessageForMeAction\x12\x18\n\x0b\x64\x65leteMedia\x18\x01 \x01(\x08H\x00\x88\x01\x01\x12\x1d\n\x10messageTimestamp\x18\x02 \x01(\x03H\x01\x88\x01\x01\x42\x0e\n\x0c_deleteMediaB\x13\n\x11_messageTimestamp\x1a\x44\n\x1c\x44\x65tectedOutcomesStatusAction\x12\x16\n\tisEnabled\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\x0c\n\n_isEnabled\x1a\x39\n\x15\x45xternalWebBetaAction\x12\x14\n\x07isOptIn\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\n\n\x08_isOptIn\x1ay\n\x0f\x46\x61voritesAction\x12\x42\n\tfavorites\x18\x01 \x03(\x0b\x32/.proto.SyncActionValue.FavoritesAction.Favorite\x1a\"\n\x08\x46\x61vorite\x12\x0f\n\x02id\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x05\n\x03_id\x1a\xd8\x01\n\x18InteractiveMessageAction\x12Z\n\x04type\x18\x01 \x01(\x0e\x32L.proto.SyncActionValue.InteractiveMessageAction.InteractiveMessageActionMode\"`\n\x1cInteractiveMessageActionMode\x12/\n+INTERACTIVE_MESSAGE_ACTION_MODE_UNSPECIFIED\x10\x00\x12\x0f\n\x0b\x44ISABLE_CTA\x10\x01\x1a\x41\n\rKeyExpiration\x12\x1c\n\x0f\x65xpiredKeyEpoch\x18\x01 \x01(\x05H\x00\x88\x01\x01\x42\x12\n\x10_expiredKeyEpoch\x1a:\n\x16LabelAssociationAction\x12\x14\n\x07labeled\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\n\n\x08_labeled\x1a\xa5\x04\n\x0fLabelEditAction\x12\x11\n\x04name\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x12\n\x05\x63olor\x18\x02 \x01(\x05H\x01\x88\x01\x01\x12\x19\n\x0cpredefinedId\x18\x03 \x01(\x05H\x02\x88\x01\x01\x12\x14\n\x07\x64\x65leted\x18\x04 \x01(\x08H\x03\x88\x01\x01\x12\x17\n\norderIndex\x18\x05 \x01(\x05H\x04\x88\x01\x01\x12\x15\n\x08isActive\x18\x06 \x01(\x08H\x05\x88\x01\x01\x12\x42\n\x04type\x18\x07 \x01(\x0e\x32/.proto.SyncActionValue.LabelEditAction.ListTypeH\x06\x88\x01\x01\x12\x18\n\x0bisImmutable\x18\x08 \x01(\x08H\x07\x88\x01\x01\x12\x1a\n\rmuteEndTimeMs\x18\t \x01(\x03H\x08\x88\x01\x01\"\x98\x01\n\x08ListType\x12\x08\n\x04NONE\x10\x00\x12\n\n\x06UNREAD\x10\x01\x12\n\n\x06GROUPS\x10\x02\x12\r\n\tFAVORITES\x10\x03\x12\x0e\n\nPREDEFINED\x10\x04\x12\n\n\x06\x43USTOM\x10\x05\x12\r\n\tCOMMUNITY\x10\x06\x12\x13\n\x0fSERVER_ASSIGNED\x10\x07\x12\x0b\n\x07\x44RAFTED\x10\x08\x12\x0e\n\nAI_HANDOFF\x10\tB\x07\n\x05_nameB\x08\n\x06_colorB\x0f\n\r_predefinedIdB\n\n\x08_deletedB\r\n\x0b_orderIndexB\x0b\n\t_isActiveB\x07\n\x05_typeB\x0e\n\x0c_isImmutableB\x10\n\x0e_muteEndTimeMs\x1a/\n\x15LabelReorderingAction\x12\x16\n\x0esortedLabelIds\x18\x01 \x03(\x05\x1a\x80\x01\n\x10LidContactAction\x12\x15\n\x08\x66ullName\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x16\n\tfirstName\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x15\n\x08username\x18\x03 \x01(\tH\x02\x88\x01\x01\x42\x0b\n\t_fullNameB\x0c\n\n_firstNameB\x0b\n\t_username\x1a/\n\rLocaleSetting\x12\x13\n\x06locale\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\t\n\x07_locale\x1a\x30\n\x0eLockChatAction\x12\x13\n\x06locked\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\t\n\x07_locked\x1a\xe7\x01\n\x1cMaibaAIFeaturesControlAction\x12\x66\n\x0f\x61iFeatureStatus\x18\x01 \x01(\x0e\x32H.proto.SyncActionValue.MaibaAIFeaturesControlAction.MaibaAIFeatureStatusH\x00\x88\x01\x01\"K\n\x14MaibaAIFeatureStatus\x12\x0b\n\x07\x45NABLED\x10\x00\x12\x18\n\x14\x45NABLED_HAS_LEARNING\x10\x01\x12\x0c\n\x08\x44ISABLED\x10\x02\x42\x12\n\x10_aiFeatureStatus\x1a\x8d\x01\n\x14MarkChatAsReadAction\x12\x11\n\x04read\x18\x01 \x01(\x08H\x00\x88\x01\x01\x12H\n\x0cmessageRange\x18\x02 \x01(\x0b\x32-.proto.SyncActionValue.SyncActionMessageRangeH\x01\x88\x01\x01\x42\x07\n\x05_readB\x0f\n\r_messageRange\x1a\x88\x03\n\x16MarketingMessageAction\x12\x11\n\x04name\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x14\n\x07message\x18\x02 \x01(\tH\x01\x88\x01\x01\x12^\n\x04type\x18\x03 \x01(\x0e\x32K.proto.SyncActionValue.MarketingMessageAction.MarketingMessagePrototypeTypeH\x02\x88\x01\x01\x12\x16\n\tcreatedAt\x18\x04 \x01(\x03H\x03\x88\x01\x01\x12\x17\n\nlastSentAt\x18\x05 \x01(\x03H\x04\x88\x01\x01\x12\x16\n\tisDeleted\x18\x06 \x01(\x08H\x05\x88\x01\x01\x12\x14\n\x07mediaId\x18\x07 \x01(\tH\x06\x88\x01\x01\"1\n\x1dMarketingMessagePrototypeType\x12\x10\n\x0cPERSONALIZED\x10\x00\x42\x07\n\x05_nameB\n\n\x08_messageB\x07\n\x05_typeB\x0c\n\n_createdAtB\r\n\x0b_lastSentAtB\x0c\n\n_isDeletedB\n\n\x08_mediaId\x1aM\n\x1fMarketingMessageBroadcastAction\x12\x19\n\x0crepliedCount\x18\x01 \x01(\x05H\x00\x88\x01\x01\x42\x0f\n\r_repliedCount\x1a\xf5\x01\n\x1cMerchantPaymentPartnerAction\x12J\n\x06status\x18\x01 \x01(\x0e\x32:.proto.SyncActionValue.MerchantPaymentPartnerAction.Status\x12\x0f\n\x07\x63ountry\x18\x02 \x01(\t\x12\x18\n\x0bgatewayName\x18\x03 \x01(\tH\x00\x88\x01\x01\x12\x19\n\x0c\x63redentialId\x18\x04 \x01(\tH\x01\x88\x01\x01\"\"\n\x06Status\x12\n\n\x06\x41\x43TIVE\x10\x00\x12\x0c\n\x08INACTIVE\x10\x01\x42\x0e\n\x0c_gatewayNameB\x0f\n\r_credentialId\x1a\xcd\x01\n\x11MusicUserIdAction\x12\x18\n\x0bmusicUserId\x18\x01 \x01(\tH\x00\x88\x01\x01\x12W\n\x11music_user_id_map\x18\x02 \x03(\x0b\x32<.proto.SyncActionValue.MusicUserIdAction.MusicUserIdMapEntry\x1a\x35\n\x13MusicUserIdMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x42\x0e\n\x0c_musicUserId\x1a\x84\x01\n\nMuteAction\x12\x12\n\x05muted\x18\x01 \x01(\x08H\x00\x88\x01\x01\x12\x1d\n\x10muteEndTimestamp\x18\x02 \x01(\x03H\x01\x88\x01\x01\x12\x16\n\tautoMuted\x18\x03 \x01(\x08H\x02\x88\x01\x01\x42\x08\n\x06_mutedB\x13\n\x11_muteEndTimestampB\x0c\n\n_autoMuted\x1a\x64\n\x1eNewsletterSavedInterestsAction\x12%\n\x18newsletterSavedInterests\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x1b\n\x19_newsletterSavedInterests\x1a\xc9\x02\n\x0eNoteEditAction\x12\x41\n\x04type\x18\x01 \x01(\x0e\x32..proto.SyncActionValue.NoteEditAction.NoteTypeH\x00\x88\x01\x01\x12\x14\n\x07\x63hatJid\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x16\n\tcreatedAt\x18\x03 \x01(\x03H\x02\x88\x01\x01\x12\x14\n\x07\x64\x65leted\x18\x04 \x01(\x08H\x03\x88\x01\x01\x12 \n\x13unstructuredContent\x18\x05 \x01(\tH\x04\x88\x01\x01\"G\n\x08NoteType\x12\x19\n\x15NOTE_TYPE_UNSPECIFIED\x10\x00\x12\x10\n\x0cUNSTRUCTURED\x10\x01\x12\x0e\n\nSTRUCTURED\x10\x02\x42\x07\n\x05_typeB\n\n\x08_chatJidB\x0c\n\n_createdAtB\n\n\x08_deletedB\x16\n\x14_unstructuredContent\x1a\xb6\x02\n!NotificationActivitySettingAction\x12~\n\x1bnotificationActivitySetting\x18\x01 \x01(\x0e\x32T.proto.SyncActionValue.NotificationActivitySettingAction.NotificationActivitySettingH\x00\x88\x01\x01\"q\n\x1bNotificationActivitySetting\x12\x18\n\x14\x44\x45\x46\x41ULT_ALL_MESSAGES\x10\x00\x12\x10\n\x0c\x41LL_MESSAGES\x10\x01\x12\x0e\n\nHIGHLIGHTS\x10\x02\x12\x16\n\x12\x44\x45\x46\x41ULT_HIGHLIGHTS\x10\x03\x42\x1e\n\x1c_notificationActivitySetting\x1a\x37\n\tNuxAction\x12\x19\n\x0c\x61\x63knowledged\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\x0f\n\r_acknowledged\x1a-\n\x11PaymentInfoAction\x12\x10\n\x03\x63pi\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x06\n\x04_cpi\x1a\x9e\x01\n\x10PaymentTosAction\x12L\n\rpaymentNotice\x18\x01 \x01(\x0e\x32\x35.proto.SyncActionValue.PaymentTosAction.PaymentNotice\x12\x10\n\x08\x61\x63\x63\x65pted\x18\x02 \x01(\x08\"*\n\rPaymentNotice\x12\x19\n\x15\x42R_PAY_PRIVACY_POLICY\x10\x00\x1a+\n\tPinAction\x12\x13\n\x06pinned\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\t\n\x07_pinned\x1a\x32\n\x12PnForLidChatAction\x12\x12\n\x05pnJid\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x08\n\x06_pnJid\x1a\x1f\n\x0ePrimaryFeature\x12\r\n\x05\x66lags\x18\x01 \x03(\t\x1a\x38\n\x14PrimaryVersionAction\x12\x14\n\x07version\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\n\n\x08_version\x1ah\n6PrivacySettingChannelsPersonalisedRecommendationAction\x12\x1b\n\x0eisUserOptedOut\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\x11\n\x0f_isUserOptedOut\x1a\x61\n\'PrivacySettingDisableLinkPreviewsAction\x12\x1f\n\x12isPreviewsDisabled\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\x15\n\x13_isPreviewsDisabled\x1a\x43\n\x1bPrivacySettingRelayAllCalls\x12\x16\n\tisEnabled\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\x0c\n\n_isEnabled\x1a\xf6\x01\n\x1ePrivateProcessingSettingAction\x12s\n\x17privateProcessingStatus\x18\x01 \x01(\x0e\x32M.proto.SyncActionValue.PrivateProcessingSettingAction.PrivateProcessingStatusH\x00\x88\x01\x01\"C\n\x17PrivateProcessingStatus\x12\r\n\tUNDEFINED\x10\x00\x12\x0b\n\x07\x45NABLED\x10\x01\x12\x0c\n\x08\x44ISABLED\x10\x02\x42\x1a\n\x18_privateProcessingStatus\x1a-\n\x0fPushNameSetting\x12\x11\n\x04name\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x07\n\x05_name\x1a\xaa\x01\n\x10QuickReplyAction\x12\x15\n\x08shortcut\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x14\n\x07message\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x10\n\x08keywords\x18\x03 \x03(\t\x12\x12\n\x05\x63ount\x18\x04 \x01(\x05H\x02\x88\x01\x01\x12\x14\n\x07\x64\x65leted\x18\x05 \x01(\x08H\x03\x88\x01\x01\x42\x0b\n\t_shortcutB\n\n\x08_messageB\x08\n\x06_countB\n\n\x08_deleted\x1a\x45\n\x18RecentEmojiWeightsAction\x12)\n\x07weights\x18\x01 \x03(\x0b\x32\x18.proto.RecentEmojiWeight\x1aQ\n\x19RemoveRecentStickerAction\x12\x1e\n\x11lastStickerSentTs\x18\x01 \x01(\x03H\x00\x88\x01\x01\x42\x14\n\x12_lastStickerSentTs\x1a.\n\nStarAction\x12\x14\n\x07starred\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\n\n\x08_starred\x1aP\n,StatusPostOptInNotificationPreferencesAction\x12\x14\n\x07\x65nabled\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\n\n\x08_enabled\x1a\xdf\x01\n\x13StatusPrivacyAction\x12T\n\x04mode\x18\x01 \x01(\x0e\x32\x41.proto.SyncActionValue.StatusPrivacyAction.StatusDistributionModeH\x00\x88\x01\x01\x12\x0f\n\x07userJid\x18\x02 \x03(\t\"X\n\x16StatusDistributionMode\x12\x0e\n\nALLOW_LIST\x10\x00\x12\r\n\tDENY_LIST\x10\x01\x12\x0c\n\x08\x43ONTACTS\x10\x02\x12\x11\n\rCLOSE_FRIENDS\x10\x03\x42\x07\n\x05_mode\x1a\xfd\x03\n\rStickerAction\x12\x10\n\x03url\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x1a\n\rfileEncSha256\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x15\n\x08mediaKey\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x12\x15\n\x08mimetype\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x13\n\x06height\x18\x05 \x01(\rH\x04\x88\x01\x01\x12\x12\n\x05width\x18\x06 \x01(\rH\x05\x88\x01\x01\x12\x17\n\ndirectPath\x18\x07 \x01(\tH\x06\x88\x01\x01\x12\x17\n\nfileLength\x18\x08 \x01(\x04H\x07\x88\x01\x01\x12\x17\n\nisFavorite\x18\t \x01(\x08H\x08\x88\x01\x01\x12\x19\n\x0c\x64\x65viceIdHint\x18\n \x01(\rH\t\x88\x01\x01\x12\x15\n\x08isLottie\x18\x0b \x01(\x08H\n\x88\x01\x01\x12\x16\n\timageHash\x18\x0c \x01(\tH\x0b\x88\x01\x01\x12\x1c\n\x0fisAvatarSticker\x18\r \x01(\x08H\x0c\x88\x01\x01\x42\x06\n\x04_urlB\x10\n\x0e_fileEncSha256B\x0b\n\t_mediaKeyB\x0b\n\t_mimetypeB\t\n\x07_heightB\x08\n\x06_widthB\r\n\x0b_directPathB\r\n\x0b_fileLengthB\r\n\x0b_isFavoriteB\x0f\n\r_deviceIdHintB\x0b\n\t_isLottieB\x0c\n\n_imageHashB\x12\n\x10_isAvatarSticker\x1a\xa2\x01\n\x12SubscriptionAction\x12\x1a\n\risDeactivated\x18\x01 \x01(\x08H\x00\x88\x01\x01\x12\x1b\n\x0eisAutoRenewing\x18\x02 \x01(\x08H\x01\x88\x01\x01\x12\x1b\n\x0e\x65xpirationDate\x18\x03 \x01(\x03H\x02\x88\x01\x01\x42\x10\n\x0e_isDeactivatedB\x11\n\x0f_isAutoRenewingB\x11\n\x0f_expirationDate\x1a\x66\n\x11SyncActionMessage\x12#\n\x03key\x18\x01 \x01(\x0b\x32\x11.proto.MessageKeyH\x00\x88\x01\x01\x12\x16\n\ttimestamp\x18\x02 \x01(\x03H\x01\x88\x01\x01\x42\x06\n\x04_keyB\x0c\n\n_timestamp\x1a\xd8\x01\n\x16SyncActionMessageRange\x12!\n\x14lastMessageTimestamp\x18\x01 \x01(\x03H\x00\x88\x01\x01\x12\'\n\x1alastSystemMessageTimestamp\x18\x02 \x01(\x03H\x01\x88\x01\x01\x12:\n\x08messages\x18\x03 \x03(\x0b\x32(.proto.SyncActionValue.SyncActionMessageB\x17\n\x15_lastMessageTimestampB\x1d\n\x1b_lastSystemMessageTimestamp\x1a`\n\x10TimeFormatAction\x12*\n\x1disTwentyFourHourFormatEnabled\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42 \n\x1e_isTwentyFourHourFormatEnabled\x1a\x30\n\x06UGCBot\x12\x17\n\ndefinition\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x42\r\n\x0b_definition\x1aG\n\x15UnarchiveChatsSetting\x12\x1b\n\x0eunarchiveChats\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\x11\n\x0f_unarchiveChats\x1a\x34\n\x14UserStatusMuteAction\x12\x12\n\x05muted\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\x08\n\x06_muted\x1a\xd0\x01\n\x1bUsernameChatStartModeAction\x12\\\n\rchatStartMode\x18\x01 \x01(\x0e\x32@.proto.SyncActionValue.UsernameChatStartModeAction.ChatStartModeH\x00\x88\x01\x01\"A\n\rChatStartMode\x12\x1f\n\x1b\x43HAT_START_MODE_UNSPECIFIED\x10\x00\x12\x07\n\x03LID\x10\x01\x12\x06\n\x02PN\x10\x02\x42\x10\n\x0e_chatStartMode\x1a\xc4\x01\n\x1cWaffleAccountLinkStateAction\x12\\\n\tlinkState\x18\x02 \x01(\x0e\x32\x44.proto.SyncActionValue.WaffleAccountLinkStateAction.AccountLinkStateH\x00\x88\x01\x01\"8\n\x10\x41\x63\x63ountLinkState\x12\n\n\x06\x41\x43TIVE\x10\x00\x12\n\n\x06PAUSED\x10\x01\x12\x0c\n\x08UNLINKED\x10\x02\x42\x0c\n\n_linkState\x1a\x42\n\x18WamoUserIdentifierAction\x12\x17\n\nidentifier\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\r\n\x0b_identifierB\x0c\n\n_timestampB\r\n\x0b_starActionB\x10\n\x0e_contactActionB\r\n\x0b_muteActionB\x0c\n\n_pinActionB\x12\n\x10_pushNameSettingB\x13\n\x11_quickReplyActionB\x1b\n\x19_recentEmojiWeightsActionB\x12\n\x10_labelEditActionB\x19\n\x17_labelAssociationActionB\x10\n\x0e_localeSettingB\x14\n\x12_archiveChatActionB\x1b\n\x19_deleteMessageForMeActionB\x10\n\x0e_keyExpirationB\x17\n\x15_markChatAsReadActionB\x12\n\x10_clearChatActionB\x13\n\x11_deleteChatActionB\x18\n\x16_unarchiveChatsSettingB\x11\n\x0f_primaryFeatureB\x1c\n\x1a_androidUnsupportedActionsB\x0e\n\x0c_agentActionB\x15\n\x13_subscriptionActionB\x17\n\x15_userStatusMuteActionB\x13\n\x11_timeFormatActionB\x0c\n\n_nuxActionB\x17\n\x15_primaryVersionActionB\x10\n\x0e_stickerActionB\x1c\n\x1a_removeRecentStickerActionB\x11\n\x0f_chatAssignmentB\x1d\n\x1b_chatAssignmentOpenedStatusB\x15\n\x13_pnForLidChatActionB\x19\n\x17_marketingMessageActionB\"\n _marketingMessageBroadcastActionB\x18\n\x16_externalWebBetaActionB\x1e\n\x1c_privacySettingRelayAllCallsB\x10\n\x0e_callLogActionB\t\n\x07_ugcBotB\x10\n\x0e_statusPrivacyB\x1a\n\x18_botWelcomeRequestActionB\x1a\n\x18_deleteIndividualCallLogB\x18\n\x16_labelReorderingActionB\x14\n\x12_paymentInfoActionB\x1d\n\x1b_customPaymentMethodsActionB\x11\n\x0f_lockChatActionB\x13\n\x11_chatLockSettingsB\x1b\n\x19_wamoUserIdentifierActionB*\n(_privacySettingDisableLinkPreviewsActionB\x15\n\x13_deviceCapabilitiesB\x11\n\x0f_noteEditActionB\x12\n\x10_favoritesActionB\x1f\n\x1d_merchantPaymentPartnerActionB\x1f\n\x1d_waffleAccountLinkStateActionB\x18\n\x16_usernameChatStartModeB$\n\"_notificationActivitySettingActionB\x13\n\x11_lidContactActionB#\n!_ctwaPerCustomerDataSharingActionB\x13\n\x11_paymentTosActionB9\n7_privacySettingChannelsPersonalisedRecommendationActionB%\n#_businessBroadcastAssociationActionB\x1f\n\x1d_detectedOutcomesStatusActionB\x1f\n\x1d_maibaAiFeaturesControlActionB\x1e\n\x1c_businessBroadcastListActionB\x14\n\x12_musicUserIdActionB/\n-_statusPostOptInNotificationPreferencesActionB\x16\n\x14_avatarUpdatedActionB!\n\x1f_privateProcessingSettingActionB!\n\x1f_newsletterSavedInterestsActionB\x17\n\x15_aiThreadRenameActionB\x1b\n\x19_interactiveMessageAction\"(\n\nSyncdIndex\x12\x11\n\x04\x62lob\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x42\x07\n\x05_blob\"\xb5\x01\n\rSyncdMutation\x12;\n\toperation\x18\x01 \x01(\x0e\x32#.proto.SyncdMutation.SyncdOperationH\x00\x88\x01\x01\x12\'\n\x06record\x18\x02 \x01(\x0b\x32\x12.proto.SyncdRecordH\x01\x88\x01\x01\"%\n\x0eSyncdOperation\x12\x07\n\x03SET\x10\x00\x12\n\n\x06REMOVE\x10\x01\x42\x0c\n\n_operationB\t\n\x07_record\"9\n\x0eSyncdMutations\x12\'\n\tmutations\x18\x01 \x03(\x0b\x32\x14.proto.SyncdMutation\"\xcb\x03\n\nSyncdPatch\x12)\n\x07version\x18\x01 \x01(\x0b\x32\x13.proto.SyncdVersionH\x00\x88\x01\x01\x12\'\n\tmutations\x18\x02 \x03(\x0b\x32\x14.proto.SyncdMutation\x12<\n\x11\x65xternalMutations\x18\x03 \x01(\x0b\x32\x1c.proto.ExternalBlobReferenceH\x01\x88\x01\x01\x12\x18\n\x0bsnapshotMac\x18\x04 \x01(\x0cH\x02\x88\x01\x01\x12\x15\n\x08patchMac\x18\x05 \x01(\x0cH\x03\x88\x01\x01\x12 \n\x05keyId\x18\x06 \x01(\x0b\x32\x0c.proto.KeyIdH\x04\x88\x01\x01\x12&\n\x08\x65xitCode\x18\x07 \x01(\x0b\x32\x0f.proto.ExitCodeH\x05\x88\x01\x01\x12\x18\n\x0b\x64\x65viceIndex\x18\x08 \x01(\rH\x06\x88\x01\x01\x12\x1c\n\x0f\x63lientDebugData\x18\t \x01(\x0cH\x07\x88\x01\x01\x42\n\n\x08_versionB\x14\n\x12_externalMutationsB\x0e\n\x0c_snapshotMacB\x0b\n\t_patchMacB\x08\n\x06_keyIdB\x0b\n\t_exitCodeB\x0e\n\x0c_deviceIndexB\x12\n\x10_clientDebugData\"\x9b\x01\n\x0bSyncdRecord\x12%\n\x05index\x18\x01 \x01(\x0b\x32\x11.proto.SyncdIndexH\x00\x88\x01\x01\x12%\n\x05value\x18\x02 \x01(\x0b\x32\x11.proto.SyncdValueH\x01\x88\x01\x01\x12 \n\x05keyId\x18\x03 \x01(\x0b\x32\x0c.proto.KeyIdH\x02\x88\x01\x01\x42\x08\n\x06_indexB\x08\n\x06_valueB\x08\n\x06_keyId\"\xb1\x01\n\rSyncdSnapshot\x12)\n\x07version\x18\x01 \x01(\x0b\x32\x13.proto.SyncdVersionH\x00\x88\x01\x01\x12#\n\x07records\x18\x02 \x03(\x0b\x32\x12.proto.SyncdRecord\x12\x10\n\x03mac\x18\x03 \x01(\x0cH\x01\x88\x01\x01\x12 \n\x05keyId\x18\x04 \x01(\x0b\x32\x0c.proto.KeyIdH\x02\x88\x01\x01\x42\n\n\x08_versionB\x06\n\x04_macB\x08\n\x06_keyId\"(\n\nSyncdValue\x12\x11\n\x04\x62lob\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x42\x07\n\x05_blob\"0\n\x0cSyncdVersion\x12\x14\n\x07version\x18\x01 \x01(\x04H\x00\x88\x01\x01\x42\n\n\x08_version\"M\n\rTapLinkAction\x12\x12\n\x05title\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x13\n\x06tapUrl\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x08\n\x06_titleB\t\n\x07_tapUrl\"\xbd\x05\n\x0eTemplateButton\x12\x12\n\x05index\x18\x04 \x01(\rH\x01\x88\x01\x01\x12\x42\n\x10quickReplyButton\x18\x01 \x01(\x0b\x32&.proto.TemplateButton.QuickReplyButtonH\x00\x12\x34\n\turlButton\x18\x02 \x01(\x0b\x32\x1f.proto.TemplateButton.URLButtonH\x00\x12\x36\n\ncallButton\x18\x03 \x01(\x0b\x32 .proto.TemplateButton.CallButtonH\x00\x1a\xb0\x01\n\nCallButton\x12@\n\x0b\x64isplayText\x18\x01 \x01(\x0b\x32&.proto.Message.HighlyStructuredMessageH\x00\x88\x01\x01\x12@\n\x0bphoneNumber\x18\x02 \x01(\x0b\x32&.proto.Message.HighlyStructuredMessageH\x01\x88\x01\x01\x42\x0e\n\x0c_displayTextB\x0e\n\x0c_phoneNumber\x1a|\n\x10QuickReplyButton\x12@\n\x0b\x64isplayText\x18\x01 \x01(\x0b\x32&.proto.Message.HighlyStructuredMessageH\x00\x88\x01\x01\x12\x0f\n\x02id\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x0e\n\x0c_displayTextB\x05\n\x03_id\x1a\x9f\x01\n\tURLButton\x12@\n\x0b\x64isplayText\x18\x01 \x01(\x0b\x32&.proto.Message.HighlyStructuredMessageH\x00\x88\x01\x01\x12\x38\n\x03url\x18\x02 \x01(\x0b\x32&.proto.Message.HighlyStructuredMessageH\x01\x88\x01\x01\x42\x0e\n\x0c_displayTextB\x06\n\x04_urlB\x08\n\x06\x62uttonB\x08\n\x06_index\"\xc3\x01\n\x08ThreadID\x12\x33\n\nthreadType\x18\x01 \x01(\x0e\x32\x1a.proto.ThreadID.ThreadTypeH\x00\x88\x01\x01\x12)\n\tthreadKey\x18\x02 \x01(\x0b\x32\x11.proto.MessageKeyH\x01\x88\x01\x01\":\n\nThreadType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x10\n\x0cVIEW_REPLIES\x10\x01\x12\r\n\tAI_THREAD\x10\x02\x42\r\n\x0b_threadTypeB\x0c\n\n_threadKey\"\xb7\x02\n\x0eUrlTrackingMap\x12K\n\x16urlTrackingMapElements\x18\x01 \x03(\x0b\x32+.proto.UrlTrackingMap.UrlTrackingMapElement\x1a\xd7\x01\n\x15UrlTrackingMapElement\x12\x18\n\x0boriginalUrl\x18\x01 \x01(\tH\x00\x88\x01\x01\x12 \n\x13unconsentedUsersUrl\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x1e\n\x11\x63onsentedUsersUrl\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x16\n\tcardIndex\x18\x04 \x01(\rH\x03\x88\x01\x01\x42\x0e\n\x0c_originalUrlB\x16\n\x14_unconsentedUsersUrlB\x14\n\x12_consentedUsersUrlB\x0c\n\n_cardIndex\"\xaf\x04\n\x0cUserPassword\x12\x33\n\x08\x65ncoding\x18\x01 \x01(\x0e\x32\x1c.proto.UserPassword.EncodingH\x00\x88\x01\x01\x12\x39\n\x0btransformer\x18\x02 \x01(\x0e\x32\x1f.proto.UserPassword.TransformerH\x01\x88\x01\x01\x12:\n\x0etransformerArg\x18\x03 \x03(\x0b\x32\".proto.UserPassword.TransformerArg\x12\x1c\n\x0ftransformedData\x18\x04 \x01(\x0cH\x02\x88\x01\x01\x1a\xb3\x01\n\x0eTransformerArg\x12\x10\n\x03key\x18\x01 \x01(\tH\x00\x88\x01\x01\x12<\n\x05value\x18\x02 \x01(\x0b\x32(.proto.UserPassword.TransformerArg.ValueH\x01\x88\x01\x01\x1a?\n\x05Value\x12\x10\n\x06\x61sBlob\x18\x01 \x01(\x0cH\x00\x12\x1b\n\x11\x61sUnsignedInteger\x18\x02 \x01(\rH\x00\x42\x07\n\x05valueB\x06\n\x04_keyB\x08\n\x06_value\"%\n\x08\x45ncoding\x12\x08\n\x04UTF8\x10\x00\x12\x0f\n\x0bUTF8_BROKEN\x10\x01\"G\n\x0bTransformer\x12\x08\n\x04NONE\x10\x00\x12\x16\n\x12PBKDF2_HMAC_SHA512\x10\x01\x12\x16\n\x12PBKDF2_HMAC_SHA384\x10\x02\x42\x0b\n\t_encodingB\x0e\n\x0c_transformerB\x12\n\x10_transformedData\"\xe8\x01\n\x0bUserReceipt\x12\x0f\n\x07userJid\x18\x01 \x01(\t\x12\x1d\n\x10receiptTimestamp\x18\x02 \x01(\x03H\x00\x88\x01\x01\x12\x1a\n\rreadTimestamp\x18\x03 \x01(\x03H\x01\x88\x01\x01\x12\x1c\n\x0fplayedTimestamp\x18\x04 \x01(\x03H\x02\x88\x01\x01\x12\x18\n\x10pendingDeviceJid\x18\x05 \x03(\t\x12\x1a\n\x12\x64\x65liveredDeviceJid\x18\x06 \x03(\tB\x13\n\x11_receiptTimestampB\x10\n\x0e_readTimestampB\x12\n\x10_playedTimestamp\"\xdf\x02\n\x17VerifiedNameCertificate\x12\x14\n\x07\x64\x65tails\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x16\n\tsignature\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x12\x1c\n\x0fserverSignature\x18\x03 \x01(\x0cH\x02\x88\x01\x01\x1a\xc9\x01\n\x07\x44\x65tails\x12\x13\n\x06serial\x18\x01 \x01(\x04H\x00\x88\x01\x01\x12\x13\n\x06issuer\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x19\n\x0cverifiedName\x18\x04 \x01(\tH\x02\x88\x01\x01\x12,\n\x0elocalizedNames\x18\x08 \x03(\x0b\x32\x14.proto.LocalizedName\x12\x16\n\tissueTime\x18\n \x01(\x04H\x03\x88\x01\x01\x42\t\n\x07_serialB\t\n\x07_issuerB\x0f\n\r_verifiedNameB\x0c\n\n_issueTimeB\n\n\x08_detailsB\x0c\n\n_signatureB\x12\n\x10_serverSignature\"Y\n\x11WallpaperSettings\x12\x15\n\x08\x66ilename\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x14\n\x07opacity\x18\x02 \x01(\rH\x01\x88\x01\x01\x42\x0b\n\t_filenameB\n\n\x08_opacity\"\xf7\x1b\n\x0bWebFeatures\x12\x33\n\rlabelsDisplay\x18\x01 \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x00\x88\x01\x01\x12<\n\x16voipIndividualOutgoing\x18\x02 \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x01\x88\x01\x01\x12.\n\x08groupsV3\x18\x03 \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x02\x88\x01\x01\x12\x34\n\x0egroupsV3Create\x18\x04 \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x03\x88\x01\x01\x12\x34\n\x0e\x63hangeNumberV2\x18\x05 \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x04\x88\x01\x01\x12<\n\x16queryStatusV3Thumbnail\x18\x06 \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x05\x88\x01\x01\x12\x33\n\rliveLocations\x18\x07 \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x06\x88\x01\x01\x12\x30\n\nqueryVname\x18\x08 \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x07\x88\x01\x01\x12<\n\x16voipIndividualIncoming\x18\t \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x08\x88\x01\x01\x12\x37\n\x11quickRepliesQuery\x18\n \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\t\x88\x01\x01\x12.\n\x08payments\x18\x0b \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\n\x88\x01\x01\x12\x36\n\x10stickerPackQuery\x18\x0c \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x0b\x88\x01\x01\x12\x38\n\x12liveLocationsFinal\x18\r \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x0c\x88\x01\x01\x12\x30\n\nlabelsEdit\x18\x0e \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\r\x88\x01\x01\x12\x31\n\x0bmediaUpload\x18\x0f \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x0e\x88\x01\x01\x12\x41\n\x1bmediaUploadRichQuickReplies\x18\x12 \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x0f\x88\x01\x01\x12-\n\x07vnameV2\x18\x13 \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x10\x88\x01\x01\x12\x36\n\x10videoPlaybackUrl\x18\x14 \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x11\x88\x01\x01\x12\x33\n\rstatusRanking\x18\x15 \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x12\x88\x01\x01\x12\x39\n\x13voipIndividualVideo\x18\x16 \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x13\x88\x01\x01\x12\x38\n\x12thirdPartyStickers\x18\x17 \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x14\x88\x01\x01\x12@\n\x1a\x66requentlyForwardedSetting\x18\x18 \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x15\x88\x01\x01\x12<\n\x16groupsV4JoinPermission\x18\x19 \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x16\x88\x01\x01\x12\x34\n\x0erecentStickers\x18\x1a \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x17\x88\x01\x01\x12-\n\x07\x63\x61talog\x18\x1b \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x18\x88\x01\x01\x12\x35\n\x0fstarredStickers\x18\x1c \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x19\x88\x01\x01\x12\x33\n\rvoipGroupCall\x18\x1d \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x1a\x88\x01\x01\x12\x35\n\x0ftemplateMessage\x18\x1e \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x1b\x88\x01\x01\x12\x42\n\x1ctemplateMessageInteractivity\x18\x1f \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x1c\x88\x01\x01\x12\x37\n\x11\x65phemeralMessages\x18  \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x1d\x88\x01\x01\x12\x39\n\x13\x65\x32\x45NotificationSync\x18! \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x1e\x88\x01\x01\x12\x36\n\x10recentStickersV2\x18\" \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\x1f\x88\x01\x01\x12\x36\n\x10recentStickersV3\x18$ \x01(\x0e\x32\x17.proto.WebFeatures.FlagH \x88\x01\x01\x12\x30\n\nuserNotice\x18% \x01(\x0e\x32\x17.proto.WebFeatures.FlagH!\x88\x01\x01\x12-\n\x07support\x18\' \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\"\x88\x01\x01\x12\x35\n\x0fgroupUiiCleanup\x18( \x01(\x0e\x32\x17.proto.WebFeatures.FlagH#\x88\x01\x01\x12\x41\n\x1bgroupDogfoodingInternalOnly\x18) \x01(\x0e\x32\x17.proto.WebFeatures.FlagH$\x88\x01\x01\x12\x32\n\x0csettingsSync\x18* \x01(\x0e\x32\x17.proto.WebFeatures.FlagH%\x88\x01\x01\x12/\n\tarchiveV2\x18+ \x01(\x0e\x32\x17.proto.WebFeatures.FlagH&\x88\x01\x01\x12@\n\x1a\x65phemeralAllowGroupMembers\x18, \x01(\x0e\x32\x17.proto.WebFeatures.FlagH\'\x88\x01\x01\x12:\n\x14\x65phemeral24HDuration\x18- \x01(\x0e\x32\x17.proto.WebFeatures.FlagH(\x88\x01\x01\x12\x34\n\x0emdForceUpgrade\x18. \x01(\x0e\x32\x17.proto.WebFeatures.FlagH)\x88\x01\x01\x12\x36\n\x10\x64isappearingMode\x18/ \x01(\x0e\x32\x17.proto.WebFeatures.FlagH*\x88\x01\x01\x12>\n\x18\x65xternalMdOptInAvailable\x18\x30 \x01(\x0e\x32\x17.proto.WebFeatures.FlagH+\x88\x01\x01\x12>\n\x18noDeleteMessageTimeLimit\x18\x31 \x01(\x0e\x32\x17.proto.WebFeatures.FlagH,\x88\x01\x01\"K\n\x04\x46lag\x12\x0f\n\x0bNOT_STARTED\x10\x00\x12\x11\n\rFORCE_UPGRADE\x10\x01\x12\x0f\n\x0b\x44\x45VELOPMENT\x10\x02\x12\x0e\n\nPRODUCTION\x10\x03\x42\x10\n\x0e_labelsDisplayB\x19\n\x17_voipIndividualOutgoingB\x0b\n\t_groupsV3B\x11\n\x0f_groupsV3CreateB\x11\n\x0f_changeNumberV2B\x19\n\x17_queryStatusV3ThumbnailB\x10\n\x0e_liveLocationsB\r\n\x0b_queryVnameB\x19\n\x17_voipIndividualIncomingB\x14\n\x12_quickRepliesQueryB\x0b\n\t_paymentsB\x13\n\x11_stickerPackQueryB\x15\n\x13_liveLocationsFinalB\r\n\x0b_labelsEditB\x0e\n\x0c_mediaUploadB\x1e\n\x1c_mediaUploadRichQuickRepliesB\n\n\x08_vnameV2B\x13\n\x11_videoPlaybackUrlB\x10\n\x0e_statusRankingB\x16\n\x14_voipIndividualVideoB\x15\n\x13_thirdPartyStickersB\x1d\n\x1b_frequentlyForwardedSettingB\x19\n\x17_groupsV4JoinPermissionB\x11\n\x0f_recentStickersB\n\n\x08_catalogB\x12\n\x10_starredStickersB\x10\n\x0e_voipGroupCallB\x12\n\x10_templateMessageB\x1f\n\x1d_templateMessageInteractivityB\x14\n\x12_ephemeralMessagesB\x16\n\x14_e2ENotificationSyncB\x13\n\x11_recentStickersV2B\x13\n\x11_recentStickersV3B\r\n\x0b_userNoticeB\n\n\x08_supportB\x12\n\x10_groupUiiCleanupB\x1e\n\x1c_groupDogfoodingInternalOnlyB\x0f\n\r_settingsSyncB\x0c\n\n_archiveV2B\x1d\n\x1b_ephemeralAllowGroupMembersB\x17\n\x15_ephemeral24HDurationB\x11\n\x0f_mdForceUpgradeB\x13\n\x11_disappearingModeB\x1b\n\x19_externalMdOptInAvailableB\x1b\n\x19_noDeleteMessageTimeLimit\"\xc7Z\n\x0eWebMessageInfo\x12\x1e\n\x03key\x18\x01 \x01(\x0b\x32\x11.proto.MessageKey\x12$\n\x07message\x18\x02 \x01(\x0b\x32\x0e.proto.MessageH\x00\x88\x01\x01\x12\x1d\n\x10messageTimestamp\x18\x03 \x01(\x04H\x01\x88\x01\x01\x12\x31\n\x06status\x18\x04 \x01(\x0e\x32\x1c.proto.WebMessageInfo.StatusH\x02\x88\x01\x01\x12\x18\n\x0bparticipant\x18\x05 \x01(\tH\x03\x88\x01\x01\x12 \n\x13messageC2STimestamp\x18\x06 \x01(\x04H\x04\x88\x01\x01\x12\x13\n\x06ignore\x18\x10 \x01(\x08H\x05\x88\x01\x01\x12\x14\n\x07starred\x18\x11 \x01(\x08H\x06\x88\x01\x01\x12\x16\n\tbroadcast\x18\x12 \x01(\x08H\x07\x88\x01\x01\x12\x15\n\x08pushName\x18\x13 \x01(\tH\x08\x88\x01\x01\x12\"\n\x15mediaCiphertextSha256\x18\x14 \x01(\x0cH\t\x88\x01\x01\x12\x16\n\tmulticast\x18\x15 \x01(\x08H\n\x88\x01\x01\x12\x14\n\x07urlText\x18\x16 \x01(\x08H\x0b\x88\x01\x01\x12\x16\n\turlNumber\x18\x17 \x01(\x08H\x0c\x88\x01\x01\x12<\n\x0fmessageStubType\x18\x18 \x01(\x0e\x32\x1e.proto.WebMessageInfo.StubTypeH\r\x88\x01\x01\x12\x17\n\nclearMedia\x18\x19 \x01(\x08H\x0e\x88\x01\x01\x12\x1d\n\x15messageStubParameters\x18\x1a \x03(\t\x12\x15\n\x08\x64uration\x18\x1b \x01(\rH\x0f\x88\x01\x01\x12\x0e\n\x06labels\x18\x1c \x03(\t\x12,\n\x0bpaymentInfo\x18\x1d \x01(\x0b\x32\x12.proto.PaymentInfoH\x10\x88\x01\x01\x12\x42\n\x11\x66inalLiveLocation\x18\x1e \x01(\x0b\x32\".proto.Message.LiveLocationMessageH\x11\x88\x01\x01\x12\x32\n\x11quotedPaymentInfo\x18\x1f \x01(\x0b\x32\x12.proto.PaymentInfoH\x12\x88\x01\x01\x12$\n\x17\x65phemeralStartTimestamp\x18  \x01(\x04H\x13\x88\x01\x01\x12\x1e\n\x11\x65phemeralDuration\x18! \x01(\rH\x14\x88\x01\x01\x12\x1d\n\x10\x65phemeralOffToOn\x18\" \x01(\x08H\x15\x88\x01\x01\x12\x1f\n\x12\x65phemeralOutOfSync\x18# \x01(\x08H\x16\x88\x01\x01\x12\x45\n\x10\x62izPrivacyStatus\x18$ \x01(\x0e\x32&.proto.WebMessageInfo.BizPrivacyStatusH\x17\x88\x01\x01\x12\x1c\n\x0fverifiedBizName\x18% \x01(\tH\x18\x88\x01\x01\x12(\n\tmediaData\x18& \x01(\x0b\x32\x10.proto.MediaDataH\x19\x88\x01\x01\x12,\n\x0bphotoChange\x18\' \x01(\x0b\x32\x12.proto.PhotoChangeH\x1a\x88\x01\x01\x12\'\n\x0buserReceipt\x18( \x03(\x0b\x32\x12.proto.UserReceipt\x12\"\n\treactions\x18) \x03(\x0b\x32\x0f.proto.Reaction\x12\x30\n\x11quotedStickerData\x18* \x01(\x0b\x32\x10.proto.MediaDataH\x1b\x88\x01\x01\x12\x1c\n\x0f\x66utureproofData\x18+ \x01(\x0cH\x1c\x88\x01\x01\x12(\n\tstatusPsa\x18, \x01(\x0b\x32\x10.proto.StatusPSAH\x1d\x88\x01\x01\x12&\n\x0bpollUpdates\x18- \x03(\x0b\x32\x11.proto.PollUpdate\x12\x42\n\x16pollAdditionalMetadata\x18. \x01(\x0b\x32\x1d.proto.PollAdditionalMetadataH\x1e\x88\x01\x01\x12\x14\n\x07\x61gentId\x18/ \x01(\tH\x1f\x88\x01\x01\x12 \n\x13statusAlreadyViewed\x18\x30 \x01(\x08H \x88\x01\x01\x12\x1a\n\rmessageSecret\x18\x31 \x01(\x0cH!\x88\x01\x01\x12*\n\nkeepInChat\x18\x32 \x01(\x0b\x32\x11.proto.KeepInChatH\"\x88\x01\x01\x12,\n\x1foriginalSelfAuthorUserJidString\x18\x33 \x01(\tH#\x88\x01\x01\x12#\n\x16revokeMessageTimestamp\x18\x34 \x01(\x04H$\x88\x01\x01\x12(\n\tpinInChat\x18\x36 \x01(\x0b\x32\x10.proto.PinInChatH%\x88\x01\x01\x12:\n\x12premiumMessageInfo\x18\x37 \x01(\x0b\x32\x19.proto.PremiumMessageInfoH&\x88\x01\x01\x12\x1e\n\x11is1PBizBotMessage\x18\x38 \x01(\x08H\'\x88\x01\x01\x12\"\n\x15isGroupHistoryMessage\x18\x39 \x01(\x08H(\x88\x01\x01\x12!\n\x14\x62otMessageInvokerJid\x18: \x01(\tH)\x88\x01\x01\x12\x34\n\x0f\x63ommentMetadata\x18; \x01(\x0b\x32\x16.proto.CommentMetadataH*\x88\x01\x01\x12,\n\x0e\x65ventResponses\x18= \x03(\x0b\x32\x14.proto.EventResponse\x12:\n\x12reportingTokenInfo\x18> \x01(\x0b\x32\x19.proto.ReportingTokenInfoH+\x88\x01\x01\x12\x1f\n\x12newsletterServerId\x18? \x01(\x04H,\x88\x01\x01\x12\x44\n\x17\x65ventAdditionalMetadata\x18@ \x01(\x0b\x32\x1e.proto.EventAdditionalMetadataH-\x88\x01\x01\x12 \n\x13isMentionedInStatus\x18\x41 \x01(\x08H.\x88\x01\x01\x12\x16\n\x0estatusMentions\x18\x42 \x03(\t\x12/\n\x0ftargetMessageId\x18\x43 \x01(\x0b\x32\x11.proto.MessageKeyH/\x88\x01\x01\x12*\n\rmessageAddOns\x18\x44 \x03(\x0b\x32\x13.proto.MessageAddOn\x12\x42\n\x18statusMentionMessageInfo\x18\x45 \x01(\x0b\x32\x1b.proto.StatusMentionMessageH0\x88\x01\x01\x12\x1f\n\x12isSupportAiMessage\x18\x46 \x01(\x08H1\x88\x01\x01\x12\x1c\n\x14statusMentionSources\x18G \x03(\t\x12+\n\x12supportAiCitations\x18H \x03(\x0b\x32\x0f.proto.Citation\x12\x18\n\x0b\x62otTargetId\x18I \x01(\tH2\x88\x01\x01\x12X\n!groupHistoryIndividualMessageInfo\x18J \x01(\x0b\x32(.proto.GroupHistoryIndividualMessageInfoH3\x88\x01\x01\x12\x42\n\x16groupHistoryBundleInfo\x18K \x01(\x0b\x32\x1d.proto.GroupHistoryBundleInfoH4\x88\x01\x01\x12^\n$interactiveMessageAdditionalMetadata\x18L \x01(\x0b\x32+.proto.InteractiveMessageAdditionalMetadataH5\x88\x01\x01\x12:\n\x12quarantinedMessage\x18M \x01(\x0b\x32\x19.proto.QuarantinedMessageH6\x88\x01\x01\"=\n\x10\x42izPrivacyStatus\x12\x08\n\x04\x45\x32\x45\x45\x10\x00\x12\x06\n\x02\x46\x42\x10\x02\x12\x07\n\x03\x42SP\x10\x01\x12\x0e\n\nBSP_AND_FB\x10\x03\"X\n\x06Status\x12\t\n\x05\x45RROR\x10\x00\x12\x0b\n\x07PENDING\x10\x01\x12\x0e\n\nSERVER_ACK\x10\x02\x12\x10\n\x0c\x44\x45LIVERY_ACK\x10\x03\x12\x08\n\x04READ\x10\x04\x12\n\n\x06PLAYED\x10\x05\"\x9b;\n\x08StubType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\n\n\x06REVOKE\x10\x01\x12\x0e\n\nCIPHERTEXT\x10\x02\x12\x0f\n\x0b\x46UTUREPROOF\x10\x03\x12\x1b\n\x17NON_VERIFIED_TRANSITION\x10\x04\x12\x19\n\x15UNVERIFIED_TRANSITION\x10\x05\x12\x17\n\x13VERIFIED_TRANSITION\x10\x06\x12\x18\n\x14VERIFIED_LOW_UNKNOWN\x10\x07\x12\x11\n\rVERIFIED_HIGH\x10\x08\x12\x1c\n\x18VERIFIED_INITIAL_UNKNOWN\x10\t\x12\x18\n\x14VERIFIED_INITIAL_LOW\x10\n\x12\x19\n\x15VERIFIED_INITIAL_HIGH\x10\x0b\x12#\n\x1fVERIFIED_TRANSITION_ANY_TO_NONE\x10\x0c\x12#\n\x1fVERIFIED_TRANSITION_ANY_TO_HIGH\x10\r\x12#\n\x1fVERIFIED_TRANSITION_HIGH_TO_LOW\x10\x0e\x12\'\n#VERIFIED_TRANSITION_HIGH_TO_UNKNOWN\x10\x0f\x12&\n\"VERIFIED_TRANSITION_UNKNOWN_TO_LOW\x10\x10\x12&\n\"VERIFIED_TRANSITION_LOW_TO_UNKNOWN\x10\x11\x12#\n\x1fVERIFIED_TRANSITION_NONE_TO_LOW\x10\x12\x12\'\n#VERIFIED_TRANSITION_NONE_TO_UNKNOWN\x10\x13\x12\x10\n\x0cGROUP_CREATE\x10\x14\x12\x18\n\x14GROUP_CHANGE_SUBJECT\x10\x15\x12\x15\n\x11GROUP_CHANGE_ICON\x10\x16\x12\x1c\n\x18GROUP_CHANGE_INVITE_LINK\x10\x17\x12\x1c\n\x18GROUP_CHANGE_DESCRIPTION\x10\x18\x12\x19\n\x15GROUP_CHANGE_RESTRICT\x10\x19\x12\x19\n\x15GROUP_CHANGE_ANNOUNCE\x10\x1a\x12\x19\n\x15GROUP_PARTICIPANT_ADD\x10\x1b\x12\x1c\n\x18GROUP_PARTICIPANT_REMOVE\x10\x1c\x12\x1d\n\x19GROUP_PARTICIPANT_PROMOTE\x10\x1d\x12\x1c\n\x18GROUP_PARTICIPANT_DEMOTE\x10\x1e\x12\x1c\n\x18GROUP_PARTICIPANT_INVITE\x10\x1f\x12\x1b\n\x17GROUP_PARTICIPANT_LEAVE\x10 \x12#\n\x1fGROUP_PARTICIPANT_CHANGE_NUMBER\x10!\x12\x14\n\x10\x42ROADCAST_CREATE\x10\"\x12\x11\n\rBROADCAST_ADD\x10#\x12\x14\n\x10\x42ROADCAST_REMOVE\x10$\x12\x18\n\x14GENERIC_NOTIFICATION\x10%\x12\x18\n\x14\x45\x32\x45_IDENTITY_CHANGED\x10&\x12\x11\n\rE2E_ENCRYPTED\x10\'\x12\x15\n\x11\x43\x41LL_MISSED_VOICE\x10(\x12\x15\n\x11\x43\x41LL_MISSED_VIDEO\x10)\x12\x1c\n\x18INDIVIDUAL_CHANGE_NUMBER\x10*\x12\x10\n\x0cGROUP_DELETE\x10+\x12&\n\"GROUP_ANNOUNCE_MODE_MESSAGE_BOUNCE\x10,\x12\x1b\n\x17\x43\x41LL_MISSED_GROUP_VOICE\x10-\x12\x1b\n\x17\x43\x41LL_MISSED_GROUP_VIDEO\x10.\x12\x16\n\x12PAYMENT_CIPHERTEXT\x10/\x12\x17\n\x13PAYMENT_FUTUREPROOF\x10\x30\x12,\n(PAYMENT_TRANSACTION_STATUS_UPDATE_FAILED\x10\x31\x12.\n*PAYMENT_TRANSACTION_STATUS_UPDATE_REFUNDED\x10\x32\x12\x33\n/PAYMENT_TRANSACTION_STATUS_UPDATE_REFUND_FAILED\x10\x33\x12\x35\n1PAYMENT_TRANSACTION_STATUS_RECEIVER_PENDING_SETUP\x10\x34\x12<\n8PAYMENT_TRANSACTION_STATUS_RECEIVER_SUCCESS_AFTER_HICCUP\x10\x35\x12)\n%PAYMENT_ACTION_ACCOUNT_SETUP_REMINDER\x10\x36\x12(\n$PAYMENT_ACTION_SEND_PAYMENT_REMINDER\x10\x37\x12*\n&PAYMENT_ACTION_SEND_PAYMENT_INVITATION\x10\x38\x12#\n\x1fPAYMENT_ACTION_REQUEST_DECLINED\x10\x39\x12\"\n\x1ePAYMENT_ACTION_REQUEST_EXPIRED\x10:\x12$\n PAYMENT_ACTION_REQUEST_CANCELLED\x10;\x12)\n%BIZ_VERIFIED_TRANSITION_TOP_TO_BOTTOM\x10<\x12)\n%BIZ_VERIFIED_TRANSITION_BOTTOM_TO_TOP\x10=\x12\x11\n\rBIZ_INTRO_TOP\x10>\x12\x14\n\x10\x42IZ_INTRO_BOTTOM\x10?\x12\x13\n\x0f\x42IZ_NAME_CHANGE\x10@\x12\x1c\n\x18\x42IZ_MOVE_TO_CONSUMER_APP\x10\x41\x12\x1e\n\x1a\x42IZ_TWO_TIER_MIGRATION_TOP\x10\x42\x12!\n\x1d\x42IZ_TWO_TIER_MIGRATION_BOTTOM\x10\x43\x12\r\n\tOVERSIZED\x10\x44\x12(\n$GROUP_CHANGE_NO_FREQUENTLY_FORWARDED\x10\x45\x12\x1c\n\x18GROUP_V4_ADD_INVITE_SENT\x10\x46\x12&\n\"GROUP_PARTICIPANT_ADD_REQUEST_JOIN\x10G\x12\x1c\n\x18\x43HANGE_EPHEMERAL_SETTING\x10H\x12\x16\n\x12\x45\x32\x45_DEVICE_CHANGED\x10I\x12\x0f\n\x0bVIEWED_ONCE\x10J\x12\x15\n\x11\x45\x32\x45_ENCRYPTED_NOW\x10K\x12\"\n\x1e\x42LUE_MSG_BSP_FB_TO_BSP_PREMISE\x10L\x12\x1e\n\x1a\x42LUE_MSG_BSP_FB_TO_SELF_FB\x10M\x12#\n\x1f\x42LUE_MSG_BSP_FB_TO_SELF_PREMISE\x10N\x12\x1e\n\x1a\x42LUE_MSG_BSP_FB_UNVERIFIED\x10O\x12\x37\n3BLUE_MSG_BSP_FB_UNVERIFIED_TO_SELF_PREMISE_VERIFIED\x10P\x12\x1c\n\x18\x42LUE_MSG_BSP_FB_VERIFIED\x10Q\x12\x37\n3BLUE_MSG_BSP_FB_VERIFIED_TO_SELF_PREMISE_UNVERIFIED\x10R\x12(\n$BLUE_MSG_BSP_PREMISE_TO_SELF_PREMISE\x10S\x12#\n\x1f\x42LUE_MSG_BSP_PREMISE_UNVERIFIED\x10T\x12<\n8BLUE_MSG_BSP_PREMISE_UNVERIFIED_TO_SELF_PREMISE_VERIFIED\x10U\x12!\n\x1d\x42LUE_MSG_BSP_PREMISE_VERIFIED\x10V\x12<\n8BLUE_MSG_BSP_PREMISE_VERIFIED_TO_SELF_PREMISE_UNVERIFIED\x10W\x12*\n&BLUE_MSG_CONSUMER_TO_BSP_FB_UNVERIFIED\x10X\x12/\n+BLUE_MSG_CONSUMER_TO_BSP_PREMISE_UNVERIFIED\x10Y\x12+\n\'BLUE_MSG_CONSUMER_TO_SELF_FB_UNVERIFIED\x10Z\x12\x30\n,BLUE_MSG_CONSUMER_TO_SELF_PREMISE_UNVERIFIED\x10[\x12#\n\x1f\x42LUE_MSG_SELF_FB_TO_BSP_PREMISE\x10\\\x12$\n BLUE_MSG_SELF_FB_TO_SELF_PREMISE\x10]\x12\x1f\n\x1b\x42LUE_MSG_SELF_FB_UNVERIFIED\x10^\x12\x38\n4BLUE_MSG_SELF_FB_UNVERIFIED_TO_SELF_PREMISE_VERIFIED\x10_\x12\x1d\n\x19\x42LUE_MSG_SELF_FB_VERIFIED\x10`\x12\x38\n4BLUE_MSG_SELF_FB_VERIFIED_TO_SELF_PREMISE_UNVERIFIED\x10\x61\x12(\n$BLUE_MSG_SELF_PREMISE_TO_BSP_PREMISE\x10\x62\x12$\n BLUE_MSG_SELF_PREMISE_UNVERIFIED\x10\x63\x12\"\n\x1e\x42LUE_MSG_SELF_PREMISE_VERIFIED\x10\x64\x12\x16\n\x12\x42LUE_MSG_TO_BSP_FB\x10\x65\x12\x18\n\x14\x42LUE_MSG_TO_CONSUMER\x10\x66\x12\x17\n\x13\x42LUE_MSG_TO_SELF_FB\x10g\x12*\n&BLUE_MSG_UNVERIFIED_TO_BSP_FB_VERIFIED\x10h\x12/\n+BLUE_MSG_UNVERIFIED_TO_BSP_PREMISE_VERIFIED\x10i\x12+\n\'BLUE_MSG_UNVERIFIED_TO_SELF_FB_VERIFIED\x10j\x12#\n\x1f\x42LUE_MSG_UNVERIFIED_TO_VERIFIED\x10k\x12*\n&BLUE_MSG_VERIFIED_TO_BSP_FB_UNVERIFIED\x10l\x12/\n+BLUE_MSG_VERIFIED_TO_BSP_PREMISE_UNVERIFIED\x10m\x12+\n\'BLUE_MSG_VERIFIED_TO_SELF_FB_UNVERIFIED\x10n\x12#\n\x1f\x42LUE_MSG_VERIFIED_TO_UNVERIFIED\x10o\x12\x36\n2BLUE_MSG_BSP_FB_UNVERIFIED_TO_BSP_PREMISE_VERIFIED\x10p\x12\x32\n.BLUE_MSG_BSP_FB_UNVERIFIED_TO_SELF_FB_VERIFIED\x10q\x12\x36\n2BLUE_MSG_BSP_FB_VERIFIED_TO_BSP_PREMISE_UNVERIFIED\x10r\x12\x32\n.BLUE_MSG_BSP_FB_VERIFIED_TO_SELF_FB_UNVERIFIED\x10s\x12\x37\n3BLUE_MSG_SELF_FB_UNVERIFIED_TO_BSP_PREMISE_VERIFIED\x10t\x12\x37\n3BLUE_MSG_SELF_FB_VERIFIED_TO_BSP_PREMISE_UNVERIFIED\x10u\x12\x1c\n\x18\x45\x32\x45_IDENTITY_UNAVAILABLE\x10v\x12\x12\n\x0eGROUP_CREATING\x10w\x12\x17\n\x13GROUP_CREATE_FAILED\x10x\x12\x11\n\rGROUP_BOUNCED\x10y\x12\x11\n\rBLOCK_CONTACT\x10z\x12!\n\x1d\x45PHEMERAL_SETTING_NOT_APPLIED\x10{\x12\x0f\n\x0bSYNC_FAILED\x10|\x12\x0b\n\x07SYNCING\x10}\x12\x1c\n\x18\x42IZ_PRIVACY_MODE_INIT_FB\x10~\x12\x1d\n\x19\x42IZ_PRIVACY_MODE_INIT_BSP\x10\x7f\x12\x1b\n\x16\x42IZ_PRIVACY_MODE_TO_FB\x10\x80\x01\x12\x1c\n\x17\x42IZ_PRIVACY_MODE_TO_BSP\x10\x81\x01\x12\x16\n\x11\x44ISAPPEARING_MODE\x10\x82\x01\x12\x1c\n\x17\x45\x32\x45_DEVICE_FETCH_FAILED\x10\x83\x01\x12\x11\n\x0c\x41\x44MIN_REVOKE\x10\x84\x01\x12$\n\x1fGROUP_INVITE_LINK_GROWTH_LOCKED\x10\x85\x01\x12 \n\x1b\x43OMMUNITY_LINK_PARENT_GROUP\x10\x86\x01\x12!\n\x1c\x43OMMUNITY_LINK_SIBLING_GROUP\x10\x87\x01\x12\x1d\n\x18\x43OMMUNITY_LINK_SUB_GROUP\x10\x88\x01\x12\"\n\x1d\x43OMMUNITY_UNLINK_PARENT_GROUP\x10\x89\x01\x12#\n\x1e\x43OMMUNITY_UNLINK_SIBLING_GROUP\x10\x8a\x01\x12\x1f\n\x1a\x43OMMUNITY_UNLINK_SUB_GROUP\x10\x8b\x01\x12\x1d\n\x18GROUP_PARTICIPANT_ACCEPT\x10\x8c\x01\x12(\n#GROUP_PARTICIPANT_LINKED_GROUP_JOIN\x10\x8d\x01\x12\x15\n\x10\x43OMMUNITY_CREATE\x10\x8e\x01\x12\x1b\n\x16\x45PHEMERAL_KEEP_IN_CHAT\x10\x8f\x01\x12+\n&GROUP_MEMBERSHIP_JOIN_APPROVAL_REQUEST\x10\x90\x01\x12(\n#GROUP_MEMBERSHIP_JOIN_APPROVAL_MODE\x10\x91\x01\x12\"\n\x1dINTEGRITY_UNLINK_PARENT_GROUP\x10\x92\x01\x12\"\n\x1d\x43OMMUNITY_PARTICIPANT_PROMOTE\x10\x93\x01\x12!\n\x1c\x43OMMUNITY_PARTICIPANT_DEMOTE\x10\x94\x01\x12#\n\x1e\x43OMMUNITY_PARENT_GROUP_DELETED\x10\x95\x01\x12\x34\n/COMMUNITY_LINK_PARENT_GROUP_MEMBERSHIP_APPROVAL\x10\x96\x01\x12\x34\n/GROUP_PARTICIPANT_JOINED_GROUP_AND_PARENT_GROUP\x10\x97\x01\x12\x1a\n\x15MASKED_THREAD_CREATED\x10\x98\x01\x12\x1b\n\x16MASKED_THREAD_UNMASKED\x10\x99\x01\x12\x18\n\x13\x42IZ_CHAT_ASSIGNMENT\x10\x9a\x01\x12\r\n\x08\x43HAT_PSA\x10\x9b\x01\x12\x1f\n\x1a\x43HAT_POLL_CREATION_MESSAGE\x10\x9c\x01\x12\x1e\n\x19\x43\x41G_MASKED_THREAD_CREATED\x10\x9d\x01\x12+\n&COMMUNITY_PARENT_GROUP_SUBJECT_CHANGED\x10\x9e\x01\x12\x18\n\x13\x43\x41G_INVITE_AUTO_ADD\x10\x9f\x01\x12!\n\x1c\x42IZ_CHAT_ASSIGNMENT_UNASSIGN\x10\xa0\x01\x12\x1b\n\x16\x43\x41G_INVITE_AUTO_JOINED\x10\xa1\x01\x12!\n\x1cSCHEDULED_CALL_START_MESSAGE\x10\xa2\x01\x12\x1a\n\x15\x43OMMUNITY_INVITE_RICH\x10\xa3\x01\x12#\n\x1e\x43OMMUNITY_INVITE_AUTO_ADD_RICH\x10\xa4\x01\x12\x1a\n\x15SUB_GROUP_INVITE_RICH\x10\xa5\x01\x12#\n\x1eSUB_GROUP_PARTICIPANT_ADD_RICH\x10\xa6\x01\x12%\n COMMUNITY_LINK_PARENT_GROUP_RICH\x10\xa7\x01\x12#\n\x1e\x43OMMUNITY_PARTICIPANT_ADD_RICH\x10\xa8\x01\x12\"\n\x1dSILENCED_UNKNOWN_CALLER_AUDIO\x10\xa9\x01\x12\"\n\x1dSILENCED_UNKNOWN_CALLER_VIDEO\x10\xaa\x01\x12\x1a\n\x15GROUP_MEMBER_ADD_MODE\x10\xab\x01\x12\x39\n4GROUP_MEMBERSHIP_JOIN_APPROVAL_REQUEST_NON_ADMIN_ADD\x10\xac\x01\x12!\n\x1c\x43OMMUNITY_CHANGE_DESCRIPTION\x10\xad\x01\x12\x12\n\rSENDER_INVITE\x10\xae\x01\x12\x14\n\x0fRECEIVER_INVITE\x10\xaf\x01\x12(\n#COMMUNITY_ALLOW_MEMBER_ADDED_GROUPS\x10\xb0\x01\x12\x1b\n\x16PINNED_MESSAGE_IN_CHAT\x10\xb1\x01\x12!\n\x1cPAYMENT_INVITE_SETUP_INVITER\x10\xb2\x01\x12.\n)PAYMENT_INVITE_SETUP_INVITEE_RECEIVE_ONLY\x10\xb3\x01\x12\x32\n-PAYMENT_INVITE_SETUP_INVITEE_SEND_AND_RECEIVE\x10\xb4\x01\x12\x1c\n\x17LINKED_GROUP_CALL_START\x10\xb5\x01\x12#\n\x1eREPORT_TO_ADMIN_ENABLED_STATUS\x10\xb6\x01\x12\x1a\n\x15\x45MPTY_SUBGROUP_CREATE\x10\xb7\x01\x12\x1a\n\x15SCHEDULED_CALL_CANCEL\x10\xb8\x01\x12+\n&SUBGROUP_ADMIN_TRIGGERED_AUTO_ADD_RICH\x10\xb9\x01\x12(\n#GROUP_CHANGE_RECENT_HISTORY_SHARING\x10\xba\x01\x12$\n\x1fPAID_MESSAGE_SERVER_CAMPAIGN_ID\x10\xbb\x01\x12\x18\n\x13GENERAL_CHAT_CREATE\x10\xbc\x01\x12\x15\n\x10GENERAL_CHAT_ADD\x10\xbd\x01\x12#\n\x1eGENERAL_CHAT_AUTO_ADD_DISABLED\x10\xbe\x01\x12 \n\x1bSUGGESTED_SUBGROUP_ANNOUNCE\x10\xbf\x01\x12!\n\x1c\x42IZ_BOT_1P_MESSAGING_ENABLED\x10\xc0\x01\x12\x14\n\x0f\x43HANGE_USERNAME\x10\xc1\x01\x12\x1f\n\x1a\x42IZ_COEX_PRIVACY_INIT_SELF\x10\xc2\x01\x12%\n BIZ_COEX_PRIVACY_TRANSITION_SELF\x10\xc3\x01\x12\x19\n\x14SUPPORT_AI_EDUCATION\x10\xc4\x01\x12!\n\x1c\x42IZ_BOT_3P_MESSAGING_ENABLED\x10\xc5\x01\x12\x1b\n\x16REMINDER_SETUP_MESSAGE\x10\xc6\x01\x12\x1a\n\x15REMINDER_SENT_MESSAGE\x10\xc7\x01\x12\x1c\n\x17REMINDER_CANCEL_MESSAGE\x10\xc8\x01\x12\x1a\n\x15\x42IZ_COEX_PRIVACY_INIT\x10\xc9\x01\x12 \n\x1b\x42IZ_COEX_PRIVACY_TRANSITION\x10\xca\x01\x12\x16\n\x11GROUP_DEACTIVATED\x10\xcb\x01\x12\'\n\"COMMUNITY_DEACTIVATE_SIBLING_GROUP\x10\xcc\x01\x12\x12\n\rEVENT_UPDATED\x10\xcd\x01\x12\x13\n\x0e\x45VENT_CANCELED\x10\xce\x01\x12\x1c\n\x17\x43OMMUNITY_OWNER_UPDATED\x10\xcf\x01\x12*\n%COMMUNITY_SUB_GROUP_VISIBILITY_HIDDEN\x10\xd0\x01\x12$\n\x1f\x43\x41PI_GROUP_NE2EE_SYSTEM_MESSAGE\x10\xd1\x01\x12\x13\n\x0eSTATUS_MENTION\x10\xd2\x01\x12!\n\x1cUSER_CONTROLS_SYSTEM_MESSAGE\x10\xd3\x01\x12\x1b\n\x16SUPPORT_SYSTEM_MESSAGE\x10\xd4\x01\x12\x0f\n\nCHANGE_LID\x10\xd5\x01\x12\x31\n,BIZ_CUSTOMER_3PD_DATA_SHARING_OPT_IN_MESSAGE\x10\xd6\x01\x12\x32\n-BIZ_CUSTOMER_3PD_DATA_SHARING_OPT_OUT_MESSAGE\x10\xd7\x01\x12\x19\n\x14\x43HANGE_LIMIT_SHARING\x10\xd8\x01\x12\x1b\n\x16GROUP_MEMBER_LINK_MODE\x10\xd9\x01\x12\x32\n-BIZ_AUTOMATICALLY_LABELED_CHAT_SYSTEM_MESSAGE\x10\xda\x01\x12\x30\n+PHONE_NUMBER_HIDING_CHAT_DEPRECATED_MESSAGE\x10\xdb\x01\x12\x18\n\x13QUARANTINED_MESSAGE\x10\xdc\x01\x12*\n%GROUP_MEMBER_SHARE_GROUP_HISTORY_MODE\x10\xdd\x01\x42\n\n\x08_messageB\x13\n\x11_messageTimestampB\t\n\x07_statusB\x0e\n\x0c_participantB\x16\n\x14_messageC2STimestampB\t\n\x07_ignoreB\n\n\x08_starredB\x0c\n\n_broadcastB\x0b\n\t_pushNameB\x18\n\x16_mediaCiphertextSha256B\x0c\n\n_multicastB\n\n\x08_urlTextB\x0c\n\n_urlNumberB\x12\n\x10_messageStubTypeB\r\n\x0b_clearMediaB\x0b\n\t_durationB\x0e\n\x0c_paymentInfoB\x14\n\x12_finalLiveLocationB\x14\n\x12_quotedPaymentInfoB\x1a\n\x18_ephemeralStartTimestampB\x14\n\x12_ephemeralDurationB\x13\n\x11_ephemeralOffToOnB\x15\n\x13_ephemeralOutOfSyncB\x13\n\x11_bizPrivacyStatusB\x12\n\x10_verifiedBizNameB\x0c\n\n_mediaDataB\x0e\n\x0c_photoChangeB\x14\n\x12_quotedStickerDataB\x12\n\x10_futureproofDataB\x0c\n\n_statusPsaB\x19\n\x17_pollAdditionalMetadataB\n\n\x08_agentIdB\x16\n\x14_statusAlreadyViewedB\x10\n\x0e_messageSecretB\r\n\x0b_keepInChatB\"\n _originalSelfAuthorUserJidStringB\x19\n\x17_revokeMessageTimestampB\x0c\n\n_pinInChatB\x15\n\x13_premiumMessageInfoB\x14\n\x12_is1PBizBotMessageB\x18\n\x16_isGroupHistoryMessageB\x17\n\x15_botMessageInvokerJidB\x12\n\x10_commentMetadataB\x15\n\x13_reportingTokenInfoB\x15\n\x13_newsletterServerIdB\x1a\n\x18_eventAdditionalMetadataB\x16\n\x14_isMentionedInStatusB\x12\n\x10_targetMessageIdB\x1b\n\x19_statusMentionMessageInfoB\x15\n\x13_isSupportAiMessageB\x0e\n\x0c_botTargetIdB$\n\"_groupHistoryIndividualMessageInfoB\x19\n\x17_groupHistoryBundleInfoB\'\n%_interactiveMessageAdditionalMetadataB\x15\n\x13_quarantinedMessage\"\xcd\x01\n\x14WebNotificationsInfo\x12\x16\n\ttimestamp\x18\x02 \x01(\x04H\x00\x88\x01\x01\x12\x18\n\x0bunreadChats\x18\x03 \x01(\rH\x01\x88\x01\x01\x12\x1f\n\x12notifyMessageCount\x18\x04 \x01(\rH\x02\x88\x01\x01\x12-\n\x0enotifyMessages\x18\x05 \x03(\x0b\x32\x15.proto.WebMessageInfoB\x0c\n\n_timestampB\x0e\n\x0c_unreadChatsB\x15\n\x13_notifyMessageCount*)\n\x11\x41\x44VEncryptionType\x12\x08\n\x04\x45\x32\x45\x45\x10\x00\x12\n\n\x06HOSTED\x10\x01*b\n\x19\x41IRichResponseMessageType\x12!\n\x1d\x41I_RICH_RESPONSE_TYPE_UNKNOWN\x10\x00\x12\"\n\x1e\x41I_RICH_RESPONSE_TYPE_STANDARD\x10\x01*\xca\x02\n\x1c\x41IRichResponseSubMessageType\x12\x1c\n\x18\x41I_RICH_RESPONSE_UNKNOWN\x10\x00\x12\x1f\n\x1b\x41I_RICH_RESPONSE_GRID_IMAGE\x10\x01\x12\x19\n\x15\x41I_RICH_RESPONSE_TEXT\x10\x02\x12!\n\x1d\x41I_RICH_RESPONSE_INLINE_IMAGE\x10\x03\x12\x1a\n\x16\x41I_RICH_RESPONSE_TABLE\x10\x04\x12\x19\n\x15\x41I_RICH_RESPONSE_CODE\x10\x05\x12\x1c\n\x18\x41I_RICH_RESPONSE_DYNAMIC\x10\x06\x12\x18\n\x14\x41I_RICH_RESPONSE_MAP\x10\x07\x12\x1a\n\x16\x41I_RICH_RESPONSE_LATEX\x10\x08\x12\"\n\x1e\x41I_RICH_RESPONSE_CONTENT_ITEMS\x10\t*\xb3\x08\n\x14\x42otMetricsEntryPoint\x12\x19\n\x15UNDEFINED_ENTRY_POINT\x10\x00\x12\x0b\n\x07\x46\x41VICON\x10\x01\x12\x0c\n\x08\x43HATLIST\x10\x02\x12#\n\x1f\x41ISEARCH_NULL_STATE_PAPER_PLANE\x10\x03\x12\"\n\x1e\x41ISEARCH_NULL_STATE_SUGGESTION\x10\x04\x12\"\n\x1e\x41ISEARCH_TYPE_AHEAD_SUGGESTION\x10\x05\x12#\n\x1f\x41ISEARCH_TYPE_AHEAD_PAPER_PLANE\x10\x06\x12\'\n#AISEARCH_TYPE_AHEAD_RESULT_CHATLIST\x10\x07\x12\'\n#AISEARCH_TYPE_AHEAD_RESULT_MESSAGES\x10\x08\x12\x16\n\x12\x41IVOICE_SEARCH_BAR\x10\t\x12\x13\n\x0f\x41IVOICE_FAVICON\x10\n\x12\x0c\n\x08\x41ISTUDIO\x10\x0b\x12\x0c\n\x08\x44\x45\x45PLINK\x10\x0c\x12\x10\n\x0cNOTIFICATION\x10\r\x12\x1a\n\x16PROFILE_MESSAGE_BUTTON\x10\x0e\x12\x0b\n\x07\x46ORWARD\x10\x0f\x12\x10\n\x0c\x41PP_SHORTCUT\x10\x10\x12\r\n\tFF_FAMILY\x10\x11\x12\n\n\x06\x41I_TAB\x10\x12\x12\x0b\n\x07\x41I_HOME\x10\x13\x12\x19\n\x15\x41I_DEEPLINK_IMMERSIVE\x10\x14\x12\x0f\n\x0b\x41I_DEEPLINK\x10\x15\x12#\n\x1fMETA_AI_CHAT_SHORTCUT_AI_STUDIO\x10\x16\x12\x1f\n\x1bUGC_CHAT_SHORTCUT_AI_STUDIO\x10\x17\x12\x16\n\x12NEW_CHAT_AI_STUDIO\x10\x18\x12 \n\x1c\x41IVOICE_FAVICON_CALL_HISTORY\x10\x19\x12\x1c\n\x18\x41SK_META_AI_CONTEXT_MENU\x10\x1a\x12!\n\x1d\x41SK_META_AI_CONTEXT_MENU_1ON1\x10\x1b\x12\"\n\x1e\x41SK_META_AI_CONTEXT_MENU_GROUP\x10\x1c\x12\x17\n\x13INVOKE_META_AI_1ON1\x10\x1d\x12\x18\n\x14INVOKE_META_AI_GROUP\x10\x1e\x12\x13\n\x0fMETA_AI_FORWARD\x10\x1f\x12\x17\n\x13NEW_CHAT_AI_CONTACT\x10 \x12$\n MESSAGE_QUICK_ACTION_1_ON_1_CHAT\x10!\x12#\n\x1fMESSAGE_QUICK_ACTION_GROUP_CHAT\x10\"\x12\x1f\n\x1b\x41TTACHMENT_TRAY_1_ON_1_CHAT\x10#\x12\x1e\n\x1a\x41TTACHMENT_TRAY_GROUP_CHAT\x10$\x12!\n\x1d\x41SK_META_AI_MEDIA_VIEWER_1ON1\x10%\x12\"\n\x1e\x41SK_META_AI_MEDIA_VIEWER_GROUP\x10&*\xd2\x01\n\x1a\x42otMetricsThreadEntryPoint\x12.\n*BOT_METRICS_THREAD_ENTRY_POINT_UNSPECIFIED\x10\x00\x12\x11\n\rAI_TAB_THREAD\x10\x01\x12\x12\n\x0e\x41I_HOME_THREAD\x10\x02\x12 \n\x1c\x41I_DEEPLINK_IMMERSIVE_THREAD\x10\x03\x12\x16\n\x12\x41I_DEEPLINK_THREAD\x10\x04\x12#\n\x1f\x41SK_META_AI_CONTEXT_MENU_THREAD\x10\x05*}\n\x10\x42otSessionSource\x12\x08\n\x04NONE\x10\x00\x12\x0e\n\nNULL_STATE\x10\x01\x12\r\n\tTYPEAHEAD\x10\x02\x12\x0e\n\nUSER_INPUT\x10\x03\x12\r\n\tEMU_FLASH\x10\x04\x12\x16\n\x12\x45MU_FLASH_FOLLOWUP\x10\x05\x12\t\n\x05VOICE\x10\x06*\x8b\x01\n\x0e\x43ollectionName\x12\x1b\n\x17\x43OLLECTION_NAME_UNKNOWN\x10\x00\x12\x0b\n\x07REGULAR\x10\x01\x12\x0f\n\x0bREGULAR_LOW\x10\x02\x12\x10\n\x0cREGULAR_HIGH\x10\x03\x12\x12\n\x0e\x43RITICAL_BLOCK\x10\x04\x12\x18\n\x14\x43RITICAL_UNBLOCK_LOW\x10\x05*@\n\x08KeepType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x10\n\x0cKEEP_FOR_ALL\x10\x01\x12\x15\n\x11UNDO_KEEP_FOR_ALL\x10\x02*/\n\x0fMediaVisibility\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x00\x12\x07\n\x03OFF\x10\x01\x12\x06\n\x02ON\x10\x02*\xfa\x10\n\rMutationProps\x12\x1e\n\x1aMUTATION_PROPS_UNSPECIFIED\x10\x00\x12\x0f\n\x0bSTAR_ACTION\x10\x02\x12\x12\n\x0e\x43ONTACT_ACTION\x10\x03\x12\x0f\n\x0bMUTE_ACTION\x10\x04\x12\x0e\n\nPIN_ACTION\x10\x05\x12!\n\x1dSECURITY_NOTIFICATION_SETTING\x10\x06\x12\x15\n\x11PUSH_NAME_SETTING\x10\x07\x12\x16\n\x12QUICK_REPLY_ACTION\x10\x08\x12\x1f\n\x1bRECENT_EMOJI_WEIGHTS_ACTION\x10\x0b\x12\x18\n\x14LABEL_MESSAGE_ACTION\x10\r\x12\x15\n\x11LABEL_EDIT_ACTION\x10\x0e\x12\x1c\n\x18LABEL_ASSOCIATION_ACTION\x10\x0f\x12\x12\n\x0eLOCALE_SETTING\x10\x10\x12\x17\n\x13\x41RCHIVE_CHAT_ACTION\x10\x11\x12 \n\x1c\x44\x45LETE_MESSAGE_FOR_ME_ACTION\x10\x12\x12\x12\n\x0eKEY_EXPIRATION\x10\x13\x12\x1c\n\x18MARK_CHAT_AS_READ_ACTION\x10\x14\x12\x15\n\x11\x43LEAR_CHAT_ACTION\x10\x15\x12\x16\n\x12\x44\x45LETE_CHAT_ACTION\x10\x16\x12\x1b\n\x17UNARCHIVE_CHATS_SETTING\x10\x17\x12\x13\n\x0fPRIMARY_FEATURE\x10\x18\x12\x1f\n\x1b\x41NDROID_UNSUPPORTED_ACTIONS\x10\x1a\x12\x10\n\x0c\x41GENT_ACTION\x10\x1b\x12\x17\n\x13SUBSCRIPTION_ACTION\x10\x1c\x12\x1b\n\x17USER_STATUS_MUTE_ACTION\x10\x1d\x12\x16\n\x12TIME_FORMAT_ACTION\x10\x1e\x12\x0e\n\nNUX_ACTION\x10\x1f\x12\x1a\n\x16PRIMARY_VERSION_ACTION\x10 \x12\x12\n\x0eSTICKER_ACTION\x10!\x12 \n\x1cREMOVE_RECENT_STICKER_ACTION\x10\"\x12\x13\n\x0f\x43HAT_ASSIGNMENT\x10#\x12!\n\x1d\x43HAT_ASSIGNMENT_OPENED_STATUS\x10$\x12\x1a\n\x16PN_FOR_LID_CHAT_ACTION\x10%\x12\x1c\n\x18MARKETING_MESSAGE_ACTION\x10&\x12&\n\"MARKETING_MESSAGE_BROADCAST_ACTION\x10\'\x12\x1c\n\x18\x45XTERNAL_WEB_BETA_ACTION\x10(\x12#\n\x1fPRIVACY_SETTING_RELAY_ALL_CALLS\x10)\x12\x13\n\x0f\x43\x41LL_LOG_ACTION\x10*\x12\x0b\n\x07UGC_BOT\x10+\x12\x12\n\x0eSTATUS_PRIVACY\x10,\x12\x1e\n\x1a\x42OT_WELCOME_REQUEST_ACTION\x10-\x12\x1e\n\x1a\x44\x45LETE_INDIVIDUAL_CALL_LOG\x10.\x12\x1b\n\x17LABEL_REORDERING_ACTION\x10/\x12\x17\n\x13PAYMENT_INFO_ACTION\x10\x30\x12!\n\x1d\x43USTOM_PAYMENT_METHODS_ACTION\x10\x31\x12\x14\n\x10LOCK_CHAT_ACTION\x10\x32\x12\x16\n\x12\x43HAT_LOCK_SETTINGS\x10\x33\x12\x1f\n\x1bWAMO_USER_IDENTIFIER_ACTION\x10\x34\x12\x30\n,PRIVACY_SETTING_DISABLE_LINK_PREVIEWS_ACTION\x10\x35\x12\x17\n\x13\x44\x45VICE_CAPABILITIES\x10\x36\x12\x14\n\x10NOTE_EDIT_ACTION\x10\x37\x12\x14\n\x10\x46\x41VORITES_ACTION\x10\x38\x12#\n\x1fMERCHANT_PAYMENT_PARTNER_ACTION\x10\x39\x12$\n WAFFLE_ACCOUNT_LINK_STATE_ACTION\x10:\x12\x1c\n\x18USERNAME_CHAT_START_MODE\x10;\x12(\n$NOTIFICATION_ACTIVITY_SETTING_ACTION\x10<\x12\x16\n\x12LID_CONTACT_ACTION\x10=\x12)\n%CTWA_PER_CUSTOMER_DATA_SHARING_ACTION\x10>\x12\x16\n\x12PAYMENT_TOS_ACTION\x10?\x12?\n;PRIVACY_SETTING_CHANNELS_PERSONALISED_RECOMMENDATION_ACTION\x10@\x12)\n%BUSINESS_BROADCAST_ASSOCIATION_ACTION\x10\x41\x12#\n\x1f\x44\x45TECTED_OUTCOMES_STATUS_ACTION\x10\x42\x12$\n MAIBA_AI_FEATURES_CONTROL_ACTION\x10\x44\x12\"\n\x1e\x42USINESS_BROADCAST_LIST_ACTION\x10\x45\x12\x18\n\x14MUSIC_USER_ID_ACTION\x10\x46\x12\x36\n2STATUS_POST_OPT_IN_NOTIFICATION_PREFERENCES_ACTION\x10G\x12\x19\n\x15\x41VATAR_UPDATED_ACTION\x10H\x12\x16\n\x12GALAXY_FLOW_ACTION\x10I\x12%\n!PRIVATE_PROCESSING_SETTING_ACTION\x10J\x12%\n!NEWSLETTER_SAVED_INTERESTS_ACTION\x10K\x12\x1b\n\x17\x41I_THREAD_RENAME_ACTION\x10L\x12\x1e\n\x1aINTERACTIVE_MESSAGE_ACTION\x10M\x12\x11\n\x0cSHARE_OWN_PN\x10\x91N\x12\x1e\n\x19\x42USINESS_BROADCAST_ACTION\x10\x92N*m\n\x14PrivacySystemMessage\x12&\n\"PRIVACY_SYSTEM_MESSAGE_UNSPECIFIED\x10\x00\x12\x0c\n\x08\x45\x32\x45\x45_MSG\x10\x01\x12\x0e\n\nNE2EE_SELF\x10\x02\x12\x0f\n\x0bNE2EE_OTHER\x10\x03*H\n\x17SessionTransparencyType\x12\x10\n\x0cUNKNOWN_TYPE\x10\x00\x12\x1b\n\x17NY_AI_SAFETY_DISCLAIMER\x10\x01*.\n\x13WebLinkRenderConfig\x12\x0b\n\x07WEBVIEW\x10\x00\x12\n\n\x06SYSTEM\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'WAProto_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_ADVKEYINDEXLIST'].fields_by_name['validIndexes']._loaded_options = None
  _globals['_ADVKEYINDEXLIST'].fields_by_name['validIndexes']._serialized_options = b'\020\001'
  _globals['_CONFIG_FIELDENTRY']._loaded_options = None
  _globals['_CONFIG_FIELDENTRY']._serialized_options = b'8\001'
  _globals['_DEVICELISTMETADATA'].fields_by_name['senderKeyIndexes']._loaded_options = None
  _globals['_DEVICELISTMETADATA'].fields_by_name['senderKeyIndexes']._serialized_options = b'\020\001'
  _globals['_DEVICELISTMETADATA'].fields_by_name['recipientKeyIndexes']._loaded_options = None
  _globals['_DEVICELISTMETADATA'].fields_by_name['recipientKeyIndexes']._serialized_options = b'\020\001'
  _globals['_FIELD_SUBFIELDENTRY']._loaded_options = None
  _globals['_FIELD_SUBFIELDENTRY']._serialized_options = b'8\001'
  _globals['_MESSAGE_APPSTATESYNCKEYFINGERPRINT'].fields_by_name['deviceIndexes']._loaded_options = None
  _globals['_MESSAGE_APPSTATESYNCKEYFINGERPRINT'].fields_by_name['deviceIndexes']._serialized_options = b'\020\001'
  _globals['_SYNCACTIONVALUE_MUSICUSERIDACTION_MUSICUSERIDMAPENTRY']._loaded_options = None
  _globals['_SYNCACTIONVALUE_MUSICUSERIDACTION_MUSICUSERIDMAPENTRY']._serialized_options = b'8\001'
  _globals['_ADVENCRYPTIONTYPE']._serialized_start=179365
  _globals['_ADVENCRYPTIONTYPE']._serialized_end=179406
  _globals['_AIRICHRESPONSEMESSAGETYPE']._serialized_start=179408
  _globals['_AIRICHRESPONSEMESSAGETYPE']._serialized_end=179506
  _globals['_AIRICHRESPONSESUBMESSAGETYPE']._serialized_start=179509
  _globals['_AIRICHRESPONSESUBMESSAGETYPE']._serialized_end=179839
  _globals['_BOTMETRICSENTRYPOINT']._serialized_start=179842
  _globals['_BOTMETRICSENTRYPOINT']._serialized_end=180917
  _globals['_BOTMETRICSTHREADENTRYPOINT']._serialized_start=180920
  _globals['_BOTMETRICSTHREADENTRYPOINT']._serialized_end=181130
  _globals['_BOTSESSIONSOURCE']._serialized_start=181132
  _globals['_BOTSESSIONSOURCE']._serialized_end=181257
  _globals['_COLLECTIONNAME']._serialized_start=181260
  _globals['_COLLECTIONNAME']._serialized_end=181399
  _globals['_KEEPTYPE']._serialized_start=181401
  _globals['_KEEPTYPE']._serialized_end=181465
  _globals['_MEDIAVISIBILITY']._serialized_start=181467
  _globals['_MEDIAVISIBILITY']._serialized_end=181514
  _globals['_MUTATIONPROPS']._serialized_start=181517
  _globals['_MUTATIONPROPS']._serialized_end=183687
  _globals['_PRIVACYSYSTEMMESSAGE']._serialized_start=183689
  _globals['_PRIVACYSYSTEMMESSAGE']._serialized_end=183798
  _globals['_SESSIONTRANSPARENCYTYPE']._serialized_start=183800
  _globals['_SESSIONTRANSPARENCYTYPE']._serialized_end=183872
  _globals['_WEBLINKRENDERCONFIG']._serialized_start=183874
  _globals['_WEBLINKRENDERCONFIG']._serialized_end=183920
  _globals['_ADVDEVICEIDENTITY']._serialized_start=25
  _globals['_ADVDEVICEIDENTITY']._serialized_end=282
  _globals['_ADVKEYINDEXLIST']._serialized_start=285
  _globals['_ADVKEYINDEXLIST']._serialized_end=508
  _globals['_ADVSIGNEDDEVICEIDENTITY']._serialized_start=511
  _globals['_ADVSIGNEDDEVICEIDENTITY']._serialized_end=730
  _globals['_ADVSIGNEDDEVICEIDENTITYHMAC']._serialized_start=733
  _globals['_ADVSIGNEDDEVICEIDENTITYHMAC']._serialized_end=892
  _globals['_ADVSIGNEDKEYINDEXLIST']._serialized_start=895
  _globals['_ADVSIGNEDKEYINDEXLIST']._serialized_end=1062
  _globals['_AIHOMESTATE']._serialized_start=1065
  _globals['_AIHOMESTATE']._serialized_end=1700
  _globals['_AIHOMESTATE_AIHOMEOPTION']._serialized_start=1231
  _globals['_AIHOMESTATE_AIHOMEOPTION']._serialized_end=1682
  _globals['_AIHOMESTATE_AIHOMEOPTION_AIHOMEACTIONTYPE']._serialized_start=1482
  _globals['_AIHOMESTATE_AIHOMEOPTION_AIHOMEACTIONTYPE']._serialized_end=1567
  _globals['_AIQUERYFANOUT']._serialized_start=1703
  _globals['_AIQUERYFANOUT']._serialized_end=1865
  _globals['_AIREGENERATEMETADATA']._serialized_start=1868
  _globals['_AIREGENERATEMETADATA']._serialized_end=2007
  _globals['_AIRICHRESPONSECODEMETADATA']._serialized_start=2010
  _globals['_AIRICHRESPONSECODEMETADATA']._serialized_end=2647
  _globals['_AIRICHRESPONSECODEMETADATA_AIRICHRESPONSECODEBLOCK']._serialized_start=2147
  _globals['_AIRICHRESPONSECODEMETADATA_AIRICHRESPONSECODEBLOCK']._serialized_end=2327
  _globals['_AIRICHRESPONSECODEMETADATA_AIRICHRESPONSECODEHIGHLIGHTTYPE']._serialized_start=2330
  _globals['_AIRICHRESPONSECODEMETADATA_AIRICHRESPONSECODEHIGHLIGHTTYPE']._serialized_end=2630
  _globals['_AIRICHRESPONSECONTENTITEMSMETADATA']._serialized_start=2650
  _globals['_AIRICHRESPONSECONTENTITEMSMETADATA']._serialized_end=3263
  _globals['_AIRICHRESPONSECONTENTITEMSMETADATA_AIRICHRESPONSECONTENTITEMMETADATA']._serialized_start=2870
  _globals['_AIRICHRESPONSECONTENTITEMSMETADATA_AIRICHRESPONSECONTENTITEMMETADATA']._serialized_end=3020
  _globals['_AIRICHRESPONSECONTENTITEMSMETADATA_AIRICHRESPONSEREELITEM']._serialized_start=3023
  _globals['_AIRICHRESPONSECONTENTITEMSMETADATA_AIRICHRESPONSEREELITEM']._serialized_end=3205
  _globals['_AIRICHRESPONSECONTENTITEMSMETADATA_CONTENTTYPE']._serialized_start=3207
  _globals['_AIRICHRESPONSECONTENTITEMSMETADATA_CONTENTTYPE']._serialized_end=3247
  _globals['_AIRICHRESPONSEDYNAMICMETADATA']._serialized_start=3266
  _globals['_AIRICHRESPONSEDYNAMICMETADATA']._serialized_end=3683
  _globals['_AIRICHRESPONSEDYNAMICMETADATA_AIRICHRESPONSEDYNAMICMETADATATYPE']._serialized_start=3455
  _globals['_AIRICHRESPONSEDYNAMICMETADATA_AIRICHRESPONSEDYNAMICMETADATATYPE']._serialized_end=3640
  _globals['_AIRICHRESPONSEGRIDIMAGEMETADATA']._serialized_start=3686
  _globals['_AIRICHRESPONSEGRIDIMAGEMETADATA']._serialized_end=3844
  _globals['_AIRICHRESPONSEIMAGEURL']._serialized_start=3847
  _globals['_AIRICHRESPONSEIMAGEURL']._serialized_end=4009
  _globals['_AIRICHRESPONSEINLINEIMAGEMETADATA']._serialized_start=4012
  _globals['_AIRICHRESPONSEINLINEIMAGEMETADATA']._serialized_end=4487
  _globals['_AIRICHRESPONSEINLINEIMAGEMETADATA_AIRICHRESPONSEIMAGEALIGNMENT']._serialized_start=4248
  _globals['_AIRICHRESPONSEINLINEIMAGEMETADATA_AIRICHRESPONSEIMAGEALIGNMENT']._serialized_end=4431
  _globals['_AIRICHRESPONSELATEXMETADATA']._serialized_start=4490
  _globals['_AIRICHRESPONSELATEXMETADATA']._serialized_end=5070
  _globals['_AIRICHRESPONSELATEXMETADATA_AIRICHRESPONSELATEXEXPRESSION']._serialized_start=4628
  _globals['_AIRICHRESPONSELATEXMETADATA_AIRICHRESPONSELATEXEXPRESSION']._serialized_end=5061
  _globals['_AIRICHRESPONSEMAPMETADATA']._serialized_start=5073
  _globals['_AIRICHRESPONSEMAPMETADATA']._serialized_end=5635
  _globals['_AIRICHRESPONSEMAPMETADATA_AIRICHRESPONSEMAPANNOTATION']._serialized_start=5329
  _globals['_AIRICHRESPONSEMAPMETADATA_AIRICHRESPONSEMAPANNOTATION']._serialized_end=5542
  _globals['_AIRICHRESPONSEMESSAGE']._serialized_start=5638
  _globals['_AIRICHRESPONSEMESSAGE']._serialized_end=5941
  _globals['_AIRICHRESPONSESUBMESSAGE']._serialized_start=5944
  _globals['_AIRICHRESPONSESUBMESSAGE']._serialized_end=6783
  _globals['_AIRICHRESPONSETABLEMETADATA']._serialized_start=6786
  _globals['_AIRICHRESPONSETABLEMETADATA']._serialized_end=6997
  _globals['_AIRICHRESPONSETABLEMETADATA_AIRICHRESPONSETABLEROW']._serialized_start=6910
  _globals['_AIRICHRESPONSETABLEMETADATA_AIRICHRESPONSETABLEROW']._serialized_end=6987
  _globals['_AIRICHRESPONSEUNIFIEDRESPONSE']._serialized_start=6999
  _globals['_AIRICHRESPONSEUNIFIEDRESPONSE']._serialized_end=7058
  _globals['_AITHREADINFO']._serialized_start=7061
  _globals['_AITHREADINFO']._serialized_end=7448
  _globals['_AITHREADINFO_AITHREADCLIENTINFO']._serialized_start=7208
  _globals['_AITHREADINFO_AITHREADCLIENTINFO']._serialized_end=7366
  _globals['_AITHREADINFO_AITHREADCLIENTINFO_AITHREADTYPE']._serialized_start=7302
  _globals['_AITHREADINFO_AITHREADCLIENTINFO_AITHREADTYPE']._serialized_end=7357
  _globals['_AITHREADINFO_AITHREADSERVERINFO']._serialized_start=7368
  _globals['_AITHREADINFO_AITHREADSERVERINFO']._serialized_end=7418
  _globals['_ACCOUNT']._serialized_start=7451
  _globals['_ACCOUNT']._serialized_end=7618
  _globals['_ACTIONLINK']._serialized_start=7620
  _globals['_ACTIONLINK']._serialized_end=7700
  _globals['_AUTODOWNLOADSETTINGS']._serialized_start=7703
  _globals['_AUTODOWNLOADSETTINGS']._serialized_end=7919
  _globals['_AVATARUSERSETTINGS']._serialized_start=7921
  _globals['_AVATARUSERSETTINGS']._serialized_end=8005
  _globals['_BIZACCOUNTLINKINFO']._serialized_start=8008
  _globals['_BIZACCOUNTLINKINFO']._serialized_end=8426
  _globals['_BIZACCOUNTLINKINFO_ACCOUNTTYPE']._serialized_start=8255
  _globals['_BIZACCOUNTLINKINFO_ACCOUNTTYPE']._serialized_end=8284
  _globals['_BIZACCOUNTLINKINFO_HOSTSTORAGETYPE']._serialized_start=8286
  _globals['_BIZACCOUNTLINKINFO_HOSTSTORAGETYPE']._serialized_end=8333
  _globals['_BIZACCOUNTPAYLOAD']._serialized_start=8429
  _globals['_BIZACCOUNTPAYLOAD']._serialized_end=8568
  _globals['_BIZIDENTITYINFO']._serialized_start=8571
  _globals['_BIZIDENTITYINFO']._serialized_end=9204
  _globals['_BIZIDENTITYINFO_ACTUALACTORSTYPE']._serialized_start=8945
  _globals['_BIZIDENTITYINFO_ACTUALACTORSTYPE']._serialized_end=8982
  _globals['_BIZIDENTITYINFO_HOSTSTORAGETYPE']._serialized_start=8286
  _globals['_BIZIDENTITYINFO_HOSTSTORAGETYPE']._serialized_end=8333
  _globals['_BIZIDENTITYINFO_VERIFIEDLEVELVALUE']._serialized_start=9033
  _globals['_BIZIDENTITYINFO_VERIFIEDLEVELVALUE']._serialized_end=9085
  _globals['_BOTAGECOLLECTIONMETADATA']._serialized_start=9207
  _globals['_BOTAGECOLLECTIONMETADATA']._serialized_end=9537
  _globals['_BOTAGECOLLECTIONMETADATA_AGECOLLECTIONTYPE']._serialized_start=9403
  _globals['_BOTAGECOLLECTIONMETADATA_AGECOLLECTIONTYPE']._serialized_end=9450
  _globals['_BOTAVATARMETADATA']._serialized_start=9540
  _globals['_BOTAVATARMETADATA']._serialized_end=9751
  _globals['_BOTCAPABILITYMETADATA']._serialized_start=9754
  _globals['_BOTCAPABILITYMETADATA']._serialized_end=11470
  _globals['_BOTCAPABILITYMETADATA_BOTCAPABILITYTYPE']._serialized_start=9850
  _globals['_BOTCAPABILITYMETADATA_BOTCAPABILITYTYPE']._serialized_end=11470
  _globals['_BOTFEEDBACKMESSAGE']._serialized_start=11473
  _globals['_BOTFEEDBACKMESSAGE']._serialized_end=15737
  _globals['_BOTFEEDBACKMESSAGE_SIDEBYSIDESURVEYMETADATA']._serialized_start=11829
  _globals['_BOTFEEDBACKMESSAGE_SIDEBYSIDESURVEYMETADATA']._serialized_end=14339
  _globals['_BOTFEEDBACKMESSAGE_SIDEBYSIDESURVEYMETADATA_SIDEBYSIDESURVEYANALYTICSDATA']._serialized_start=12313
  _globals['_BOTFEEDBACKMESSAGE_SIDEBYSIDESURVEYMETADATA_SIDEBYSIDESURVEYANALYTICSDATA']._serialized_end=12488
  _globals['_BOTFEEDBACKMESSAGE_SIDEBYSIDESURVEYMETADATA_SIDEBYSIDESURVEYMETAAIANALYTICSDATA']._serialized_start=12491
  _globals['_BOTFEEDBACKMESSAGE_SIDEBYSIDESURVEYMETADATA_SIDEBYSIDESURVEYMETAAIANALYTICSDATA']._serialized_end=14144
  _globals['_BOTFEEDBACKMESSAGE_SIDEBYSIDESURVEYMETADATA_SIDEBYSIDESURVEYMETAAIANALYTICSDATA_SIDEBYSIDESURVEYABANDONEVENTDATA']._serialized_start=13418
  _globals['_BOTFEEDBACKMESSAGE_SIDEBYSIDESURVEYMETADATA_SIDEBYSIDESURVEYMETAAIANALYTICSDATA_SIDEBYSIDESURVEYABANDONEVENTDATA']._serialized_end=13520
  _globals['_BOTFEEDBACKMESSAGE_SIDEBYSIDESURVEYMETADATA_SIDEBYSIDESURVEYMETAAIANALYTICSDATA_SIDEBYSIDESURVEYCTACLICKEVENTDATA']._serialized_start=13523
  _globals['_BOTFEEDBACKMESSAGE_SIDEBYSIDESURVEYMETADATA_SIDEBYSIDESURVEYMETAAIANALYTICSDATA_SIDEBYSIDESURVEYCTACLICKEVENTDATA']._serialized_end=13672
  _globals['_BOTFEEDBACKMESSAGE_SIDEBYSIDESURVEYMETADATA_SIDEBYSIDESURVEYMETAAIANALYTICSDATA_SIDEBYSIDESURVEYCTAIMPRESSIONEVENTDATA']._serialized_start=13674
  _globals['_BOTFEEDBACKMESSAGE_SIDEBYSIDESURVEYMETADATA_SIDEBYSIDESURVEYMETAAIANALYTICSDATA_SIDEBYSIDESURVEYCTAIMPRESSIONEVENTDATA']._serialized_end=13764
  _globals['_BOTFEEDBACKMESSAGE_SIDEBYSIDESURVEYMETADATA_SIDEBYSIDESURVEYMETAAIANALYTICSDATA_SIDEBYSIDESURVEYCARDIMPRESSIONEVENTDATA']._serialized_start=13766
  _globals['_BOTFEEDBACKMESSAGE_SIDEBYSIDESURVEYMETADATA_SIDEBYSIDESURVEYMETAAIANALYTICSDATA_SIDEBYSIDESURVEYCARDIMPRESSIONEVENTDATA']._serialized_end=13807
  _globals['_BOTFEEDBACKMESSAGE_SIDEBYSIDESURVEYMETADATA_SIDEBYSIDESURVEYMETAAIANALYTICSDATA_SIDEBYSIDESURVEYRESPONSEEVENTDATA']._serialized_start=13810
  _globals['_BOTFEEDBACKMESSAGE_SIDEBYSIDESURVEYMETADATA_SIDEBYSIDESURVEYMETAAIANALYTICSDATA_SIDEBYSIDESURVEYRESPONSEEVENTDATA']._serialized_end=13971
  _globals['_BOTFEEDBACKMESSAGE_BOTFEEDBACKKIND']._serialized_start=14342
  _globals['_BOTFEEDBACKMESSAGE_BOTFEEDBACKKIND']._serialized_end=14941
  _globals['_BOTFEEDBACKMESSAGE_BOTFEEDBACKKINDMULTIPLENEGATIVE']._serialized_start=14944
  _globals['_BOTFEEDBACKMESSAGE_BOTFEEDBACKKINDMULTIPLENEGATIVE']._serialized_end=15456
  _globals['_BOTFEEDBACKMESSAGE_BOTFEEDBACKKINDMULTIPLEPOSITIVE']._serialized_start=15459
  _globals['_BOTFEEDBACKMESSAGE_BOTFEEDBACKKINDMULTIPLEPOSITIVE']._serialized_end=15589
  _globals['_BOTFEEDBACKMESSAGE_REPORTKIND']._serialized_start=15591
  _globals['_BOTFEEDBACKMESSAGE_REPORTKIND']._serialized_end=15626
  _globals['_BOTIMAGINEMETADATA']._serialized_start=15740
  _globals['_BOTIMAGINEMETADATA']._serialized_end=15913
  _globals['_BOTIMAGINEMETADATA_IMAGINETYPE']._serialized_start=15827
  _globals['_BOTIMAGINEMETADATA_IMAGINETYPE']._serialized_end=15897
  _globals['_BOTLINKEDACCOUNT']._serialized_start=15916
  _globals['_BOTLINKEDACCOUNT']._serialized_end=16064
  _globals['_BOTLINKEDACCOUNT_BOTLINKEDACCOUNTTYPE']._serialized_start=16001
  _globals['_BOTLINKEDACCOUNT_BOTLINKEDACCOUNTTYPE']._serialized_end=16055
  _globals['_BOTLINKEDACCOUNTSMETADATA']._serialized_start=16067
  _globals['_BOTLINKEDACCOUNTSMETADATA']._serialized_end=16223
  _globals['_BOTMEDIAMETADATA']._serialized_start=16226
  _globals['_BOTMEDIAMETADATA']._serialized_end=16673
  _globals['_BOTMEDIAMETADATA_ORIENTATIONTYPE']._serialized_start=16473
  _globals['_BOTMEDIAMETADATA_ORIENTATIONTYPE']._serialized_end=16557
  _globals['_BOTMEMORYFACT']._serialized_start=16675
  _globals['_BOTMEMORYFACT']._serialized_end=16750
  _globals['_BOTMEMORYMETADATA']._serialized_start=16753
  _globals['_BOTMEMORYMETADATA']._serialized_end=16898
  _globals['_BOTMEMUMETADATA']._serialized_start=16900
  _globals['_BOTMEMUMETADATA']._serialized_end=16962
  _globals['_BOTMESSAGEORIGIN']._serialized_start=16965
  _globals['_BOTMESSAGEORIGIN']._serialized_end=17123
  _globals['_BOTMESSAGEORIGIN_BOTMESSAGEORIGINTYPE']._serialized_start=17050
  _globals['_BOTMESSAGEORIGIN_BOTMESSAGEORIGINTYPE']._serialized_end=17114
  _globals['_BOTMESSAGEORIGINMETADATA']._serialized_start=17125
  _globals['_BOTMESSAGEORIGINMETADATA']._serialized_end=17193
  _globals['_BOTMESSAGESHARINGINFO']._serialized_start=17196
  _globals['_BOTMESSAGESHARINGINFO']._serialized_end=17350
  _globals['_BOTMETADATA']._serialized_start=17353
  _globals['_BOTMETADATA']._serialized_end=20086
  _globals['_BOTMETRICSMETADATA']._serialized_start=20089
  _globals['_BOTMETRICSMETADATA']._serialized_end=20325
  _globals['_BOTMODESELECTIONMETADATA']._serialized_start=20328
  _globals['_BOTMODESELECTIONMETADATA']._serialized_end=20484
  _globals['_BOTMODESELECTIONMETADATA_BOTUSERSELECTIONMODE']._serialized_start=20424
  _globals['_BOTMODESELECTIONMETADATA_BOTUSERSELECTIONMODE']._serialized_end=20484
  _globals['_BOTMODELMETADATA']._serialized_start=20487
  _globals['_BOTMODELMETADATA']._serialized_end=20884
  _globals['_BOTMODELMETADATA_MODELTYPE']._serialized_start=20675
  _globals['_BOTMODELMETADATA_MODELTYPE']._serialized_end=20744
  _globals['_BOTMODELMETADATA_PREMIUMMODELSTATUS']._serialized_start=20746
  _globals['_BOTMODELMETADATA_PREMIUMMODELSTATUS']._serialized_end=20825
  _globals['_BOTPLUGINMETADATA']._serialized_start=20887
  _globals['_BOTPLUGINMETADATA']._serialized_end=21794
  _globals['_BOTPLUGINMETADATA_PLUGINTYPE']._serialized_start=21436
  _globals['_BOTPLUGINMETADATA_PLUGINTYPE']._serialized_end=21491
  _globals['_BOTPLUGINMETADATA_SEARCHPROVIDER']._serialized_start=21493
  _globals['_BOTPLUGINMETADATA_SEARCHPROVIDER']._serialized_end=21557
  _globals['_BOTPROGRESSINDICATORMETADATA']._serialized_start=21797
  _globals['_BOTPROGRESSINDICATORMETADATA']._serialized_end=23538
  _globals['_BOTPROGRESSINDICATORMETADATA_BOTPLANNINGSTEPMETADATA']._serialized_start=21948
  _globals['_BOTPROGRESSINDICATORMETADATA_BOTPLANNINGSTEPMETADATA']._serialized_end=23514
  _globals['_BOTPROGRESSINDICATORMETADATA_BOTPLANNINGSTEPMETADATA_BOTPLANNINGSEARCHSOURCEMETADATA']._serialized_start=22414
  _globals['_BOTPROGRESSINDICATORMETADATA_BOTPLANNINGSTEPMETADATA_BOTPLANNINGSEARCHSOURCEMETADATA']._serialized_end=22676
  _globals['_BOTPROGRESSINDICATORMETADATA_BOTPLANNINGSTEPMETADATA_BOTPLANNINGSEARCHSOURCESMETADATA']._serialized_start=22679
  _globals['_BOTPROGRESSINDICATORMETADATA_BOTPLANNINGSTEPMETADATA_BOTPLANNINGSEARCHSOURCESMETADATA']._serialized_end=23037
  _globals['_BOTPROGRESSINDICATORMETADATA_BOTPLANNINGSTEPMETADATA_BOTPLANNINGSEARCHSOURCESMETADATA_BOTPLANNINGSEARCHSOURCEPROVIDER']._serialized_start=22915
  _globals['_BOTPROGRESSINDICATORMETADATA_BOTPLANNINGSTEPMETADATA_BOTPLANNINGSEARCHSOURCESMETADATA_BOTPLANNINGSEARCHSOURCEPROVIDER']._serialized_end=22994
  _globals['_BOTPROGRESSINDICATORMETADATA_BOTPLANNINGSTEPMETADATA_BOTPLANNINGSTEPSECTIONMETADATA']._serialized_start=23040
  _globals['_BOTPROGRESSINDICATORMETADATA_BOTPLANNINGSTEPMETADATA_BOTPLANNINGSTEPSECTIONMETADATA']._serialized_end=23276
  _globals['_BOTPROGRESSINDICATORMETADATA_BOTPLANNINGSTEPMETADATA_BOTSEARCHSOURCEPROVIDER']._serialized_start=23278
  _globals['_BOTPROGRESSINDICATORMETADATA_BOTPLANNINGSTEPMETADATA_BOTSEARCHSOURCEPROVIDER']._serialized_end=23358
  _globals['_BOTPROGRESSINDICATORMETADATA_BOTPLANNINGSTEPMETADATA_PLANNINGSTEPSTATUS']._serialized_start=23360
  _globals['_BOTPROGRESSINDICATORMETADATA_BOTPLANNINGSTEPMETADATA_PLANNINGSTEPSTATUS']._serialized_end=23435
  _globals['_BOTPROMOTIONMESSAGEMETADATA']._serialized_start=23541
  _globals['_BOTPROMOTIONMESSAGEMETADATA']._serialized_end=23779
  _globals['_BOTPROMOTIONMESSAGEMETADATA_BOTPROMOTIONTYPE']._serialized_start=23679
  _globals['_BOTPROMOTIONMESSAGEMETADATA_BOTPROMOTIONTYPE']._serialized_end=23745
  _globals['_BOTPROMPTSUGGESTION']._serialized_start=23781
  _globals['_BOTPROMPTSUGGESTION']._serialized_end=23870
  _globals['_BOTPROMPTSUGGESTIONS']._serialized_start=23872
  _globals['_BOTPROMPTSUGGESTIONS']._serialized_end=23943
  _globals['_BOTQUOTAMETADATA']._serialized_start=23946
  _globals['_BOTQUOTAMETADATA']._serialized_end=24348
  _globals['_BOTQUOTAMETADATA_BOTFEATUREQUOTAMETADATA']._serialized_start=24049
  _globals['_BOTQUOTAMETADATA_BOTFEATUREQUOTAMETADATA']._serialized_end=24348
  _globals['_BOTQUOTAMETADATA_BOTFEATUREQUOTAMETADATA_BOTFEATURETYPE']._serialized_start=24229
  _globals['_BOTQUOTAMETADATA_BOTFEATUREQUOTAMETADATA_BOTFEATURETYPE']._serialized_end=24289
  _globals['_BOTREMINDERMETADATA']._serialized_start=24351
  _globals['_BOTREMINDERMETADATA']._serialized_end=24908
  _globals['_BOTREMINDERMETADATA_REMINDERACTION']._serialized_start=24613
  _globals['_BOTREMINDERMETADATA_REMINDERACTION']._serialized_end=24710
  _globals['_BOTREMINDERMETADATA_REMINDERFREQUENCY']._serialized_start=24712
  _globals['_BOTREMINDERMETADATA_REMINDERFREQUENCY']._serialized_end=24827
  _globals['_BOTRENDERINGMETADATA']._serialized_start=24911
  _globals['_BOTRENDERINGMETADATA']._serialized_end=25056
  _globals['_BOTRENDERINGMETADATA_KEYWORD']._serialized_start=24990
  _globals['_BOTRENDERINGMETADATA_KEYWORD']._serialized_end=25056
  _globals['_BOTSESSIONMETADATA']._serialized_start=25059
  _globals['_BOTSESSIONMETADATA']._serialized_end=25188
  _globals['_BOTSIGNATUREVERIFICATIONMETADATA']._serialized_start=25190
  _globals['_BOTSIGNATUREVERIFICATIONMETADATA']._serialized_end=25285
  _globals['_BOTSIGNATUREVERIFICATIONUSECASEPROOF']._serialized_start=25288
  _globals['_BOTSIGNATUREVERIFICATIONUSECASEPROOF']._serialized_end=25579
  _globals['_BOTSIGNATUREVERIFICATIONUSECASEPROOF_BOTSIGNATUREUSECASE']._serialized_start=25487
  _globals['_BOTSIGNATUREVERIFICATIONUSECASEPROOF_BOTSIGNATUREUSECASE']._serialized_end=25541
  _globals['_BOTSOURCESMETADATA']._serialized_start=25582
  _globals['_BOTSOURCESMETADATA']._serialized_end=26129
  _globals['_BOTSOURCESMETADATA_BOTSOURCEITEM']._serialized_start=25663
  _globals['_BOTSOURCESMETADATA_BOTSOURCEITEM']._serialized_end=26129
  _globals['_BOTSOURCESMETADATA_BOTSOURCEITEM_SOURCEPROVIDER']._serialized_start=25930
  _globals['_BOTSOURCESMETADATA_BOTSOURCEITEM_SOURCEPROVIDER']._serialized_end=26005
  _globals['_BOTSUGGESTEDPROMPTMETADATA']._serialized_start=26132
  _globals['_BOTSUGGESTEDPROMPTMETADATA']._serialized_end=26379
  _globals['_BOTUNIFIEDRESPONSEMUTATION']._serialized_start=26382
  _globals['_BOTUNIFIEDRESPONSEMUTATION']._serialized_end=26920
  _globals['_BOTUNIFIEDRESPONSEMUTATION_MEDIADETAILSMETADATA']._serialized_start=26583
  _globals['_BOTUNIFIEDRESPONSEMUTATION_MEDIADETAILSMETADATA']._serialized_end=26767
  _globals['_BOTUNIFIEDRESPONSEMUTATION_SIDEBYSIDEMETADATA']._serialized_start=26770
  _globals['_BOTUNIFIEDRESPONSEMUTATION_SIDEBYSIDEMETADATA']._serialized_end=26904
  _globals['_CALLLOGRECORD']._serialized_start=26923
  _globals['_CALLLOGRECORD']._serialized_end=28095
  _globals['_CALLLOGRECORD_PARTICIPANTINFO']._serialized_start=27450
  _globals['_CALLLOGRECORD_PARTICIPANTINFO']._serialized_end=27574
  _globals['_CALLLOGRECORD_CALLRESULT']._serialized_start=27577
  _globals['_CALLLOGRECORD_CALLRESULT']._serialized_end=27752
  _globals['_CALLLOGRECORD_CALLTYPE']._serialized_start=27754
  _globals['_CALLLOGRECORD_CALLTYPE']._serialized_end=27813
  _globals['_CALLLOGRECORD_SILENCEREASON']._serialized_start=27815
  _globals['_CALLLOGRECORD_SILENCEREASON']._serialized_end=27885
  _globals['_CERTCHAIN']._serialized_start=28098
  _globals['_CERTCHAIN']._serialized_end=28532
  _globals['_CERTCHAIN_NOISECERTIFICATE']._serialized_start=28228
  _globals['_CERTCHAIN_NOISECERTIFICATE']._serialized_end=28506
  _globals['_CERTCHAIN_NOISECERTIFICATE_DETAILS']._serialized_start=28295
  _globals['_CERTCHAIN_NOISECERTIFICATE_DETAILS']._serialized_end=28480
  _globals['_CHATLOCKSETTINGS']._serialized_start=28535
  _globals['_CHATLOCKSETTINGS']._serialized_end=28664
  _globals['_CHATROWOPAQUEDATA']._serialized_start=28667
  _globals['_CHATROWOPAQUEDATA']._serialized_end=29956
  _globals['_CHATROWOPAQUEDATA_DRAFTMESSAGE']._serialized_start=28755
  _globals['_CHATROWOPAQUEDATA_DRAFTMESSAGE']._serialized_end=29939
  _globals['_CHATROWOPAQUEDATA_DRAFTMESSAGE_CTWACONTEXTDATA']._serialized_start=29014
  _globals['_CHATROWOPAQUEDATA_DRAFTMESSAGE_CTWACONTEXTDATA']._serialized_end=29695
  _globals['_CHATROWOPAQUEDATA_DRAFTMESSAGE_CTWACONTEXTDATA_CONTEXTINFOEXTERNALADREPLYINFOMEDIATYPE']._serialized_start=29435
  _globals['_CHATROWOPAQUEDATA_DRAFTMESSAGE_CTWACONTEXTDATA_CONTEXTINFOEXTERNALADREPLYINFOMEDIATYPE']._serialized_end=29508
  _globals['_CHATROWOPAQUEDATA_DRAFTMESSAGE_CTWACONTEXTLINKDATA']._serialized_start=29698
  _globals['_CHATROWOPAQUEDATA_DRAFTMESSAGE_CTWACONTEXTLINKDATA']._serialized_end=29861
  _globals['_CITATION']._serialized_start=29958
  _globals['_CITATION']._serialized_end=30034
  _globals['_CLIENTPAIRINGPROPS']._serialized_start=30037
  _globals['_CLIENTPAIRINGPROPS']._serialized_end=30257
  _globals['_CLIENTPAYLOAD']._serialized_start=30260
  _globals['_CLIENTPAYLOAD']._serialized_end=36171
  _globals['_CLIENTPAYLOAD_DNSSOURCE']._serialized_start=31526
  _globals['_CLIENTPAYLOAD_DNSSOURCE']._serialized_end=31764
  _globals['_CLIENTPAYLOAD_DNSSOURCE_DNSRESOLUTIONMETHOD']._serialized_start=31639
  _globals['_CLIENTPAYLOAD_DNSSOURCE_DNSRESOLUTIONMETHOD']._serialized_end=31736
  _globals['_CLIENTPAYLOAD_DEVICEPAIRINGREGISTRATIONDATA']._serialized_start=31767
  _globals['_CLIENTPAYLOAD_DEVICEPAIRINGREGISTRATIONDATA']._serialized_end=32084
  _globals['_CLIENTPAYLOAD_INTEROPDATA']._serialized_start=32087
  _globals['_CLIENTPAYLOAD_INTEROPDATA']._serialized_end=32224
  _globals['_CLIENTPAYLOAD_USERAGENT']._serialized_start=32227
  _globals['_CLIENTPAYLOAD_USERAGENT']._serialized_end=33976
  _globals['_CLIENTPAYLOAD_USERAGENT_APPVERSION']._serialized_start=32835
  _globals['_CLIENTPAYLOAD_USERAGENT_APPVERSION']._serialized_end=33029
  _globals['_CLIENTPAYLOAD_USERAGENT_DEVICETYPE']._serialized_start=33031
  _globals['_CLIENTPAYLOAD_USERAGENT_DEVICETYPE']._serialized_end=33101
  _globals['_CLIENTPAYLOAD_USERAGENT_PLATFORM']._serialized_start=33104
  _globals['_CLIENTPAYLOAD_USERAGENT_PLATFORM']._serialized_end=33653
  _globals['_CLIENTPAYLOAD_USERAGENT_RELEASECHANNEL']._serialized_start=33655
  _globals['_CLIENTPAYLOAD_USERAGENT_RELEASECHANNEL']._serialized_end=33716
  _globals['_CLIENTPAYLOAD_WEBINFO']._serialized_start=33979
  _globals['_CLIENTPAYLOAD_WEBINFO']._serialized_end=34959
  _globals['_CLIENTPAYLOAD_WEBINFO_WEBDPAYLOAD']._serialized_start=34178
  _globals['_CLIENTPAYLOAD_WEBINFO_WEBDPAYLOAD']._serialized_end=34795
  _globals['_CLIENTPAYLOAD_WEBINFO_WEBSUBPLATFORM']._serialized_start=34797
  _globals['_CLIENTPAYLOAD_WEBINFO_WEBSUBPLATFORM']._serialized_end=34899
  _globals['_CLIENTPAYLOAD_ACCOUNTTYPE']._serialized_start=34961
  _globals['_CLIENTPAYLOAD_ACCOUNTTYPE']._serialized_end=34998
  _globals['_CLIENTPAYLOAD_CONNECTREASON']._serialized_start=35001
  _globals['_CLIENTPAYLOAD_CONNECTREASON']._serialized_end=35135
  _globals['_CLIENTPAYLOAD_CONNECTTYPE']._serialized_start=35138
  _globals['_CLIENTPAYLOAD_CONNECTTYPE']._serialized_end=35442
  _globals['_CLIENTPAYLOAD_IOSAPPEXTENSION']._serialized_start=35444
  _globals['_CLIENTPAYLOAD_IOSAPPEXTENSION']._serialized_end=35528
  _globals['_CLIENTPAYLOAD_PRODUCT']._serialized_start=35530
  _globals['_CLIENTPAYLOAD_PRODUCT']._serialized_end=35617
  _globals['_CLIENTPAYLOAD_TRAFFICANONYMIZATION']._serialized_start=35619
  _globals['_CLIENTPAYLOAD_TRAFFICANONYMIZATION']._serialized_end=35664
  _globals['_COMMENTMETADATA']._serialized_start=36174
  _globals['_COMMENTMETADATA']._serialized_end=36302
  _globals['_COMPANIONCOMMITMENT']._serialized_start=36304
  _globals['_COMPANIONCOMMITMENT']._serialized_end=36353
  _globals['_COMPANIONEPHEMERALIDENTITY']._serialized_start=36356
  _globals['_COMPANIONEPHEMERALIDENTITY']._serialized_end=36521
  _globals['_CONFIG']._serialized_start=36524
  _globals['_CONFIG']._serialized_end=36667
  _globals['_CONFIG_FIELDENTRY']._serialized_start=36597
  _globals['_CONFIG_FIELDENTRY']._serialized_end=36655
  _globals['_CONTEXTINFO']._serialized_start=36670
  _globals['_CONTEXTINFO']._serialized_end=44545
  _globals['_CONTEXTINFO_ADREPLYINFO']._serialized_start=39199
  _globals['_CONTEXTINFO_ADREPLYINFO']._serialized_end=39465
  _globals['_CONTEXTINFO_ADREPLYINFO_MEDIATYPE']._serialized_start=39359
  _globals['_CONTEXTINFO_ADREPLYINFO_MEDIATYPE']._serialized_end=39402
  _globals['_CONTEXTINFO_BUSINESSMESSAGEFORWARDINFO']._serialized_start=39467
  _globals['_CONTEXTINFO_BUSINESSMESSAGEFORWARDINFO']._serialized_end=39547
  _globals['_CONTEXTINFO_DATASHARINGCONTEXT']._serialized_start=39550
  _globals['_CONTEXTINFO_DATASHARINGCONTEXT']._serialized_end=40182
  _globals['_CONTEXTINFO_DATASHARINGCONTEXT_PARAMETERS']._serialized_start=39749
  _globals['_CONTEXTINFO_DATASHARINGCONTEXT_PARAMETERS']._serialized_end=39985
  _globals['_CONTEXTINFO_DATASHARINGCONTEXT_DATASHARINGFLAGS']._serialized_start=39987
  _globals['_CONTEXTINFO_DATASHARINGCONTEXT_DATASHARINGFLAGS']._serialized_end=40106
  _globals['_CONTEXTINFO_EXTERNALADREPLYINFO']._serialized_start=40185
  _globals['_CONTEXTINFO_EXTERNALADREPLYINFO']._serialized_end=41626
  _globals['_CONTEXTINFO_EXTERNALADREPLYINFO_ADTYPE']._serialized_start=41064
  _globals['_CONTEXTINFO_EXTERNALADREPLYINFO_ADTYPE']._serialized_end=41092
  _globals['_CONTEXTINFO_EXTERNALADREPLYINFO_MEDIATYPE']._serialized_start=39359
  _globals['_CONTEXTINFO_EXTERNALADREPLYINFO_MEDIATYPE']._serialized_end=39402
  _globals['_CONTEXTINFO_FEATUREELIGIBILITIES']._serialized_start=41629
  _globals['_CONTEXTINFO_FEATUREELIGIBILITIES']._serialized_end=41915
  _globals['_CONTEXTINFO_FORWARDEDNEWSLETTERMESSAGEINFO']._serialized_start=41918
  _globals['_CONTEXTINFO_FORWARDEDNEWSLETTERMESSAGEINFO']._serialized_end=42342
  _globals['_CONTEXTINFO_FORWARDEDNEWSLETTERMESSAGEINFO_CONTENTTYPE']._serialized_start=42160
  _globals['_CONTEXTINFO_FORWARDEDNEWSLETTERMESSAGEINFO_CONTENTTYPE']._serialized_end=42247
  _globals['_CONTEXTINFO_QUESTIONREPLYQUOTEDMESSAGE']._serialized_start=42345
  _globals['_CONTEXTINFO_QUESTIONREPLYQUOTEDMESSAGE']._serialized_end=42553
  _globals['_CONTEXTINFO_STATUSAUDIENCEMETADATA']._serialized_start=42556
  _globals['_CONTEXTINFO_STATUSAUDIENCEMETADATA']._serialized_end=42728
  _globals['_CONTEXTINFO_STATUSAUDIENCEMETADATA_AUDIENCETYPE']._serialized_start=42665
  _globals['_CONTEXTINFO_STATUSAUDIENCEMETADATA_AUDIENCETYPE']._serialized_end=42711
  _globals['_CONTEXTINFO_UTMINFO']._serialized_start=42730
  _globals['_CONTEXTINFO_UTMINFO']._serialized_end=42819
  _globals['_CONTEXTINFO_FORWARDORIGIN']._serialized_start=42821
  _globals['_CONTEXTINFO_FORWARDORIGIN']._serialized_end=42907
  _globals['_CONTEXTINFO_PAIREDMEDIATYPE']._serialized_start=42910
  _globals['_CONTEXTINFO_PAIREDMEDIATYPE']._serialized_end=43125
  _globals['_CONTEXTINFO_QUOTEDTYPE']._serialized_start=43127
  _globals['_CONTEXTINFO_QUOTEDTYPE']._serialized_end=43163
  _globals['_CONTEXTINFO_STATUSATTRIBUTIONTYPE']._serialized_start=43166
  _globals['_CONTEXTINFO_STATUSATTRIBUTIONTYPE']._serialized_end=43312
  _globals['_CONTEXTINFO_STATUSSOURCETYPE']._serialized_start=43314
  _globals['_CONTEXTINFO_STATUSSOURCETYPE']._serialized_end=43406
  _globals['_CONVERSATION']._serialized_start=44548
  _globals['_CONVERSATION']._serialized_end=47417
  _globals['_CONVERSATION_ENDOFHISTORYTRANSFERTYPE']._serialized_start=46293
  _globals['_CONVERSATION_ENDOFHISTORYTRANSFERTYPE']._serialized_end=46481
  _globals['_DEVICECAPABILITIES']._serialized_start=47420
  _globals['_DEVICECAPABILITIES']._serialized_end=48313
  _globals['_DEVICECAPABILITIES_BUSINESSBROADCAST']._serialized_start=47835
  _globals['_DEVICECAPABILITIES_BUSINESSBROADCAST']._serialized_end=47908
  _globals['_DEVICECAPABILITIES_LIDMIGRATION']._serialized_start=47910
  _globals['_DEVICECAPABILITIES_LIDMIGRATION']._serialized_end=47992
  _globals['_DEVICECAPABILITIES_USERHASAVATAR']._serialized_start=47994
  _globals['_DEVICECAPABILITIES_USERHASAVATAR']._serialized_end=48055
  _globals['_DEVICECAPABILITIES_CHATLOCKSUPPORTLEVEL']._serialized_start=48057
  _globals['_DEVICECAPABILITIES_CHATLOCKSUPPORTLEVEL']._serialized_end=48112
  _globals['_DEVICECAPABILITIES_MEMBERNAMETAGPRIMARYSUPPORT']._serialized_start=48114
  _globals['_DEVICECAPABILITIES_MEMBERNAMETAGPRIMARYSUPPORT']._serialized_end=48199
  _globals['_DEVICECONSISTENCYCODEMESSAGE']._serialized_start=48315
  _globals['_DEVICECONSISTENCYCODEMESSAGE']._serialized_end=48423
  _globals['_DEVICELISTMETADATA']._serialized_start=48426
  _globals['_DEVICELISTMETADATA']._serialized_end=48877
  _globals['_DEVICEPROPS']._serialized_start=48880
  _globals['_DEVICEPROPS']._serialized_end=51012
  _globals['_DEVICEPROPS_APPVERSION']._serialized_start=32835
  _globals['_DEVICEPROPS_APPVERSION']._serialized_end=33029
  _globals['_DEVICEPROPS_HISTORYSYNCCONFIG']._serialized_start=49323
  _globals['_DEVICEPROPS_HISTORYSYNCCONFIG']._serialized_end=50580
  _globals['_DEVICEPROPS_PLATFORMTYPE']._serialized_start=50583
  _globals['_DEVICEPROPS_PLATFORMTYPE']._serialized_end=50934
  _globals['_DISAPPEARINGMODE']._serialized_start=51015
  _globals['_DISAPPEARINGMODE']._serialized_end=51511
  _globals['_DISAPPEARINGMODE_INITIATOR']._serialized_start=51210
  _globals['_DISAPPEARINGMODE_INITIATOR']._serialized_end=51315
  _globals['_DISAPPEARINGMODE_TRIGGER']._serialized_start=51317
  _globals['_DISAPPEARINGMODE_TRIGGER']._serialized_end=51444
  _globals['_EMBEDDEDCONTENT']._serialized_start=51513
  _globals['_EMBEDDEDCONTENT']._serialized_end=51639
  _globals['_EMBEDDEDMESSAGE']._serialized_start=51641
  _globals['_EMBEDDEDMESSAGE']._serialized_end=51744
  _globals['_EMBEDDEDMUSIC']._serialized_start=51747
  _globals['_EMBEDDEDMUSIC']._serialized_end=52458
  _globals['_ENCRYPTEDPAIRINGREQUEST']._serialized_start=52460
  _globals['_ENCRYPTEDPAIRINGREQUEST']._serialized_end=52561
  _globals['_EPHEMERALSETTING']._serialized_start=52563
  _globals['_EPHEMERALSETTING']._serialized_end=52655
  _globals['_EVENTADDITIONALMETADATA']._serialized_start=52657
  _globals['_EVENTADDITIONALMETADATA']._serialized_end=52716
  _globals['_EVENTRESPONSE']._serialized_start=52719
  _globals['_EVENTRESPONSE']._serialized_end=52990
  _globals['_EXITCODE']._serialized_start=52992
  _globals['_EXITCODE']._serialized_end=53058
  _globals['_EXTERNALBLOBREFERENCE']._serialized_start=53061
  _globals['_EXTERNALBLOBREFERENCE']._serialized_end=53324
  _globals['_FIELD']._serialized_start=53327
  _globals['_FIELD']._serialized_end=53627
  _globals['_FIELD_SUBFIELDENTRY']._serialized_start=53494
  _globals['_FIELD_SUBFIELDENTRY']._serialized_end=53555
  _globals['_FORWARDEDAIBOTMESSAGEINFO']._serialized_start=53630
  _globals['_FORWARDEDAIBOTMESSAGEINFO']._serialized_end=53765
  _globals['_GLOBALSETTINGS']._serialized_start=53768
  _globals['_GLOBALSETTINGS']._serialized_end=55285
  _globals['_GROUPHISTORYBUNDLEINFO']._serialized_start=55288
  _globals['_GROUPHISTORYBUNDLEINFO']._serialized_end=55640
  _globals['_GROUPHISTORYBUNDLEINFO_PROCESSSTATE']._serialized_start=55467
  _globals['_GROUPHISTORYBUNDLEINFO_PROCESSSTATE']._serialized_end=55588
  _globals['_GROUPHISTORYINDIVIDUALMESSAGEINFO']._serialized_start=55643
  _globals['_GROUPHISTORYINDIVIDUALMESSAGEINFO']._serialized_end=55825
  _globals['_GROUPMENTION']._serialized_start=55827
  _globals['_GROUPMENTION']._serialized_end=55921
  _globals['_GROUPPARTICIPANT']._serialized_start=55924
  _globals['_GROUPPARTICIPANT']._serialized_end=56127
  _globals['_GROUPPARTICIPANT_RANK']._serialized_start=56056
  _globals['_GROUPPARTICIPANT_RANK']._serialized_end=56102
  _globals['_HANDSHAKEMESSAGE']._serialized_start=56130
  _globals['_HANDSHAKEMESSAGE']._serialized_end=56913
  _globals['_HANDSHAKEMESSAGE_CLIENTFINISH']._serialized_start=56342
  _globals['_HANDSHAKEMESSAGE_CLIENTFINISH']._serialized_end=56478
  _globals['_HANDSHAKEMESSAGE_CLIENTHELLO']._serialized_start=56481
  _globals['_HANDSHAKEMESSAGE_CLIENTHELLO']._serialized_end=56696
  _globals['_HANDSHAKEMESSAGE_SERVERHELLO']._serialized_start=56699
  _globals['_HANDSHAKEMESSAGE_SERVERHELLO']._serialized_end=56864
  _globals['_HISTORYSYNC']._serialized_start=56916
  _globals['_HISTORYSYNC']._serialized_end=58061
  _globals['_HISTORYSYNC_BOTAIWAITLISTSTATE']._serialized_start=57683
  _globals['_HISTORYSYNC_BOTAIWAITLISTSTATE']._serialized_end=57738
  _globals['_HISTORYSYNC_HISTORYSYNCTYPE']._serialized_start=57741
  _globals['_HISTORYSYNC_HISTORYSYNCTYPE']._serialized_end=57879
  _globals['_HISTORYSYNCMSG']._serialized_start=58063
  _globals['_HISTORYSYNCMSG']._serialized_end=58176
  _globals['_HYDRATEDTEMPLATEBUTTON']._serialized_start=58179
  _globals['_HYDRATEDTEMPLATEBUTTON']._serialized_end=59055
  _globals['_HYDRATEDTEMPLATEBUTTON_HYDRATEDCALLBUTTON']._serialized_start=58451
  _globals['_HYDRATEDTEMPLATEBUTTON_HYDRATEDCALLBUTTON']._serialized_end=58555
  _globals['_HYDRATEDTEMPLATEBUTTON_HYDRATEDQUICKREPLYBUTTON']._serialized_start=58557
  _globals['_HYDRATEDTEMPLATEBUTTON_HYDRATEDQUICKREPLYBUTTON']._serialized_end=58649
  _globals['_HYDRATEDTEMPLATEBUTTON_HYDRATEDURLBUTTON']._serialized_start=58652
  _globals['_HYDRATEDTEMPLATEBUTTON_HYDRATEDURLBUTTON']._serialized_end=59027
  _globals['_HYDRATEDTEMPLATEBUTTON_HYDRATEDURLBUTTON_WEBVIEWPRESENTATIONTYPE']._serialized_start=58856
  _globals['_HYDRATEDTEMPLATEBUTTON_HYDRATEDURLBUTTON_WEBVIEWPRESENTATIONTYPE']._serialized_end=58957
  _globals['_IDENTITYKEYPAIRSTRUCTURE']._serialized_start=59057
  _globals['_IDENTITYKEYPAIRSTRUCTURE']._serialized_end=59161
  _globals['_INTHREADSURVEYMETADATA']._serialized_start=59164
  _globals['_INTHREADSURVEYMETADATA']._serialized_end=60569
  _globals['_INTHREADSURVEYMETADATA_INTHREADSURVEYOPTION']._serialized_start=59822
  _globals['_INTHREADSURVEYMETADATA_INTHREADSURVEYOPTION']._serialized_end=59978
  _globals['_INTHREADSURVEYMETADATA_INTHREADSURVEYPRIVACYSTATEMENTPART']._serialized_start=59980
  _globals['_INTHREADSURVEYMETADATA_INTHREADSURVEYPRIVACYSTATEMENTPART']._serialized_end=60070
  _globals['_INTHREADSURVEYMETADATA_INTHREADSURVEYQUESTION']._serialized_start=60073
  _globals['_INTHREADSURVEYMETADATA_INTHREADSURVEYQUESTION']._serialized_end=60258
  _globals['_INTERACTIVEANNOTATION']._serialized_start=60572
  _globals['_INTERACTIVEANNOTATION']._serialized_end=61197
  _globals['_INTERACTIVEANNOTATION_STATUSLINKTYPE']._serialized_start=60981
  _globals['_INTERACTIVEANNOTATION_STATUSLINKTYPE']._serialized_end=61121
  _globals['_INTERACTIVEMESSAGEADDITIONALMETADATA']._serialized_start=61199
  _globals['_INTERACTIVEMESSAGEADDITIONALMETADATA']._serialized_end=61299
  _globals['_KEEPINCHAT']._serialized_start=61302
  _globals['_KEEPINCHAT']._serialized_end=61608
  _globals['_KEYEXCHANGEMESSAGE']._serialized_start=61611
  _globals['_KEYEXCHANGEMESSAGE']._serialized_end=61823
  _globals['_KEYID']._serialized_start=61825
  _globals['_KEYID']._serialized_end=61856
  _globals['_LIDMIGRATIONMAPPING']._serialized_start=61858
  _globals['_LIDMIGRATIONMAPPING']._serialized_end=61950
  _globals['_LIDMIGRATIONMAPPINGSYNCMESSAGE']._serialized_start=61952
  _globals['_LIDMIGRATIONMAPPINGSYNCMESSAGE']._serialized_end=62046
  _globals['_LIDMIGRATIONMAPPINGSYNCPAYLOAD']._serialized_start=62049
  _globals['_LIDMIGRATIONMAPPINGSYNCPAYLOAD']._serialized_end=62202
  _globals['_LEGACYMESSAGE']._serialized_start=62205
  _globals['_LEGACYMESSAGE']._serialized_end=62385
  _globals['_LIMITSHARING']._serialized_start=62388
  _globals['_LIMITSHARING']._serialized_end=62733
  _globals['_LIMITSHARING_TRIGGERTYPE']._serialized_start=62559
  _globals['_LIMITSHARING_TRIGGERTYPE']._serialized_end=62651
  _globals['_LOCALIZEDNAME']._serialized_start=62735
  _globals['_LOCALIZEDNAME']._serialized_end=62842
  _globals['_LOCATION']._serialized_start=62845
  _globals['_LOCATION']._serialized_end=62985
  _globals['_MEDIADATA']._serialized_start=62987
  _globals['_MEDIADATA']._serialized_end=63036
  _globals['_MEDIANOTIFYMESSAGE']._serialized_start=63039
  _globals['_MEDIANOTIFYMESSAGE']._serialized_end=63193
  _globals['_MEDIARETRYNOTIFICATION']._serialized_start=63196
  _globals['_MEDIARETRYNOTIFICATION']._serialized_end=63499
  _globals['_MEDIARETRYNOTIFICATION_RESULTTYPE']._serialized_start=63361
  _globals['_MEDIARETRYNOTIFICATION_RESULTTYPE']._serialized_end=63442
  _globals['_MEMBERLABEL']._serialized_start=63501
  _globals['_MEMBERLABEL']._serialized_end=63592
  _globals['_MESSAGE']._serialized_start=63596
  _globals['_MESSAGE']._serialized_end=123191
  _globals['_MESSAGE_ALBUMMESSAGE']._serialized_start=70180
  _globals['_MESSAGE_ALBUMMESSAGE']._serialized_end=70368
  _globals['_MESSAGE_APPSTATEFATALEXCEPTIONNOTIFICATION']._serialized_start=70370
  _globals['_MESSAGE_APPSTATEFATALEXCEPTIONNOTIFICATION']._serialized_end=70469
  _globals['_MESSAGE_APPSTATESYNCKEY']._serialized_start=70472
  _globals['_MESSAGE_APPSTATESYNCKEY']._serialized_end=70623
  _globals['_MESSAGE_APPSTATESYNCKEYDATA']._serialized_start=70626
  _globals['_MESSAGE_APPSTATESYNCKEYDATA']._serialized_end=70804
  _globals['_MESSAGE_APPSTATESYNCKEYFINGERPRINT']._serialized_start=70807
  _globals['_MESSAGE_APPSTATESYNCKEYFINGERPRINT']._serialized_end=70936
  _globals['_MESSAGE_APPSTATESYNCKEYID']._serialized_start=70938
  _globals['_MESSAGE_APPSTATESYNCKEYID']._serialized_end=70987
  _globals['_MESSAGE_APPSTATESYNCKEYREQUEST']._serialized_start=70989
  _globals['_MESSAGE_APPSTATESYNCKEYREQUEST']._serialized_end=71063
  _globals['_MESSAGE_APPSTATESYNCKEYSHARE']._serialized_start=71065
  _globals['_MESSAGE_APPSTATESYNCKEYSHARE']._serialized_end=71133
  _globals['_MESSAGE_AUDIOMESSAGE']._serialized_start=71136
  _globals['_MESSAGE_AUDIOMESSAGE']._serialized_end=71897
  _globals['_MESSAGE_BCALLMESSAGE']._serialized_start=71900
  _globals['_MESSAGE_BCALLMESSAGE']._serialized_end=72149
  _globals['_MESSAGE_BCALLMESSAGE_MEDIATYPE']._serialized_start=72049
  _globals['_MESSAGE_BCALLMESSAGE_MEDIATYPE']._serialized_end=72095
  _globals['_MESSAGE_BUTTONSMESSAGE']._serialized_start=72152
  _globals['_MESSAGE_BUTTONSMESSAGE']._serialized_end=73299
  _globals['_MESSAGE_BUTTONSMESSAGE_BUTTON']._serialized_start=72630
  _globals['_MESSAGE_BUTTONSMESSAGE_BUTTON']._serialized_end=73129
  _globals['_MESSAGE_BUTTONSMESSAGE_BUTTON_BUTTONTEXT']._serialized_start=72881
  _globals['_MESSAGE_BUTTONSMESSAGE_BUTTON_BUTTONTEXT']._serialized_end=72935
  _globals['_MESSAGE_BUTTONSMESSAGE_BUTTON_NATIVEFLOWINFO']._serialized_start=72937
  _globals['_MESSAGE_BUTTONSMESSAGE_BUTTON_NATIVEFLOWINFO']._serialized_end=73021
  _globals['_MESSAGE_BUTTONSMESSAGE_BUTTON_TYPE']._serialized_start=73023
  _globals['_MESSAGE_BUTTONSMESSAGE_BUTTON_TYPE']._serialized_end=73073
  _globals['_MESSAGE_BUTTONSMESSAGE_HEADERTYPE']._serialized_start=73131
  _globals['_MESSAGE_BUTTONSMESSAGE_HEADERTYPE']._serialized_end=73227
  _globals['_MESSAGE_BUTTONSRESPONSEMESSAGE']._serialized_start=73302
  _globals['_MESSAGE_BUTTONSRESPONSEMESSAGE']._serialized_end=73594
  _globals['_MESSAGE_BUTTONSRESPONSEMESSAGE_TYPE']._serialized_start=73499
  _globals['_MESSAGE_BUTTONSRESPONSEMESSAGE_TYPE']._serialized_end=73536
  _globals['_MESSAGE_CALL']._serialized_start=73597
  _globals['_MESSAGE_CALL']._serialized_end=74071
  _globals['_MESSAGE_CALLLOGMESSAGE']._serialized_start=74074
  _globals['_MESSAGE_CALLLOGMESSAGE']._serialized_end=74746
  _globals['_MESSAGE_CALLLOGMESSAGE_CALLPARTICIPANT']._serialized_start=74343
  _globals['_MESSAGE_CALLLOGMESSAGE_CALLPARTICIPANT']._serialized_end=74471
  _globals['_MESSAGE_CALLLOGMESSAGE_CALLOUTCOME']._serialized_start=74474
  _globals['_MESSAGE_CALLLOGMESSAGE_CALLOUTCOME']._serialized_end=74627
  _globals['_MESSAGE_CALLLOGMESSAGE_CALLTYPE']._serialized_start=27754
  _globals['_MESSAGE_CALLLOGMESSAGE_CALLTYPE']._serialized_end=27813
  _globals['_MESSAGE_CANCELPAYMENTREQUESTMESSAGE']._serialized_start=74748
  _globals['_MESSAGE_CANCELPAYMENTREQUESTMESSAGE']._serialized_end=74822
  _globals['_MESSAGE_CHAT']._serialized_start=74824
  _globals['_MESSAGE_CHAT']._serialized_end=74896
  _globals['_MESSAGE_CLOUDAPITHREADCONTROLNOTIFICATION']._serialized_start=74899
  _globals['_MESSAGE_CLOUDAPITHREADCONTROLNOTIFICATION']._serialized_end=75663
  _globals['_MESSAGE_CLOUDAPITHREADCONTROLNOTIFICATION_CLOUDAPITHREADCONTROLNOTIFICATIONCONTENT']._serialized_start=75300
  _globals['_MESSAGE_CLOUDAPITHREADCONTROLNOTIFICATION_CLOUDAPITHREADCONTROLNOTIFICATIONCONTENT']._serialized_end=75446
  _globals['_MESSAGE_CLOUDAPITHREADCONTROLNOTIFICATION_CLOUDAPITHREADCONTROL']._serialized_start=75448
  _globals['_MESSAGE_CLOUDAPITHREADCONTROLNOTIFICATION_CLOUDAPITHREADCONTROL']._serialized_end=75523
  _globals['_MESSAGE_COMMENTMESSAGE']._serialized_start=75666
  _globals['_MESSAGE_COMMENTMESSAGE']._serialized_end=75803
  _globals['_MESSAGE_CONTACTMESSAGE']._serialized_start=75806
  _globals['_MESSAGE_CONTACTMESSAGE']._serialized_end=75956
  _globals['_MESSAGE_CONTACTSARRAYMESSAGE']._serialized_start=75959
  _globals['_MESSAGE_CONTACTSARRAYMESSAGE']._serialized_end=76134
  _globals['_MESSAGE_DECLINEPAYMENTREQUESTMESSAGE']._serialized_start=76136
  _globals['_MESSAGE_DECLINEPAYMENTREQUESTMESSAGE']._serialized_end=76211
  _globals['_MESSAGE_DEVICESENTMESSAGE']._serialized_start=76214
  _globals['_MESSAGE_DEVICESENTMESSAGE']._serialized_end=76361
  _globals['_MESSAGE_DOCUMENTMESSAGE']._serialized_start=76364
  _globals['_MESSAGE_DOCUMENTMESSAGE']._serialized_end=77386
  _globals['_MESSAGE_ENCCOMMENTMESSAGE']._serialized_start=77389
  _globals['_MESSAGE_ENCCOMMENTMESSAGE']._serialized_end=77549
  _globals['_MESSAGE_ENCEVENTRESPONSEMESSAGE']._serialized_start=77552
  _globals['_MESSAGE_ENCEVENTRESPONSEMESSAGE']._serialized_end=77732
  _globals['_MESSAGE_ENCREACTIONMESSAGE']._serialized_start=77735
  _globals['_MESSAGE_ENCREACTIONMESSAGE']._serialized_end=77896
  _globals['_MESSAGE_EVENTMESSAGE']._serialized_start=77899
  _globals['_MESSAGE_EVENTMESSAGE']._serialized_end=78461
  _globals['_MESSAGE_EVENTRESPONSEMESSAGE']._serialized_start=78464
  _globals['_MESSAGE_EVENTRESPONSEMESSAGE']._serialized_end=78740
  _globals['_MESSAGE_EVENTRESPONSEMESSAGE_EVENTRESPONSETYPE']._serialized_start=78622
  _globals['_MESSAGE_EVENTRESPONSEMESSAGE_EVENTRESPONSETYPE']._serialized_end=78691
  _globals['_MESSAGE_EXTENDEDTEXTMESSAGE']._serialized_start=78743
  _globals['_MESSAGE_EXTENDEDTEXTMESSAGE']._serialized_end=81070
  _globals['_MESSAGE_EXTENDEDTEXTMESSAGE_FONTTYPE']._serialized_start=80130
  _globals['_MESSAGE_EXTENDEDTEXTMESSAGE_FONTTYPE']._serialized_end=80294
  _globals['_MESSAGE_EXTENDEDTEXTMESSAGE_INVITELINKGROUPTYPE']._serialized_start=80296
  _globals['_MESSAGE_EXTENDEDTEXTMESSAGE_INVITELINKGROUPTYPE']._serialized_end=80368
  _globals['_MESSAGE_EXTENDEDTEXTMESSAGE_PREVIEWTYPE']._serialized_start=80370
  _globals['_MESSAGE_EXTENDEDTEXTMESSAGE_PREVIEWTYPE']._serialized_end=80464
  _globals['_MESSAGE_FULLHISTORYSYNCONDEMANDREQUESTMETADATA']._serialized_start=81072
  _globals['_MESSAGE_FULLHISTORYSYNCONDEMANDREQUESTMETADATA']._serialized_end=81150
  _globals['_MESSAGE_FUTUREPROOFMESSAGE']._serialized_start=81152
  _globals['_MESSAGE_FUTUREPROOFMESSAGE']._serialized_end=81222
  _globals['_MESSAGE_GROUPINVITEMESSAGE']._serialized_start=81225
  _globals['_MESSAGE_GROUPINVITEMESSAGE']._serialized_end=81674
  _globals['_MESSAGE_GROUPINVITEMESSAGE_GROUPTYPE']._serialized_start=81515
  _globals['_MESSAGE_GROUPINVITEMESSAGE_GROUPTYPE']._serialized_end=81551
  _globals['_MESSAGE_HIGHLYSTRUCTUREDMESSAGE']._serialized_start=81677
  _globals['_MESSAGE_HIGHLYSTRUCTUREDMESSAGE']._serialized_end=83544
  _globals['_MESSAGE_HIGHLYSTRUCTUREDMESSAGE_HSMLOCALIZABLEPARAMETER']._serialized_start=82030
  _globals['_MESSAGE_HIGHLYSTRUCTUREDMESSAGE_HSMLOCALIZABLEPARAMETER']._serialized_end=83428
  _globals['_MESSAGE_HIGHLYSTRUCTUREDMESSAGE_HSMLOCALIZABLEPARAMETER_HSMCURRENCY']._serialized_start=82271
  _globals['_MESSAGE_HIGHLYSTRUCTUREDMESSAGE_HSMLOCALIZABLEPARAMETER_HSMCURRENCY']._serialized_end=82368
  _globals['_MESSAGE_HIGHLYSTRUCTUREDMESSAGE_HSMLOCALIZABLEPARAMETER_HSMDATETIME']._serialized_start=82371
  _globals['_MESSAGE_HIGHLYSTRUCTUREDMESSAGE_HSMLOCALIZABLEPARAMETER_HSMDATETIME']._serialized_end=83402
  _globals['_MESSAGE_HIGHLYSTRUCTUREDMESSAGE_HSMLOCALIZABLEPARAMETER_HSMDATETIME_HSMDATETIMECOMPONENT']._serialized_start=82623
  _globals['_MESSAGE_HIGHLYSTRUCTUREDMESSAGE_HSMLOCALIZABLEPARAMETER_HSMDATETIME_HSMDATETIMECOMPONENT']._serialized_end=83323
  _globals['_MESSAGE_HIGHLYSTRUCTUREDMESSAGE_HSMLOCALIZABLEPARAMETER_HSMDATETIME_HSMDATETIMECOMPONENT_CALENDARTYPE']._serialized_start=83021
  _globals['_MESSAGE_HIGHLYSTRUCTUREDMESSAGE_HSMLOCALIZABLEPARAMETER_HSMDATETIME_HSMDATETIMECOMPONENT_CALENDARTYPE']._serialized_end=83098
  _globals['_MESSAGE_HIGHLYSTRUCTUREDMESSAGE_HSMLOCALIZABLEPARAMETER_HSMDATETIME_HSMDATETIMECOMPONENT_DAYOFWEEKTYPE']._serialized_start=83101
  _globals['_MESSAGE_HIGHLYSTRUCTUREDMESSAGE_HSMLOCALIZABLEPARAMETER_HSMDATETIME_HSMDATETIMECOMPONENT_DAYOFWEEKTYPE']._serialized_end=83242
  _globals['_MESSAGE_HIGHLYSTRUCTUREDMESSAGE_HSMLOCALIZABLEPARAMETER_HSMDATETIME_HSMDATETIMEUNIXEPOCH']._serialized_start=83325
  _globals['_MESSAGE_HIGHLYSTRUCTUREDMESSAGE_HSMLOCALIZABLEPARAMETER_HSMDATETIME_HSMDATETIMEUNIXEPOCH']._serialized_end=83385
  _globals['_MESSAGE_HISTORYSYNCMESSAGEACCESSSTATUS']._serialized_start=83546
  _globals['_MESSAGE_HISTORYSYNCMESSAGEACCESSSTATUS']._serialized_end=83640
  _globals['_MESSAGE_HISTORYSYNCNOTIFICATION']._serialized_start=83643
  _globals['_MESSAGE_HISTORYSYNCNOTIFICATION']._serialized_end=84592
  _globals['_MESSAGE_IMAGEMESSAGE']._serialized_start=84595
  _globals['_MESSAGE_IMAGEMESSAGE']._serialized_end=86187
  _globals['_MESSAGE_IMAGEMESSAGE_IMAGESOURCETYPE']._serialized_start=85606
  _globals['_MESSAGE_IMAGEMESSAGE_IMAGESOURCETYPE']._serialized_end=85702
  _globals['_MESSAGE_INITIALSECURITYNOTIFICATIONSETTINGSYNC']._serialized_start=86189
  _globals['_MESSAGE_INITIALSECURITYNOTIFICATIONSETTINGSYNC']._serialized_end=86303
  _globals['_MESSAGE_INTERACTIVEMESSAGE']._serialized_start=86306
  _globals['_MESSAGE_INTERACTIVEMESSAGE']._serialized_end=88636
  _globals['_MESSAGE_INTERACTIVEMESSAGE_BODY']._serialized_start=86933
  _globals['_MESSAGE_INTERACTIVEMESSAGE_BODY']._serialized_end=86967
  _globals['_MESSAGE_INTERACTIVEMESSAGE_CAROUSELMESSAGE']._serialized_start=86970
  _globals['_MESSAGE_INTERACTIVEMESSAGE_CAROUSELMESSAGE']._serialized_end=87274
  _globals['_MESSAGE_INTERACTIVEMESSAGE_CAROUSELMESSAGE_CAROUSELCARDTYPE']._serialized_start=87167
  _globals['_MESSAGE_INTERACTIVEMESSAGE_CAROUSELMESSAGE_CAROUSELCARDTYPE']._serialized_end=87234
  _globals['_MESSAGE_INTERACTIVEMESSAGE_COLLECTIONMESSAGE']._serialized_start=87276
  _globals['_MESSAGE_INTERACTIVEMESSAGE_COLLECTIONMESSAGE']._serialized_end=87399
  _globals['_MESSAGE_INTERACTIVEMESSAGE_FOOTER']._serialized_start=87402
  _globals['_MESSAGE_INTERACTIVEMESSAGE_FOOTER']._serialized_end=87556
  _globals['_MESSAGE_INTERACTIVEMESSAGE_HEADER']._serialized_start=87559
  _globals['_MESSAGE_INTERACTIVEMESSAGE_HEADER']._serialized_end=88004
  _globals['_MESSAGE_INTERACTIVEMESSAGE_NATIVEFLOWMESSAGE']._serialized_start=88007
  _globals['_MESSAGE_INTERACTIVEMESSAGE_NATIVEFLOWMESSAGE']._serialized_end=88315
  _globals['_MESSAGE_INTERACTIVEMESSAGE_NATIVEFLOWMESSAGE_NATIVEFLOWBUTTON']._serialized_start=88176
  _globals['_MESSAGE_INTERACTIVEMESSAGE_NATIVEFLOWMESSAGE_NATIVEFLOWBUTTON']._serialized_end=88274
  _globals['_MESSAGE_INTERACTIVEMESSAGE_SHOPMESSAGE']._serialized_start=88318
  _globals['_MESSAGE_INTERACTIVEMESSAGE_SHOPMESSAGE']._serialized_end=88548
  _globals['_MESSAGE_INTERACTIVEMESSAGE_SHOPMESSAGE_SURFACE']._serialized_start=88456
  _globals['_MESSAGE_INTERACTIVEMESSAGE_SHOPMESSAGE_SURFACE']._serialized_end=88510
  _globals['_MESSAGE_INTERACTIVERESPONSEMESSAGE']._serialized_start=88639
  _globals['_MESSAGE_INTERACTIVERESPONSEMESSAGE']._serialized_end=89238
  _globals['_MESSAGE_INTERACTIVERESPONSEMESSAGE_BODY']._serialized_start=88889
  _globals['_MESSAGE_INTERACTIVERESPONSEMESSAGE_BODY']._serialized_end=89051
  _globals['_MESSAGE_INTERACTIVERESPONSEMESSAGE_BODY_FORMAT']._serialized_start=88992
  _globals['_MESSAGE_INTERACTIVERESPONSEMESSAGE_BODY_FORMAT']._serialized_end=89031
  _globals['_MESSAGE_INTERACTIVERESPONSEMESSAGE_NATIVEFLOWRESPONSEMESSAGE']._serialized_start=89054
  _globals['_MESSAGE_INTERACTIVERESPONSEMESSAGE_NATIVEFLOWRESPONSEMESSAGE']._serialized_end=89183
  _globals['_MESSAGE_INVOICEMESSAGE']._serialized_start=89241
  _globals['_MESSAGE_INVOICEMESSAGE']._serialized_end=89885
  _globals['_MESSAGE_INVOICEMESSAGE_ATTACHMENTTYPE']._serialized_start=89627
  _globals['_MESSAGE_INVOICEMESSAGE_ATTACHMENTTYPE']._serialized_end=89663
  _globals['_MESSAGE_KEEPINCHATMESSAGE']._serialized_start=89888
  _globals['_MESSAGE_KEEPINCHATMESSAGE']._serialized_end=90047
  _globals['_MESSAGE_LINKPREVIEWMETADATA']._serialized_start=90050
  _globals['_MESSAGE_LINKPREVIEWMETADATA']._serialized_end=90794
  _globals['_MESSAGE_LINKPREVIEWMETADATA_SOCIALMEDIAPOSTTYPE']._serialized_start=90497
  _globals['_MESSAGE_LINKPREVIEWMETADATA_SOCIALMEDIAPOSTTYPE']._serialized_end=90602
  _globals['_MESSAGE_LISTMESSAGE']._serialized_start=90797
  _globals['_MESSAGE_LISTMESSAGE']._serialized_end=91997
  _globals['_MESSAGE_LISTMESSAGE_PRODUCT']._serialized_start=91142
  _globals['_MESSAGE_LISTMESSAGE_PRODUCT']._serialized_end=91189
  _globals['_MESSAGE_LISTMESSAGE_PRODUCTLISTHEADERIMAGE']._serialized_start=91191
  _globals['_MESSAGE_LISTMESSAGE_PRODUCTLISTHEADERIMAGE']._serialized_end=91299
  _globals['_MESSAGE_LISTMESSAGE_PRODUCTLISTINFO']._serialized_start=91302
  _globals['_MESSAGE_LISTMESSAGE_PRODUCTLISTINFO']._serialized_end=91532
  _globals['_MESSAGE_LISTMESSAGE_PRODUCTSECTION']._serialized_start=91534
  _globals['_MESSAGE_LISTMESSAGE_PRODUCTSECTION']._serialized_end=91634
  _globals['_MESSAGE_LISTMESSAGE_ROW']._serialized_start=91636
  _globals['_MESSAGE_LISTMESSAGE_ROW']._serialized_end=91743
  _globals['_MESSAGE_LISTMESSAGE_SECTION']._serialized_start=91745
  _globals['_MESSAGE_LISTMESSAGE_SECTION']._serialized_end=91830
  _globals['_MESSAGE_LISTMESSAGE_LISTTYPE']._serialized_start=91832
  _globals['_MESSAGE_LISTMESSAGE_LISTTYPE']._serialized_end=91892
  _globals['_MESSAGE_LISTRESPONSEMESSAGE']._serialized_start=92000
  _globals['_MESSAGE_LISTRESPONSEMESSAGE']._serialized_end=92455
  _globals['_MESSAGE_LISTRESPONSEMESSAGE_SINGLESELECTREPLY']._serialized_start=92269
  _globals['_MESSAGE_LISTRESPONSEMESSAGE_SINGLESELECTREPLY']._serialized_end=92334
  _globals['_MESSAGE_LISTRESPONSEMESSAGE_LISTTYPE']._serialized_start=91832
  _globals['_MESSAGE_LISTRESPONSEMESSAGE_LISTTYPE']._serialized_end=91874
  _globals['_MESSAGE_LIVELOCATIONMESSAGE']._serialized_start=92458
  _globals['_MESSAGE_LIVELOCATIONMESSAGE']._serialized_end=92989
  _globals['_MESSAGE_LOCATIONMESSAGE']._serialized_start=92992
  _globals['_MESSAGE_LOCATIONMESSAGE']._serialized_end=93551
  _globals['_MESSAGE_MMSTHUMBNAILMETADATA']._serialized_start=93554
  _globals['_MESSAGE_MMSTHUMBNAILMETADATA']._serialized_end=94007
  _globals['_MESSAGE_MESSAGEHISTORYBUNDLE']._serialized_start=94010
  _globals['_MESSAGE_MESSAGEHISTORYBUNDLE']._serialized_end=94449
  _globals['_MESSAGE_MESSAGEHISTORYMETADATA']._serialized_start=94452
  _globals['_MESSAGE_MESSAGEHISTORYMETADATA']._serialized_end=94610
  _globals['_MESSAGE_MESSAGEHISTORYNOTICE']._serialized_start=94613
  _globals['_MESSAGE_MESSAGEHISTORYNOTICE']._serialized_end=94800
  _globals['_MESSAGE_NEWSLETTERADMININVITEMESSAGE']._serialized_start=94803
  _globals['_MESSAGE_NEWSLETTERADMININVITEMESSAGE']._serialized_end=95121
  _globals['_MESSAGE_NEWSLETTERFOLLOWERINVITEMESSAGE']._serialized_start=95124
  _globals['_MESSAGE_NEWSLETTERFOLLOWERINVITEMESSAGE']._serialized_end=95393
  _globals['_MESSAGE_ORDERMESSAGE']._serialized_start=95396
  _globals['_MESSAGE_ORDERMESSAGE']._serialized_end=96294
  _globals['_MESSAGE_ORDERMESSAGE_ORDERSTATUS']._serialized_start=95917
  _globals['_MESSAGE_ORDERMESSAGE_ORDERSTATUS']._serialized_end=96001
  _globals['_MESSAGE_ORDERMESSAGE_ORDERSURFACE']._serialized_start=96003
  _globals['_MESSAGE_ORDERMESSAGE_ORDERSURFACE']._serialized_end=96061
  _globals['_MESSAGE_PAYMENTEXTENDEDMETADATA']._serialized_start=96297
  _globals['_MESSAGE_PAYMENTEXTENDEDMETADATA']._serialized_end=96440
  _globals['_MESSAGE_PAYMENTINVITEMESSAGE']._serialized_start=96443
  _globals['_MESSAGE_PAYMENTINVITEMESSAGE']._serialized_end=96664
  _globals['_MESSAGE_PAYMENTINVITEMESSAGE_SERVICETYPE']._serialized_start=96572
  _globals['_MESSAGE_PAYMENTINVITEMESSAGE_SERVICETYPE']._serialized_end=96628
  _globals['_MESSAGE_PAYMENTLINKMETADATA']._serialized_start=96667
  _globals['_MESSAGE_PAYMENTLINKMETADATA']._serialized_end=97270
  _globals['_MESSAGE_PAYMENTLINKMETADATA_PAYMENTLINKBUTTON']._serialized_start=96919
  _globals['_MESSAGE_PAYMENTLINKMETADATA_PAYMENTLINKBUTTON']._serialized_end=96980
  _globals['_MESSAGE_PAYMENTLINKMETADATA_PAYMENTLINKHEADER']._serialized_start=96983
  _globals['_MESSAGE_PAYMENTLINKMETADATA_PAYMENTLINKHEADER']._serialized_end=97172
  _globals['_MESSAGE_PAYMENTLINKMETADATA_PAYMENTLINKHEADER_PAYMENTLINKHEADERTYPE']._serialized_start=97105
  _globals['_MESSAGE_PAYMENTLINKMETADATA_PAYMENTLINKHEADER_PAYMENTLINKHEADERTYPE']._serialized_end=97157
  _globals['_MESSAGE_PAYMENTLINKMETADATA_PAYMENTLINKPROVIDER']._serialized_start=97174
  _globals['_MESSAGE_PAYMENTLINKMETADATA_PAYMENTLINKPROVIDER']._serialized_end=97235
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTMESSAGE']._serialized_start=97273
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTMESSAGE']._serialized_end=99960
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTMESSAGE_FULLHISTORYSYNCONDEMANDREQUEST']._serialized_start=98305
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTMESSAGE_FULLHISTORYSYNCONDEMANDREQUEST']._serialized_end=98534
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTMESSAGE_GALAXYFLOWACTION']._serialized_start=98537
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTMESSAGE_GALAXYFLOWACTION']._serialized_end=98821
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTMESSAGE_GALAXYFLOWACTION_GALAXYFLOWACTIONTYPE']._serialized_start=98706
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTMESSAGE_GALAXYFLOWACTION_GALAXYFLOWACTIONTYPE']._serialized_end=98788
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTMESSAGE_HISTORYSYNCCHUNKRETRYREQUEST']._serialized_start=98824
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTMESSAGE_HISTORYSYNCCHUNKRETRYREQUEST']._serialized_end=99070
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTMESSAGE_HISTORYSYNCONDEMANDREQUEST']._serialized_start=99073
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTMESSAGE_HISTORYSYNCONDEMANDREQUEST']._serialized_end=99379
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTMESSAGE_PLACEHOLDERMESSAGERESENDREQUEST']._serialized_start=99381
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTMESSAGE_PLACEHOLDERMESSAGERESENDREQUEST']._serialized_end=99473
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTMESSAGE_REQUESTSTICKERREUPLOAD']._serialized_start=99475
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTMESSAGE_REQUESTSTICKERREUPLOAD']._serialized_end=99539
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTMESSAGE_REQUESTURLPREVIEW']._serialized_start=99541
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTMESSAGE_REQUESTURLPREVIEW']._serialized_end=99642
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTMESSAGE_SYNCDCOLLECTIONFATALRECOVERYREQUEST']._serialized_start=99644
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTMESSAGE_SYNCDCOLLECTIONFATALRECOVERYREQUEST']._serialized_end=99767
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE']._serialized_start=99963
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE']._serialized_end=104842
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE_PEERDATAOPERATIONRESULT']._serialized_start=100231
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE_PEERDATAOPERATIONRESULT']._serialized_end=104796
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE_PEERDATAOPERATIONRESULT_COMPANIONCANONICALUSERNONCEFETCHRESPONSE']._serialized_start=101684
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE_PEERDATAOPERATIONRESULT_COMPANIONCANONICALUSERNONCEFETCHRESPONSE']._serialized_end=101832
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE_PEERDATAOPERATIONRESULT_COMPANIONMETANONCEFETCHRESPONSE']._serialized_start=101834
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE_PEERDATAOPERATIONRESULT_COMPANIONMETANONCEFETCHRESPONSE']._serialized_end=101897
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE_PEERDATAOPERATIONRESULT_FULLHISTORYSYNCONDEMANDREQUESTRESPONSE']._serialized_start=101900
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE_PEERDATAOPERATIONRESULT_FULLHISTORYSYNCONDEMANDREQUESTRESPONSE']._serialized_end=102206
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE_PEERDATAOPERATIONRESULT_HISTORYSYNCCHUNKRETRYRESPONSE']._serialized_start=102209
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE_PEERDATAOPERATIONRESULT_HISTORYSYNCCHUNKRETRYRESPONSE']._serialized_end=102585
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE_PEERDATAOPERATIONRESULT_LINKPREVIEWRESPONSE']._serialized_start=102588
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE_PEERDATAOPERATIONRESULT_LINKPREVIEWRESPONSE']._serialized_end=103644
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE_PEERDATAOPERATIONRESULT_LINKPREVIEWRESPONSE_LINKPREVIEWHIGHQUALITYTHUMBNAIL']._serialized_start=103068
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE_PEERDATAOPERATIONRESULT_LINKPREVIEWRESPONSE_LINKPREVIEWHIGHQUALITYTHUMBNAIL']._serialized_end=103399
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE_PEERDATAOPERATIONRESULT_LINKPREVIEWRESPONSE_PAYMENTLINKPREVIEWMETADATA']._serialized_start=103402
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE_PEERDATAOPERATIONRESULT_LINKPREVIEWRESPONSE_PAYMENTLINKPREVIEWMETADATA']._serialized_end=103530
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE_PEERDATAOPERATIONRESULT_PLACEHOLDERMESSAGERESENDRESPONSE']._serialized_start=103646
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE_PEERDATAOPERATIONRESULT_PLACEHOLDERMESSAGERESENDRESPONSE']._serialized_end=103738
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE_PEERDATAOPERATIONRESULT_SYNCDSNAPSHOTFATALRECOVERYRESPONSE']._serialized_start=103741
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE_PEERDATAOPERATIONRESULT_SYNCDSNAPSHOTFATALRECOVERYRESPONSE']._serialized_end=103877
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE_PEERDATAOPERATIONRESULT_WAFFLENONCEFETCHRESPONSE']._serialized_start=103879
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE_PEERDATAOPERATIONRESULT_WAFFLENONCEFETCHRESPONSE']._serialized_end=103973
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE_PEERDATAOPERATIONRESULT_FULLHISTORYSYNCONDEMANDRESPONSECODE']._serialized_start=103976
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE_PEERDATAOPERATIONRESULT_FULLHISTORYSYNCONDEMANDRESPONSECODE']._serialized_end=104230
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE_PEERDATAOPERATIONRESULT_HISTORYSYNCCHUNKRETRYRESPONSECODE']._serialized_start=104233
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTRESPONSEMESSAGE_PEERDATAOPERATIONRESULT_HISTORYSYNCCHUNKRETRYRESPONSECODE']._serialized_end=104447
  _globals['_MESSAGE_PININCHATMESSAGE']._serialized_start=104845
  _globals['_MESSAGE_PININCHATMESSAGE']._serialized_end=105090
  _globals['_MESSAGE_PININCHATMESSAGE_TYPE']._serialized_start=104991
  _globals['_MESSAGE_PININCHATMESSAGE_TYPE']._serialized_end=105051
  _globals['_MESSAGE_PLACEHOLDERMESSAGE']._serialized_start=105093
  _globals['_MESSAGE_PLACEHOLDERMESSAGE']._serialized_end=105236
  _globals['_MESSAGE_PLACEHOLDERMESSAGE_PLACEHOLDERTYPE']._serialized_start=105185
  _globals['_MESSAGE_PLACEHOLDERMESSAGE_PLACEHOLDERTYPE']._serialized_end=105227
  _globals['_MESSAGE_POLLCREATIONMESSAGE']._serialized_start=105239
  _globals['_MESSAGE_POLLCREATIONMESSAGE']._serialized_end=105828
  _globals['_MESSAGE_POLLCREATIONMESSAGE_OPTION']._serialized_start=105626
  _globals['_MESSAGE_POLLCREATIONMESSAGE_OPTION']._serialized_end=105714
  _globals['_MESSAGE_POLLENCVALUE']._serialized_start=105830
  _globals['_MESSAGE_POLLENCVALUE']._serialized_end=105914
  _globals['_MESSAGE_POLLRESULTSNAPSHOTMESSAGE']._serialized_start=105917
  _globals['_MESSAGE_POLLRESULTSNAPSHOTMESSAGE']._serialized_end=106267
  _globals['_MESSAGE_POLLRESULTSNAPSHOTMESSAGE_POLLVOTE']._serialized_start=106129
  _globals['_MESSAGE_POLLRESULTSNAPSHOTMESSAGE_POLLVOTE']._serialized_end=106229
  _globals['_MESSAGE_POLLUPDATEMESSAGE']._serialized_start=106270
  _globals['_MESSAGE_POLLUPDATEMESSAGE']._serialized_end=106561
  _globals['_MESSAGE_POLLUPDATEMESSAGEMETADATA']._serialized_start=106563
  _globals['_MESSAGE_POLLUPDATEMESSAGEMETADATA']._serialized_end=106590
  _globals['_MESSAGE_POLLVOTEMESSAGE']._serialized_start=106592
  _globals['_MESSAGE_POLLVOTEMESSAGE']._serialized_end=106634
  _globals['_MESSAGE_PRODUCTMESSAGE']._serialized_start=106637
  _globals['_MESSAGE_PRODUCTMESSAGE']._serialized_end=107711
  _globals['_MESSAGE_PRODUCTMESSAGE_CATALOGSNAPSHOT']._serialized_start=106911
  _globals['_MESSAGE_PRODUCTMESSAGE_CATALOGSNAPSHOT']._serialized_end=107073
  _globals['_MESSAGE_PRODUCTMESSAGE_PRODUCTSNAPSHOT']._serialized_start=107076
  _globals['_MESSAGE_PRODUCTMESSAGE_PRODUCTSNAPSHOT']._serialized_end=107630
  _globals['_MESSAGE_PROTOCOLMESSAGE']._serialized_start=107714
  _globals['_MESSAGE_PROTOCOLMESSAGE']._serialized_end=110640
  _globals['_MESSAGE_PROTOCOLMESSAGE_TYPE']._serialized_start=109254
  _globals['_MESSAGE_PROTOCOLMESSAGE_TYPE']._serialized_end=110035
  _globals['_MESSAGE_QUESTIONRESPONSEMESSAGE']._serialized_start=110642
  _globals['_MESSAGE_QUESTIONRESPONSEMESSAGE']._serialized_end=110740
  _globals['_MESSAGE_REACTIONMESSAGE']._serialized_start=110743
  _globals['_MESSAGE_REACTIONMESSAGE']._serialized_end=110929
  _globals['_MESSAGE_REQUESTPAYMENTMESSAGE']._serialized_start=110932
  _globals['_MESSAGE_REQUESTPAYMENTMESSAGE']._serialized_end=111315
  _globals['_MESSAGE_REQUESTPHONENUMBERMESSAGE']._serialized_start=111317
  _globals['_MESSAGE_REQUESTPHONENUMBERMESSAGE']._serialized_end=111406
  _globals['_MESSAGE_REQUESTWELCOMEMESSAGEMETADATA']._serialized_start=111409
  _globals['_MESSAGE_REQUESTWELCOMEMESSAGEMETADATA']._serialized_end=111593
  _globals['_MESSAGE_REQUESTWELCOMEMESSAGEMETADATA_LOCALCHATSTATE']._serialized_start=111532
  _globals['_MESSAGE_REQUESTWELCOMEMESSAGEMETADATA_LOCALCHATSTATE']._serialized_end=111574
  _globals['_MESSAGE_SCHEDULEDCALLCREATIONMESSAGE']._serialized_start=111596
  _globals['_MESSAGE_SCHEDULEDCALLCREATIONMESSAGE']._serialized_end=111853
  _globals['_MESSAGE_SCHEDULEDCALLCREATIONMESSAGE_CALLTYPE']._serialized_start=111760
  _globals['_MESSAGE_SCHEDULEDCALLCREATIONMESSAGE_CALLTYPE']._serialized_end=111805
  _globals['_MESSAGE_SCHEDULEDCALLEDITMESSAGE']._serialized_start=111856
  _globals['_MESSAGE_SCHEDULEDCALLEDITMESSAGE']._serialized_end=112050
  _globals['_MESSAGE_SCHEDULEDCALLEDITMESSAGE_EDITTYPE']._serialized_start=111994
  _globals['_MESSAGE_SCHEDULEDCALLEDITMESSAGE_EDITTYPE']._serialized_end=112029
  _globals['_MESSAGE_SECRETENCRYPTEDMESSAGE']._serialized_start=112053
  _globals['_MESSAGE_SECRETENCRYPTEDMESSAGE']._serialized_end=112381
  _globals['_MESSAGE_SECRETENCRYPTEDMESSAGE_SECRETENCTYPE']._serialized_start=112255
  _globals['_MESSAGE_SECRETENCRYPTEDMESSAGE_SECRETENCTYPE']._serialized_end=112317
  _globals['_MESSAGE_SENDPAYMENTMESSAGE']._serialized_start=112384
  _globals['_MESSAGE_SENDPAYMENTMESSAGE']._serialized_end=112651
  _globals['_MESSAGE_SENDERKEYDISTRIBUTIONMESSAGE']._serialized_start=112654
  _globals['_MESSAGE_SENDERKEYDISTRIBUTIONMESSAGE']._serialized_end=112808
  _globals['_MESSAGE_STATUSNOTIFICATIONMESSAGE']._serialized_start=112811
  _globals['_MESSAGE_STATUSNOTIFICATIONMESSAGE']._serialized_end=113198
  _globals['_MESSAGE_STATUSNOTIFICATIONMESSAGE_STATUSNOTIFICATIONTYPE']._serialized_start=113028
  _globals['_MESSAGE_STATUSNOTIFICATIONMESSAGE_STATUSNOTIFICATIONTYPE']._serialized_end=113143
  _globals['_MESSAGE_STATUSQUESTIONANSWERMESSAGE']._serialized_start=113200
  _globals['_MESSAGE_STATUSQUESTIONANSWERMESSAGE']._serialized_end=113302
  _globals['_MESSAGE_STATUSQUOTEDMESSAGE']._serialized_start=113305
  _globals['_MESSAGE_STATUSQUOTEDMESSAGE']._serialized_end=113643
  _globals['_MESSAGE_STATUSQUOTEDMESSAGE_STATUSQUOTEDMESSAGETYPE']._serialized_start=113500
  _globals['_MESSAGE_STATUSQUOTEDMESSAGE_STATUSQUOTEDMESSAGETYPE']._serialized_end=113590
  _globals['_MESSAGE_STATUSSTICKERINTERACTIONMESSAGE']._serialized_start=113646
  _globals['_MESSAGE_STATUSSTICKERINTERACTIONMESSAGE']._serialized_end=113906
  _globals['_MESSAGE_STATUSSTICKERINTERACTIONMESSAGE_STATUSSTICKERTYPE']._serialized_start=113828
  _globals['_MESSAGE_STATUSSTICKERINTERACTIONMESSAGE_STATUSSTICKERTYPE']._serialized_end=113874
  _globals['_MESSAGE_STICKERMESSAGE']._serialized_start=113909
  _globals['_MESSAGE_STICKERMESSAGE']._serialized_end=114852
  _globals['_MESSAGE_STICKERPACKMESSAGE']._serialized_start=114855
  _globals['_MESSAGE_STICKERPACKMESSAGE']._serialized_end=116272
  _globals['_MESSAGE_STICKERPACKMESSAGE_STICKER']._serialized_start=115596
  _globals['_MESSAGE_STICKERPACKMESSAGE_STICKER']._serialized_end=115825
  _globals['_MESSAGE_STICKERPACKMESSAGE_STICKERPACKORIGIN']._serialized_start=115827
  _globals['_MESSAGE_STICKERPACKMESSAGE_STICKERPACKORIGIN']._serialized_end=115898
  _globals['_MESSAGE_STICKERSYNCRMRMESSAGE']._serialized_start=116275
  _globals['_MESSAGE_STICKERSYNCRMRMESSAGE']._serialized_end=116406
  _globals['_MESSAGE_TEMPLATEBUTTONREPLYMESSAGE']._serialized_start=116409
  _globals['_MESSAGE_TEMPLATEBUTTONREPLYMESSAGE']._serialized_end=116713
  _globals['_MESSAGE_TEMPLATEMESSAGE']._serialized_start=116716
  _globals['_MESSAGE_TEMPLATEMESSAGE']._serialized_end=118260
  _globals['_MESSAGE_TEMPLATEMESSAGE_FOURROWTEMPLATE']._serialized_start=117133
  _globals['_MESSAGE_TEMPLATEMESSAGE_FOURROWTEMPLATE']._serialized_end=117644
  _globals['_MESSAGE_TEMPLATEMESSAGE_HYDRATEDFOURROWTEMPLATE']._serialized_start=117647
  _globals['_MESSAGE_TEMPLATEMESSAGE_HYDRATEDFOURROWTEMPLATE']._serialized_end=118198
  _globals['_MESSAGE_URLMETADATA']._serialized_start=118262
  _globals['_MESSAGE_URLMETADATA']._serialized_end=118323
  _globals['_MESSAGE_VIDEOENDCARD']._serialized_start=118325
  _globals['_MESSAGE_VIDEOENDCARD']._serialized_end=118428
  _globals['_MESSAGE_VIDEOMESSAGE']._serialized_start=118431
  _globals['_MESSAGE_VIDEOMESSAGE']._serialized_end=120146
  _globals['_MESSAGE_VIDEOMESSAGE_ATTRIBUTION']._serialized_start=119531
  _globals['_MESSAGE_VIDEOMESSAGE_ATTRIBUTION']._serialized_end=119587
  _globals['_MESSAGE_VIDEOMESSAGE_VIDEOSOURCETYPE']._serialized_start=119589
  _globals['_MESSAGE_VIDEOMESSAGE_VIDEOSOURCETYPE']._serialized_end=119640
  _globals['_MESSAGE_HISTORYSYNCTYPE']._serialized_start=120149
  _globals['_MESSAGE_HISTORYSYNCTYPE']._serialized_end=120330
  _globals['_MESSAGE_MEDIAKEYDOMAIN']._serialized_start=120332
  _globals['_MESSAGE_MEDIAKEYDOMAIN']._serialized_end=120405
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTTYPE']._serialized_start=120408
  _globals['_MESSAGE_PEERDATAOPERATIONREQUESTTYPE']._serialized_end=120818
  _globals['_MESSAGE_POLLCONTENTTYPE']._serialized_start=120820
  _globals['_MESSAGE_POLLCONTENTTYPE']._serialized_end=120871
  _globals['_MESSAGE_POLLTYPE']._serialized_start=120873
  _globals['_MESSAGE_POLLTYPE']._serialized_end=120903
  _globals['_MESSAGEADDON']._serialized_start=123194
  _globals['_MESSAGEADDON']._serialized_end=123852
  _globals['_MESSAGEADDON_MESSAGEADDONTYPE']._serialized_start=123599
  _globals['_MESSAGEADDON_MESSAGEADDONTYPE']._serialized_end=123700
  _globals['_MESSAGEADDONCONTEXTINFO']._serialized_start=123855
  _globals['_MESSAGEADDONCONTEXTINFO']._serialized_end=124066
  _globals['_MESSAGEASSOCIATION']._serialized_start=124069
  _globals['_MESSAGEASSOCIATION']._serialized_end=124767
  _globals['_MESSAGEASSOCIATION_ASSOCIATIONTYPE']._serialized_start=124242
  _globals['_MESSAGEASSOCIATION_ASSOCIATIONTYPE']._serialized_end=124709
  _globals['_MESSAGECONTEXTINFO']._serialized_start=124770
  _globals['_MESSAGECONTEXTINFO']._serialized_end=125939
  _globals['_MESSAGECONTEXTINFO_MESSAGEADDONEXPIRYTYPE']._serialized_start=125503
  _globals['_MESSAGECONTEXTINFO_MESSAGEADDONEXPIRYTYPE']._serialized_end=125607
  _globals['_MESSAGEKEY']._serialized_start=125942
  _globals['_MESSAGEKEY']._serialized_end=126090
  _globals['_MESSAGESECRETMESSAGE']._serialized_start=126092
  _globals['_MESSAGESECRETMESSAGE']._serialized_end=126218
  _globals['_MONEY']._serialized_start=126220
  _globals['_MONEY']._serialized_end=126333
  _globals['_MSGOPAQUEDATA']._serialized_start=126336
  _globals['_MSGOPAQUEDATA']._serialized_end=129173
  _globals['_MSGOPAQUEDATA_EVENTLOCATION']._serialized_start=127773
  _globals['_MSGOPAQUEDATA_EVENTLOCATION']._serialized_end=128024
  _globals['_MSGOPAQUEDATA_POLLOPTION']._serialized_start=128026
  _globals['_MSGOPAQUEDATA_POLLOPTION']._serialized_end=128094
  _globals['_MSGOPAQUEDATA_POLLVOTESNAPSHOT']._serialized_start=128097
  _globals['_MSGOPAQUEDATA_POLLVOTESNAPSHOT']._serialized_end=128230
  _globals['_MSGOPAQUEDATA_POLLVOTESSNAPSHOT']._serialized_start=128232
  _globals['_MSGOPAQUEDATA_POLLVOTESSNAPSHOT']._serialized_end=128309
  _globals['_MSGOPAQUEDATA_POLLCONTENTTYPE']._serialized_start=120820
  _globals['_MSGOPAQUEDATA_POLLCONTENTTYPE']._serialized_end=120871
  _globals['_MSGOPAQUEDATA_POLLTYPE']._serialized_start=120873
  _globals['_MSGOPAQUEDATA_POLLTYPE']._serialized_end=120903
  _globals['_MSGROWOPAQUEDATA']._serialized_start=129176
  _globals['_MSGROWOPAQUEDATA']._serialized_end=129316
  _globals['_NOISECERTIFICATE']._serialized_start=129319
  _globals['_NOISECERTIFICATE']._serialized_end=129579
  _globals['_NOISECERTIFICATE_DETAILS']._serialized_start=129386
  _globals['_NOISECERTIFICATE_DETAILS']._serialized_end=129553
  _globals['_NOTIFICATIONMESSAGEINFO']._serialized_start=129582
  _globals['_NOTIFICATIONMESSAGEINFO']._serialized_end=129796
  _globals['_NOTIFICATIONSETTINGS']._serialized_start=129799
  _globals['_NOTIFICATIONSETTINGS']._serialized_end=130115
  _globals['_PAIRINGREQUEST']._serialized_start=130118
  _globals['_PAIRINGREQUEST']._serialized_end=130288
  _globals['_PASTPARTICIPANT']._serialized_start=130291
  _globals['_PASTPARTICIPANT']._serialized_end=130492
  _globals['_PASTPARTICIPANT_LEAVEREASON']._serialized_start=130416
  _globals['_PASTPARTICIPANT_LEAVEREASON']._serialized_end=130452
  _globals['_PASTPARTICIPANTS']._serialized_start=130494
  _globals['_PASTPARTICIPANTS']._serialized_end=130598
  _globals['_PATCHDEBUGDATA']._serialized_start=130601
  _globals['_PATCHDEBUGDATA']._serialized_end=131346
  _globals['_PATCHDEBUGDATA_PLATFORM']._serialized_start=130985
  _globals['_PATCHDEBUGDATA_PLATFORM']._serialized_end=131123
  _globals['_PAYMENTBACKGROUND']._serialized_start=131349
  _globals['_PAYMENTBACKGROUND']._serialized_end=132055
  _globals['_PAYMENTBACKGROUND_MEDIADATA']._serialized_start=131666
  _globals['_PAYMENTBACKGROUND_MEDIADATA']._serialized_end=131893
  _globals['_PAYMENTBACKGROUND_TYPE']._serialized_start=131895
  _globals['_PAYMENTBACKGROUND_TYPE']._serialized_end=131927
  _globals['_PAYMENTINFO']._serialized_start=132058
  _globals['_PAYMENTINFO']._serialized_end=133724
  _globals['_PAYMENTINFO_CURRENCY']._serialized_start=132572
  _globals['_PAYMENTINFO_CURRENCY']._serialized_end=132613
  _globals['_PAYMENTINFO_STATUS']._serialized_start=132616
  _globals['_PAYMENTINFO_STATUS']._serialized_end=132820
  _globals['_PAYMENTINFO_TXNSTATUS']._serialized_start=132823
  _globals['_PAYMENTINFO_TXNSTATUS']._serialized_end=133488
  _globals['_PHONENUMBERTOLIDMAPPING']._serialized_start=133726
  _globals['_PHONENUMBERTOLIDMAPPING']._serialized_end=133813
  _globals['_PHOTOCHANGE']._serialized_start=133815
  _globals['_PHOTOCHANGE']._serialized_end=133940
  _globals['_PININCHAT']._serialized_start=133943
  _globals['_PININCHAT']._serialized_end=134318
  _globals['_PININCHAT_TYPE']._serialized_start=104991
  _globals['_PININCHAT_TYPE']._serialized_end=105051
  _globals['_POINT']._serialized_start=134321
  _globals['_POINT']._serialized_end=134456
  _globals['_POLLADDITIONALMETADATA']._serialized_start=134458
  _globals['_POLLADDITIONALMETADATA']._serialized_end=134532
  _globals['_POLLENCVALUE']._serialized_start=105830
  _globals['_POLLENCVALUE']._serialized_end=105914
  _globals['_POLLUPDATE']._serialized_start=134621
  _globals['_POLLUPDATE']._serialized_end=134912
  _globals['_PREKEYRECORDSTRUCTURE']._serialized_start=134914
  _globals['_PREKEYRECORDSTRUCTURE']._serialized_end=135039
  _globals['_PREKEYSIGNALMESSAGE']._serialized_start=135042
  _globals['_PREKEYSIGNALMESSAGE']._serialized_end=135305
  _globals['_PREMIUMMESSAGEINFO']._serialized_start=135307
  _globals['_PREMIUMMESSAGEINFO']._serialized_end=135379
  _globals['_PRIMARYEPHEMERALIDENTITY']._serialized_start=135381
  _globals['_PRIMARYEPHEMERALIDENTITY']._serialized_end=135475
  _globals['_PROCESSEDVIDEO']._serialized_start=135478
  _globals['_PROCESSEDVIDEO']._serialized_end=135861
  _globals['_PROCESSEDVIDEO_VIDEOQUALITY']._serialized_start=135714
  _globals['_PROCESSEDVIDEO_VIDEOQUALITY']._serialized_end=135771
  _globals['_PROLOGUEPAYLOAD']._serialized_start=135864
  _globals['_PROLOGUEPAYLOAD']._serialized_end=136021
  _globals['_PUSHNAME']._serialized_start=136023
  _globals['_PUSHNAME']._serialized_end=136093
  _globals['_QUARANTINEDMESSAGE']._serialized_start=136095
  _globals['_QUARANTINEDMESSAGE']._serialized_end=136205
  _globals['_REACTION']._serialized_start=136208
  _globals['_REACTION']._serialized_end=136419
  _globals['_RECENTEMOJIWEIGHT']._serialized_start=136421
  _globals['_RECENTEMOJIWEIGHT']._serialized_end=136502
  _globals['_RECORDSTRUCTURE']._serialized_start=136505
  _globals['_RECORDSTRUCTURE']._serialized_end=136646
  _globals['_REPORTABLE']._serialized_start=136649
  _globals['_REPORTABLE']._serialized_end=136837
  _globals['_REPORTINGTOKENINFO']._serialized_start=136839
  _globals['_REPORTINGTOKENINFO']._serialized_end=136903
  _globals['_SENDERKEYDISTRIBUTIONMESSAGE']._serialized_start=136906
  _globals['_SENDERKEYDISTRIBUTIONMESSAGE']._serialized_end=137074
  _globals['_SENDERKEYMESSAGE']._serialized_start=137076
  _globals['_SENDERKEYMESSAGE']._serialized_end=137196
  _globals['_SENDERKEYRECORDSTRUCTURE']._serialized_start=137198
  _globals['_SENDERKEYRECORDSTRUCTURE']._serialized_end=137281
  _globals['_SENDERKEYSTATESTRUCTURE']._serialized_start=137284
  _globals['_SENDERKEYSTATESTRUCTURE']._serialized_end=137879
  _globals['_SENDERKEYSTATESTRUCTURE_SENDERCHAINKEY']._serialized_start=137569
  _globals['_SENDERKEYSTATESTRUCTURE_SENDERCHAINKEY']._serialized_end=137651
  _globals['_SENDERKEYSTATESTRUCTURE_SENDERMESSAGEKEY']._serialized_start=137653
  _globals['_SENDERKEYSTATESTRUCTURE_SENDERMESSAGEKEY']._serialized_end=137737
  _globals['_SENDERKEYSTATESTRUCTURE_SENDERSIGNINGKEY']._serialized_start=137739
  _globals['_SENDERKEYSTATESTRUCTURE_SENDERSIGNINGKEY']._serialized_end=137823
  _globals['_SERVERERRORRECEIPT']._serialized_start=137881
  _globals['_SERVERERRORRECEIPT']._serialized_end=137937
  _globals['_SESSIONSTRUCTURE']._serialized_start=137940
  _globals['_SESSIONSTRUCTURE']._serialized_end=139730
  _globals['_SESSIONSTRUCTURE_CHAIN']._serialized_start=138490
  _globals['_SESSIONSTRUCTURE_CHAIN']._serialized_end=138961
  _globals['_SESSIONSTRUCTURE_CHAIN_CHAINKEY']._serialized_start=138694
  _globals['_SESSIONSTRUCTURE_CHAIN_CHAINKEY']._serialized_end=138760
  _globals['_SESSIONSTRUCTURE_CHAIN_MESSAGEKEY']._serialized_start=138763
  _globals['_SESSIONSTRUCTURE_CHAIN_MESSAGEKEY']._serialized_end=138899
  _globals['_SESSIONSTRUCTURE_PENDINGKEYEXCHANGE']._serialized_start=138964
  _globals['_SESSIONSTRUCTURE_PENDINGKEYEXCHANGE']._serialized_end=139354
  _globals['_SESSIONSTRUCTURE_PENDINGPREKEY']._serialized_start=139357
  _globals['_SESSIONSTRUCTURE_PENDINGPREKEY']._serialized_end=139490
  _globals['_SESSIONTRANSPARENCYMETADATA']._serialized_start=139733
  _globals['_SESSIONTRANSPARENCYMETADATA']._serialized_end=139938
  _globals['_SIGNALMESSAGE']._serialized_start=139941
  _globals['_SIGNALMESSAGE']._serialized_end=140120
  _globals['_SIGNEDPREKEYRECORDSTRUCTURE']._serialized_start=140123
  _globals['_SIGNEDPREKEYRECORDSTRUCTURE']._serialized_end=140330
  _globals['_STATUSATTRIBUTION']._serialized_start=140333
  _globals['_STATUSATTRIBUTION']._serialized_end=142523
  _globals['_STATUSATTRIBUTION_AICREATEDATTRIBUTION']._serialized_start=140813
  _globals['_STATUSATTRIBUTION_AICREATEDATTRIBUTION']._serialized_end=140964
  _globals['_STATUSATTRIBUTION_AICREATEDATTRIBUTION_SOURCE']._serialized_start=140912
  _globals['_STATUSATTRIBUTION_AICREATEDATTRIBUTION_SOURCE']._serialized_end=140953
  _globals['_STATUSATTRIBUTION_EXTERNALSHARE']._serialized_start=140967
  _globals['_STATUSATTRIBUTION_EXTERNALSHARE']._serialized_end=141362
  _globals['_STATUSATTRIBUTION_EXTERNALSHARE_SOURCE']._serialized_start=141132
  _globals['_STATUSATTRIBUTION_EXTERNALSHARE_SOURCE']._serialized_end=141302
  _globals['_STATUSATTRIBUTION_GROUPSTATUS']._serialized_start=141364
  _globals['_STATUSATTRIBUTION_GROUPSTATUS']._serialized_end=141415
  _globals['_STATUSATTRIBUTION_MUSIC']._serialized_start=141418
  _globals['_STATUSATTRIBUTION_MUSIC']._serialized_end=141653
  _globals['_STATUSATTRIBUTION_RLATTRIBUTION']._serialized_start=141656
  _globals['_STATUSATTRIBUTION_RLATTRIBUTION']._serialized_end=141847
  _globals['_STATUSATTRIBUTION_RLATTRIBUTION_SOURCE']._serialized_start=141741
  _globals['_STATUSATTRIBUTION_RLATTRIBUTION_SOURCE']._serialized_end=141836
  _globals['_STATUSATTRIBUTION_STATUSRESHARE']._serialized_start=141850
  _globals['_STATUSATTRIBUTION_STATUSRESHARE']._serialized_end=142328
  _globals['_STATUSATTRIBUTION_STATUSRESHARE_METADATA']._serialized_start=142008
  _globals['_STATUSATTRIBUTION_STATUSRESHARE_METADATA']._serialized_end=142204
  _globals['_STATUSATTRIBUTION_STATUSRESHARE_SOURCE']._serialized_start=142206
  _globals['_STATUSATTRIBUTION_STATUSRESHARE_SOURCE']._serialized_end=142304
  _globals['_STATUSATTRIBUTION_TYPE']._serialized_start=142331
  _globals['_STATUSATTRIBUTION_TYPE']._serialized_end=142481
  _globals['_STATUSMENTIONMESSAGE']._serialized_start=142525
  _globals['_STATUSMENTIONMESSAGE']._serialized_end=142607
  _globals['_STATUSPSA']._serialized_start=142609
  _globals['_STATUSPSA']._serialized_end=142714
  _globals['_STICKERMETADATA']._serialized_start=142717
  _globals['_STICKERMETADATA']._serialized_end=143270
  _globals['_SYNCACTIONDATA']._serialized_start=143273
  _globals['_SYNCACTIONDATA']._serialized_end=143441
  _globals['_SYNCACTIONVALUE']._serialized_start=143445
  _globals['_SYNCACTIONVALUE']._serialized_end=160269
  _globals['_SYNCACTIONVALUE_AGENTACTION']._serialized_start=149035
  _globals['_SYNCACTIONVALUE_AGENTACTION']._serialized_end=149150
  _globals['_SYNCACTIONVALUE_AITHREADRENAMEACTION']._serialized_start=149152
  _globals['_SYNCACTIONVALUE_AITHREADRENAMEACTION']._serialized_end=149210
  _globals['_SYNCACTIONVALUE_ANDROIDUNSUPPORTEDACTIONS']._serialized_start=149212
  _globals['_SYNCACTIONVALUE_ANDROIDUNSUPPORTEDACTIONS']._serialized_end=149273
  _globals['_SYNCACTIONVALUE_ARCHIVECHATACTION']._serialized_start=149276
  _globals['_SYNCACTIONVALUE_ARCHIVECHATACTION']._serialized_end=149422
  _globals['_SYNCACTIONVALUE_AVATARUPDATEDACTION']._serialized_start=149425
  _globals['_SYNCACTIONVALUE_AVATARUPDATEDACTION']._serialized_end=149670
  _globals['_SYNCACTIONVALUE_AVATARUPDATEDACTION_AVATAREVENTTYPE']._serialized_start=149600
  _globals['_SYNCACTIONVALUE_AVATARUPDATEDACTION_AVATAREVENTTYPE']._serialized_end=149656
  _globals['_SYNCACTIONVALUE_BOTWELCOMEREQUESTACTION']._serialized_start=149672
  _globals['_SYNCACTIONVALUE_BOTWELCOMEREQUESTACTION']._serialized_end=149729
  _globals['_SYNCACTIONVALUE_BROADCASTLISTPARTICIPANT']._serialized_start=149731
  _globals['_SYNCACTIONVALUE_BROADCASTLISTPARTICIPANT']._serialized_end=149803
  _globals['_SYNCACTIONVALUE_BUSINESSBROADCASTASSOCIATIONACTION']._serialized_start=149805
  _globals['_SYNCACTIONVALUE_BUSINESSBROADCASTASSOCIATIONACTION']._serialized_end=149875
  _globals['_SYNCACTIONVALUE_BUSINESSBROADCASTLISTACTION']._serialized_start=149878
  _globals['_SYNCACTIONVALUE_BUSINESSBROADCASTLISTACTION']._serialized_end=150048
  _globals['_SYNCACTIONVALUE_CALLLOGACTION']._serialized_start=150050
  _globals['_SYNCACTIONVALUE_CALLLOGACTION']._serialized_end=150133
  _globals['_SYNCACTIONVALUE_CHATASSIGNMENTACTION']._serialized_start=150135
  _globals['_SYNCACTIONVALUE_CHATASSIGNMENTACTION']._serialized_end=150203
  _globals['_SYNCACTIONVALUE_CHATASSIGNMENTOPENEDSTATUSACTION']._serialized_start=150205
  _globals['_SYNCACTIONVALUE_CHATASSIGNMENTOPENEDSTATUSACTION']._serialized_end=150279
  _globals['_SYNCACTIONVALUE_CLEARCHATACTION']._serialized_start=150281
  _globals['_SYNCACTIONVALUE_CLEARCHATACTION']._serialized_end=150389
  _globals['_SYNCACTIONVALUE_CONTACTACTION']._serialized_start=150392
  _globals['_SYNCACTIONVALUE_CONTACTACTION']._serialized_end=150647
  _globals['_SYNCACTIONVALUE_CTWAPERCUSTOMERDATASHARINGACTION']._serialized_start=150649
  _globals['_SYNCACTIONVALUE_CTWAPERCUSTOMERDATASHARINGACTION']._serialized_end=150773
  _globals['_SYNCACTIONVALUE_CUSTOMPAYMENTMETHOD']._serialized_start=150776
  _globals['_SYNCACTIONVALUE_CUSTOMPAYMENTMETHOD']._serialized_end=150920
  _globals['_SYNCACTIONVALUE_CUSTOMPAYMENTMETHODMETADATA']._serialized_start=150922
  _globals['_SYNCACTIONVALUE_CUSTOMPAYMENTMETHODMETADATA']._serialized_end=150979
  _globals['_SYNCACTIONVALUE_CUSTOMPAYMENTMETHODSACTION']._serialized_start=150981
  _globals['_SYNCACTIONVALUE_CUSTOMPAYMENTMETHODSACTION']._serialized_end=151083
  _globals['_SYNCACTIONVALUE_DELETECHATACTION']._serialized_start=151085
  _globals['_SYNCACTIONVALUE_DELETECHATACTION']._serialized_end=151194
  _globals['_SYNCACTIONVALUE_DELETEINDIVIDUALCALLLOGACTION']._serialized_start=151196
  _globals['_SYNCACTIONVALUE_DELETEINDIVIDUALCALLLOGACTION']._serialized_end=151301
  _globals['_SYNCACTIONVALUE_DELETEMESSAGEFORMEACTION']._serialized_start=151303
  _globals['_SYNCACTIONVALUE_DELETEMESSAGEFORMEACTION']._serialized_end=151423
  _globals['_SYNCACTIONVALUE_DETECTEDOUTCOMESSTATUSACTION']._serialized_start=151425
  _globals['_SYNCACTIONVALUE_DETECTEDOUTCOMESSTATUSACTION']._serialized_end=151493
  _globals['_SYNCACTIONVALUE_EXTERNALWEBBETAACTION']._serialized_start=151495
  _globals['_SYNCACTIONVALUE_EXTERNALWEBBETAACTION']._serialized_end=151552
  _globals['_SYNCACTIONVALUE_FAVORITESACTION']._serialized_start=151554
  _globals['_SYNCACTIONVALUE_FAVORITESACTION']._serialized_end=151675
  _globals['_SYNCACTIONVALUE_FAVORITESACTION_FAVORITE']._serialized_start=151641
  _globals['_SYNCACTIONVALUE_FAVORITESACTION_FAVORITE']._serialized_end=151675
  _globals['_SYNCACTIONVALUE_INTERACTIVEMESSAGEACTION']._serialized_start=151678
  _globals['_SYNCACTIONVALUE_INTERACTIVEMESSAGEACTION']._serialized_end=151894
  _globals['_SYNCACTIONVALUE_INTERACTIVEMESSAGEACTION_INTERACTIVEMESSAGEACTIONMODE']._serialized_start=151798
  _globals['_SYNCACTIONVALUE_INTERACTIVEMESSAGEACTION_INTERACTIVEMESSAGEACTIONMODE']._serialized_end=151894
  _globals['_SYNCACTIONVALUE_KEYEXPIRATION']._serialized_start=151896
  _globals['_SYNCACTIONVALUE_KEYEXPIRATION']._serialized_end=151961
  _globals['_SYNCACTIONVALUE_LABELASSOCIATIONACTION']._serialized_start=151963
  _globals['_SYNCACTIONVALUE_LABELASSOCIATIONACTION']._serialized_end=152021
  _globals['_SYNCACTIONVALUE_LABELEDITACTION']._serialized_start=152024
  _globals['_SYNCACTIONVALUE_LABELEDITACTION']._serialized_end=152573
  _globals['_SYNCACTIONVALUE_LABELEDITACTION_LISTTYPE']._serialized_start=152302
  _globals['_SYNCACTIONVALUE_LABELEDITACTION_LISTTYPE']._serialized_end=152454
  _globals['_SYNCACTIONVALUE_LABELREORDERINGACTION']._serialized_start=152575
  _globals['_SYNCACTIONVALUE_LABELREORDERINGACTION']._serialized_end=152622
  _globals['_SYNCACTIONVALUE_LIDCONTACTACTION']._serialized_start=152625
  _globals['_SYNCACTIONVALUE_LIDCONTACTACTION']._serialized_end=152753
  _globals['_SYNCACTIONVALUE_LOCALESETTING']._serialized_start=152755
  _globals['_SYNCACTIONVALUE_LOCALESETTING']._serialized_end=152802
  _globals['_SYNCACTIONVALUE_LOCKCHATACTION']._serialized_start=152804
  _globals['_SYNCACTIONVALUE_LOCKCHATACTION']._serialized_end=152852
  _globals['_SYNCACTIONVALUE_MAIBAAIFEATURESCONTROLACTION']._serialized_start=152855
  _globals['_SYNCACTIONVALUE_MAIBAAIFEATURESCONTROLACTION']._serialized_end=153086
  _globals['_SYNCACTIONVALUE_MAIBAAIFEATURESCONTROLACTION_MAIBAAIFEATURESTATUS']._serialized_start=152991
  _globals['_SYNCACTIONVALUE_MAIBAAIFEATURESCONTROLACTION_MAIBAAIFEATURESTATUS']._serialized_end=153066
  _globals['_SYNCACTIONVALUE_MARKCHATASREADACTION']._serialized_start=153089
  _globals['_SYNCACTIONVALUE_MARKCHATASREADACTION']._serialized_end=153230
  _globals['_SYNCACTIONVALUE_MARKETINGMESSAGEACTION']._serialized_start=153233
  _globals['_SYNCACTIONVALUE_MARKETINGMESSAGEACTION']._serialized_end=153625
  _globals['_SYNCACTIONVALUE_MARKETINGMESSAGEACTION_MARKETINGMESSAGEPROTOTYPETYPE']._serialized_start=153491
  _globals['_SYNCACTIONVALUE_MARKETINGMESSAGEACTION_MARKETINGMESSAGEPROTOTYPETYPE']._serialized_end=153540
  _globals['_SYNCACTIONVALUE_MARKETINGMESSAGEBROADCASTACTION']._serialized_start=153627
  _globals['_SYNCACTIONVALUE_MARKETINGMESSAGEBROADCASTACTION']._serialized_end=153704
  _globals['_SYNCACTIONVALUE_MERCHANTPAYMENTPARTNERACTION']._serialized_start=153707
  _globals['_SYNCACTIONVALUE_MERCHANTPAYMENTPARTNERACTION']._serialized_end=153952
  _globals['_SYNCACTIONVALUE_MERCHANTPAYMENTPARTNERACTION_STATUS']._serialized_start=153885
  _globals['_SYNCACTIONVALUE_MERCHANTPAYMENTPARTNERACTION_STATUS']._serialized_end=153919
  _globals['_SYNCACTIONVALUE_MUSICUSERIDACTION']._serialized_start=153955
  _globals['_SYNCACTIONVALUE_MUSICUSERIDACTION']._serialized_end=154160
  _globals['_SYNCACTIONVALUE_MUSICUSERIDACTION_MUSICUSERIDMAPENTRY']._serialized_start=154091
  _globals['_SYNCACTIONVALUE_MUSICUSERIDACTION_MUSICUSERIDMAPENTRY']._serialized_end=154144
  _globals['_SYNCACTIONVALUE_MUTEACTION']._serialized_start=154163
  _globals['_SYNCACTIONVALUE_MUTEACTION']._serialized_end=154295
  _globals['_SYNCACTIONVALUE_NEWSLETTERSAVEDINTERESTSACTION']._serialized_start=154297
  _globals['_SYNCACTIONVALUE_NEWSLETTERSAVEDINTERESTSACTION']._serialized_end=154397
  _globals['_SYNCACTIONVALUE_NOTEEDITACTION']._serialized_start=154400
  _globals['_SYNCACTIONVALUE_NOTEEDITACTION']._serialized_end=154729
  _globals['_SYNCACTIONVALUE_NOTEEDITACTION_NOTETYPE']._serialized_start=154587
  _globals['_SYNCACTIONVALUE_NOTEEDITACTION_NOTETYPE']._serialized_end=154658
  _globals['_SYNCACTIONVALUE_NOTIFICATIONACTIVITYSETTINGACTION']._serialized_start=154732
  _globals['_SYNCACTIONVALUE_NOTIFICATIONACTIVITYSETTINGACTION']._serialized_end=155042
  _globals['_SYNCACTIONVALUE_NOTIFICATIONACTIVITYSETTINGACTION_NOTIFICATIONACTIVITYSETTING']._serialized_start=154897
  _globals['_SYNCACTIONVALUE_NOTIFICATIONACTIVITYSETTINGACTION_NOTIFICATIONACTIVITYSETTING']._serialized_end=155010
  _globals['_SYNCACTIONVALUE_NUXACTION']._serialized_start=155044
  _globals['_SYNCACTIONVALUE_NUXACTION']._serialized_end=155099
  _globals['_SYNCACTIONVALUE_PAYMENTINFOACTION']._serialized_start=155101
  _globals['_SYNCACTIONVALUE_PAYMENTINFOACTION']._serialized_end=155146
  _globals['_SYNCACTIONVALUE_PAYMENTTOSACTION']._serialized_start=155149
  _globals['_SYNCACTIONVALUE_PAYMENTTOSACTION']._serialized_end=155307
  _globals['_SYNCACTIONVALUE_PAYMENTTOSACTION_PAYMENTNOTICE']._serialized_start=155265
  _globals['_SYNCACTIONVALUE_PAYMENTTOSACTION_PAYMENTNOTICE']._serialized_end=155307
  _globals['_SYNCACTIONVALUE_PINACTION']._serialized_start=155309
  _globals['_SYNCACTIONVALUE_PINACTION']._serialized_end=155352
  _globals['_SYNCACTIONVALUE_PNFORLIDCHATACTION']._serialized_start=155354
  _globals['_SYNCACTIONVALUE_PNFORLIDCHATACTION']._serialized_end=155404
  _globals['_SYNCACTIONVALUE_PRIMARYFEATURE']._serialized_start=155406
  _globals['_SYNCACTIONVALUE_PRIMARYFEATURE']._serialized_end=155437
  _globals['_SYNCACTIONVALUE_PRIMARYVERSIONACTION']._serialized_start=155439
  _globals['_SYNCACTIONVALUE_PRIMARYVERSIONACTION']._serialized_end=155495
  _globals['_SYNCACTIONVALUE_PRIVACYSETTINGCHANNELSPERSONALISEDRECOMMENDATIONACTION']._serialized_start=155497
  _globals['_SYNCACTIONVALUE_PRIVACYSETTINGCHANNELSPERSONALISEDRECOMMENDATIONACTION']._serialized_end=155601
  _globals['_SYNCACTIONVALUE_PRIVACYSETTINGDISABLELINKPREVIEWSACTION']._serialized_start=155603
  _globals['_SYNCACTIONVALUE_PRIVACYSETTINGDISABLELINKPREVIEWSACTION']._serialized_end=155700
  _globals['_SYNCACTIONVALUE_PRIVACYSETTINGRELAYALLCALLS']._serialized_start=155702
  _globals['_SYNCACTIONVALUE_PRIVACYSETTINGRELAYALLCALLS']._serialized_end=155769
  _globals['_SYNCACTIONVALUE_PRIVATEPROCESSINGSETTINGACTION']._serialized_start=155772
  _globals['_SYNCACTIONVALUE_PRIVATEPROCESSINGSETTINGACTION']._serialized_end=156018
  _globals['_SYNCACTIONVALUE_PRIVATEPROCESSINGSETTINGACTION_PRIVATEPROCESSINGSTATUS']._serialized_start=155923
  _globals['_SYNCACTIONVALUE_PRIVATEPROCESSINGSETTINGACTION_PRIVATEPROCESSINGSTATUS']._serialized_end=155990
  _globals['_SYNCACTIONVALUE_PUSHNAMESETTING']._serialized_start=156020
  _globals['_SYNCACTIONVALUE_PUSHNAMESETTING']._serialized_end=156065
  _globals['_SYNCACTIONVALUE_QUICKREPLYACTION']._serialized_start=156068
  _globals['_SYNCACTIONVALUE_QUICKREPLYACTION']._serialized_end=156238
  _globals['_SYNCACTIONVALUE_RECENTEMOJIWEIGHTSACTION']._serialized_start=156240
  _globals['_SYNCACTIONVALUE_RECENTEMOJIWEIGHTSACTION']._serialized_end=156309
  _globals['_SYNCACTIONVALUE_REMOVERECENTSTICKERACTION']._serialized_start=156311
  _globals['_SYNCACTIONVALUE_REMOVERECENTSTICKERACTION']._serialized_end=156392
  _globals['_SYNCACTIONVALUE_STARACTION']._serialized_start=156394
  _globals['_SYNCACTIONVALUE_STARACTION']._serialized_end=156440
  _globals['_SYNCACTIONVALUE_STATUSPOSTOPTINNOTIFICATIONPREFERENCESACTION']._serialized_start=156442
  _globals['_SYNCACTIONVALUE_STATUSPOSTOPTINNOTIFICATIONPREFERENCESACTION']._serialized_end=156522
  _globals['_SYNCACTIONVALUE_STATUSPRIVACYACTION']._serialized_start=156525
  _globals['_SYNCACTIONVALUE_STATUSPRIVACYACTION']._serialized_end=156748
  _globals['_SYNCACTIONVALUE_STATUSPRIVACYACTION_STATUSDISTRIBUTIONMODE']._serialized_start=156651
  _globals['_SYNCACTIONVALUE_STATUSPRIVACYACTION_STATUSDISTRIBUTIONMODE']._serialized_end=156739
  _globals['_SYNCACTIONVALUE_STICKERACTION']._serialized_start=156751
  _globals['_SYNCACTIONVALUE_STICKERACTION']._serialized_end=157260
  _globals['_SYNCACTIONVALUE_SUBSCRIPTIONACTION']._serialized_start=157263
  _globals['_SYNCACTIONVALUE_SUBSCRIPTIONACTION']._serialized_end=157425
  _globals['_SYNCACTIONVALUE_SYNCACTIONMESSAGE']._serialized_start=157427
  _globals['_SYNCACTIONVALUE_SYNCACTIONMESSAGE']._serialized_end=157529
  _globals['_SYNCACTIONVALUE_SYNCACTIONMESSAGERANGE']._serialized_start=157532
  _globals['_SYNCACTIONVALUE_SYNCACTIONMESSAGERANGE']._serialized_end=157748
  _globals['_SYNCACTIONVALUE_TIMEFORMATACTION']._serialized_start=157750
  _globals['_SYNCACTIONVALUE_TIMEFORMATACTION']._serialized_end=157846
  _globals['_SYNCACTIONVALUE_UGCBOT']._serialized_start=157848
  _globals['_SYNCACTIONVALUE_UGCBOT']._serialized_end=157896
  _globals['_SYNCACTIONVALUE_UNARCHIVECHATSSETTING']._serialized_start=157898
  _globals['_SYNCACTIONVALUE_UNARCHIVECHATSSETTING']._serialized_end=157969
  _globals['_SYNCACTIONVALUE_USERSTATUSMUTEACTION']._serialized_start=157971
  _globals['_SYNCACTIONVALUE_USERSTATUSMUTEACTION']._serialized_end=158023
  _globals['_SYNCACTIONVALUE_USERNAMECHATSTARTMODEACTION']._serialized_start=158026
  _globals['_SYNCACTIONVALUE_USERNAMECHATSTARTMODEACTION']._serialized_end=158234
  _globals['_SYNCACTIONVALUE_USERNAMECHATSTARTMODEACTION_CHATSTARTMODE']._serialized_start=158151
  _globals['_SYNCACTIONVALUE_USERNAMECHATSTARTMODEACTION_CHATSTARTMODE']._serialized_end=158216
  _globals['_SYNCACTIONVALUE_WAFFLEACCOUNTLINKSTATEACTION']._serialized_start=158237
  _globals['_SYNCACTIONVALUE_WAFFLEACCOUNTLINKSTATEACTION']._serialized_end=158433
  _globals['_SYNCACTIONVALUE_WAFFLEACCOUNTLINKSTATEACTION_ACCOUNTLINKSTATE']._serialized_start=158363
  _globals['_SYNCACTIONVALUE_WAFFLEACCOUNTLINKSTATEACTION_ACCOUNTLINKSTATE']._serialized_end=158419
  _globals['_SYNCACTIONVALUE_WAMOUSERIDENTIFIERACTION']._serialized_start=158435
  _globals['_SYNCACTIONVALUE_WAMOUSERIDENTIFIERACTION']._serialized_end=158501
  _globals['_SYNCDINDEX']._serialized_start=160271
  _globals['_SYNCDINDEX']._serialized_end=160311
  _globals['_SYNCDMUTATION']._serialized_start=160314
  _globals['_SYNCDMUTATION']._serialized_end=160495
  _globals['_SYNCDMUTATION_SYNCDOPERATION']._serialized_start=160433
  _globals['_SYNCDMUTATION_SYNCDOPERATION']._serialized_end=160470
  _globals['_SYNCDMUTATIONS']._serialized_start=160497
  _globals['_SYNCDMUTATIONS']._serialized_end=160554
  _globals['_SYNCDPATCH']._serialized_start=160557
  _globals['_SYNCDPATCH']._serialized_end=161016
  _globals['_SYNCDRECORD']._serialized_start=161019
  _globals['_SYNCDRECORD']._serialized_end=161174
  _globals['_SYNCDSNAPSHOT']._serialized_start=161177
  _globals['_SYNCDSNAPSHOT']._serialized_end=161354
  _globals['_SYNCDVALUE']._serialized_start=161356
  _globals['_SYNCDVALUE']._serialized_end=161396
  _globals['_SYNCDVERSION']._serialized_start=161398
  _globals['_SYNCDVERSION']._serialized_end=161446
  _globals['_TAPLINKACTION']._serialized_start=161448
  _globals['_TAPLINKACTION']._serialized_end=161525
  _globals['_TEMPLATEBUTTON']._serialized_start=161528
  _globals['_TEMPLATEBUTTON']._serialized_end=162229
  _globals['_TEMPLATEBUTTON_CALLBUTTON']._serialized_start=161745
  _globals['_TEMPLATEBUTTON_CALLBUTTON']._serialized_end=161921
  _globals['_TEMPLATEBUTTON_QUICKREPLYBUTTON']._serialized_start=161923
  _globals['_TEMPLATEBUTTON_QUICKREPLYBUTTON']._serialized_end=162047
  _globals['_TEMPLATEBUTTON_URLBUTTON']._serialized_start=162050
  _globals['_TEMPLATEBUTTON_URLBUTTON']._serialized_end=162209
  _globals['_THREADID']._serialized_start=162232
  _globals['_THREADID']._serialized_end=162427
  _globals['_THREADID_THREADTYPE']._serialized_start=162340
  _globals['_THREADID_THREADTYPE']._serialized_end=162398
  _globals['_URLTRACKINGMAP']._serialized_start=162430
  _globals['_URLTRACKINGMAP']._serialized_end=162741
  _globals['_URLTRACKINGMAP_URLTRACKINGMAPELEMENT']._serialized_start=162526
  _globals['_URLTRACKINGMAP_URLTRACKINGMAPELEMENT']._serialized_end=162741
  _globals['_USERPASSWORD']._serialized_start=162744
  _globals['_USERPASSWORD']._serialized_end=163303
  _globals['_USERPASSWORD_TRANSFORMERARG']._serialized_start=162963
  _globals['_USERPASSWORD_TRANSFORMERARG']._serialized_end=163142
  _globals['_USERPASSWORD_TRANSFORMERARG_VALUE']._serialized_start=163061
  _globals['_USERPASSWORD_TRANSFORMERARG_VALUE']._serialized_end=163124
  _globals['_USERPASSWORD_ENCODING']._serialized_start=163144
  _globals['_USERPASSWORD_ENCODING']._serialized_end=163181
  _globals['_USERPASSWORD_TRANSFORMER']._serialized_start=163183
  _globals['_USERPASSWORD_TRANSFORMER']._serialized_end=163254
  _globals['_USERRECEIPT']._serialized_start=163306
  _globals['_USERRECEIPT']._serialized_end=163538
  _globals['_VERIFIEDNAMECERTIFICATE']._serialized_start=163541
  _globals['_VERIFIEDNAMECERTIFICATE']._serialized_end=163892
  _globals['_VERIFIEDNAMECERTIFICATE_DETAILS']._serialized_start=163645
  _globals['_VERIFIEDNAMECERTIFICATE_DETAILS']._serialized_end=163846
  _globals['_WALLPAPERSETTINGS']._serialized_start=163894
  _globals['_WALLPAPERSETTINGS']._serialized_end=163983
  _globals['_WEBFEATURES']._serialized_start=163986
  _globals['_WEBFEATURES']._serialized_end=167561
  _globals['_WEBFEATURES_FLAG']._serialized_start=166531
  _globals['_WEBFEATURES_FLAG']._serialized_end=166606
  _globals['_WEBMESSAGEINFO']._serialized_start=167564
  _globals['_WEBMESSAGEINFO']._serialized_end=179155
  _globals['_WEBMESSAGEINFO_BIZPRIVACYSTATUS']._serialized_start=170289
  _globals['_WEBMESSAGEINFO_BIZPRIVACYSTATUS']._serialized_end=170350
  _globals['_WEBMESSAGEINFO_STATUS']._serialized_start=170352
  _globals['_WEBMESSAGEINFO_STATUS']._serialized_end=170440
  _globals['_WEBMESSAGEINFO_STUBTYPE']._serialized_start=170443
  _globals['_WEBMESSAGEINFO_STUBTYPE']._serialized_end=178022
  _globals['_WEBNOTIFICATIONSINFO']._serialized_start=179158
  _globals['_WEBNOTIFICATIONSINFO']._serialized_end=179363
# @@protoc_insertion_point(module_scope)
